#!/usr/bin/perl -w
#
# Explain to the user that the URL is blocked and by which rule set
#
# By P?l Baltzersen 1999 (pal.baltzersen@ost.eltele.no)
# French texts thanks to Fabrice Prigent (fabrice.prigent@univ-tlse1.fr)
# Dutch texts thanks to Anneke Sicherer-Roetman (sicherer@sichemsoft.nl)
# German texts thanks to Buergernetz Pfaffenhofen (www.bn-paf.de/filter/)
# Italian texts thanks to Valerio M. Felici (valerio@bgweb.it)
#
# The last version may be found anytime at:
#    http://ftp.ost.eltele.no/pub/www/proxy/squidGuard/contrib/
#

# By accepting this notice, you agree to be bound by the following
# agreements:
# 
# This software product, squidGuard, is copyrighted (C) 1999 by ElTele
# ?st AS, Oslo, Norway, with all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License (version 2) as
# published by the Free Software Foundation.  It is distributed in the
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even
# the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License (GPL) for more details.
# 
# You should have received a copy of the GNU General Public License
# (GPL) along with this program.

use strict;
use Socket;
# -----------------------------------------------------------------------
# GLOBAL VALUES:
# -----------------------------------------------------------------------
my ($clientaddr,$clientname,$clientident,$srcclass,$targetclass,$url);
my ($lang,@supported,$image,$redirect,$autoinaddr,$proxy,$proxymaster);
my (%msgconf,%title,%logo,%msg,%tab,%word);
my ($protocol,$address,$port,$path,$refererhost,$referer);
sub msginit();
sub getpreferedlang(@);
sub parsequery($$);
sub status($);
sub redirect($);
sub content($);
sub expires($);
sub title($);
sub terminator();
sub msg($$);
sub table($$@);
sub href($);
sub gethostnames($);
sub spliturl($);
sub showhtml($);
sub showimage($$$);
sub showinaddr($$$$$);

# ------------------------------------------------------------------------
# CONFIGURABLE OPTIONS:
# ------------------------------------------------------------------------

# SUPPORTED LANGUAGES
# at this time:
# "en", "fr", "de", "it", "nl", "no"
#
# add more, if you want... ;-)
#
@supported   = (
		"en (English),",
		"fr (French),",
		"de (German),",
		"it (Italian),",
		"nl (Dutch),",	
		"no (Norwegian)."
	       );

# SOME MORE OPTIONS
# ALL PATHS ARE RELATIVE TO DOCUMENT_ROOT
$image       = "/stop.gif\"";			# THE IMAGE SHOWN ON THE "default" ERROR SITE
$redirect    = "";				# "" TO AVOID REDIRECTION
$proxy       = "proxy.#DOMAIN#";		# THE NAME OF THE HOST WHO RUNS THE SCRIPT
$proxymaster = "mailadmin\@#DOMAIN#";	# THE ADMINISTRATOR MAIL - USE \@ to ESCAPE THE @
$autoinaddr  =  0;				# 0|1|2;
						# 0 TO NOT REDIRECT
						# 1 TO AUTORESOLVE & REDIRECT IF UNIQUE
						# 2 TO AUTORESOLVE & REDIRECT TO FIRST NAME


# ------------------------------------------------------------------------
# CONFIGURABLE MESSAGES:
# ------------------------------------------------------------------------
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# !!! NOTE1: ALLWAYS ESCAPE EMBEDDED VARIABLES (I.E. \$var)         !!!
# !!!	     IF YOU DON'T YOU MAY OPEN A SECURITY HOLE	            !!!
# !!! NOTE2: TRIPLE ESCAPE EMBEDDED `\', `"', `$', `@', `%' and `&' !!!
# !!!	     (I.E. \\\\, \\\", \\\$, \\\@, \\\&)	     	        !!!
# !!! NOTE3: ESCAPE OTHER SPECIAL INLINE CHARACTERS	          		!!!
# !!!	     (I.E. \;, \')				           					!!!
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# ------------------------------------------------------------------------

sub msginit() {
  $lang = getpreferedlang(@supported);
  ($clientaddr,$clientname,$clientident,$srcclass,$targetclass,$url) = parsequery($ENV{"QUERY_STRING"}, $lang);
  ($protocol,$address,$port,$path) = spliturl($url);


  # THE WORD "unknown" IN DIFFERENT LANGUAGES
  $word{"unknown"}->{"en"}= "unknown";
  $word{"unknown"}->{"fr"}= "inconnu";
  $word{"unknown"}->{"de"}= "unbekannt"; 
  $word{"unknown"}->{"it"}= "sconosciuto"; 
  $word{"unknown"}->{"nl"}= "onbekend"; 
  $word{"unknown"}->{"no"}= "ukjent"; 

  # TITLE OF THE "default" SITE
  $title{"default"}->{"en"}= [ "403 Access denied" ]; 
  $title{"default"}->{"fr"}= [ "403 Interdit" ]; 
  $title{"default"}->{"de"}= [ "403 Zugriff verweigert" ];
  $title{"default"}->{"it"}= [ "403 Vietato" ]; 
  $title{"default"}->{"nl"}= [ "403 Verboden" ]; 
  $title{"default"}->{"no"}= [ "403 Vietato" ];


  # ---------------------------------------------------------------------
  # MESSAGE LAYOUT CONFIGURATION
  # The language specific entries are defined below
  # ---------------------------------------------------------------------

  # The "default" message config (used when no other message applies)
  #
  # msg:H1:default    ==> show "default" message as <H1>
  # tab:R,C,L:info    ==> show the next elements as <TABLE>;
  #                       columns aligned R,C,L
  # clientaddr 
  # clientname 
  # clientident       ==> table content (defined below) 
  # srcclass 
  # url 
  # targetclass
  #
  # msg:P:proxymaster ==> show "proxymaster" as <P>
  # msg:P:refresh     ==> show "refresh" as <P>
  #
  $msgconf{"default"}	= [ "msg:H1:default",							
							"tab:R,C,L:info"
							. ":clientaddr"
							. ":clientname"	
							. ":clientident"
							. ":srcclass"
							. ":url"
							. ":targetclass",
							"msg:P:proxymaster",
							"msg:P:refresh"	
      					  ];

  # The "unknown" client message config (used, if no ACL matches)
  #
  # msg:H1:unknown    ==> show "unknown" message as <H1>
  # tab:R,C,L:info    ==> show the next elements as <TABLE>;
  #                       columns aligned R,C,L
  # clientaddr
  # clientname        ==> table content (defined below)
  # clientident
  # srcclass
  # 
  # msg:P:proxymaster ==> show "proxymaster" as <P>
  # msg:P:refresh     ==> show "refresh" as <P>
  #
  $msgconf{"unknown"}	= [ "msg:H1:unknown",
							"tab:R,C,L:info"
							. ":clientaddr"	
							. ":clientname"
							. ":clientident"
							. ":srcclass",
							"msg:P:proxymaster",
							"msg:P:refresh"	
				          ];

  # Define a language specific word for "unknown"
  $msgconf{$word{"unknown"}->{$lang}}= $msgconf{"unknown"};

  # The "in-addr destination group" client message config (used for in-addr ACLs)
  # 
  # msg:H1:alternatives ==> show "alternatives" message as <H1> 
  # alternatives        ==> show the alternativ domain address(es)
  # referermaster       ==> show "referermaster"
  # msg:P:refresh       ==> show "refresh" as <P>
  #
  $msgconf{"in-addr"}	= [ "msg:H1:alternatives",
							"alternatives",	
							"referermaster",
							"msg:P:refresh"	
				      	  ];
  
  # The "in-addr destination group" client message config - if there are no domain 
  # address alternatives (used for in-addr ACLs)
  #
  # msg:H1:in-addr        ==> show "in-addr" as <H1>
  # tab:R,C,L:info        ==> show the next elements as <TABLE>;
  #                           columns aligned R,C,L
  # clientaddr
  # clientname
  # clientident           ==> table content (defined below)
  # srcclass
  # domainurl
  # targetclass
  #
  # msg:H3:noalternatives ==> show "noalternatives" as <H3>
  # msg:P:webmaster       ==> show "webmaster" as <P>
  # msg:P:refresh         ==> show "refresh" as <P>
  #
  $msgconf{"noalternatives"} = [ 	"msg:H1:in-addr",
									"tab:R,C,L:info"
									. ":clientaddr"
									. ":clientname"
									. ":clientident"
									. ":srcclass"
									. ":domainurl"
									. ":targetclass",
									"msg:H3:noalternatives",
									"msg:P:webmaster",
									"msg:P:refresh"	
      							 ];

  # ---------------------------------------------------------------------
  # MESSAGE LANGUAGE CONFIGURATION
  # ---------------------------------------------------------------------
 

  # The message text "default" in different languages
  $msg{"default"}->{"en"}	= [ "Access to this site is blocked" ];	
  $msg{"default"}->{"fr"}	= [ "L\'acc?s ? ce site est bloqu?" ];	
  $msg{"default"}->{"de"}	= [ "Der Zugriff zu dieser Seite wird verweigert" ];
  $msg{"default"}->{"it"}   = [ "Spiacente ma l\'accesso a questo sito non ? 
								conforme allo stile di questa Istituzione ed ? 
								pertanto interdetto" ];	
  $msg{"default"}->{"nl"}	= [ "De toegang is geblokkeerd" ];
  $msg{"default"}->{"no"}   = [ "Siden er sperret" ];

  #
  # The "info" table in different languages
  #

  # English (en)
  $tab{"info"}->{"caption"}->{"en"} 	= [ "Additional information:" ];
  if (defined($clientaddr))
  {
  	$tab{"info"}->{"clientaddr"}->{"en"} 	= [ "Client address", "=", "\$clientaddr" ];
  }
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"en"} 	= [ "Client name", "=", "\$clientname" ];
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"en"} = [ "Client user", "=", "\$clientident" ];
  }
  if (defined($srcclass))
  {
  	$tab{"info"}->{"srcclass"}->{"en"} 	= [ "Client group", "=", "\$srcclass" ];
  }
  if (defined($url))
  {
  	$tab{"info"}->{"url"}->{"en"} 		= [ "URL", "=", "\$url" ];
  }
  $tab{"info"}->{"domainurl"}->{"en"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
  	$tab{"info"}->{"targetclass"}->{"en"} = [ "Target group", "=", "\$targetclass" ];
  }

  # French (fr)  
  $tab{"info"}->{"caption"}->{"fr"}		= [ "Information compl?mentaire:" ];			
  if (defined($clientaddr))
  {
  	$tab{"info"}->{"clientaddr"}->{"fr"}	= [ "Adresse de la machine", "=", "\$clientaddr" ];
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"fr"}	= [ "Nom de la machine", "=", "\$clientname" ]; 
  }
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"fr"}	= [ "Utilisateur", "=", "\$clientident" ];		
  }
  if (defined($srcclass))
  {
  	$tab{"info"}->{"srcclass"}->{"fr"}	= [ "Groupe", "=", "\$srcclass" ];
  }
  if (defined($url))
  {
  	$tab{"info"}->{"url"}->{"fr"}		    = [ "URL", "=", "\$url" ];			
  }
  $tab{"info"}->{"domainurl"}->{"fr"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
  	$tab{"info"}->{"targetclass"}->{"fr"}	= [ "Groupe cible", "=", "\$targetclass" ];	
  }

  # German (de)
  $tab{"info"}->{"caption"}->{"de"}		= [ "Zusatzinformationen:" ];		
  if (defined($clientaddr))
  {
  	$tab{"info"}->{"clientaddr"}->{"de"}	= [ "IP-Adresse", "=", "\$clientaddr" ];
  }
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"de"}	= [ "Rechnername", "=", "\$clientname" ];
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"de"}	= [ "Benutzer", "=", "\$clientident" ];	
  }
  if (defined($srcclass))
  {
  	$tab{"info"}->{"srcclass"}->{"de"}	= [ "Gruppe", "=", "\$srcclass" ];	
  }
  if (defined($url))
  {
  	$tab{"info"}->{"url"}->{"de"}		    = [ "URL", "=", "\$url" ];			
  }
  $tab{"info"}->{"domainurl"}->{"de"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
  	$tab{"info"}->{"targetclass"}->{"de"}	= [ "Klassifizierung", "=", "\$targetclass" ];
  }

  # Italian (de)  
  $tab{"info"}->{"caption"}->{"it"}		= [ "Informazioni aggiuntive:" ];		
  if (defined($clientaddr))
  {
  	$tab{"info"}->{"clientaddr"}->{"it"}	= [ "Indirizzo IP del tuo PC", "=", "\$clientaddr" ];
  }
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"it"}	= [ "Nome in rete locale del PC", "=", "\$clientname" ];	
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"it"}	= [ "Utente", "=", "\$clientident" ];	
  }
  if (defined($srcclass))
  {
  	$tab{"info"}->{"srcclass"}->{"it"}    = [ "Gruppo", "=", "\$srcclass" ];
  }
  if (defined($url))
  {
	$tab{"info"}->{"url"}->{"it"}	        = [ "URL", "=", "\$url" ];			
  }
  $tab{"info"}->{"domainurl"}->{"it"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
  	$tab{"info"}->{"targetclass"}->{"it"} = [ "Target dell\'URL", "=", "\$targetclass" ];	
  }

  
  # Dutch (nl)
  $tab{"info"}->{"caption"}->{"nl"}		= [ "Extra informatie:" ];			
  if (defined($clientaddr))
  {
  	$tab{"info"}->{"clientaddr"}->{"nl"}	= [ "Computeradres", "=", "\$clientaddr" ];	
  }
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"nl"}	= [ "Computernaam", "=", "\$clientname" ];	
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"nl"} = [ "Gebruiker", "=", "\$clientident" ];
  }
  if (defined($srcclass))
  {
  	$tab{"info"}->{"srcclass"}->{"nl"}	= [ "Groep", "=", "\$srcclass" ];	
  }
  if (defined($url))
  {
	$tab{"info"}->{"url"}->{"nl"}		    = [ "URL", "=", "\$url" ];			
  }
  $tab{"info"}->{"domainurl"}->{"nl"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
	  $tab{"info"}->{"targetclass"}->{"nl"} = [ "Doelgroep", "=", "\$targetclass" ];
  }

  # Norwegian (no)
  $tab{"info"}->{"caption"}->{"no"}		= [ "Tilleggsinformasjon:" ];		
  if (defined($clientaddr))
  {
	$tab{"info"}->{"clientaddr"}->{"no"}	= [ "Klientadresse", "=", "\$clientaddr" ];	
  }
  if (defined($clientname))
  {
  	$tab{"info"}->{"clientname"}->{"no"}	= [ "Klientnavn", "=", "\$clientname" ];
  }
  if (defined($clientident))
  {
  	$tab{"info"}->{"clientident"}->{"no"}	= [ "Brukerident", "=", "\$clientident" ];
  }
  if (defined($srcclass))
  {
	$tab{"info"}->{"srcclass"}->{"no"}    = [ "Klientgruppe", "=", "\$srcclass" ];
  }
  if (defined($url))
  {
  	$tab{"info"}->{"url"}->{"no"}         = [ "URL", "=", "\$url" ];	
  }
  $tab{"info"}->{"domainurl"}->{"no"}	= [ "URL", "=", "\$protocol://<U>\$address</U>\$port\$path" ];
  if (defined($targetclass))
  {
	  $tab{"info"}->{"targetclass"}->{"no"} = [ "M?lkategori", "=", "\$targetclass" ];
  }

  #
  # The "proxymaster" message in different languages
  #

  # English (en)
  $msg{"proxymaster"}->{"en"}
    = [ "If you think this is an error, send <A HREF=\\\"\$url\\\"><U>this page</U></A> to",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  # French (fr)
  $msg{"proxymaster"}->{"fr"}
    = [ "Si vous pensez qu\'il s\'agit d\'une erreur, envoyez <A HREF=\\\"\$url\\\"><U>cette page</U></A> ?",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  # German (de)
  $msg{"proxymaster"}->{"de"}
    = [ "Falls ein Fehler vorliegt schicken Sie die Adresse <A HREF=\\\"\$url\\\"><U>dieser Seite</U></A> bitte an",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  # Italian (it)
  $msg{"proxymaster"}->{"it"}
    = [ "Se pensi che sia un errore invia <A HREF=\\\"\$url\\\"><U>questa pagina</U></A> a",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  # Dutch (nl)
  $msg{"proxymaster"}->{"nl"}
    = [ "Als u denkt dat dit onjuist is, zend <A HREF=\\\"\$url\\\"><U>deze bladzijde</U></A> aan",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  # Norwegian (no)
  $msg{"proxymaster"}->{"no"}
    = [ "Om du mener dette er feil, s? send <A HREF=\\\"\$url\\\"><U>denne siden</U></A> til",
	"<A HREF=\\\"mailto:\$proxymaster\\\">\$proxymaster</A>" ];

  #
  # The "refresh" message in different languages
  #

  # English (en)
  $msg{"refresh"}->{"en"}
    = [ "You may need to use the browser's \&lt\;Reload\&gt\; button<BR>",
	"or even \&lt\;Keyboard Shift\&gt\;+\&lt\;Browser Reload\&gt\;<BR>",
	"to get rid of this page after an access rule change" ];

  # French (fr)
  $msg{"refresh"}->{"fr"}
    = [ "Vous avez peut-?tre besoin d\'utiliser le bouton \&lt\;Recharger\&gt\;<BR>",
	"ou m?me \&lt\;Shift\&gt\;+\&lt\;Recharger\&gt\;<BR>",
	"apr?s un changement de r?gles" ];

  # German (de)
  $msg{"refresh"}->{"de"}
    = [ "Nach einer &Auml;nderung der Zugriffsrechte m&uuml;ssen Sie evtl. die Seite<BR>",
	"mit dem \&lt\;Aktualisieren\&gt\; bzw. \&lt\;Neu laden\&gt\; Button<BR>",
	"des Browsers oder sogar mit \&lt\;Strg\&gt\;+\&lt\;F5\&gt\;<BR>",
	"erneut laden lassen." ];

  # Italian (it)
  $msg{"refresh"}->{"it"}
    = [ "Pu? essere necessario usare il pulsante \&lt\;Aggiorna\&gt\; del browser<BR>",
	"o anche \&lt\;Shift\&gt\;+\&lt\;Aggiorna\&gt\;<BR>",
	"per ottenere l\'accesso alla pagina desiderata dopo il cambiamento delle regole" ];

  # Dutch (nl)
  $msg{"refresh"}->{"nl"}
    = [ "U moet waarschijnlijk de browser's \&lt\;Reload\&gt\; knop gebruiken<BR>",
	"of zelfs \&lt\;Shift\&gt\;+\&lt\;Reload\&gt\;<BR>",
	"na een verandering in de squidGuard regels" ];

  # Norwegian (no)
  $msg{"refresh"}->{"no"}
    = [ "Du kan trenge ? bruke browserens \&lt\;Reload\&gt\; knapp<BR>",
	"eller til og med",
	"\&lt\;Tastatur Shift\&gt\;+\&lt\;Browser Reload\&gt\;<BR>",
	"for ? bli kvitt denne siden etter endring i adgangskontrollen" ];
  
  #
  # The "timerefresh" message in different languages
  #

  # English (en)
  $msg{"timerefresh"}->{"en"}
    = [ "You may need to use the browser's \&lt\;Reload\&gt\; button<BR>",
	"or even \&lt\;Keyboard Shift\&gt\;+\&lt\;Browser Reload\&gt\;<BR>",
	"to get rid of this page after transition from<BR>",
	"a time zone with access restrictions" ];

  # French (fr)
  $msg{"timerefresh"}->{"fr"}
    = [ "Vous avez peut-?tre besoin d\'utiliser le bouton \&lt\;Recharger\&gt\;<BR>",
	"ou m?me \&lt\;Shift\&gt\;+\&lt\;Recharger\&gt\;<BR>",
	"apr?s un changement de zone temporelle d\'interdiction" ];

  # German (de)
  $msg{"timerefresh"}->{"de"}
    = [ "Nach dem Wechsel in eine erlaubte Zeitperiode m&uuml;ssen Sie evtl. die Seite<BR>",
	"mit dem \&lt\;Aktualisieren\&gt\; bzw. \&lt\;Neu laden\&gt\; Button des Browsers<BR>",
	"oder sogar mit \&lt\;Strg\&gt\;+\&lt\;F5\&gt\; erneut laden lassen." ];

  # Italian (it)
  $msg{"timerefresh"}->{"it"}
    = [ "Pu? essere necessario usare il pulsante \&lt\;Aggiorna\&gt\; del browser<BR>",
	"o anche \&lt\;Shift\&gt\;+\&lt\;Aggiorna\&gt\;<BR>",
	"per ottenere l\'accesso alla pagina desiderata dopo la fine<BR>",
	"della restrizione per una pagina con accesso limitato a determinati orari" ];

  # Dutch (nl)
  $msg{"timerefresh"}->{"nl"}
    = [ "U moet waarschijnlijk de browser's \&lt\;Reload\&gt\; knop gebruiken<BR>",
	"of zelfs \&lt\;Shift\&gt\;+\&lt\;Reload\&gt\;<BR>",
	"na beeindiging van een periode met beperkingen" ];

  # Norwegian (no)
  $msg{"timerefresh"}->{"no"}
    = [ "Du kan trenge ? bruke browserens \&lt\;Reload\&gt\; knapp<BR>",
	"eller til og med",
	"\&lt\;Tastatur Shift\&gt\;+\&lt\;Browser Reload\&gt\;<BR>",
	"for ? bli kvitt denne siden ved overgang fra",
	"et tidsrom med sperring" ];
  
  # 
  # Message "unknown" in different languages 
  #

  # English (en)
  $msg{"unknown"}->{"en"}
    = [ "Access denied because<BR>",
	"your clienten is<BR>",
	"unknown to \$proxy"];

  # French (fr)
  $msg{"unknown"}->{"fr"}
    = [ "Acc?s interdit car <BR>",
	"votre client est <BR>",
	"inconnu de \$proxy"];

  # German (de)
  $msg{"unknown"}->{"de"}
    = [ "Zugriff verweigert,<BR>",
	"da Ihr Rechner bei<BR>",
	"\$proxy unbekannt ist."];

  # Italian (it)
  $msg{"unknown"}->{"it"}
    = [ "Accesso NEGATO poich?<BR>",
	"il tuo pc ?<BR>",
	"sconosciuto al \$proxy"];

  # Dutch (nl)
  $msg{"unknown"}->{"nl"}
    = [ "Toegand geweigerd omdat <BR>",
	"uw client niet <BR>",
	"bekend is bij \$proxy"];

  # Norwegain (no)
  $msg{"unknown"}->{"no"}
    = [ "Adgang nektes fordi<BR>",
	"denne klienten ikke er<BR>",
	"definert p? \$proxy" ];
  
  #
  # Message "in-addr" in different languages"
  #

  # English (en)
  $msg{"in-addr"}->{"en"}
    = [ "Surfing on plain <U>IP-addresses</U><BR>",
	"is denied from this client<BR>",
	"for security reasons" ];

  # French (fr)
  $msg{"in-addr"}->{"fr"}
    = [ "Naviguer sur des <U>adresses IP</U><BR>",
	"est refus? ? cette machine<BR>",
	"pour des raisons de s?curit?" ];

  # German (de)
  $msg{"in-addr"}->{"de"}
    = [ "Die direkte Verwendung von <U>IP-Adressen</U><BR>",
	"ist von diesem Rechner aus Sicherheitsg&uuml;nden<BR>",
	"nicht erlaubt." ];

  # Italian (it)
  $msg{"in-addr"}->{"it"}
    = [ "Navigare inserendo <U>indirizzi IP</U><BR>",
	"? vietato dal tuo pc<BR>",
	"per motivi di sicurezza" ];

  # Dutch (nl)
  $msg{"in-addr"}->{"nl"}
    = [ "Surfen naar harde <U>IP adressen</U><BR>",
	"wordt op deze client geweigerd<BR>",
	"om veiligheidsredenen" ];

  # Norwegian (no)
  $msg{"in-addr"}->{"no"}
    = [ "Av sikkerhetsgrunner er<BR>",
	"surfing p? <U>IP-adresser</U><BR>",
	"ikke tillatt fra denne klienten" ];
  
  #
  # Message "alternatives" in different languages
  #
  $msg{"alternatives"}->{"en"} = [ "The following possible alternatives were found:" ];
  $msg{"alternatives"}->{"fr"} = [ "Les alternatives suivantes sont possibles:" ];
  $msg{"alternatives"}->{"de"} = [ "Die folgenden Alternativen wurden gefunden:" ];
  $msg{"alternatives"}->{"it"} = [ "Sono state trovate le seguenti possibili alternative:" ];  
  $msg{"alternatives"}->{"nl"} = [ "De volgende alternatieven zijn mogelijk:" ];
  $msg{"alternatives"}->{"no"} = [ "F?lgende mulige alternativer ble funnet:" ];

  #
  # Message "noalternatives" in different languages
  #
  $msg{"noalternatives"}->{"en"} = [ "No alternative domainname were found<BR>",
	"for the server <U>\$address</U>" ];
  $msg{"noalternatives"}->{"fr"} = [ "Aucun nom de domaine alternatif n\'a ?t?<BR>",
	"trouv? pour le serveur <U>\$address</U>" ];
  $msg{"noalternatives"}->{"de"} = [ "Es konnte kein alternativer Domainname f&uuml;r den<BR>",
	"Server <U>\$address</U> gefunden werden" ];
  $msg{"noalternatives"}->{"it"} = [ "Non sono stati trovati siti<BR>",
	"alternativi a <U>\$address</U>" ];
  $msg{"noalternatives"}->{"nl"} = [ "Geen alternatieve domeinnaam gevonden<BR>",
	"voor de server <U>\$address</U>" ];
  $msg{"noalternatives"}->{"no"} = [ "Finner ingen alternative domenenavn<BR>",
	" for serveren <U>\$address</U>" ];

  #
  # Message "referermaster" in different languages
  # 

  # English (en)
  $msg{"referermaster"}->{"en"}
    = [ "Send complaints to the",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">webmaster</A><BR>",
	"of <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"and ask him to correct the link(s) that points to \$url<BR>",
	"in <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"with the supposedly correct alternative above" ];

  # French (fr)
  $msg{"referermaster"}->{"fr"}
    = [ "Envoyez les demandes au",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">webmaster</A><BR>",
	"de <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"et demandez lui corriger les liens qui pointent sur \$url<BR>",
	"dans <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"avec l\'alternative (suppos?e correcte) suivante" ];

  # German (de)
  $msg{"referermaster"}->{"de"}
    = [ "Benachrichtigen Sie den",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">Webmaster</A><BR>",
	"von <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"und bitten Sie ihn die auf \$url verweisenden Links<BR>",
	"in <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"auf die vermutlich korrekte oben angezeigte Alternative zu setzen." ];

  # Italian (it)
  $msg{"referermaster"}->{"it"}
    = [ "Invia la tua richiesta a",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">webmaster</A><BR>",
	"of <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"e chiedigli di correggere il link(s) che puntano a \$url<BR>",
	"in <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"con le corrette (spero!) alternative sopra proposte" ];

  # Dutch (nl)
  $msg{"referermaster"}->{"nl"}
    = [ "Zend klachten aan",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">webmaster</A><BR>",
	"de <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"en vraag deze de link te verbeteren die verwijst naar \$url<BR>",
	"op <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"met het waarschijnlijk correcte alternatief" ];

  # Norwegian (no)
  $msg{"referermaster"}->{"no"}
    = [ "Send evt. klager til",
	"<A HREF=\\\"mailto:webmaster\@\$refererhost\\\">webmaster</A><BR>",
	"for <A HREF=\\\"\$referer\\\">\$referer</A><BR>",
	"og be ham rette linken(e) som peker til \$url<BR>",
	"i <A HREF=\\\"\$referer\\\">\$referer</A>,<BR>",
	"med det antatt korrekte alternativet over" ];
  
  #
  # Message "webmaster" in different languages
  #

  # English (en)
  $msg{"webmaster"}->{"en"} = [ "Send complaints to the <U>webmaster</U>",
	"for <U>\$protocol://\$address</U><BR>",
	"and request for a <EM>domainname</EM> to the server" ];

  # French (fr)
  $msg{"webmaster"}->{"fr"} = [ "Envoyez les demandes au <U>webmaster</U>",
	"pour <U>\$protocol://\$address</U><BR>",
	"et demandez un <EM>nom de domaine</EM> pour le serveur" ];

  # German (de)
  $msg{"webmaster"}->{"de"} = [ "Fragen Sie den <U>Webmaster</U>",
	"von <U>\$protocol://\$address</U><BR>",
	"nach einem <EM>Domainnamen</EM> f&uuml;r den Server" ];

  # Italian (it)
  $msg{"webmaster"}->{"it"}
    = [ "Invia la tua richiesta a <U>webmaster</U>",
	"per <U>\$protocol://\$address</U><BR>",
	"e richiedi un <EM>nome di dominio</EM> per il sito" ];

  # Dutch (nl)
  $msg{"webmaster"}->{"nl"} = [ "Stuur klachten aan de <U>webmaster</U>",
	"voor <U>\$protocol://\$address</U><BR>",
	"en vraag om een <EM>domeinnaam</EM> voor de server" ];

  # Norwegian (no)
  $msg{"webmaster"}->{"no"} = [ "Send evt. klager til <U>webmaster</U>",
	"for <U>\$protocol://\$address</U><BR>",
	"og anmod om ? f? knyttet serveren til et <EM>domenenavn</EM>" ];
  
  #
  # Message "deflang" in different languages
  #

  # English (en)
  $msg{"deflang"}->{"en"} = [ "This message is in English because \\\"en\\\"",
	"is the first supported language<BR>",
	"of those your browser is set up",
	"to report as prefered.<BR>",
	"Supported languages are:",
	@supported ];

  # French (fr)
  $msg{"deflang"}->{"fr"} = [ "Ce message est en fran?ais car \\\"fr\\\"",
	"est la premi?re langue support?e<BR>",
	"parmi celles que votre navigateur signale comme",
	"pr?f?r?e.<BR>",
	"Les langues support?es sont:",
	@supported ];

  # German (de)
  $msg{"deflang"}->{"de"} = [ "Dieser Text erscheint in Deutsch, \\\"de\\\"",
	"da Ihr Browser dies als bevorzugte<BR>",
	"(erste) Sprache einstellt hat.<BR>",
	"Unterst&uuml;tzte Sprachen:",
	@supported ];

  # Italian (it)
  $msg{"deflang"}->{"it"} = [ "Questo messaggio ? in Italiano perch? \\\"it\\\"",
	"? la prima lingua supportata<BR>",
	"dal tuo browser",
	" e impostata come preferita<BR>",
	"Le lingue supportate sono:",
	@supported ];

  # Dutch (nl)
  $msg{"deflang"}->{"nl"} = [ "Deze melding is in het Nederlands want \\\"nl\\\"",
	"is de eerst ondersteunde taal<BR>",
	"van de talen die uw browser ondersteunt.",
	"<BR>",
	"De ondersteunde talen zijn:",
	@supported ];

  # Norwegian (no)
  $msg{"deflang"}->{"no"} = [ "Denne meldingen er p? norsk fordi \\\"no\\\"",
	"er det f?rste st?ttede sproget<BR>",
	"av de din nettleser er satt opp til",
	"? rapportere som foretrukket.<BR>",
	"St?ttede sprog er:",
	@supported ];
  
  #
  # Path to the default logo shown on right top of the page 
  # and the referring URL for that logo
  #
  #  %logo->{"default"}->{"url"} 	= "http://www.squidguard.org/images/squidGuard.gif";
  #  %logo->{"default"}->{"href"}	= "http://www.squidguard.org/";

  $logo{"default"}->{"url"}	= "https://admin.#DOMAIN#/stop.gif";
  $logo{"default"}->{"href"}	= "https://schulserver.#DOMAIN#/";
}


# ------------------------------------------------------------------------
# END OF CONFIGURABLE OPTIONS
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# SUBROUTINES:
# ------------------------------------------------------------------------

#
# getpreferedlang()
# 
# RETURN THE FIRST SUPPORTED LANGUAGE OF THE BROWSERS PREFERRED OR THE
# DEFAULT: en
#

sub getpreferedlang(@) 
{
  my @supported = @_;
  my @languages = split(/\s*,\s*/,$ENV{"HTTP_ACCEPT_LANGUAGE"}) if(defined($ENV{"HTTP_ACCEPT_LANGUAGE"}));
  my $lang;
  my $supp;
  push(@languages,$supported[0]);
  for $lang (@languages) 
  {
    $lang =~ s/\s.*//;
    for $supp (@supported) 
    {
      $supp =~ s/\s.*//;
      return($lang) if ($lang eq $supp);
    }
  }
 return ('en');
}


#
# parsequery($)
#
# PARSE THE QUERY_STRING FOR KNOWN KEYS:
#
sub parsequery($$) 
{
  my $query       = shift;
  my $lang		  = shift;
  my $clientaddr  = $word{"unknown"}->{$lang};
  my $clientname  = $word{"unknown"}->{$lang};
  my $clientident = $word{"unknown"}->{$lang};
  my $srcclass    = $word{"unknown"}->{$lang};
  my $targetclass = $word{"unknown"}->{$lang};
  my $url         = $word{"unknown"}->{$lang};
  if (defined($query)) 
  {
    while ($query =~ /^\&?([^\&=]+)=\"([^\"]*)\"(.*)/ || $query =~ /^\&?([^\&=]+)=([^\&=]*)(.*)/) 
    {
      my $key 	= $1;
      my $value = $2;
      $value 	= $word{"unknown"}->{$lang} unless(defined($value) && $value && $value ne "unknown");
      $query 	= $3;
      if ($key 	=~ /^(clientaddr|clientname|clientident|srcclass|targetclass|url)$/) 
      {
		eval "\$$key = \$value";
      } 
      if ($query =~ /^url=(.*)/) 
	  {
		$url = $1;
		last;
      }
    }
  } 
    return($clientaddr,$clientname,$clientident,$srcclass,$targetclass,$url);
}


#
# status($)
#
# PRINT HTTP STATUS HEARER:
#
sub status($) 
{
  my $status = shift;
  print "Status: $status\n";
}


#
# redirect($)
#
# PRINT HTTP LOCATION HEARER:
#
sub redirect($) {
  my $location = shift;
  print "Location: $location\n";
}

#
# content($)
#
# PRINT HTTP CONTENT-TYPE HEARER:
#
sub content($) 
{
  my $contenttype = shift;
  print "Content-Type: $contenttype\n";
}


#
# expires($)
#
# PRINT HTTP LAST-MODIFIED AND EXPIRES HEARER:
#
sub expires($) 
{
  my $ttl 	= shift;
  my $time 	= time;
  my @day 	= ("Sun","Mon","Tue","Wed","Thu","Fri","Sat");
  my @month = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
  my ($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($time);
  printf "Last-Modified: %s, %d %s %d", $day[$wday],$mday,$month[$mon],$year+1900;
  printf " %02d:%02d:%02d GMT\n", $hour,$min,$sec;
  ($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($time+$ttl);
  printf "Expires: %s, %d %s %d", $day[$wday],$mday,$month[$mon],$year+1900;
  printf " %02d:%02d:%02d GMT\n", $hour,$min,$sec;
}


#
# title($)
#
# PRINT THE INITIAL HTML TAGS FOR HTML, HEAD, TITLE BODY AND H1:
#
sub title($) 
{
  my $msgid = shift;
  my $defl  = $supported[0];
  my $text  = "";
  $defl 	=~ s/\s.*//;
  print "\n<HTML>\n";
  print " <HEAD>\n  <TITLE>\n";
  if (defined($msg{$msgid}{$lang})) 
  {
    for $text (@{$title{$msgid}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "       $text\n";
    }
  } 
  else 
  {
    for $text (@{$title{"default"}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "       $text\n";
    }
  }
  print "  </TITLE>\n </HEAD>\n";
  print " <BODY BGCOLOR=\"#FFFFFF\">\n";
  print "  <TABLE BORDER=0 ALIGN=CENTER WIDTH=100%>\n";
  print "   <TR>\n";
  print "    <TD ALIGN=LEFT VALIGN=BOTTOM>\n";
  print "     <FONT SIZE=7>\n";
  print "      <B>\n       <U>\n";
  if (defined($msg{$msgid}{$lang})) 
  {
    for $text (@{$title{$msgid}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "       $text\n";
    }
  } 
  else 
  {
    for $text (@{$title{"default"}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "       $text\n";
    }
  }
  print "       </U>\n      </B>\n";
  print "     </FONT>\n";
  print "    </TD>\n";
  print "    <TD ROWSPAN=2 ALIGN=RIGHT>\n";
  if (defined($logo{$msgid}{"url"})) 
  {
    print "     <A HREF=\"$logo{$msgid}{\"href\"}\"><IMG\n";
  } 
  else 
  {
    print "     <A HREF=\"$logo{\"default\"}{\"href\"}\"><IMG\n";
  }
  if (defined($logo{$msgid}{"url"})) 
  {
    print "     SRC=\"$logo{$msgid}{\"url\"}\" BORDER=0 ALIGN=TOP></A>\n";
  } 
  else 
  {
    print "     SRC=\"$logo{\"default\"}{\"url\"}\" BORDER=0 ALIGN=TOP></A>\n";
  }
  print "    </TD>\n";
  print "   </TR>\n";
  if ($lang eq $defl && defined($msg{"deflang"}{$lang})) 
  {
    print "   <TR><!-- \$msg{\"deflang\"}{$lang} -->\n";
    print "    <TH ALIGN=LEFT VALIGN=TOP>\n";
    print "     <FONT SIZE=-1>\n";
    print "      <B>\n";
    for $text (@{$msg{"deflang"}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "       $text\n";
    }
    print "      </B>\n";
    print "     </FONT>\n";
    print "    </TH>\n";
    print "   </TR>\n";
  }
  print "  </TABLE>\n";
}


#
# terminator()
#
# PRINT THE ENDING HTML TAGS FOR BODY AND HTML:
#
sub terminator() 
{
  print " </BODY>\n</HTML>\n";
}


#
# msg($$)
#
# PRINT A MESSAGE WITH THE SPECIFIED TYPE (P,H1,H2,..):
#
sub msg($$) 
{
  my ($type,$msgid) = @_;
  my $text;
  print "  <$type ALIGN=CENTER><!-- \$msg{$msgid}{$lang} -->\n";
  if (defined($msg{$msgid}{$lang})) 
  {
    for $text (@{$msg{$msgid}{$lang}}) 
    {
      eval "\$text = \"$text\"";
      print "   $text\n";
    }
  } 
  else 
  {
    print "   <EM><B>ERROR: missing message \"$msgid\"</B></EM>\n";
  }
  print "  </$type>\n";
}

#
# table($$@)
#
# PRINT A TABLE WITH THE SPECIFIED FORMAT:
#
sub table($$@) {
  my @format = split(/,/,shift);
  my $table  = shift;
  my $cols   = @format;
  my @msgids = @_;
  my $msgid;
  my $text;
  my %type;
  $type{"L"} = [ "<TH ALIGN=LEFT>", "</TH>" ];
  $type{"C"} = [ "<TH ALIGN=CENTER>", "</TH>" ];
  $type{"R"} = [ "<TH ALIGN=RIGHT>", "</TH>" ];
  $type{"l"} = [ "<TD ALIGN=LEFT>", "</TD>" ];
  $type{"c"} = [ "<TD ALIGN=CENTER>", "</TD>" ];
  $type{"r"} = [ "<TD ALIGN=RIGHT>", "</TD>" ];
  print "  <TABLE BORDER=0 ALIGN=CENTER><!-- table(\"$table\") -->\n";
  if (defined($tab{$table})) 
  {
    if (defined($tab{$table}{"caption"}{$lang})) 
    {
      #print "   <CAPTION ALIGN=LEFT>\n";
      print "   <TH ALIGN=LEFT>\n";
      print "    <FONT SIZE=+1>\n";
      for $text (@{$tab{$table}{"caption"}{$lang}}) 
      {
		eval "\$text = \"$text\"";
		print "    $text\n";
      }
      print "    </FONT>\n";
      #print "   </CAPTION>\n";
      print "   </TH>\n";
    }
    for $msgid (@msgids) 
    {
      print "   <TR>\n";
      if (defined($tab{$table}{$msgid}{$lang})) 
      {
		my $i = 0;
		for $text (@{$tab{$table}{$msgid}{$lang}}) 
        {
	  		eval "\$text = \"$text\"";
	  			print "    $type{$format[$i]}[0]\n";
	  			print "     $text\n";
	  			print "    $type{$format[$i]}[1]\n";
	  			$i++;
		}
      }
      print "   </TR>\n";
    }
  } 
  else 
  {
    print "   <TR>\n";
    print "    <TH ALIGN=CENTER>\n";
    print "     <EM><B>ERROR: missing message \"$msgid\"</B></EM>\n";
    print "    </TH>\n";
    print "   </TR>\n";
  }
  print "  </TABLE>\n";
}


#
# href($)
#
# PRINT A LINK HREF:
#
sub href($) 
{
  my $href = shift;
  print "<A HREF=\"$href\">$href</A>";
}


#
# gethostnames($)
#
# REVERSE LOOKUP AND RETURN NAMES:
#
sub gethostnames($) 
{
  my $address = shift;
  my ($name,$aliases) = gethostbyaddr(inet_aton($address), AF_INET);
  my @names;
  if (defined($name)) 
  {
    push(@names,$name);
    if (defined($aliases) && $aliases) 
    {
      for(split(/\s+/,$aliases)) 
      {
	     next unless(/\./);
	     push(@names,$_);
      }
    }
  }
  return(@names);
}


#
# spliturl($)
#
# SPLIT AN URL INTO PROTOCOL, ADDRESS, PORT AND PATH:
#
sub spliturl($) 
{
  my $url      = shift;
  my $protocol = "";
  my $address  = "";
  my $port     = "";
  my $path     = "";
  if (defined($url))
  {
  	$url =~ /^([^\/:]+):\/\/([^\/:]+)(:\d*)?(.*)/;
  } else 
  {
  	$url = "";
  }
  $protocol = $1 if(defined($1));
  $address  = $2 if(defined($2));
  $port     = $3 if(defined($3));
  $path     = $4 if(defined($4));
  return($protocol,$address,$port,$path);
}


#
# showhtml($)
#
# SHOW THE CONFIGURED MESSAGE AS HTML:
#
sub showhtml($) {
  my $msgid = shift;
  status("403 Forbidden");
  content("text/html");
  expires(0);
  title($msgid);
  $msgid = "default" unless(defined($msgconf{$msgid}));
  if (defined($msgconf{$msgid})) 
  {
    print "  <!-- showhtml(\"$msgid\") -->\n";
    for (@{$msgconf{$msgid}}) 
    {
      my @config = split(/:/);
      my $type = shift(@config);
      if ($type eq "msg") {
	  msg($config[0],$config[1]);
      } 
      elsif ($type eq "tab") 
      {
         table(shift(@config),shift(@config),@config);
      }
    }
  } 
  else 
  {
    print "  <P><EM><B>ERROR: missing msgconf for \"$msgid\"</B></EM></P>\n";
  }
  terminator();
}


#
# showimage($$$)
#
# SEND OUT AN IMAGE:
#
sub showimage($$$) 
{
  my ($type,$file,$redirect) = @_;
  content("image/$type");
  expires(300);
  redirect($redirect) if($redirect);
  print "\n";
  open(GIF, "$ENV{\"DOCUMENT_ROOT\"}$file");
  print <GIF>;
  close(GIF)
}


#
# showinaddr($$$$$)
#
# SHOW THE INADDR ALERNATIVES WITH OPTIONAL ATOREDIRECT:
#
sub showinaddr($$$$$) 
{
  my ($targetclass,$protocol,$address,$port,$path) = @_;
  my $msgid = $targetclass;
  my @names = gethostnames($address);
  if($autoinaddr == 2 && @names || $autoinaddr && @names==1) 
  {
    status("301 Moved Permanently");
    redirect("$protocol://$names[0]$port$path");
  } elsif (@names>1) 
  {
    status("300 Multiple Choices");
  } elsif (@names) 
  {
    status("301 Moved Permanently");
  } else 
  {
    status("404 Not Found");
  }
  if ($path =~ /\.(gif|jpg|jpeg|mp3|mpg|mpeg|avi|mov)$/i) 
  {
    showimage("gif",$image,$redirect);
  } elsif (@names) 
  {
    content("text/html");
    expires(0);
    title($msgid);
    $msgid = "in-addr" unless(defined($msgconf{$msgid}));
    if (defined($msgconf{$msgid})) 
    {
      print "  <!-- showinaddr(\"$msgid\") -->\n";
      for (@{$msgconf{$msgid}}) 
      {
    	my @config = split(/:/);
	    my $type = shift(@config);
	    if ($type eq "msg") 
        {
	       msg($config[0],$config[1]);
	    }  elsif ($type eq "tab") 
        {
	       table(shift(@config),shift(@config),@config);
	    } elsif ($type eq "alternatives") 
        {
	       print "  <TABLE BORDER=0 ALIGN=CENTER>\n";
	       for (@names) 
           {
	           print "   <TR>\n    <TH ALIGN=LEFT>\n     <FONT SIZE=+1>";
	           href("$protocol://$_$port$path");
	           print "\n     </FONT>\n    </TH>\n   </TR>\n";
	       }
	       print "  </TABLE>\n\n";
	       if (defined($ENV{"HTTP_REFERER"}) && $ENV{"HTTP_REFERER"} =~ /:\/\/([^\/:]+)/) 
           {
	           $refererhost = $1;
	           $referer = $ENV{"HTTP_REFERER"};
	           msg("H4","referermaster");
	       }
	    }
      }
    } else 
    {
      print "  <P><EM><B>ERROR: missing msgconf for \"$msgid\"</B></EM></P>\n";
    }
    terminator();
  } else 
  {
    showhtml("noalternatives");
  }
}



# ------------------------------------------------------------------------
# NOW JUST DO IT:
# ------------------------------------------------------------------------
msginit();

if (! defined($targetclass))
{
	 $targetclass = "unknown";
}

if (! defined($url))
{
	$url="http://unknown";
}

if ("$targetclass" eq "in-addr") 
{
	showinaddr($targetclass,$protocol,$address,$port,$path);
} elsif ($url =~ /\.(gif|jpg|jpeg|mp3|mpg|mpeg|avi|mov)$/i) 
{
	status("403 Forbidden");
	showimage("gif",$image,$redirect);
} else 
{
	if (defined($srcclass))
	{
 			showhtml($srcclass);
	} else 
	{
		showhtml("");
	}
}

exit 0;


