#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: Filterlist.pm,v 1.4 2003/10/29 10:35:39 varkoly Exp $
#
package Filterlist;

use Exporter;
use LDAPUtils;
use strict;

sub new {
    my ($this, $cgi, $session, $ldap, $display) = @_;
    my $class= ref($this) || $this;
    my $self = {};
    $self->{'cgi'} = $cgi;
    $self->{'session'} = $session;
    $self->{'ldap'} = $ldap;
    $self->{'display'} = $display;

    return bless($self,$class);
}

sub make_filter_view {
    my $this = shift;

    my $cgi  = $this->{"cgi"};
    my $ldap = $this->{"ldap"};

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};

    my $param    = shift;
    my $textlabel   = $param->{'textlabel'};
    my $listlabel_left  = $param->{'listlabel_left'};
    my $name1    = $param->{'textfield'};     #name of the textfield
    my $name2    = $param->{'listname'};      #name of the list
    my $name3    = $param->{'classname'};      #name of the list
    my $buttonlabel     = $param->{'buttonlabel'};
    if( ! defined $buttonlabel ) {
	$cgi->param(-name=>'filtered', -value=>"1");
    }
    my $listlabel_right = $param->{'listlabel_right'};
    my $nofilter = $param->{"nofilter-msg"}||"set filter";  #message if no filter has been set
    my $listfilter  = $param->{"listfilter"};   #ldap filter-rule for list (array: attr,value)
    my $listattrs   = $param->{"listattrs"};   #ldap filter-rule for list (array: attr,value)
    my $labelexp = $$param{"labelexp"};      #expression for labels
    my $width    = $$param{"width"};         #width of text-field
    my $height   = $$param{"height"};        #number of lines in select-box
    my $multi    = $$param{"multiple"}||"";  #select-box with multiple selection
    my $selectedfilter  = $$param{"selectedfilter"}; #ldap filter-rule for selected values (array: attr,value)
#    my $selectedattrs   = $$param{"selectedattrs"}; #ldap filter-rule for selected values (array: attr,value)
    my $selectedregex    = $$param{"selectedregex"}; #optional parameter  this array holds the 
						     #ldap attribute and the regular expression for defaults
    my $url      = $$param{"error-url"};     #url to return after errors
    
    my $output  = "";
    my $values=[];
    my $defaults=[];
    my $result;
    my $result2;
    my $textfield  = $cgi->param($name1) || '*';
    my $classfield = $cgi->param('name3') || '*';

    my $addr  = $cgi->param('__FilterADD.x');
    my $delr  = $cgi->param('__FilterDEL.x');
    my $beenhere = $cgi->param('beenhere');
    my @__defaults = $cgi->param("$name2");

    #print STDERR "@__defaults\n";
    
    my @rn2selected = $cgi->param("__right_$name2");
    my @n2selected  = $cgi->param("__left_$name2");

    if (defined $selectedfilter && $beenhere ne "1" ) {
	
	#print STDERR "<$selectedfilter> <$selectedregex->[0]>\n";

	if( defined $ldap ) {
	    $result2=$ldap->search(base => $confParam->{"baseDN"},
				   scope => "one",
				   filter => $selectedfilter,
				   attrs => $selectedregex->[0]);
	    if ($result2->code != 0) {
		my $errs = "search1: failed\n\n";
		$errs .= &LDAPerror($result2);
		$this->{"display"}->display_error($errs,$url);
		$ldap->unbind;   # take down session
		exit;
	    }
	    my @entries = $result2->entries();
	    foreach my $act_entry (@entries){
		#print STDERR "value: ".$act_entry->get_value($selectedregex->[0])."\n".
		#"attr: ".$selectedregex->[0]."\nregex: ".$selectedregex->[1]."\n";
		if((defined $selectedregex) && (@$selectedregex == 2)){
		    my @values = $act_entry->get_value($selectedregex->[0]);
		    if( @values > 0 ){
			foreach my $value (@values){
			    #print STDERR $value."\n";
			    if( $value =~ /$selectedregex->[1]/ ){
				if(defined $1){ 
				    #print STDERR $1."\n";
				    push @$defaults, $1;
				}else{
				    #print STDERR $act_entry->get_value($$selectedregex[0])."\n";
				    push @$defaults, $act_entry->get_value($$selectedregex[0]);
				}
			    }
			}
		    }
		}
	    }
	} elsif ( ref($selectedfilter) eq "ARRAY" ) {
	    #print STDERR "selectedfilter = ARRAY\n";
	    foreach my $e ( @$selectedfilter ) {
		push @$defaults, $e;
	    }
	}

    } elsif( $beenhere eq "1" ) {
	@$defaults = @__defaults;

	if( defined $delr ) {
	    my @tmp;
	    foreach my $e ( @$defaults ) {
		my $found = 0;
		foreach my $re ( @rn2selected ) {
		    if( $e eq $re ) {
			$found = 1;
		    }
		}
		push @tmp, $e if ! $found;
	    }
	    @$defaults = @tmp;
	}

	if( defined $addr ) {
	    foreach my $e ( @n2selected ) {
		push @$defaults, $e;
	    }
	}
    }

    if( ( ( ! defined $cgi->param($buttonlabel) ) || ( $cgi->param($buttonlabel) eq "" ) ) && 
	( ( ! defined $cgi->param("filtered") ) || ( $cgi->param("filtered") != 1 ) ) ){
        $values=[$nofilter];
        #$defaults=[$nofilter];
    } else {
        my $ldapfilter=$listfilter;
        my $defaultfilter="(&";
        
	if( defined $ldap ) {
	   if ($classfield ne "*" ) {
		   my $classfilter = "";
		   if($textfield eq '*') {
			$listfilter = '';
		   }
		   if ($confParam->{uid}  == "cyrus" ) {
			$classfilter = "(&(objectclass=posixgroup)(|(description=$classfield)(cn=$classfield)))";
		   } else {
			$classfilter = "(&(objectclass=posixgroup)(cn=$classfield)(memberUid=$confParam->{uid})";
		   }
		   $result=$ldap->search(base   => $confParam->{"baseDN"},
				      scope  => "one",
				      filter => $classfilter,
				      attrs  => ['memberUid','gidnumber']
				     );
		   my @entr = $result->entries;
		   # Jetzerla suchen wir die Benutzer mssen jedoch dafr sorgen, dass jeder uid nur ein mal vorkommt:
		   my $ind = {};
		   foreach my $act_entry ( @entr ){
			my @uids = $act_entry->get_value('memberUid');
			foreach my $uid ( @uids ) {
			   if( not exists($ind->{$uid}) ) {
				$result=$ldap->search(base   => $confParam->{"baseDN"},
					scope  => "one",
					filter => "(&(&(objectclass=posixAccount)(objectclass=SuSEeMailObject)(uid=$uid))".
						   $listfilter.")",
					attrs  => $listattrs
				);
				if ($result->count() > 0) {
				  push @$values , $uid;
				}
				$ind->{$uid} = 1;
			   }
			}
			# And now we are searching the users with primary group
			my $gidnumber = $act_entry->get_value('gidnumber');
			$result=$ldap->search(base   => $confParam->{"baseDN"},
					scope  => "one",
					filter => "(&(&(objectclass=posixAccount)(objectclass=SuSEeMailObject)".
						  "(gidnumber=$gidnumber))".$listfilter.")",
					attrs  => $listattrs
			);
			foreach my $ent ($result->entries()){
			    if ($result->count() > 0) {
				    my $uid = $ent->get_value('uid');
				    if( not exists($ind->{$uid}) ) {
					push @$values , $uid;
					$ind->{$uid} = 1;
				    }
			    }
			}
		   }
		   if( scalar @$values  == -1 ){
                	$values=[$nofilter];
	           }
        	} else {
		    $result=$ldap->search(base   => $confParam->{"baseDN"},
					  scope  => "one",
					  filter => $ldapfilter,
					  attrs  => $listattrs  
					  );
		    if ($result->code != 0) {
			my $errs = "search2: failed\n\n";
			$errs .= &LDAPerror($result);
			$this->{"display"}->display_error($errs,$url);
			$ldap->unbind;   # take down session
			exit;
		    }
		    if ($result->count==0) {
			#no results => set new filter
			$values=[$nofilter];
			#$defaults=[$nofilter];
		    } else {
			#otherwise show results
			my @entries = $result->entries;
			foreach my $act_entry (@entries){
			    my $entry = $act_entry->get_value($listattrs->[0]);
			    my $found = 0;
			    foreach my $v ( @$defaults ) {
				$found = 1 if $entry eq $v;
			    }
			    if( ! $found ) {
				push @$values, $entry;
			    }
			    if( (! (defined $selectedfilter)) && (defined $selectedregex) && (@$selectedregex == 2)){
	#                    print STDERR "value: ".$act_entry->get_value($selectedregex->[0])."\n".
	#				 "attr: ".$selectedregex->[0]."\nregex: ".$selectedregex->[1]."\n";
				my @values = $act_entry->get_value($selectedregex->[0]);
				if( @values > 0 ){
				    foreach my $value (@values){
					if( $value =~ /$selectedregex->[1]/ ){
					    if(defined $1){
						push @$defaults, $1;
					    }else{
						push @$defaults, $act_entry->get_value($selectedregex->[0]);
					    }
					}
				    }
				}
			    }
			}
		    }
		}
	} elsif ( ref($listfilter) eq "ARRAY" ) {
	    #print STDERR "listfilter = ARRAY\n";
	    foreach my $le ( @$listfilter ) {
		my $found = 0;
		foreach my $v ( @$defaults ) {
		    $found = 1 if $le eq $v;
		}
		if( ! $found ) {
		    push @$values, $le;
		}
	    }
	}

    }

    if( defined $buttonlabel ) {
	$output .= $cgi->start_Tr();
	if( (defined $textlabel) || (defined $listlabel_right) ){
	    $output .= $cgi->start_td();
	    if(defined $textlabel){
		$output .= $textlabel;
	    }else{
		$output .= "&nbsp;"
		}
	    $output .= $cgi->end_td();
	}
	$output .= $cgi->start_td({-align => 'center'}).
	    $cgi->textfield(-name=>"$name1",-default=>$textfield || "*",-size=>"$width").
	    $cgi->end_td().
	    $cgi->start_td({-align => 'center'}).
	    $cgi->submit(-class=>"button",-name => $buttonlabel).
	    $cgi->end_td().
	    $cgi->end_Tr();
    }
    if( defined $name3 ) {
	$output .= $cgi->start_Tr().
            $cgi->start_td().$name3.$cgi->end_td().
            $cgi->start_td({-align => 'center'}).
            $cgi->textfield(-name=>'name3',-default=>$classfield || "*",-size=>"$width").
            $cgi->end_td().
            $cgi->end_Tr();
     }

    #create the labels hash for the scrolling list
    my %labels;
    if( $values->[0] ne $nofilter ){
	
	if( defined $ldap ) {
	    my @entries = $result->entries;
	    foreach my $act_entry ( @entries ){
		for (my $j=0; $j <= $#$labelexp; $j = $j + 2) {
		    my $attrname = $act_entry->get_value($$labelexp[$j+1]);
		    my $string = $$labelexp[$j];
		    if ($string=~/\$\$/)  {
			  $string=~s/\$\$/$attrname/;
		    } elsif ($string=~/%.%/)  {
			#typ is FLAG
			if (defined $attrname) {
			    $string=~s/%(.)%/$1/;
			}
			else {
			    $string=~s/%(.)%//;
			}
		    }
		    $string=~s/\\\$/\$/;
		    $string=~s/\\%/%/;
		    $labels{$act_entry->get_value($listattrs->[0])} .= $string;
		}
	    }
	} 
    }
    @$values=sort(@$values);
    if(defined $defaults){
	@$defaults=sort(@$defaults);
    }
    

    $output .= $cgi->start_Tr();
    if( (defined $textlabel) || (defined $listlabel_right) ){
        $output .= $cgi->start_td({-valign => "top"});
        if(defined $listlabel_left){
            $output .= $listlabel_left;
	}else{
	    $output .= "&nbsp;"
	}
        $output .= $cgi->end_td();
    }

    if (defined $selectedfilter) {
	$output .= $cgi->start_td();

	$output .= $cgi->start_table({-border => 0, -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
	#$output .= $cgi->start_form(-action=>$confParam->{cgi_path}."/browse_groups.pl");
	$output .= $cgi->start_Tr();
    }	

    my $leftname  = ( defined $selectedfilter ? "__left_$name2" : "$name2" );

    $output .= $cgi->start_td({-align => 'center'}).
	$cgi->scrolling_list(-name => $leftname,
			     -values => $values, 
			     -labels => \%labels,
			     -override => 1, 
			     -size => $height,
			     -width => ($width*8),
			     -multiple   => ( $multi ne "" ? 'true' : "" )
			     );

    if (defined $selectedfilter) {
	$output .= $cgi->end_td();

	$output .= $cgi->start_td();
	$output .= $cgi->start_table({-border => 0, -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
	$output .= $cgi->start_Tr();
	$output .= $cgi->start_td({-align=>'center', -valign=>'center'});
	#$output .= $cgi->submit(-class=>"BlockButton", -name=> "__FilterADD", -value=>"&nbsp;--->&nbsp;");
        $output .= $cgi->image_button(-name=>"__FilterADD",
				      -src=>"$confParam->{image_path}/arrow_next.gif",
				      -align=>'MIDDLE',
				      -border=>0
				      );
	$output .= $cgi->end_td();
	$output .= $cgi->end_Tr();
	
	$output .= $cgi->start_Tr();
	$output .= $cgi->start_td({-align=>'center', -valign=>'center'});
	#$output .= $cgi->submit(-class=>"BlockButton", -name=> "__FilterDEL", -value=>"&nbsp;<---&nbsp;");
	$output .= $cgi->image_button(-name=>"__FilterDEL",
				      -src=>"$confParam->{image_path}/arrow_prev.gif",
				      -align=>'MIDDLE',
				      -border=>0
				      );
	$output .= $cgi->end_td();
	$output .= $cgi->end_Tr();

	$output .= $cgi->end_td();
	$output .= $cgi->end_Tr();
	$output .= $cgi->end_table();

	$output .= $cgi->end_td();

	$output .= $cgi->start_td({-align => 'center'});
	$output .= $cgi->scrolling_list(-name => "__right_$name2",
					-values => $defaults, 
					-labels => \%labels,
					-override => 1, 
					-size => $height,
					-width => ($width*8),
					-multiple   => ( $multi ne "" ? 'true' : "" )
					).
					$cgi->hidden(-name=>"$buttonlabel", -value=>"$buttonlabel", -override=>1).
					$cgi->hidden(-name=>"filtered", -value=>"1", -override=>1).
					$cgi->hidden(-name=>"beenhere", -value=>"1", -override=>1).
					$cgi->hidden(-name=>"$name2", -value=>$defaults, -override=>1).
					$cgi->hidden(-name=>"filtered", -value=>1,-override=>1);
	$output .= $cgi->end_Tr();
	#$output .= $cgi->end_form();
	$output .= $cgi->end_table();
    }

    if( (defined $values) && (defined $values->[0]) && ($values->[0] ne $nofilter) ){
        $output .= $cgi->hidden( -name => "filtered",
                                 -value => 1,
                                 -override => 1);
    }
    $output .= $cgi->end_td().
            $cgi->start_td({-align => 'center'});
    if(defined $listlabel_right){
        $output .= $listlabel_right;
    }else{
        $output .= "&nbsp;";
    }
    $output .= $cgi->end_td().
        $cgi->end_Tr();

    return $output;
}

1;

