#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: IMAPUtils.pm,v 1.4 2003/10/29 10:35:39 varkoly Exp $
#
require Exporter;
package IMAPUtils;

use strict;
use CGI;
use Net::IMAP;
use MIME::Base64;

# This is global shit (see getFolder)
my @__folder = "";
my $__acl    = "";
my $_delimiter = ".";

#-----------------------------------------------------------------------
sub collect_folder {
    my $self = shift;
    my $resp = shift;

    $_delimiter = $resp->delimiter;

    if(! $resp->has_flag('\Noselect')) {
	push @__folder, $resp->mailbox;
    }
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub namespace($$$) {
    my ($user, $passwd, $host ) = @_;

    my $imap = new Net::IMAP($host, Debug => 0);
    unless ($imap) {
	print STDERR "can't connect to $host: $!\n";
	return undef;
    }

    my $ret = $imap->login($user, $passwd);


    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }    

    my $namespaces;
    my $nscb = sub {
	my $self = shift;
	my $resp = shift;
	#print STDERR "---------------- ".$resp->personal("")."\n";
	#print STDERR "---------------- ".$resp->other_users("user/")."\n";
	#print STDERR "---------------- ".$resp->shared("")."\n";
	$namespaces = $resp;
    };

    $imap->set_untagged_callback('namespace', $nscb);

    $ret = $imap->namespace();

    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }    

    return $namespaces;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub myacl {
    my $self = shift;
    my $resp = shift;

    $__acl = $resp->rights;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getDelimiter {
    return $_delimiter;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getFolder($$$$) {
    #     l - lookup (mailbox is visible to LIST/LSUB commands)
    #     r - read (SELECT the mailbox, perform CHECK, FETCH, PARTIAL,
    #         SEARCH, COPY from mailbox)
    #     s - keep seen/unseen information across sessions (STORE SEEN flag)
    #     w - write (STORE flags other than SEEN and DELETED)
    #     i - insert (perform APPEND, COPY into mailbox)
    #     p - post (send mail to submission address for mailbox,
    #         not enforced by IMAP4 itself)
    #     c - create (CREATE new sub-mailboxes in any implementation-defined
    #         hierarchy)
    #     d - delete (STORE DELETED flag, perform EXPUNGE)
    #     a - administer (perform SETACL)

    my ($user, $acl, $passwd, $host ) = @_;
    my $folders;

    my $imap = new Net::IMAP($host, Debug => 0);
	#or die ("can't connect to $host: $!\n");
    unless ($imap) {
	print STDERR "can't connect to $host: $!\n";
	return undef;
    }

    # clear it, because it's global
    @__folder = ();
    $__acl    = "";

    my $ret = $imap->login($user, $passwd);


    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }    

    $imap->set_untagged_callback('list', \&collect_folder);
    $imap->set_untagged_callback('myrights', \&myacl);

    $ret = $imap->list('*','%');

    if( $ret->status ne 'ok' ) {
	print STDERR "fetching mailboxes failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }

    foreach( @__folder ) {

	#print STDERR "Foldername=<$_>\n";
        # I SCHULSERVER
        # nobody may manipulate workgroups
        if( $_ eq 'workgroups' ) {
                next;
        }
        # E SCHULSERVER

	$ret = $imap->myrights($_);

	if( $ret->status ne 'ok' ) {
	    print STDERR "fetching acl for mailbox $_ failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	} else {
	    my $match = 1;
	    for(my $i=0; $i<length($acl); $i++) {
		my $t = substr($acl, $i, 1);
		# print STDERR "REAL=<$__acl> match with=<$t>\n";
		if( ! ($__acl =~ /$t/) ) {
		    # print STDERR "NOMATCH: $_\n";
		    $match = 0;
		}
	    }
	    my $enc = utf7_decode($_);
	    #print STDERR "Folder=<$enc>, laenge=".length($enc)."\n";
	    $folders->{$enc} = $_ if $match;
	}
    }
    $imap->logout();

    return $folders;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getFolderList($$$$) {
    my ($user, $passwd, $mode, $host ) = @_;
    my $folders;

    my $imap = new Net::IMAP($host, Debug => 0);
	#or die ("can't connect to $host: $!\n");
    unless ($imap) {
	print STDERR "can't connect to $host: $!\n";
	return undef;
    }

    # clear it, because it's global
    @__folder = ();
    $__acl    = "";

    my $ret = $imap->login($user, $passwd);


    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }    

    if( $mode eq "list" ) {
	$imap->set_untagged_callback($mode, \&collect_folder);
	$ret = $imap->list('*','%');
    } else {
	$imap->set_untagged_callback("lsub", \&collect_folder);
	$ret = $imap->lsub('*','%');
    }

    if( $ret->status ne 'ok' ) {
	print STDERR "Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return undef;
    }

    foreach( @__folder ) {
	my $enc = utf7_decode($_);
	push @$folders, $enc;
    }
    $imap->logout();

    return $folders;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub subscribe($$$$) {
    my ($user, $passwd, $folder, $host) = @_;

    my $imap = new Net::IMAP($host, Debug => 0);
	#or die ("can't connect to $host: $!\n");
    unless ($imap) {
	print STDERR "can't connect to $host: $!\n";
	return "can't connect to $host: $!\n";
    }

    my $ret = $imap->login($user, $passwd);


    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
    }    

    my $subs = sub {
	my $self = shift;
	my $resp = shift;
    };

    $imap->set_untagged_callback("subscribe", $subs);

    $imap->subscribe(utf7_encode($folder));
    
    $imap->logout();

    return undef;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub unsubscribe($$$$) {
    my ($user, $passwd, $folder, $host) = @_;

    my $imap = new Net::IMAP($host, Debug => 0);
	#or die ("can't connect to $host: $!\n");
    unless ($imap) {
	print STDERR "can't connect to $host: $!\n";
	return "can't connect to $host: $!\n";
    }

    my $ret = $imap->login($user, $passwd);


    if($$ret{Status} ne "ok") {
	print STDERR "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
	return "login failed: Serverresponse:$$ret{Status} => $$ret{Text}\n";
    }    

    my $unsubs = sub {
	my $self = shift;
	my $resp = shift;
    };

    $imap->set_untagged_callback("unsubscribe", $unsubs);

    $imap->unsubscribe(utf7_encode($folder));
    
    $imap->logout();

    return undef;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub mustENC($) {
    my $c = shift; 
    return ( $c =~ /[\x00-\x1f]/ ) || ( ord($c) > 127 );
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub utf7_encode($) {
    # see rfc 2060
    my $ustr = shift;
    my $str = &Encode::decode("utf8", $ustr);

    my $nstr = "";
    for(my $i=0; $i<length($str); $i++) {
	my $l = substr($str,$i,1);
	if( mustENC($l) ) {
            my $allenc = "";
            while( mustENC($l) ) {
                $allenc .= $l;
                $l = substr($str,++$i,1);
	    }
            
	    $i--;
            # make 16 bit representation of allenc (UCS-2)
            my @chars = unpack( "U*", $allenc);
            $allenc = pack( "n*", @chars );

	    my $enc = encode_base64( $allenc ,"");
	    $enc =~ s/=//g;
	    $enc =~ s/\//,/g;
	    $nstr .= "&".$enc."-";
	} elsif( $l eq "&" ) {
	    $nstr .= $l."-";
	} else {
	    $nstr .= $l;
	}
    }

    return $nstr;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub utf7_decode($) {
    my $str = shift;

    #print STDERR "STR=$str\n";
    my $nstr = "";
    foreach my $token ( split(/-/, $str, -1) ) {
        #print STDERR "TOK=$token\n";
        if( $token =~ /&/ ) {
	    $token =~ /(.*)&(.*)/;
	    if( ! defined $2 || $2 eq "" ) {
		$token = $1."&";
	    } else {
		my $foo = $2;
		my $prefix = $1;
		$foo =~ s/,/\//;
                # pad with "=" to make base64 happy
                my $num = 4 - ( length($foo) % 4 );
                for( my $i=0; $i < $num && $num < 4; $i++ ){
                    $foo .= "=";
                }
                my $decode = decode_base64($foo);
                # each char is 16bit (UCS-2 AFAIK)
                my @chars = unpack("n*", $decode );
                # generate the corresponding Unicode String in Perl's
                # internal encoding
                my $decode = pack("U*", @chars);
		$token = $prefix.$decode;
	    }
        } else {
	    $token .= "-";
        }
        $nstr .= $token;
    }
    $nstr =~ s/\-$// ;
    
    # remove binary \0 trash
    $nstr =~ s/\0//g;

    $nstr = &Encode::encode("utf-8",$nstr);
    return $nstr;
}
#-----------------------------------------------------------------------



@IMAPUtils::ISA = qw(Exporter);
@IMAPUtils::EXPORT = qw(
			&namespace
			&getDelimiter
			&getFolder
			&utf7_decode
			&utf7_encode
			&getFolderList
			&subscribe
			&unsubscribe
			);
$IMAPUtils::VERSION = "0.1";

1; 

