#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: LDAPUtils.pm,v 1.2 2003/05/06 11:30:44 varkoly Exp $
#
require Exporter;
package LDAPUtils;

use strict;
use Net::LDAP qw(:all);
use Net::LDAP::Util qw( ldap_error_name
			ldap_error_text) ;

# -attrs
# -dn
# -confParam
sub LDAP_read_entry($$$){
    my ($confParam, $dn, $attrs) = @_;
    my $ldap= Net::LDAP->new($confParam->{'LDAPserver'}, 
	    port => $confParam->{'LDAPport'},
	    version => 3);
    if(! (defined $ldap) ) {
	return ( -1 , "init failed");
    }

    my $mesg=$ldap->bind ("uid=".$confParam->{'uid'}.",".$confParam->{'baseDN'}, 
			  password=>$confParam->{'passwd'});
    if($mesg->code != LDAP_SUCCESS) {
        my $text = "bind failed\n".&LDAPerror($mesg);
	return ( -1 , $text );
    }

    $mesg = $ldap->search(base => $dn,
			  scope => "base", 
			  filter => "objectclass=*",
			  attrs => $attrs);
    if( $mesg->code ) {
        my $text = "search failed\n".&LDAPerror($mesg);
	return ( -1 , $text );
    }
    my $entry = $mesg->entry(0);
    if( ! (defined($entry) )){
	return( -2, "search failed\n"."Entry does not exist" );
    }
    $ldap->unbind();
    return(0, $entry);
}

sub LDAP_is_samba_user($$$){
    my ($confParam, $ldap, $uid) = @_;
    my $mesg = undef;
    my $do_unbind = 0;
    if( ! (defined $ldap) ){
	$do_unbind = 1;
	$ldap= Net::LDAP->new($confParam->{'LDAPserver'},
		port => $confParam->{'LDAPport'},
		version => 3);
	if(! (defined $ldap) ) {
	    return ( -1 , "init failed");
	}

	$mesg=$ldap->bind ("uid=".$confParam->{'uid'}.",".$confParam->{'baseDN'}, 
			      password=>$confParam->{'passwd'});
	if($mesg->code != LDAP_SUCCESS) {
	    my $text = "bind failed\n".&LDAPerror($mesg);
	    return ( -1 , $text );
	}
    }
    # look whether this user is smb enabled
    $mesg = $ldap->search( base => "uid=".$uid.",".$confParam->{'baseDN'},
			   scope => "base", 
			   filter => "objectClass=sambaaccount");
    if( $mesg->code != LDAP_SUCCESS ){
        my $text = "search failed\n".&LDAPerror($mesg);
	if($do_unbind){
	    $ldap->unbind();
	}
	return ( -1 , $text );
    }else{
	if( $mesg->count == 1){
	    if($do_unbind){
		$ldap->unbind();
	    }
	    return ( 1, undef );
	}elsif( $mesg->count > 1 ){
	    # uid is present more than once
	    if($do_unbind){
		$ldap->unbind();
	    }
	    return ( -1, "Something weird happened: \"$uid\" is present more than once");
	}else{
	    if($do_unbind){
		$ldap->unbind();
	    }
	    return ( 0, undef );
	}
    }
}

sub LDAP_filter_escape($){
    my $input = shift;
    #print STDERR "Escape Filter: $input";
    $input =~ s/\\/\\\\/g;
    $input =~ s/\0/\\\0/g;
    $input =~ s/\(/\\(/g;
    $input =~ s/\)/\\)/g;
    #print STDERR " $input\n";
    return $input;
}

sub LDAPerror($)  {
    my $mesg = shift;
    my $err_msg = "";
    
    $err_msg .= " LDAP-Server return code: ".$mesg->code ;
    $err_msg .= " Message: ".ldap_error_name($mesg->code);
    $err_msg .= "\n :".ldap_error_text($mesg->code);
    $err_msg .= "\n";
    return $err_msg;
}

@LDAPUtils::ISA = qw(Exporter);
@LDAPUtils::EXPORT = qw(
		   &LDAP_read_entry
		   &LDAP_is_samba_user
		   &LDAP_filter_escape
		   &LDAPerror
		  );
$LDAPUtils::VERSION = "0.1";

1;
