#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: RefSecMod.pm,v 1.2 2003/05/06 11:30:44 varkoly Exp $
#
package RefSecMod;

use strict;
use Apache::Constants ':common';
use MIME::Base64;
use Session;
use dataSession;
use Socket;
use FileHandle;
use URI::Escape;

sub handler {
  my $r = shift;

  my %args              = $r->args;
  my $req_filename      = $r->filename;
  my $c                 = $r->connection;
  my $remote_ip         = $c->remote_ip;

  
  return DECLINED if $c->remote_ip eq "127.0.0.1";

  #-- access to startup page is ok
  return DECLINED if $req_filename =~ /index\.pl/;

  #-- access to login page is ok
  if ( %args ) {
    return DECLINED if $args{'doit'} eq 'login';
    return DECLINED if ( ( $req_filename =~ /menu\.pl/ ) &&
                         ( $args{'tab'} eq 'logout' ) );
  }
  my $sessionID = $args{'sessionID'};
  $sessionID =~ s/\.htm// if defined $sessionID;

  my $_USECOOKIE;
  if ( $::CONF_GLOBAL{"SECURITY"}{"UseCookie"} eq "true"){
	$_USECOOKIE = 1;
  }else{
	$_USECOOKIE = 0;
  }

  #-- sessionID in request args ?
  #-- compare the client ip from the request with ip in the session
  my $client_ip = undef;
  if ( $sessionID ) {

      if( $_USECOOKIE ) {
	  my ($s_cookie, $uid) = split(/,/,__get_session_client_cookie($sessionID));

	  $uid = uri_escape($uid);

	  my $cookie = "";
	  if( $_USECOOKIE ) {
	      $cookie = $r->header_in('Cookie');
	      $cookie = substr($cookie,index($cookie,"sessionCookie-$uid-$sessionID=")
			       +length("sessionCookie-$uid-$sessionID="),32);
	  }

	  #print STDERR "s_cookie=<$s_cookie> cookie=<$cookie>\n";
	  if( ! defined $s_cookie || $s_cookie eq "" ) {
	      $r->internal_redirect( "/start/index.pl" );
	      return DONE;
	  }

	  if( $s_cookie ne $cookie ) {
	      my $q = new CGI();
	      $r->internal_redirect( "/start/warncookie.pl" );
	      $r->log_error("RefSecMod: COOKIES activated, but session cookie and client cookie are different");
	      return DONE;
	  }
      }
      
      if( $::CONF_GLOBAL{"SECURITY"}{"CheckClientIP"} ne "false" ){
	  $client_ip = __get_session_client_ip($sessionID);
	  if( ! defined $client_ip ) {
	      $r->log_error("RefSecMod: client_ip is undef");
	      $r->internal_redirect( "/start/index.pl" );
	      return DONE;
	  }
	  if ( $client_ip eq $remote_ip) {
	      return DECLINED;
	  } else {
	      $r->log_error("RefSecMod: client $remote_ip differently to ip in session from $client_ip");
	      return FORBIDDEN;
	  }
      }
  }

  return DECLINED;
}

sub __get_session_client_ip {
  my $sessionID = shift;
  return undef if ! $sessionID;

  my $line = s_getauth($sessionID);
  return undef  if ! $line;
  $line = decode_base64($line);
  my ($user, $pass, $lang, $ip) = split("\1",$line);

  return $ip    if $ip;
  return undef  if ! $ip;
}

sub __get_session_client_cookie {
  my $sessionID = shift;
  return undef if ! $sessionID;

  my $line = s_getcookie($sessionID);
  return undef if ! $line;
  return undef if $line =~ /^ERROR/;
  return $line;
}

1;
