#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: Session.pm,v 1.6 2004/09/16 08:22:44 varkoly Exp $
#
package SessionReply;
use strict;

sub new {
    my $this = shift;
    my $data = shift;
    my $code = shift || "OK";

    my $class = ref($this) || $this;
    my $self = {};
    $self->{"data"} = $data;
    $self->{"code"} = $code;

    bless $self, $class;
    return $self;
}

sub code {
    my $this = shift;
    return $this->{"code"};
}

sub data {
    my $this = shift;
    return ( defined $this->{"data"} ? $this->{"data"} : undef );
}

sub is_lsb_code() {
    my $this = shift;
    if($this->code() =~ /^OK:\s+\d+/) {
      return 1;
    }
    return 0;
}

sub lsb_code() {
    my $this = shift;
    if($this->code() =~ /^OK:\s+(\d+)/) {
      return $1;
    }
    return undef;
}

sub lsb_status() {
    my $this      = shift;
    my $is_status = shift;

    if($this->code() =~ /^OK:\s+(\d+)/) {
      my $lsb_code= $1;
      my @lsb_str = $is_status ? (
                    'LSB_S_RUNNING', 'LSB_S_DEAD_PID',
                    'LSB_S_DEAD_LCK','LSB_S_NOT_RUN'
                  ) : (
                    'LSB_SUCCESS',   'LSB_GENERIC',
                    'LSB_ARGUMENT',  'LSB_FEATURE',
                    'LSB_PRIVS',     'LSB_PROG_INST',
                    'LSB_PROG_CONF', 'LSB_PROG_NRUN'
                  );
      if(exists($lsb_str[$lsb_code])) {
        return  $lsb_str[$lsb_code];
      } else {
        return  'LSB_UNKNOWN';
      }
    }
    return undef;
}
#-----------------------------------------------------------------------


package Session;
use strict;
use Time::Local;
use Digest::MD5  qw(md5_hex);
use dataSession;
use Storable qw(thaw freeze);
use MIME::Base64;
use FileHandle;
use Socket;
use Carp qw(cluck);
use IO::Socket::SSL;

my $_image_path     = "/images";
my $_cgi_path       = ( $ENV{MOD_PERL} ? "/suse/" : "/imap-bin/" );
my $_lib_path       = "/srv/www/lib";
my $_help_path      = $_lib_path."/help";
my $_shorthelp_path = $_lib_path."/shorthelp";
my $_bin_path       = "/usr/sbin";


#-----------------------------------------------------------------------
sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {};
    bless $self, $class;
    return $self;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub image_path {
    my $this = shift;
    return $_image_path;
}

sub cgi_path {
    my $this = shift;
    return $_cgi_path;
}

sub lib_path {
    my $this = shift;
    return $_lib_path;
}

sub help_path {
    my $this = shift;
    return $_help_path;
}

sub bin_path {
    my $this = shift;
    return $_bin_path;
}

#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub create() {
    my $this = shift;
    my $dref = shift;

    my $timestamp = timelocal(localtime());
    # TODO: make this configurable
    my $livetime = ($::CONF_GLOBAL{'SESSIOND'}{'SessionTimeout'}*60) || 3600; # 1h
    my $ID = "";
    
    
    my $rand = rand((time)*$$);
    $ID = md5_hex($rand.$dref);
    
    #print STDERR "ID:$ID\n";

    my $authdata =  encode_base64($dref->{uid}."\1".$dref->{passwd}."\1".$dref->{LANG}."\1".$ENV{REMOTE_ADDR});

    my $line = s_add($timestamp, $livetime, $ID, $authdata, $dref);
    
    #print STDERR "sessionReturn:$line\n";

    if($line =~ /^ERROR:(.*)/) {
	return new SessionReply(undef, "FAILED: $1");
    }

    $line =~ s/\0//g;
    #print STDERR "LINE=<$line>\n";

    $line =~ /.*REPLACEDID=([a-z0-9]*)/;
    my $replacedid = $1;

    $this->{"sessionID"} = $ID;
    return new SessionReply("$ID|$replacedid");
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub load() {
    my $this = shift;
    my $sessionID = shift;

    my $_sessionID = $sessionID;

    my $confParam = s_read($_sessionID);

    if( ! defined $confParam ) {
	cluck("FAILED: s_read returned undef\n");
	return new SessionReply(undef, "FAILED: s_read returned undef");
    }

    if( $confParam =~ /^ERROR:(.*)/) {
	#print STDERR "FAILED: $1\n";
	return new SessionReply(undef, "FAILED: $1");
    }

    $this->{"confParam"} = $confParam;
    $this->{"confParam"}->{"image_path"}     = $_image_path;
    $this->{"confParam"}->{"cgi_path"}       = $_cgi_path;
    $this->{"confParam"}->{"lib_path"}       = $_lib_path;
    $this->{"confParam"}->{"help_path"}      = $_help_path;
    $this->{"confParam"}->{"shorthelp_path"} = $_shorthelp_path;
    $this->{"confParam"}->{"bin_path"}       = $_bin_path;

    # also store sessionID to use it later in Display.pm
    $this->{"confParam"}->{"sessionID"} = $_sessionID;
    # and store also the sessionID of the loaded session
    $this->{"sessionID"} = $_sessionID;
    
    return new SessionReply($this->{"confParam"});
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub modify() {
    my $this = shift;
    my $mod_data = shift;

    my $timestamp = timelocal(localtime());
    # TODO: make this configurable
    my $livetime = ($::CONF_GLOBAL{'SESSIOND'}{'SessionTimeout'}*60) || 3600; # 1h
    
    if( ! defined $this->{"sessionID"} ) {
	return new SessionReply(undef, "No such Session");
    }

    my $confParam = s_read($this->{"sessionID"});
    my $authd = s_getauth($this->{"sessionID"});
    my $rem_ip = (split("\1",decode_base64($authd)))[3];

    if( ! defined $confParam ) {
	return new SessionReply(undef, "FAILED: s_read returned undef");
    }

    if( $confParam =~ /^ERROR:(.*)/ ) {
	return new SessionReply(undef, "FAILED: $1");
    }
    
    foreach my $key (keys %$mod_data) {
	$confParam->{"$key"} = $mod_data->{"$key"};
    }
    
    my $authdata =
	encode_base64($confParam->{uid}."\1".$confParam->{passwd}."\1".$confParam->{LANG}."\1".$rem_ip);

    my $line = s_add($timestamp, $livetime, $this->{"sessionID"}, $authdata, $confParam);
 
    if( ! defined $line ) {
	return new SessionReply(undef, "FAILED: s_add returned undef");
    }
    
    if($line =~ /^ERROR:(.*)/) {
	return new SessionReply(undef, "FAILED: $1");
    }

    return new SessionReply();
}		     
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub clear() {
    my $this = shift;

    my $ID = shift;
    my $killown = 0;
    if( ! defined $ID || $ID eq "" ) {
	$ID = $this->{"sessionID"};
	$killown = 1;
    }

# C SCHULSERVER
#    if( $this->{"sessionID"} ne $ID &&
#       $this->{"confParam"}->{"uid"} ne "cyrus" ) {
#       return new SessionReply(undef, "FAILED: Permission denied");
#    } 
    if( $this->{"sessionID"} ne $ID && 
        $this->{"confParam"}->{"uid"} ne "cyrus" &&
        $this->{"confParam"}->{"admin"} ne "1" ) {

	return new SessionReply(undef, "FAILED: Permission denied");
    }

    if( ! $ID ) {
	return new SessionReply(undef, "FAILED: No such Session");
    }
    
    s_clear($ID);

    if( $killown ) {
	# clean up...
	$this->{"sessionID"} = undef;
	$this->{"confParam"} = undef;
    }
    
    return new SessionReply();
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub monitor() {
    my $this = shift;

    my $confParam = $this->{"confParam"};
    
    my $data = s_monitor($confParam->{uid}, $confParam->{passwd});

    if( $data =~ /^ERROR:(.*)/ ) {
	return new SessionReply(undef, "FAILED: $1");
    }

    my $mdata;
    foreach my $line ( split(/\n/,$data) ) {
	next if ! defined $line;
	next if $line eq "";
	my ( $id, $ttl, $lt, $ad, $dl ) = $line =~ /ID=(.*) TTL=(.*) LT=(.*) AD=(.*) DL=(.*)/;
	my $authdata = decode_base64($ad);
	$mdata->{$id}->{uid} = (split("\1", $authdata))[0];
	$mdata->{$id}->{lang} = (split("\1", $authdata))[2];
	$mdata->{$id}->{remoteaddr} = (split("\1", $authdata))[3];
	$mdata->{$id}->{ttl} = $ttl;
	$mdata->{$id}->{lt} = $lt;
	$mdata->{$id}->{dl} = $dl;
    }

    return new SessionReply($mdata);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub setcookie() {
    my $this = shift;

    my $cookie    = shift;
    my $sessionID = shift;

    my $ret = s_setcookie($sessionID, $cookie);
    
    return new SessionReply(undef, $ret);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getcookie() {
    my $this = shift;

    my $confParam = $this->{"confParam"};
    
    my $ret = s_getcookie($confParam->{sessionID});
    
    return new SessionReply(undef, $ret);
}
#-----------------------------------------------------------------------
		     
#-----------------------------------------------------------------------
sub save_clear2session() {
    my $this = shift;
    my $s_key     = shift;

    if( ! defined $this->{"sessionID"} ) {
	return new SessionReply(undef, "No such Session");
    }

    my %tmp = ( "$s_key" => "" );
    $tmp{"md5.$s_key"} = "";
    if( ($this->modify(\%tmp))->code() ne "OK" ) {
	return new SessionReply(undef, "unable to store data in session");
    }
    return new SessionReply();
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub save_read2session() {
    my $this  = shift;
    my $s_key = shift;
    my $file  = shift;
    
    my $_confParam = $this->{"confParam"};
    my $_sessionID = $this->{"sessionID"};
    
    if( ! defined $_sessionID ) {
	return new SessionReply(undef, "No such Session");
    }

    my $filedata = $this->suad_read($file);
    if( $filedata->code() =~ /^ERROR:(.*)/ ) {
	return new SessionReply(undef, "$1");
    }
    my $all = $filedata->data();

    #if( ! open(IN, "$file") ) {
	#return new SessionReply(undef, "unable to open $file");
    #}
    #my @all = <IN>;
    #close(IN);
    
    # if file has already been loaded into our session, lets look, if
    # someone modified it
    if( defined $_confParam->{"md5.$s_key"} && $_confParam->{"md5.$s_key"} ne "" ) {
	#if( md5_hex(@all) ne $_confParam->{"md5.$s_key"} ) {
	if( md5_hex($all) ne $_confParam->{"md5.$s_key"} ) {
	    # if so, free the old data and md5sum
	    my %tmp = ( "$s_key" => "" );
	    $tmp{"md5.$s_key"} = "";
	    ($this->modify(\%tmp))->code();
	    return new SessionReply(undef, "MD5SUM");
	}
    }
    
    return new SessionReply($_confParam->{"$s_key"});
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub save_store2session() {
    my $this = shift;
    my $s_key     = shift;
    my $file      = shift;
    my $data      = shift;

    my $_confParam = $this->{"confParam"};
    my $_sessionID = $this->{"sessionID"};

    if( ! defined $_sessionID ) {
	return new SessionReply(undef, "No such Session");
    }

    my $filedata = $this->suad_read($file);
    if( $filedata->code() =~ /^ERROR:(.*)/ ) {
	return new SessionReply(undef, "$1");
    }
    my $all = $filedata->data();

    #if( ! open(IN, "$file") ) {
	#return new SessionReply(undef, "unable to open $file");
    #}
    #my @all = <IN>;
    #close(IN);

    # if file has already been loaded into our session, lets look, if
    # someone modified it
    if( defined $_confParam->{"md5.$s_key"} && $_confParam->{"md5.$s_key"} ne "" ) {
	#if( md5_hex(@all) ne $_confParam->{"md5.$s_key"} ) {
	if( md5_hex($all) ne $_confParam->{"md5.$s_key"} ) {
	    # if so, free the old data and md5sum
	    my %tmp = ( "$s_key" => "" );
	    $tmp{"md5.$s_key"} = "";
	    ($this->modify(\%tmp))->code();
	    return new SessionReply(undef, "MD5SUM");
	}
    }
	    
    my %tmp = ( "$s_key" => $data );
    #$tmp{"md5.$s_key"} = md5_hex(@all);
    $tmp{"md5.$s_key"} = md5_hex($all);

    if( ($this->modify(\%tmp))->code() ne "OK" ) {
	return new SessionReply(undef,"unable to store data in session");
    }
    return new SessionReply();
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getLanguage() {
    my $this = shift;
    my $lang = shift;
    my @part = @_;
    
    unshift @part, "GLOBAL";
    
    #print STDERR @part." LANG=$lang\n";
    
    # -- Alle Meldungs/Feldtexte in HASH holen
    my %message=();
    
    foreach my $section (@part) {
	my $section = uc($section);
	
	# $lmsg is initialized in startup.perl
	if( ! defined $::lmsg->{$lang} ) {
	    $lang = "EN";
	}
	my @parameters = $::lmsg->{$lang}->Parameters($section);

	foreach(@parameters){
	    my $value = $::lmsg->{$lang}->val($section, $_);
	    #print STDERR "Key:$_ Val:$value \n";
	    $message{$_} = $value;
	}
    }

    return new SessionReply(\%message);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_save() {
    my $this = shift;
    my ($data, $file) = @_;
    
    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "write\n";
    print $SOCK $file."\n";
    foreach my $l ( @$data ) {
	#print STDERR "$l\n";
	print $SOCK "$l\n";
    }
    print $SOCK "\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_read() {
    my $this = shift;
    my $file = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "read\n";
    print $SOCK $file."\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=sysread($SOCK,$buf,4096)) ) {
	$line .= $buf;
    }
    #chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    if( $line =~ /^ERROR:/ ) {
	return new SessionReply(undef, "$line");
    }else{
	return new SessionReply(decode_base64($line), "OK");
    }
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_mkdir() {
    my $this = shift;
    my $dir    = shift;
    my $mode   = shift || "0700";

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "mkdir\n";
    print $SOCK "$dir\n";
    print $SOCK "$mode\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_chown() {
    my $this = shift;
    my $dir   = shift;
    my $owner = shift;
    my $group = shift || "users";

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "chown\n";
    print $SOCK "$dir\n";
    print $SOCK "$owner\n";
    print $SOCK "$group\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_unlink() {
    my $this = shift;
    my $file   = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "unlink\n";
    print $SOCK "$file\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_chmod() {
    my $this = shift;
    my $path    = shift;
    my $mode   = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "chmod\n";
    print $SOCK "$path\n";
    print $SOCK "$mode\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_change_smbpw() {
    my $this = shift;
    my $uid       = shift;
    my $oldpasswd = shift;
    my $newpasswd = shift;

    if( ! defined $newpasswd || $newpasswd eq "" ) {
	return new SessionReply(undef, "ERROR: no or empty new password: $!");
    }

# C SCHULSERVER
#    if( $this->{"confParam"}->{"uid"} ne "cyrus" && ( ! defined $oldpasswd || $oldpasswd eq "" ) ) {
#       return new SessionReply(undef, "ERROR: no or empty old password: $!");
#    }
# E SCHULSERVER
    if( $this->{"confParam"}->{"uid"} ne "cyrus" && ( ! defined $oldpasswd || $oldpasswd eq "" ) &&
        $this->{"confParam"}->{"gidnumber"} ne "102" ) {
	return new SessionReply(undef, "ERROR: no or empty old password: $!");
    }

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "smbchangepw\n";
# C SCHULSERVER
#    if( $this->{"confParam"}->{"uid"} eq "cyrus" ) {
# E SCHULSERVER
    if( $this->{"confParam"}->{"uid"} eq "cyrus" || $this->{"confParam"}->{"gidnumber"} eq "102" ) {
	print $SOCK $uid."\n";
	print $SOCK $newpasswd."\0";
    } else {
	print $SOCK $oldpasswd."\n";
	print $SOCK $newpasswd."\0";
    }
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_smbadduser() {
    my $this = shift;
    my $uid    = shift;
    my $passwd = shift;

    if( ! defined $passwd || $passwd eq "" ) {
	return new SessionReply(undef, "ERROR: no or empty password: $!");
    }

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "smbadduser\n";
    print $SOCK $uid."\n";
    print $SOCK $passwd."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_smbaddmachine() {
    my $this = shift;
    my $name    = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "smbaddmachine\n";
    print $SOCK $name."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_smbdeluser() {
    my $this = shift;
    my $uid    = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "smbdeluser\n";
    print $SOCK $uid."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_saslsetpw() {
    my $this = shift;
    my ($user, $passwd) = @_;

    if( ! defined $passwd || $passwd eq "" ) {
	return new SessionReply(undef, "ERROR: no or empty password: $!");
    }

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "saslpw\n";
    print $SOCK $user."\n";
    print $SOCK $passwd."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_copy() {
    my $this = shift;
    my ($src, $dst) = @_;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "copy\n";
    print $SOCK $src."\n";
    print $SOCK $dst."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_move() {
    my $this = shift;
    my ($src, $dst) = @_;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "move\n";
    print $SOCK $src."\n";
    print $SOCK $dst."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_copydir() {
    my $this = shift;
    my ($src, $dst) = @_;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "copydir\n";
    print $SOCK $src."\n";
    print $SOCK $dst."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_exec() {
    my $this = shift;
    my $cmd = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "exec\n";
    print $SOCK $cmd."\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_service_exec() {
    my $this = shift;
    my $service = shift;
    my $arg     = shift || "start";
    my $opt     = shift || undef;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "service\n$service\n$arg";
    if( defined $opt ) {
	print $SOCK "\n$opt\0";
    } else {
	print $SOCK "\0";
    }
    $SOCK->flush();

    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	$line .= $buf;
	# silly workaround:
	# read blocks from time to time because it makes another loop without additional
	# data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_listfile() {
    my $this = shift;
    my $path = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "listfile\n$path\0";
    $SOCK->flush();

    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	  $line .= $buf;
	  # silly workaround:
	  # read blocks from time to time because it makes another loop without additional
	  # data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	  last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_listdir() {
    my $this = shift;
    my $path = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "listdir\n$path\0";
    $SOCK->flush();

    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	  $line .= $buf;
	  # silly workaround:
	  # read blocks from time to time because it makes another loop without additional
	  # data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	  last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_setfacl() {
    my $this = shift;
    my $file = shift;
    my $acl  = shift;
    my $opt  = shift || undef;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "setfacl\n$file\n$acl";
    if( defined $opt ) {
	  print $SOCK "\n$opt\0";
    } else {
	  print $SOCK "\0";
    }
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_getfacl() {
    my $this = shift;
    my $file = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "getfacl\n$file\0";
    $SOCK->flush();

    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	  $line .= $buf;
	  # silly workaround:
	  # read blocks from time to time because it makes another loop without additional
	  # data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	  last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_setfquota() {
    my $this   = shift;
    my $fs     = shift;
    my $user   = shift;
    my $quota  = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "setfquota\n";
    print $SOCK "$fs\n";
    print $SOCK "$user\n";
    print $SOCK "$quota\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);
    
    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_getfquota() {
    my $this   = shift;
    my $fs     = shift;
    my $user   = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "getfquota\n";
    print $SOCK "$fs\n";
    print $SOCK "$user\0";
    $SOCK->flush();
    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	  $line .= $buf;
	  # silly workaround:
	  # read blocks from time to time because it makes another loop without additional
	  # data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	  last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);

}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_symlink() {
    my $this = shift;
    my $src  = shift;
    my $dst  = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");

    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "symlink\n";
    print $SOCK "$src\n";
    print $SOCK "$dst\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub suad_exists() {
    my $this = shift;
    my $name = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");

    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "exists\n";
    print $SOCK "$name\0";
    $SOCK->flush();

    my $line = "";
    my $ret;

    while( $ret=read($SOCK,my $buf,4096) ) {
	  $line .= $buf;
	  # silly workaround:
	  # read blocks from time to time because it makes another loop without additional
	  # data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	  last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    if($line =~  /^OK:\s+/) {
       $line =~ s/^OK:\s+//;
       #
       # returns lines with, e.g.:
       #  type=FILE
       #  size=1111
       # or "NONE" if not exists
       #
       return new SessionReply("$line");
    } else {
       return new SessionReply(undef, $line);
    }
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub rm_mail() {
    my $this = shift;
    my $qid = shift;
    my $qdepth = shift || 2;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");
    
    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "rmmail\n";
    print $SOCK "$qdepth $qid";
    print $SOCK "\0";
    $SOCK->flush();
    my $l = <$SOCK>;
    chomp($l);
    #print STDERR "<$l>\n";
    close($SOCK);

    return new SessionReply(undef, $l);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub comFireLogout {
    my $this = shift;

    my $sessionID = shift || $this->{"confParam"}->{"sessionID"};

    my $remotehost = $::CONF_GLOBAL{'GENERAL'}{'GroupwareHostname'};
    if( ! defined $remotehost || $remotehost eq "" ) {
	$remotehost = "localhost";
    }

    my $request = "GET /servlet/intranet?NAS_ID=$sessionID&SITE=logout&NAS_END_SESSION=A&sessionID=$sessionID HTTP/1.0\r\ncontent-length: 0\r\n\r\n";
    $request   .= "GET /servlet/webmail?NAS_ID=$sessionID&SITE=logout&NAS_END_SESSION=A&sessionID=$sessionID HTTP/1.0\r\ncontent-length: 0\r\n\r\n";

    my $addr = $remotehost;
    my $port = "80";
    my $ipaddr = inet_aton($addr) || return new SessionReply(undef, "ERROR: no host $addr");
    my $paddr = sockaddr_in($port, $ipaddr);
    socket(SSOCK, PF_INET, SOCK_STREAM,0) || return new SessionReply(undef, "ERROR: socket: $!");
    connect(SSOCK, $paddr)  || return new SessionReply(undef, "ERROR: connect: $!");
    
    print SSOCK $request;
    SSOCK->flush();
    
    my $line = "";
    my $ret = "";
    my $buf = "";
    while( ($ret=sysread(SSOCK,$buf,4096)) ) {
	$line .= $buf;
    }
    
    #print STDERR "<$line>\n";

    close(SSOCK);
}
#-----------------------------------------------------------------------

sub my_connect {
    my $sock;
    my $socket = "/var/run/suad.sock";

    if( $::CONF_GLOBAL{'SUAD'}{'SuadHost'} eq "unix" || ! defined $::CONF_GLOBAL{'SUAD'}{'SuadHost'} ) {
	socket($sock, PF_UNIX, SOCK_STREAM,0) || return undef;
	connect($sock, sockaddr_un($socket))  || return undef;
    } else {
	#$IO::Socket::SSL::DEBUG=1;
	$sock = IO::Socket::SSL->new( PeerAddr => $::CONF_GLOBAL{'SUAD'}{'SuadHost'},
				      PeerPort => $::CONF_GLOBAL{'SUAD'}{'SuadPort'},
				      Proto    => 'tcp',
				      SSL_key_file => $::CONF_GLOBAL{'SUAD'}{'SSL_key_file'},
				      SSL_cert_file => $::CONF_GLOBAL{'SUAD'}{'SSL_cert_file'},
				      SSL_ca_file => $::CONF_GLOBAL{'SUAD'}{'SSL_ca_file'},
				      SSL_use_cert => 1,
				      SSL_verify_mode => 0x01 );

    }

    if( ! defined $sock ) {
	print STDERR "ERRNO=<$!> in my_connect\n";
    }

    return $sock;
}

sub my_read {
    my ( $FD, $refbuf, $nr ) = @_;

    #my $ret=read($FD,$$refbuf,$nr);
    my $ret=sysread($FD,$$refbuf,$nr);

    if( ! defined $ret ) {
	print STDERR "ERRNO=<$!> in my_read\n";
	if( $::CONF_GLOBAL{'SUAD'}{'SuadHost'} eq "unix" ) {
	    print STDERR "_get_SSL_err_str()=<".$FD->_get_SSL_err_str().">\n";
	    print STDERR "_SSL_want_read=<".${*$FD}{'_SSL_want_read'}.">\n";

	    if( $FD->want_read() ) {
		print STDERR "SSL_WANT_READ, read again...\n";
		$ret = sysread($FD,$$refbuf,$nr);
	    }
	}
    }

    return $ret;
}

sub my_close {
    my $FD = shift;
    
    close($FD);
    #if( $use_ssl ) {
	#shutdown($FD, 1);
    #}
}


sub DESTROY {
    my $this = shift;
    #if( defined $this->{"sessionID"} ) {
	#print STDERR "DESTROYING Object with ID=".$this->{"sessionID"}."...\n";
    #}
}

#-----------------------------------------------------------------------
sub suad_stdout_exec() {
    my $this = shift;
    my $command = shift;

    my $SOCK = my_connect() || return new SessionReply(undef, "ERROR: my_connect: $!");

    print $SOCK $this->{"confParam"}->{"uid"}."\n";
    print $SOCK $this->{"confParam"}->{"passwd"}."\n";
    print $SOCK $this->{"confParam"}->{"sessionID"}."\n";
    print $SOCK "stdout_exec\n$command\0";
    $SOCK->flush();

    my $line = "";
    my $ret;
    while( $ret=read($SOCK,my $buf,4096) ) {
	$line .= $buf;
	# silly workaround:
	# read blocks from time to time because it makes another loop without additional
	# data waiting to read, so suad will send \1 as EOF and we definitely quit here!
	last if index($buf,"\1");
    }
    close($SOCK);
    $line =~ s/\0//g;
    $line =~ s/\x1//g;
    return new SessionReply(undef, $line);
}
#-----------------------------------------------------------------------


1;
