#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: Utils.pm,v 1.7 2004/05/03 20:04:36 varkoly Exp $
#
require Exporter;
package Utils;

use strict;
use CGI;
use Config::IniFiles;
use Net::LDAP::Util qw( ldap_error_name
			ldap_error_text) ;
use Fcntl ':flock';
use File::Copy 'cp';
use MIME::Base64;
use vars qw($lmsg %CONF_GLOBAL);

my $globalIniFile = "/etc/imap/globals.conf";
my $libdir = "/srv/www/lib";

# I SCHULSERVER


#-----------------------------------------------------------------------
sub ConvertImportFile($$) {
        my ($lines, $format) = @_;
        open OUT, ">/tmp/userlist_in.txt";
        foreach (@$lines) {
                print OUT $_."\n";
        }
        close OUT;
        my $rc = 0xffff & system("(cd /tmp; /usr/sbin/convert$format.pl userlist_in.txt)");
        system ("rm /tmp/userlist_in.txt");

        if($rc != 0){
                return  0;
        }

        @$lines = ();
        @$lines = read_file("/tmp/userlist.txt");
        system ("rm /tmp/userlist.txt");

        return 1;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub read_rcconfig {
    my $searchfor = shift;

    my $erg = "";
    my $found = 0;

    my @a = read_file("/etc/sysconfig/schoolserver");

    foreach (@a) {
        if(/^$searchfor/) {
                /$searchfor=(.*\s+.*)/;
                $erg = $1;
                chomp($erg);
                $erg =~ s/"//g;
                $found = 1;
                last;
        }
    }
    return $erg;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub to_ascii {
    $_ = shift;

#       s//ue/g; s//ss/g; s//oe/g; s//ae/g;
#       s//ue/g; s//oe/g;
#       s//ue/g; s//Oe/g; s//Ae/g;
#       s//ue/g; s//Oe/g;
	Encode::from_to($_ , 'utf8', 'iso-8859-2');
        tr//uuUUsoOOoAa/;
        tr//aeeiouAEIOU/;

    return $_;
}
#-----------------------------------------------------------------------

# E SCHULSERVER


#-----------------------------------------------------------------------
sub read_Config {
    tie %CONF_GLOBAL, 'Config::IniFiles', ( -file => $globalIniFile );
    if( %CONF_GLOBAL ){
	print STDERR "Unable to load global configuration file\n";
	return undef;
    }
    return 1;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub read_LangINI {
    if( ! opendir(LIB, $libdir) ) {
	print STDERR "ERROR: unable to opendir $libdir\n";
	return undef;
    }
    
    print STDERR "\n";
    foreach my $langfile ( grep(/^imap_lang_[A-Z][A-Z]\.ini$/, readdir(LIB)) ) {
	$langfile =~ /imap_lang_(\w+)\.ini/;
	print STDERR "adding language $1\n";
	$lmsg->{$1} = new Config::IniFiles( -file => $libdir."/".$langfile );
    }
    
    closedir(LIB);
    
    if( ! defined $lmsg ) {
	print STDERR "Unable to load a language file\n";
	return undef;
    }

    return 1;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub parse_file {
    my $file = shift;
    my @searchfor = @_;

    my @erg = ();
    my $found = 0;

    my @a = read_file($file);

    for(my $i=0; $i<=$#searchfor; $i++) {
	foreach (@a) {
	    if($_ =~ /^$searchfor[$i]/i) {
		$_ =~ s/$searchfor[$i]\s*(.*)/$1/i;
		chomp($_);
		push @erg, $_;
		$found = 1;
		last;
	    }
	}
	if($found != 1) {
	    push @erg, "";
	} else {
	    $found = 0;
	}
    }
    return @erg;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub read_file($) {
  my $file = shift;
  local *F;
  open F, $file || die "Couldn't open file '$file' for reading: $!; aborting";
  local $/ unless wantarray;
  <F>;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub is_address_unique($$$$) {
    my $ldap = shift;
    my $alias = shift;
    my $uid = shift;
    my $basedn = shift;

    my $errs = "";

    my $filter = "(& (| (alias=$alias) (uid=$alias) ) (! (uid=$uid)))";
    my $mesg = $ldap->search (	# perform a search
			      base   => "$basedn",
			      scope => "one",
			      attrs => ['objectclass'],
			      filter => "$filter" 
			     );
    if($mesg->code == 0) {
	if($mesg->count != 0) {
	    return 0;
	}
    } else {
	print STDERR &LDAPerror($mesg);
	return 0;
    }
    
    return 1;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub cmd_pipe
{
        my $cmd = shift;
        my $arg = shift;
        my $err = "";
        my $pid = open(KID, "-|");

	#print STDERR "CMD=$cmd, ARG=$arg, PID=$pid\n";

        if( defined( $pid)) {
	    if ($pid) { # parent
		while(<KID>) {
		    chomp;
		    $err .= "$_ ";
		}
		close(KID);
	    } else {    # child
		if( !open(CMD, "| $cmd 2>&1")) {
			    print STDERR "can not open $cmd cmd-pipe";
                        } else {
			    print CMD "$arg";
			    close(CMD);
                        }
	      CORE::exit;
	    }
        } else { # open failed
	    $err = "cannot fork $cmd";
        }
        return "$err";
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub readFilesRecursive {
    my $currentdir = shift;
    my $allFiles   = shift;;
    
    opendir(DIR,$currentdir) || die "Unable to opendir $currentdir: $!";
    
    my @ret = readdir(DIR);
    foreach my $f ( @ret ) {
        next if $f =~ /^\./;
        my $fqp = $currentdir."/".$f;
        if( -d $fqp ) {
            readFilesRecursive($fqp,$allFiles);
        } else {
            push @$allFiles, $fqp;
        }
    }
    
    closedir(DIR);
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub usesSPAMFilter {
    my $masterraw = shift;

    my $mastercf = "/etc/postfix/master.cf";
    if( ! defined $masterraw ) {
	if( ! open(IN, "< $mastercf") ) {
	    print STDERR "usesSPAMFilter: $!\n";
	    return 0;
	}
	
	my @cf = <IN>;
	chomp(@cf);
	close(IN);
	$masterraw = \@cf;
    }

    my $count=0;
    for(my $i=0; $i<scalar(@$masterraw); $i++) {
	my $l = $masterraw->[$i];
	$count++ if $l =~ /^[\w:]+\s+inet.*content_filter=(?:procmail|spamcheck):spamfilter/;
	if( $l =~ /^(?:procmail|spamcheck)\s+unix/ ) {
	    $count++ if $masterraw->[$i+1] =~ /^\s+flags=.*(?:procmail|spamcheck).*sender/;
	}
    }
    return ( $count >= 2 ? 1 : 0 );
}

#-----------------------------------------------------------------------
# Extract the Timezone-Name from /usr/share/zoneinfo/zone.tab
sub getTimeZones{
    my $tzf = '/usr/share/zoneinfo/zone.tab';
    my @timezones;
    open(TZ, "$tzf") || return undef;
    while(<TZ>) {
	next if /^#/;
	next if /^\s*$/;
	m/(\w\w)\s+(.*?)\s+(.*?)\s+.*/;
	push @timezones, $3;
    }
    @timezones = sort(@timezones);
    #add the Standard 3 Letter timezones just to make sure
    push @timezones,( "ACT", "AET", "AGT", "ART", "AST", "BET", "BST", "CAT",
	    "CET", "CNT", "CST", "CTT", "EAT", "ECT", "EET", "EST", "GMT",
	    "HST", "IET", "IST", "JST", "MET", "MIT", "MST", "NET", "NST",
	    "PLT", "PNT", "PRT", "PST", "SST", "UTC", "VST", "WET", "Etc/GMT",
	    "Etc/GMT+1", "Etc/GMT+2", "Etc/GMT+3", "Etc/GMT+4", "Etc/GMT+5",
	    "Etc/GMT+6", "Etc/GMT+7", "Etc/GMT+8", "Etc/GMT+9", "Etc/GMT+10",
	    "Etc/GMT+11", "Etc/GMT+12", "Etc/GMT-1", "Etc/GMT-2", "Etc/GMT-3",
	    "Etc/GMT-4", "Etc/GMT-5", "Etc/GMT-6", "Etc/GMT-7", "Etc/GMT-8",
	    "Etc/GMT-9", "Etc/GMT-10", "Etc/GMT-11", "Etc/GMT-12",
	    "Etc/GMT-13", "Etc/GMT-14", "Etc/UCT", "Etc/UTC");
    return @timezones;
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub getPROTO {
    return ( defined $ENV{'HTTPS'} && $ENV{'HTTPS'} eq "on" ? "https://" : "http://" );
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub sync_comfire_contact($) {
    my $user = shift;

    my $command = "/opt/comfire/application/scripts/insUpdSystemUser $user";

    system($command);
    my $exit_value  = $? >> 8;

    if($exit_value != 0) {
	# error
        return (-1,"insUpdSystemUser");
    }
    return (0, "");
}
#-----------------------------------------------------------------------

@Utils::ISA = qw(Exporter);
@Utils::EXPORT = qw(
		    &read_rcconfig
		    &ConvertImportFile
		    &read_file
		    &parse_file
		    &is_address_unique
		    &cmd_pipe
		    &read_LangINI
		    &read_Config
		    &readFilesRecursive
		    &usesSPAMFilter
		    &getTimeZones
		    &getPROTO
		    &sync_comfire_contact
                    &to_ascii
		    %CONF_GLOBAL
		    $lmsg
		    );
$Utils::VERSION = "0.1";

1; 

