#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: cfreader.pm,v 1.1.1.1 2003/03/25 10:31:38 ray Exp $
#
package cfreader;
use strict;


sub new {
    my $this      = shift;
    my $delimiter = shift || "=";

    my $class = ref($this) || $this;
    my $self = {};
    $self->{"delimiter"} = $delimiter;

    bless $self, $class;
    return $self;
}

sub setAllCMD($$$$) {
    my $this = shift;
    my ($hr, $ar, $deak, $ak) = @_;

    foreach my $k ( @$ar ) {
	next if ! defined $k;
	$k->{value} = $hr->{$k->{cmd}};

	if( defined @$deak ) {
	    foreach my $d ( @$deak ) {
		$k->{status} = "C_CMD" if $k->{cmd} eq $d;
	    }
	}
	if( defined @$ak ) {
	    foreach my $d ( @$ak ) {
		$k->{status} = "CMD" if $k->{cmd} eq $d;
	    }
	}
    }
}

sub getAllArray($) {
    my $this = shift;
    my $r    = shift;
    my $ret;

    foreach my $l (@$r) {
	next if ! defined $l;
	next if $l->{status} eq "COMMENT";
	next if $l->{status} eq "BLANK";
	push @$ret, { cmd => $l->{cmd},
		      value => $l->{value},
		      multi => $l->{multi},
		      status => $l->{status}
		  };
    }
    
    return $ret;
}

sub setAllArray($$) {
    my $this = shift;
    my ($ar, $r) = @_;
    
    my $cnt = 0;
    foreach my $rh (@$r) {
	next if $rh->{status} eq "COMMENT";
	next if $rh->{status} eq "BLANK";
	if( ! defined $ar->[$cnt] ) {
	    $rh = undef;
	} else {
	    $rh->{status} = $ar->[$cnt]->{status};
	    $rh->{cmd} = $ar->[$cnt]->{cmd};
	    $rh->{value} = $ar->[$cnt]->{value};
	    $rh->{multi} = $ar->[$cnt]->{multi};
	}
	$cnt++;
    }
}

sub getAllCMD($) {
    my $this = shift;
    my $r    = shift;
    my $ret;

    foreach my $l (@$r) {
	next if ! defined $l;
	next if $l->{status} eq "COMMENT";
	next if $l->{status} eq "BLANK";
	$ret->{$l->{cmd}} = $l->{value};
    }

    return $ret;
}

sub getAllSTATUS($) {
    my $this = shift;
    my $r = shift;
    my $ret;

    foreach my $l (@$r) {
	next if ! defined $l;
	next if $l->{status} eq "COMMENT";
	next if $l->{status} eq "BLANK";
	$ret->{$l->{cmd}} = $l->{status};
    }

    return $ret;
}

sub isComment($) {
    my $this = shift;
    my $l = shift;

    return $l =~ /^[\#]/;
}

sub isCommentedCommand($$$$) {
    my $this = shift;
    my ( $l, $refc, $refv, $rcom ) = @_;

    $$refc = "";
    $$refv = "";

    # improve performance: skip if there is no delimiter in line
    # then it can't be a commented command at all 
    return 0 if index($l,$this->{delimiter}) < 0;

    foreach( @$rcom ) {
	# improve performance: only if command is somewhere in line
	# we should use the time expensive expression
	if( index($l,$_) > 0 ) {
	    if( $l =~ /^\#+\s*$_\s*$this->{delimiter}\s*(.*)$/ ) {
		$$refc  = $_;
		$$refv  = $1;
	    }
	}
    }

    return ( $$refc eq "" ? 0 : 1 );
}

sub isBlank($) {
    my $this = shift;
    my $l = shift;

    return ( ! defined $l || $l eq "" ? 1 : 0 );
}

sub getCommands($) {
    my $this = shift;
    my $shellcmd = shift;
    my @commands;

    open(IN, "$shellcmd |") ||
	return undef;

    while( <IN> ) {
	chomp;
	s/(^\w+).*$/$1/;
	push @commands, $_;
    }

    close(IN);

    return \@commands;
}

sub HASH2CF($) {
    my $this = shift;
    my $hr = shift;
    my $ra;

    for(my $i=0; $i < @$hr; $i++ ) {

	next if ! defined $hr->[$i]->{cmd};

	if( $hr->[$i]->{status} eq "COMMENT" ) {
	    push @$ra, $hr->[$i]->{cmd};
	} elsif( $hr->[$i]->{status} eq "C_CMD" ) {
	    push @$ra, "# ".$hr->[$i]->{cmd}.$this->{delimiter}." ".$hr->[$i]->{value};
	} elsif( $hr->[$i]->{status} eq "BLANK" ) {
	    push @$ra, "";
	} elsif( $hr->[$i]->{status} eq "CMD" ) {
	    if( $hr->[$i]->{multi} ) {
		my @v = split("\n", $hr->[$i]->{value});
		for(my $x=0; $x < $#v + 1; $x++) {
		    $v[$x] = "     ".$v[$x] if $x>0;
		}
		push @$ra, $hr->[$i]->{cmd}.$this->{delimiter}." ".join("\n", @v);
	    } else {
		push @$ra, $hr->[$i]->{cmd}.$this->{delimiter}." ".$hr->[$i]->{value};
	    }
	} else {
	    push @$ra, "# THIS LINE SHOULD NOT BE HERE";
	}
    }
    return $ra;
}

sub CF2HASH($$) {
    my $this = shift;
    my ($t, $u) = @_;
    my @all = @$t;
    my $com;
    my @ready;

    $com = ( defined $u ? $u : undef );

    my $tmp = "";
    my $cnt = 0;
    my $cmd;
    my $value;
    my @cmds;
    my %c_cmdcnt = ();
    my %cmdcnt = ();

    while( $cnt < $#all + 1 ) {
	
	if( $this->isComment($all[$cnt]) ) {
	    
	    # if second arg is not defined, we don't want to
	    # look for commented commands
	    if( defined $u ) {
		if( $this->isCommentedCommand($all[$cnt], \$cmd, \$value, $com) ) {
		    #print STDERR "COMMENTED: <$cmd> = <$value>\n";
		    # drop duplicate commented commants
		    $c_cmdcnt{$cmd}->{cnt} = 0 if ! defined $c_cmdcnt{$cmd}->{cnt};
		    if( $c_cmdcnt{$cmd}->{cnt} < 1 ) {
			push @ready, 
			{ status => "C_CMD",
			  cmd => $cmd,
			  value => $value,
			  multi => 0 };
		    } else {
			#print STDERR "DROP: $cmd\n";
		    }
		    $c_cmdcnt{$cmd}->{cnt}++;
		} else {
		    #print "COMMENT\n";
		    push @ready,
		    { status => "COMMENT",
		      cmd => $all[$cnt],
		      value => "",
		      multi => 0 };
		}
	    } else {
		#print "COMMENT\n";
		push @ready,
		{ status => "COMMENT",
		  cmd => $all[$cnt],
		  value => "",
		  multi => 0 };
	    }
	} elsif( $this->isBlank($all[$cnt]) ) {
	    #print "BLANK\n";
	    push @ready,
	    { status => "BLANK",
	      cmd => "",
	      value => "",
	      multi => 0 };
	} else {
	    $tmp = $all[$cnt];
	    # values can preceed in the next line, so let's look
	    # if the next line is still a value
	    my $multiline=0;
	    while( (! $this->isBlank($all[$cnt+1]))   &&
		   (! $this->isComment($all[$cnt+1])) &&
		   ( $all[$cnt+1] =~ /^\s+/  ) &&
		   ( $cnt < $#all + 1 )           ) {
		
		$tmp .= ( $all[$cnt+1] =~ /^\s+/ ? "\n" : "" );

		$cnt++;
		$multiline++;
		
		$all[$cnt] =~ s/^\s+//;
		$tmp .= $all[$cnt];

	    }
	    # extract key and value out of $tmp with
	    # a multiline match
	    if ($this->{delimiter} eq " ") {
		($cmd, $value) = split /\s+/, $tmp, 2;
		# print STDERR "<$cmd>  <$value>\n";
	    } else {
		    ($cmd, $value) = $tmp =~ /(^[\w-]+)\s*$this->{delimiter}\s*(.*)$/ms;
		    #print STDERR "<$cmd> = <$value>\n";
    	    }
 
	    push @ready,
	    { status => "CMD",
	      cmd => $cmd,
	      value => $value,
	      multi => $multiline };

	    $cmdcnt{$cmd}->{cnt}++ if defined $u;
	}
	$cnt++;
    }
    
    # eleminate commented commands that are already present
    # as uncommented commands
    for( my $k=0; $k < @ready; $k++ ) {
	next if $ready[$k]->{status} eq "BLANK";
	next if $ready[$k]->{status} eq "COMMENT";
	next if $ready[$k]->{status} eq "CMD";
	my $c = $ready[$k]->{cmd};
	$cmdcnt{$c}->{cnt} = 0 if ! defined $cmdcnt{$c}->{cnt};
	if( $cmdcnt{$c}->{cnt} >= 1 ) {
	    # print STDERR "DROP COMMENTED: ".$ready[$k]->{cmd}."\n";
	    $ready[$k] = undef;
	}
    }

    return \@ready;
}

=head1 NAME
    
    cfreader.pm - reading and manipulating of key = value configuration files

=head1 SYNOPSIS
  
  use cfreader;

=head1 Variales

    none

=head1 METHODS

=head2 getCommands

B<Explanation>

    determines the available commands with help of an external shell
    command. The Output of this external command must be 'key =
    value'. getCommands reads all available keys into an array and
    returns its reference. The resulting array reference can be used
    to determine commented commands, which could be usefull if one
    wants to uncommend and activate them with maybe a new value. The
    CF2HASH method can use it to do this job.

B<Example>

    my $c = getCommands("/usr/sbin/postconf");
    die "unable to read available commands: $?" if not defined $c;

=head2 CF2HASH

B<Explanation>

    converts an array of lines of a configuration file into a hash of
    the following form:

    %array => <line_number> => status 
                            => cmd 
                            => value
                            => multi

    status can be one of "COMMENT", "BLANK", "C_CMD" and "CMD".

    "COMMENT": an ordinary commented line
    "BLANK":   an empty line
    "C_CMD":   a commented command
    "CMD":     an active command

    cmd is the key and value the value. A value can be preceded at a
    new line, e.g.:

    mykey = monday, tuesday, wednesday, thursday,
            friday, saturday, sunday

    the next line(s) MUST start with a whitespace

B<Example>

    my $cf = "/etc/postfix/main.cf";

    my $c = getCommands("/usr/sbin/postconf");
    die "unable to read available commands: $?" if not defined $c;

    open(IN, "< $cf") || die "unable to read $cf: $?";
    my @raw = <IN>;
    chomp(@raw);
    close(IN);

    my $r = CF2HASH(\@raw, $c);


    foreach( @$r ) {
	if( $r->{cmd} eq "mailbox_command" ) {
	    $r->{value} = "/bin/myMDA" ;
	    $r->{status} = "CMD" ;
	}
    }

    This code determines the commands available to postfix with help
    of the postconf command of postfix. After that it reads main.cf in
    a hash and we change the value of mailbox_command to
    /bin/myMDA. Then we set status to "CMD". 

=head2 HASH2CF

B<Explanation>

    This method converts the array-hash back to an array that can be written
    back to a file. The new file will be very similar to the original
    one, expect your modifications and a little exception:

    key=value
    #key= value

    will all be formated to exact

    key = value
    # key = value

    so don't be curious... :-) 

B<Example>

    my $ar = HASH2CF($r);

    open(OUT, "> test.cf") || die "unable to write test.cf: $?";

    foreach( @$ar ) {
	print OUT $_."\n";
    }

    close(OUT);

=head2 getAllCMD

B<Explanation>

    Flattens the array-hash to a simple hash in the form

    $h->{command} = value;

    It ONLY returns active commands, NOT commented commands. This can
    be used, if one want to check only all active commands.

=head2 getAllSTATUS

B<Explanation>

    Flattens the array-hash to the form

    $h->{command} = status;

    where status can be either CMD or C_CMD. BLANKS and COMMENTS are ignored.

=head2 setAllCMD

B<Explanation>

    This one can be used to "recombine" a hash-array for CF2HASH,
    e.g. if one has a manipulated only the hash returned from
    getAllCMD and want's to get it back to the hash-array from. Due to
    the fact, that there is no status field in the simple hash from
    getAllCMD, one may also specify one array for commands that must
    be commented and one for commands that must be uncommented.

B<Example>

    my @active = ( "cmd1", "cmd2", "cmd3" );
    my @inactive = ( "cmd4", "cmd5", "cmd6" );

    [...] do something

    $r  = CF2HASH(\@ar, \@cmds);
    $sr = getAllCMD($r);

    [...] do some other thing

    setAllCMD($sr, $r, \@inactive, \@active);

    or

    setAllCMD($sr, $r, undef, \@active);

=cut

1;
