#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: dataSession.pm,v 1.2 2003/05/06 11:30:44 varkoly Exp $
#
require Exporter;
package dataSession;
use strict;
use Socket;
use IO::Socket::SSL;
use FileHandle;
use Storable qw(thaw freeze);
use Carp qw(cluck);
use Net::SSLeay::Handle qw/shutdown/;

my $rendezvous = '/var/run/sessiond.sock';
my $readcount = 4096;

sub my_connect {
    my $sock;

    if( $::CONF_GLOBAL{'SESSIOND'}{'SessiondHost'} eq "unix" || ! defined $::CONF_GLOBAL{'SESSIOND'}{'SessiondHost'} ) {
	socket($sock, PF_UNIX, SOCK_STREAM,0);
	connect($sock, sockaddr_un($rendezvous));
    } else {
	#$IO::Socket::SSL::DEBUG=1;
	$sock = IO::Socket::SSL->new( PeerAddr => $::CONF_GLOBAL{'SESSIOND'}{'SessiondHost'},
				      PeerPort => $::CONF_GLOBAL{'SESSIOND'}{'SessiondPort'},
				      Proto    => 'tcp',
				      SSL_key_file => $::CONF_GLOBAL{'SESSIOND'}{'SSL_key_file'},
				      SSL_cert_file => $::CONF_GLOBAL{'SESSIOND'}{'SSL_cert_file'},
				      SSL_ca_file => $::CONF_GLOBAL{'SESSIOND'}{'SSL_ca_file'},
				      SSL_use_cert => 1,
				      SSL_verify_mode => 0x01 );
    }

    if( ! defined $sock ) {
	print STDERR "ERRNO=<$!> in my_connect\n";
    }

    return $sock;
}

sub my_read {
    my ( $FD, $refbuf, $nr ) = @_;

    #my $ret=read($FD,$$refbuf,$nr);
    my $ret=sysread($FD,$$refbuf,$nr);

    if( ! defined $ret ) {
	print STDERR "ERRNO=<$!> in my_read\n";
	if( $::CONF_GLOBAL{'SESSIOND'}{'SessiondHost'} eq "unix" ) {
	    print STDERR "_get_SSL_err_str()=<".$FD->_get_SSL_err_str().">\n";
	    print STDERR "_SSL_want_read=<".${*$FD}{'_SSL_want_read'}.">\n";

	    if( $FD->want_read() ) {
		print STDERR "SSL_WANT_READ, read again...\n";
		$ret = sysread($FD,$$refbuf,$nr);
	    }
	}
    }

    return $ret;
}

sub my_close {
    my $FD = shift;
    
    close($FD);
    #if( $use_ssl ) {
	#shutdown($FD, 1);
    #}
}

sub s_add {
    my ($timestamp, $livetime, $ID, $uid, $data) = @_;

    my $frozen = freeze($data);
    $frozen = unpack('H*',$frozen);

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "add: $timestamp $livetime $ID $uid $frozen\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);

    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    return $line;
}

sub s_read {
    my $ID = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";

    print $SOCK "read: $ID\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);

    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    return $line if $line =~ /^ERROR/;

    # remove binary seros
    #$line =~ s/\0//g;

    my $upline = pack('H*',$line); # convert back to binary
    my $data = thaw($upline);     # melt down... :-)

    if( ! defined $data ) {
	print STDERR "ERR: data is undef!!!\n";
	#print STDERR "LINE=$line\n";
	cluck("\n\nLINE=$line\n\n");
    }

    return $data;
}

sub s_getauth {
    my $ID = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "getauth: $ID\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);

    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    return $line if $line =~ /^ERROR/;
    
    if( ! defined $line ) {
	print STDERR "ERR: line is undef!!!\n";
	#print STDERR "LINE=$line\n";
	cluck("\n\nLINE=$line\n\n");
    }

    return $line;
}

sub s_remove {
    my $timestamp = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "remove: $timestamp\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);
    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    return $line;
}

sub s_clear {
    my $ID = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "clear: $ID\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);
    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    return $line;
}

sub s_monitor {
    my $uid    = shift;
    my $passwd = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "monitor: $uid $passwd\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);

    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    $line =~ s/\0//g;
    return $line;
}

sub s_setcookie {
    my $ID     = shift;
    my $COOKIE = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "setcookie: $ID $COOKIE\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);

    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    $line =~ s/\0//g;
    return $line if $line =~ /^ERROR/;
    
    if( ! defined $line ) {
	print STDERR "ERR: line is undef!!!\n";
	#print STDERR "LINE=$line\n";
	cluck("\n\nLINE=$line\n\n");
    }

    return $line;
}

sub s_getcookie {
    my $ID     = shift;

    my $SOCK = my_connect() || return "ERROR: my_connect: $!";
    print $SOCK "getcookie: $ID\0";
    $SOCK->flush();
    my $buf;
    my $line;
    my $ret;
    while( ($ret=my_read($SOCK,\$buf,$readcount)) ) {
	$line .= $buf;
    }
    my_close($SOCK);
    
    if( ! defined $ret ) {
	return "ERROR: read return undef!";
    }
    $line =~ s/\0//g;
    return $line if $line =~ /^ERROR/;
    
    if( ! defined $line ) {
	print STDERR "ERR: line is undef!!!\n";
	#print STDERR "LINE=$line\n";
	cluck("\n\nLINE=$line\n\n");
    }

    return $line;
}

@dataSession::ISA = qw(Exporter);
@dataSession::EXPORT = qw(
			  &s_add
			  &s_read
			  &s_remove
			  &s_clear
			  &s_getauth
			  &s_monitor
			  &s_setcookie
			  &s_getcookie
			  );
$dataSession::VERSION = "0.1";

=head1 NAME

    dataSession.pm - interface to session daemon

    The session daemon is a very simple daemon for session management.
    You can insert, read and manipulate data with id and timestamp.
    Cool use for temporary data in a websession...

    What ever ...

=head1 SYNOPSIS

    use dataSession;

=head1 METHODS

=head2 s_add

    - create a new session

    $return = &s_add("timestamp", "32charID", "data")
    return code is "OK: ID=32charID"

=head2 s_read

    - read session data

    $return = &s_read("32charID")
    return the data

=head2 s_remove

    - remove all sessions befor timestamp

    $return = &s_remove("timestamp")
    return code is "OK: removed 1 sessions"

=head2 s_clear

    - remove a session with 32charID

    $return = &s_clear("32charID")
    return code is "OK: removed session 32charID"

=head1 COPYRIGHT

    Copyright (c) 2000-2001 SuSE GmbH, Nuernberg, Germany. All rights reserved.

=cut


1;
