#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: IMAPFetch.pm,v 1.3 2003/09/10 12:27:33 varkoly Exp $
#
package IMAPFetch;
use Error;
use strict;
use Net::IMAP;

my $__msgnum;
my $__msgsizearr;
my $__msgarr;

my $lockdir = "/var/lock";

sub new {
    my $this = shift;

    my $data    = shift;

    my $class = ref($this) || $this;
    my $self = {};
    $self->{"data"}    = $data;
    
    bless $self, $class;
    return $self;
}

#
# _imap_status callback funtion for Net::IMAP::status
#
# This function is only used to determine the total number of messages
# in the selected mailbox. It sets $__msgnum
#
sub _imap_status {
    my $self = shift;
    my $resp = shift;

    $__msgnum = $resp->{Items}->{messages};
}

#
# _imap_fetch callback function for Net::IMAP::fetch
# This CB function is used for three jobs:
#
# 1. fetch a complete set messages                -> stored in $__msgarr
# 2. fetch the message header of set of messages  -> stored in $__msgarr
# 3. fetch the size of a set of messages          -> stored in $__msgsizearr
#
sub _imap_fetch {
    my $self = shift;
    my $resp = shift;

    my $msgarr;
    if( defined $resp->{Items}->{rfc822} ) {
	@$msgarr = split(/\n/, $resp->{Items}->{rfc822});
	push @$__msgarr, $msgarr;
    } elsif( defined $resp->{Items}->{"rfc822.header"} ) {
	@$msgarr = split(/\n/, $resp->{Items}->{"rfc822.header"});
	push @$__msgarr, $msgarr;
    } elsif( defined $resp->{Items}->{"rfc822.size"} ) {
	push @$__msgsizearr, $resp->{Items}->{"rfc822.size"};
    }
}

sub fetch {
    my $this = shift;
    my @args = @_;
    my $statmode = shift || 0;

    if( ! $statmode ) {
	if( ! &::addThread("IMAPFetch, for $this->{data}->{user}\@$this->{data}->{host}") ) {
	    return new Error(undef, "ERROR(IMAPFetch): max Threads reached!!!");
	}
    }

    my $pid;
    if (!defined($pid = fork)) {
	return new Error(undef, "ERROR(IMAPFetch): cannot fork: $!");
    } elsif ($pid) {
	return new Error(undef, "ERROR(IMAPFetch): I am the parent with my childs pid=$pid");
    }
    # else I'm the child -- go spawn
    
    my $ret = $this->fetch_thread(@args);

    return new Error($ret, "OK");
}

sub fetch_thread {
    my $this = shift;

    my $statmode = shift || 0;

    my $keeponserver = $::keep;
    if( !defined $keeponserver ) {
        $keeponserver = 1;
    }

    my $user     = $this->{"data"}->{"user"};
    my $password = $this->{"data"}->{"passwd"};
    my $host     = $this->{"data"}->{"host"};
    my $msga;

    my $lret = &::dolock($user, $host);
    if( $lret->code() ne "OK" ) {
	return $lret;
    }

    my $ih = new Net::IMAP($host);
    if( ! defined $ih ) {
	&::unlock($user, $host);
	return new Error(undef, "ERROR(IMAPFetch): connection to <$host> failed");
    }

    # We need to catch alerts, so we have to provide our own
    # callback function to "NO" responses of cyrus-imapd
    my $alert = undef;

    my $nocb = sub {
	my $self = shift;
	my $resp = shift;

	if( defined $resp->{RespCode}->[0] ) {
	    $alert = $resp->{Text};
	}
	
    };

    $ih->set_untagged_callback('no', $nocb);

    my $ret = $ih->login($user, $password);
    if( $ret->{Status} ne "ok" ) {
	&::unlock($user, $host);
	return new Error(undef, "ERROR(IMAPFetch): unable to login as user <$user> at <$host>");
    }

    $ih->set_untagged_callback('status', \&_imap_status);

    # the following will set global $__msgnum within _imap_status callback function
    $ret = $ih->status("INBOX", ( "messages" ));
    if( ! defined $ret ) {
	&::unlock($user, $host);
	$ih->logout;
	return new Error(undef, "ERROR(IMAPFetch): unable to issue inbox status of <$user\@$host>");
    }

    if( $__msgnum == 0 ) {
	&::unlock($user, $host);
	if( $statmode ) {
	    $msga = "No messages\n";
	}
	$ih->logout;
	return new Error($msga, "OK: no mail to fetch");
    }

    $ret = $ih->select("INBOX");
    if( $ret->{Status} ne "ok" ) {
	&::unlock($user, $host);
	$ih->logout;
	return new Error(undef, "ERROR(IMAPFetch): unable select inbox of <$user\@$host>");
    }

    $ih->set_untagged_callback('fetch', \&_imap_fetch);

    # first fetch the size of each message
    $__msgsizearr = ();
    $ret = $ih->fetch("1:$__msgnum", 'rfc822.size');
    if( $ret->{Status} ne "ok" ) {
	&::unlock($user, $host);
	$ih->logout;
	return new Error(undef, "ERROR(IMAPFetch): unable to fetch msg size of <$user\@$host>: $ret->{Status}");
    }

    # build the messageset of fetched messages with size < maxsize
    # and the messageset of too big messages

    # from Net::IMAP:
    #
    #  The `$msgset' parameter identifies the set of messages
    #  from which to retrieve the items of interest.  The
    #  notation accepted is similar to that found in `.newsrc'
    #  files, except that `:' is used to specify ranges, instead
    #  of `-'.  Thus, to specify messages 1, 2, 3, 5, 7, 8, 9,
    #  the following string could be used: `'1:3,5,7:9''.  The
    #  character `*' can be used to indicate the highest message
    #  number in the mailbox.  Thus, to specify the last 4
    #  messages in an 8-message mailbox, you can use `'5-*''.

    my $msgsetok = "";
    my $msgsettobig = "";
    for( my $i=0; $i<$__msgnum; $i++ ) {
	if( $__msgsizearr->[$i] <= $::maxsize ) {
	    if( $statmode ) {
		$msga .= "Message ".($i+1).": Size=$__msgsizearr->[$i]\n";
	    }
	    $msgsetok .= ($i+1).",";
	} else {
	    if( $statmode ) {
		$msga .= "Message ".($i+1).
		    ": Size=$__msgsizearr->[$i] > maxsize=$::maxsize (will not be fetched)\n";
	    }
	    $msgsettobig .= ($i+1).",";
	}
    }
    $msgsetok =~ s/(.*),$/$1/; # remove trailing ","
    $msgsettobig =~ s/(.*),$/$1/; # remove trailing ","

    #print "TOBIG=$msgsettobig, OKAY=$msgsetok\n";

    # fetch complete mails that are not over sizelimit
    if( $msgsetok ne "" && ! $statmode ) {
	$__msgarr = ();
	$ret = $ih->fetch("$msgsetok", 'rfc822');
	if( $ret->{Status} ne "ok" ) {
	    &::unlock($user, $host);
	    $ih->logout;
	    return new Error(undef, "ERROR(IMAPFetch): unable to fetch mail bodies of <$user\@$host>");
	}
	
	foreach my $m ( @$__msgarr ) {
	    push @$msga, { "oversize" => 0,
			   "msg" => $m };
	}
	
	if( ! $keeponserver ) {
	    # mark fetched mails with \Deleted flag
	    $ret = $ih->store("$msgsetok", '+flags', ('\\Deleted') );
	    if( $ret->{Status} ne "ok" ) {
		&::unlock($user, $host);
		$ih->logout;
		return new Error(undef,
				 "ERROR(IMAPFetch): unable to set \\Deleted flag on messages from <$user\@$host>");
	    }
	    
	    # remove mails marked as \Deleted
	    $ret = $ih->expunge();
	    if( $ret->{Status} ne "ok" ) {
		&::unlock($user, $host);
		$ih->logout;
		return new Error(undef, "ERROR(IMAPFetch): unable to expunge \\Deleted mails for <$user\@$host>");
	    }
	}
    }

    # fetch only headers of messages over sizelimit
    if( $msgsettobig ne "" && ! $statmode ) {
	$__msgarr = ();
	$ret = $ih->fetch("$msgsettobig", 'rfc822.header');
	if( $ret->{Status} ne "ok" ) {
	    &::unlock($user, $host);
	    $ih->logout;
	    return new Error(undef, "ERROR(IMAPFetch): unable to fetch mail headers of <$user\@$host>");
	}
	
	foreach my $m ( @$__msgarr ) {
	    push @$msga, { "oversize" => 1,
			   "msg" => $m };
	}
    }
    
    &::logit("IMAPFetch: ALERT for $user\@$host: $alert\n") if defined $alert;

    &::unlock($user, $host);
    $ih->logout;
    return new Error($msga, "OK: fetched $__msgnum mail(s)");
}


1;
