#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: POPFetch.pm,v 1.3 2003/09/10 12:27:33 varkoly Exp $
#
package POPFetch;
use strict;
use Error;
use Net::POP3;


sub new {
    my $this = shift;

    my $data    = shift;

    my $class = ref($this) || $this;
    my $self = {};
    $self->{"data"}    = $data;
    
    bless $self, $class;
    return $self;
}

sub fetch {
    my $this = shift;
    my @args = @_;

    my $statmode = shift || 0;

    if( ! $statmode ) {
	if( ! &::addThread("POPFetch, for $this->{data}->{user}\@$this->{data}->{host}") ) {
	    return new Error(undef, "ERROR(POPFetch): max Threads reached!!!");
	}
    }

    my $pid;
    if (!defined($pid = fork)) {
	return new Error(undef, "ERROR(POPFetch): cannot fork: $!");
    } elsif ($pid) {
	return new Error(undef, "ERROR(POPFetch): I am the parent with my childs pid=$pid");
    }
    # else I'm the child -- go spawn

    my $ret = $this->fetch_thread(@args);

    return new Error($ret, "OK");
}

sub fetch_thread {
    my $this = shift;

    my $statmode = shift || 0;

    my $keeponserver = $::keep;
    if( !defined $keeponserver ) {
        $keeponserver = 1;
    }

    my $user     = $this->{"data"}->{"user"};
    my $password = $this->{"data"}->{"passwd"};
    my $host     = $this->{"data"}->{"host"};
    my $msga;

    my $lret = &::dolock($user, $host);
    if( $lret->code() ne "OK" ) {
	return $lret;
    }

    my $ph = new Net::POP3($host);
    if( ! defined $ph ) {
	&::unlock($user, $host);
	return new Error(undef, "ERROR(POPFetch): connection to <$host> failed");
    }
    
    my $ret = $ph->user($user);
    if( ! defined $ret ) {
	&::unlock($user, $host);
	return new Error(undef, "ERROR(POPFetch): sending username <$user> to <$host>");
    }
    
    my $nmsg = $ph->pass($password);
    if( ! defined $nmsg ) {
	&::unlock($user, $host);
	return new Error(undef, "ERROR(POPFetch): sending password of <$user> to <$host>");
    }

    for(my $i=1; $i<=$nmsg; $i++) {
	my $msize = $ph->list($i);
	my $msg;
	#if( ! defined $msize ) {
	#    return new Error(undef, "ERROR(POPFetch): fetching size of msg <$i> of <$user\@$host>");
	#}
	if( $msize > $::maxsize ) {
	    if( ! $statmode ) {
		$msg = $ph->top($i);
		#if( ! defined $msize ) {
		#return new Error(undef, "ERROR(POPFetch): fetching header of msg <$i> of <$user\@$host>");
		#}
		push @$msga, { "oversize" => 1,
			       "msg" => $msg };
	    } else {
		$msga .= "Message $i: Size=$msize > maxsize=$::maxsize (will not be fetched)\n";
	    }
	} else {
	    if( ! $statmode ) {
		$msg = $ph->get($i);
		push @$msga, { "oversize" => 0,
			       "msg" => $msg };
		$ph->delete($i) if ! $keeponserver;
	    } else {
		$msga .= "Message $i: Size=$msize\n";
	    }
	}
    }

    if( $statmode && $nmsg eq "0E0" ) {
	$msga = "No messages\n";
    }

    $ph->quit();

    &::unlock($user, $host);
    return new Error($msga, "OK: fetched $nmsg mail(s)");
}


1;
