#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: SMTPSender.pm,v 1.4 2003/09/10 12:27:33 varkoly Exp $
#
package SMTPSender;
use strict;
use Error;
use Net::SMTP;
use Mail::Header;
use Mail::Internet;

sub new {
    my $this = shift;

    my $data     = shift;
    my $mailhost = shift || "localhost";

    my $class = ref($this) || $this;
    my $self = {};
    $self->{"data"}     = $data;
    $self->{"mailhost"} = $mailhost;
    
    bless $self, $class;
    return $self;
}

sub send {
    my $this = shift;

    my $data = shift || return new Error(undef, "ERROR(SMTPSender): no data to send");

    my $pid;
    if (!defined($pid = fork)) {
	return new Error(undef, "ERROR(SMTPSender): cannot fork: $!");
    } elsif ($pid) {
	return new Error(undef, "ERROR(SMTPSender): I am the parent with my childs pid=$pid");
    }
    # else I'm the child -- go spawn
    
    my $ret = $this->send_thread($data);

    return new Error($ret, "OK");
}

sub smtp_connect {
    my $this = shift;

    my $sh;
    my $max_count = 10;
    my $interval = 6;
    my $count = 1;
    my $firstattempt = 1;
    while( ( ! defined $sh ) || $count > $max_count ) {
	if( ! $firstattempt ) {
	    sleep($interval);
	}
	#$sh = new Net::SMTP($this->{mailhost}, Debug => 1);
	$sh = new Net::SMTP($this->{mailhost});
	$firstattempt = 0;
	$count++;
    }
    return $sh;
}

sub send_thread {
    my $this = shift;

    my $data = shift;

    my $localaddr   = $this->{"data"}->{"localaddr"};
    my $user        = $this->{"data"}->{"user"};
    my $host        = $this->{"data"}->{"host"};
    my $proto       = $this->{"data"}->{"proto"};
    my $multidrop   = $this->{"data"}->{"multidrop"};
    my $qmailprefix = $this->{"data"}->{"qmailprefix"} eq "" ? undef : $this->{"data"}->{"qmailprefix"};
    my $tbcnt = 0;
    
    my $oversize_txt = 'Subject: A mail was to big from $user\@$host to $localaddr
To: SuSE Linux School Server Administrator <$::madmin>

A message exceeds the maximum message size of $::maxsize bytes.

Fetch host:             $host
User account:           $user
Local delivery address: $localaddr
Protocol:               $proto

The header of this message is included below:

';

    my $norcpt_txt = 'Subject: No recipient found
To: SuSE Linux School Server Administrator <$::madmin>

I was not able to determine the recipient of the following message:

Fetch host:             $host
User account:           $user
Local delivery address: $localaddr
Protocol:               $proto

';


    my $rcpt_reject = 'Subject: A message has been rejected
To: SuSE Linux School Server Administrator <$::madmin>

I was not able to deliver the following message:

Fetch host:             $host
User account:           $user
Local delivery address: $localaddr
Protocol:               $proto

Check /var/log/mail for the reason.

The message is included below:

';

    my $sh = $this->smtp_connect();
    if( ! defined $sh ) {
	return new Error(undef, "ERROR(SMTPSender): connecting to <$this->{mailhost}> for <$localaddr> failed");
    }
    

    if( ! $multidrop ) {
	# no multidrop, no problem...
	foreach my $d ( @$data ) {
	    my $m = $d->{msg};
	    
	    # if message was to big, send notification to mailadmin
	    if( $d->{oversize} ) {
		$sh->mail($::madmin);
		$sh->to($::madmin);
		$sh->data();
		$sh->datasend(eval "sprintf \"%s\", \"$oversize_txt\"");
		$sh->datasend($m);
		$sh->dataend();
	    } else {
		my $head = new Mail::Header($m, Modify => 0);
		my $body = (new Mail::Internet($m))->body();
		my $hh   = $head->header_hashref();
		my $envrcpt = $hh->{"Return-Path"}->[0];
		$head->delete("Delivered-To");
		if( $::fetchheader ) {
		    $head->add(undef, "X-Fetched: by SuSE Linux School Server from $user\@$host for $localaddr via $proto");
		}

		my $rcpt;
		my $send;
		$rcpt = $sh->mail($envrcpt);
		$send = $sh->to($localaddr);

		if( ! $rcpt || ! $send ) {
		    $sh->quit();
		    $sh = $this->smtp_connect();
		    if( ! defined $sh ) {
			return new Error(undef, "ERROR(SMTPSender): connecting to <$this->{mailhost}> for <$localaddr> failed");
		    }
		    if( ! $::dropundeliverable ) {
			$sh->mail($::madmin);
			$sh->to($::madmin);
			$sh->data();
			$sh->datasend(eval "sprintf \"%s\", \"$rcpt_reject\"");
			$sh->datasend($head->header());
			$sh->datasend("\n");
			$sh->datasend($body);
			$sh->dataend();
		    } else {
			&main::logit("WARNING: dropped mail from <$host> to <$localaddr>, SENDER=$envrcpt\n");
		    }
		} else {
		    $sh->data();
		    $sh->datasend($head->header());
		    $sh->datasend("\n");
		    $sh->datasend($body);
		    $sh->dataend();
		}
	    }
	}
    } elsif( $multidrop ) {
	# welcome to hell :-(
	my $msgids = undef;
	foreach my $m ( @$data ) {
	    my $rcpt = undef;
	    my $found = 0;
	    my $dupfound = 0;
	    #print "----------------------------------------------------------\n";
	    if( ! $m->{oversize} ) {
		my $head = new Mail::Header($m->{msg}, Modify => 0);
		my $body = (new Mail::Internet($m->{msg}))->body();
		
		if( $::fetchheader ) {
		    $head->add(undef, "X-Fetched: by SuSE Linux School Server from $user\@$host for $localaddr via $proto");
		}

		my $hh = $head->header_hashref();
		if( defined $qmailprefix ) {
		    # congratiulations! your provider uses qmail!
		    my $delivered = $hh->{"Delivered-To"}->[0];
		    if( ! defined $delivered ) {
			&::logit("WARNING: Job=<$user\@$host for $localaddr via $proto>:\n");
			&::logit("WARNING: qmailprefix is set, but no Delivered-To found in header!!!\n");
		    }
		    $delivered =~ /^$qmailprefix(.*)/ if defined $delivered;
		    if( defined $1 ) {
			push @$rcpt, $1;
			$found = 1;
		    }
		    #if( defined $1 ) {
			#print("Recipient=$1\n");
		    #}
		}
		$head->delete("Delivered-To");

		# postfix appends X-Original-To since version 2.x
		if( ! $found ) {
		    my $xorigto = $hh->{"X-Original-To"}->[0];
		    #&::logit("X-Original-To detected, RCPT=<$xorigto>\n");
		    if( defined $xorigto ) {
			push @$rcpt, $xorigto;
			$found = 1;
		    }
		}

		if( ! $found ) {
		    my $xenvto = $hh->{"X-Envelope-To"}->[0];
		    if( defined $xenvto ) {
			push @$rcpt, $xenvto;
			$found = 1;
		    }
		}

		if( ! $found ) {
		    my $xenvto = $hh->{"Envelope-To"}->[0];
		    if( defined $xenvto ) {
			# there could be more then one recipient... well...
			if( $xenvto =~ /,/ ) {
			    foreach my $r ( split(/,/, $xenvto) ) {
				$r =~ s/\s//g; # remove ws
				push @$rcpt, $r;
			    }
			} else {
			    push @$rcpt, $xenvto;
			}
			$found = 1;
		    }
		}

		if( ! $found ) {
		    # exim seems to insert an X-RCPT-TO which contains the final recipient
		    # nice... :-)
		    my $xrcpt = $hh->{"X-RCPT-To"}->[0];
		    #&::logit("X-RCPT-TO detected, RCPT=<$xrcpt>\n");
		    if( defined $xrcpt ) {
			push @$rcpt, $xrcpt;
			$found = 1;
		    }
		}

		if( ! $found ) {
		    # Now to something completely different....
		    # Try to find out, to whom this mail may be addressed...
		    
		    #
		    # Mystery Part 1
		    #
		    # search for local recipients in the received lines
		    #print "MYST, PART 1\n";

		    my $msgid = $hh->{"Message-Id"}->[0];
		    chomp($msgid);
		    #print "MSGID=<$msgid>\n";
		    foreach my $rl ( @{$hh->{"Received"}} ) {
			foreach my $mydomain ( @{$::maildomain} ) {
			    if( $rl =~ /for.*$mydomain/msi ) {
				#print "RL=$rl\n";
				$rl =~ /.*for\s+[<]*(.*\@$mydomain).*/msi;
				if( defined $1 ) {
				    if( ! defined $msgids->{"$msgid"}->{"$1"} ) {
					#print "IN RECIEVED: pushing $1\n";
					push @$rcpt, $1;
					$msgids->{"$msgid"}->{"$1"} = 1;
				    } else {
					$dupfound = 1;
				    }
				}
			    }
			}
		    }
		
		    #print "MYST, PART 2\n";
		    #
		    # Mystery Part 2
		    #
		    # Check "To:" and "Cc:" Header
		    foreach my $hline ( ( "To", "Cc" ) ) {
			foreach my $tl ( @{$hh->{$hline}} ) {
			    foreach my $mydomain ( @{$::maildomain} ) {
				if( $tl =~ /\@$mydomain/msi ) {
				    foreach my $fr ( split(/,\s*/ ,$tl ) ) {
					chomp($fr);
					next if $fr !~ /$mydomain/;
					$fr =~ s/<//;
					# remove all stuff following the email address
					$fr =~ s/(^.*\@$mydomain).*/$1/i;
					# remove all stuff before the last whitespace in front
					# of the email address
					if( $fr =~ /\s+/ ) {
					    $fr =~ s/.*\s+//i;
					}
					if( defined $fr ) {
					    if( ! defined $msgids->{"$msgid"}->{"$fr"} ) {
						#print "IN $hline pushing $fr\n";
						push @$rcpt, $fr;
						$msgids->{"$msgid"}->{"$fr"} = 1;
					    } else {
						$dupfound = 1;
					    }
					}
				    }
				}
			    }
			}
		    }
		}

		#
                # Maybe some additional magic inbetween....
                #

		if( ( ! defined $rcpt || ! scalar(@$rcpt) ) && ! $dupfound ) {
		    #
		    # Mystery Last Resort
		    #
		    # no receipient found? Not that I am shocked...
		    &main::logit("NO RECIPIENT FOUND\n");
		    $sh->mail($::madmin);
		    $sh->to($::madmin);
		    $sh->data();
		    $sh->datasend(eval "sprintf \"%s\", \"$norcpt_txt\"");
		    $sh->datasend($head->header());
		    $sh->datasend("\n");
		    $sh->datasend($body);
		    $sh->dataend();
		} else {
		    foreach my $r ( @$rcpt ) {
			# blow out...
			# and don't send twice
			if( $r ne "" ) {
			    #&::logit("SENDING TO <$r>\n");
			    my $envfrom = $hh->{"Return-Path"}->[0];

			    my $rcpt;
			    my $send;
			    $rcpt = $sh->mail($envfrom);
			    $send = $sh->to($r);

			    if( ! $rcpt || ! $send ) {
				$sh->quit();
				$sh = $this->smtp_connect();
				if( ! defined $sh ) {
				    return new Error(undef, "ERROR(SMTPSender): connecting to <$this->{mailhost}> for <$localaddr> failed");
				}
				# override localaddr
				$localaddr = $r;
				if( ! $::dropundeliverable ) {
				    $sh->mail($::madmin);
				    $sh->to($::madmin);
				    $sh->data();
				    $sh->datasend(eval "sprintf \"%s\", \"$rcpt_reject\"");
				    $sh->datasend($head->header());
				    $sh->datasend("\n");
				    $sh->datasend($body);
				    $sh->dataend();
				} else {
				    &main::logit("WARNING: dropped mail from <$host> to <$localaddr>, SENDER=$envfrom\n");
				}
			    } else {
				$sh->data();
				$sh->datasend($head->header());
				$sh->datasend("\n");
				$sh->datasend($body);
				$sh->dataend();
			    }
                        }
		    }
		}
	    } else {
		$sh->mail($::madmin);
		$sh->to($::madmin);
		$sh->data();
		$sh->datasend(eval "sprintf \"%s\", \"$oversize_txt\"");
		$sh->datasend($m->{msg});
		$sh->dataend();
		$tbcnt++;
	    }
	    #print "----------------------------------------------------------\n";
	}
    }

    $sh->quit();
    if( $tbcnt ) {
	return new Error(undef, "ERROR(SMTPSender): $tbcnt to big mails left on server!");
    } else {
	return new Error(undef, "OK");
    }
}

1;
