#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: browse_groups.pm,v 1.19 2004/02/26 18:10:57 varkoly Exp $
#

package browse_groups;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Net::IMAP;
use Display;
use Utils;
use vars qw(@ISA);
use subs qw(exit);
use Filterlist;

# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();
    
    my $doit = $cgi->param('doit');
    my $nob  = $cgi->param('nob') || '';
    my $update = $cgi->param('update');
    my $do_filter = $cgi->param($message->{'apply_filter'});

    if( defined $do_filter && ( ! defined $update || $update eq ""  ) ) {
        $this->change_members();
    } elsif( defined $doit && $doit eq "g_select" && $nob eq "" ) {
        $this->group_selected();
    } elsif(defined $update) {
        $this->group_update();
    } else {
	$this->startup();
    }
}

sub startup {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();

    my $doit      = $q->param('doit');
    my $yesb      = $q->param('yesb') || '';
    my $nob       = $q->param('nob') || '';
    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    
    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    
    my $button_width = 150;

    my @cn = ();
    my $text = "";

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    
    if(!defined $ldap) {
    $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (    # bind to a directory with dn and password
                dn       => "$bind_dn",
                password => "$ADMINpasswd"
                );
    
    if($mesg->code != 0) {
    $text = "bind $message->{failed}\n\n";
    $text .= &LDAPerror($mesg);
    $this->display_error($text,
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    exit;
    }
    
    my $filter = "(&(cn=*)(objectclass=posixGroup))";
    if( $ADMINuid ne 'cyrus' ) {
	$filter = "(&(cn=*)(objectclass=SuSESchoolGroup)(|(!(writerdn=*))(writerdn=$bind_dn)))"
    }
    my $result = $ldap->search ( # perform a search
                 base   => "$ldapbase",
                 scope  => "one",
                 attrs => ['cn'],
                 filter => $filter
                 );
    
    if($result->code == 0) {
	foreach my $entry ( $result->all_entries) {
	    push @cn, $entry->get_value("cn");
	}
	@cn = sort @cn;
    } else {
	$text = "search $message->{failed}\n\n";
	$text .= &LDAPerror($result);
	$ldap->unbind;   # take down session
	$this->display_error($text,
	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $size = 20;
    if($#cn < 17) {
	$size = $#cn + 3; 
    }

    $ldap->unbind;   # take down session

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->Tr({},
            $q->td({},
               $q->start_form(-action=>$confParam->{cgi_path}."/browse_groups.pl").
               $q->table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"},
                     $q->Tr([ $this->help_th(2, $message->{'edit_group'}, "SELECT_GROUP"),
                        $q->td({-class=>'AdminSubHeader', -colspan => "2"}, $message->{'select_group'}) ]),
                     $q->Tr(
                        $q->td({-colspan=>2, -align=>"center"},
                           $q->scrolling_list(-name=>"group",
                                      -value=>[@cn],
                                      -size=>$size,
                                      -default=>$cn[0],
                                      -width=>200
                                      )
                           )
                        ),
                     $q->Tr(
                        $q->td({-align=>"center"},
                           $q->submit(-class=>"button",-name=>"delete",
                                  -value=>"$message->{b_delete}",
                                  -width=>$button_width).
                           $q->hidden(-name=>"sessionID", -value=>"$sessionID").
                           $q->hidden(-name=>"doit", -value=>"g_select").
                           $q->hidden(-name=>"tab", -value=>"$tab").
                           $q->hidden(-name=>"stab", -value=>"$stab")
                           ),
                        $q->td({-align=>"center"},
                           $q->submit(-class=>"button",-name=>"change",
                                  -value=>"$message->{edit}",
                                  -width=>$button_width)
                           )
                        )
                     )
               ).
               $q->end_form()
            );
    
    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub confirm {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $Folder    = $q->param('group');

    my $button_width =100;

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->Tr({},
            $q->td(
	       $q->start_form(-action=>$confParam->{cgi_path}."/browse_groups.pl").
               $q->table({-cellspacing=>0, -cellpadding=>5, -width=>"100%"},
                     $q->Tr(
                        [
                         $q->td({-class=>'AdminHeader', -colspan=>2 },
                            "$message->{confirm_delete_head}&nbsp;$Folder"),
                         $q->td({-class=>'AdminSubHeaderRed', -colspan=>2},
                            [$message->{confirm_delete_warn}]),
                         ]
                        ),
                     $q->Tr(
                        $q->td({-align=>"center"},
                           $q->submit(-class=>"button",-name=>"nob",
                                  -value=>"$message->{no}",
                                  -width=>$button_width).
                           $q->hidden(-name=>'sessionID', -value=>$sessionID).
                           $q->hidden(-name=>'group', -value=>$Folder).
                           $q->hidden(-name=>'delete', -value=>$message->{b_delete}).
                           $q->hidden(-name=>'doit', -value=>"g_select").
                           $q->hidden(-name=>'tab', -value=>"$tab").
                           $q->hidden(-name=>'stab', -value=>"$stab")
                           ),
                        $q->td({-align=>"center"},
                           $q->submit(-class=>"button",-name=>"yesb",
                                  -value=>"$message->{yes}",
                                  -width=>$button_width) ) 
                        )
                     ).
	       $q->end_form()
               )
            );

    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub group_selected {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();

    my $group     = $q->param('group');
    my $change    = $q->param('change');
    my $delete    = $q->param('delete');
    my $yesb      = $q->param('yesb') || '';
    my $nob       = $q->param('nob') || '';
    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    
    if(!defined $group || $group eq "") {
	$this->display_error($message->{must_select_group},
			     "$confParam->{cgi_path}/browse_groups.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
			     2);
	exit;
    }

    if(defined $change && $change eq $message->{edit}) {
	$this->change_members();
    } elsif(defined $delete && $delete eq $message->{b_delete} && $yesb eq "" && $nob eq "") {
	$this->confirm();
    } elsif(defined $delete && $delete eq $message->{b_delete} && $yesb ne "") {
	$this->delete_group($group);
    }
}


sub delete_group() {
    my $this = shift;
    my $group = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $imaphost      = $confParam->{IMAPserver};
    my $imapport      = $confParam->{IMAPport};
    my $groupware   = read_rcconfig("SCHOOL_GROUPWARE") || 'all';

    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    my $text = "";

    if(!defined $ldap) {
    $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (    # bind to a directory with dn and password
                dn       => "$bind_dn",
                password => "$ADMINpasswd"
                );
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
		  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    # read the GID to update the comFireGroupID
    $mesg = $ldap->search( "base"  => 'cn='.$group.",".$ldapbase,
			   "scope" => 'base',
			   "filter"=> 'objectClass=posixGroup',
			   "attrs" => 'gidNumber');
    if($mesg->code != 0) {
	$text = "delete $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $gidNumber = $mesg->entry(0)->get_value("gidNumber");
    my $imap = new Net::IMAP($imaphost, Debug => 0) or die ("can't connect to $imaphost: $!\n");
    my $resp = $imap->login($ADMINuid, $ADMINpasswd);
       $resp = $imap->delete($group);

    if( $groupware eq 'phpgw' || $groupware eq 'all') {
	    my $mysqlc = "mysql -u $ADMINuid -p\'$ADMINpasswd\' -h mailserver phpgroupware";
	    my $mysqla = " DELETE FROM phpgw_acl WHERE acl_location=\"$gidNumber\" OR acl_account=$gidNumber";
	       system("echo \'$mysqla\' | $mysqlc"); 
    }

    # get the entry with comFireGroupID = GID
    my $gidNumber = $mesg->entry(0)->get_value("gidNumber");
    $mesg = $ldap->search( "base" => $ldapbase,
			   "scope" => "one",
			   "filter" => "(&(objectClass=posixAccount)(comFireGroupID=$gidNumber))",
			   "attrs" => ["uid"] );
    if($mesg->code != 0) {
	$text = "delete $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    # update the comFireGroupId of those entries
    if( $groupware eq 'comfire' || $groupware eq 'all') {
	    foreach my $entry ($mesg->entries()){
		my $dn = $entry->dn();
		my $result = $ldap->modify( $dn, delete => {'comFireGroupID' => $gidNumber});
#		if( $result->code != 0 ){
#		    $text .= "failed to update $dn\n";
#		    $text .= &LDAPerror($result)."\n";
#		}
	    }
    }
    my $is_SuSESchoolGroup = 0;
    $mesg =  $ldap->search(  base => $ldapbase, 
			    "scope" => 'base',
                            "filter"=> 'objectClass=SuSESchoolGroup',
			     "attrs" => 'gidNumber'			     
			  );
    if( $mesg->count > 0 ) {
	$is_SuSESchoolGroup = 1;
    }
    $mesg = $ldap->delete("cn=".$group.",".$ldapbase);
    if($mesg->code != 0) {
	$text = "delete $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    $mesg = $ldap->delete("fn=".$group.",".$ldapbase);
    if($mesg->code == 0) {
	    my $imap = new Net::IMAP($imaphost, Debug => 0) or die ("can't connect to $imaphost: $!\n");
	    my $resp = $imap->login($ADMINuid, $ADMINpasswd);
	    my $mailbox = $group;
	    if($is_SuSESchoolGroup) {
		$mailbox = "workgroups/".$group; 
	    }
	    $resp = $imap->delete($mailbox);
	    $resp = $imap->logout();
    }

    $ldap->unbind;   # take down session

    if( $groupware eq 'comfire' || $groupware eq 'all') {
	    my $command = "/opt/comfire/application/scripts/deleteGroup $group";
	    
	    system($command);
	    my $exit_value  = $? >> 8;

#	    if($exit_value != 0) {
#		$this->display_error("Unable to delete group $group: $!",
#				     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
#		exit;
#	    }
    }
    if( $group ne '' && $group ne '/' ) {
	my $ret = $session->suad_unlink("/home/groups/$group");
	   $ret = $session->suad_unlink("/home/$group");
	if($is_SuSESchoolGroup) {
	   $ret = $session->suad_unlink("/home/classes/$group");
	}
    }
    $text .= "$group: $message->{group_delete_txt}\n";
    $q->param(-name=>'doit', -value=>"");
    $q->param(-name=>'group', -value=>"");

    $this->display_msg($text,
        "$confParam->{cgi_path}/browse_groups.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
        2);

}

sub change_members() {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP","BROWSE_USER"))->data();

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $group     = $q->param('group');

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my @uid = ();
    my $desc = "";
    my $text = "";
    my $filter     = $q->param('filter') || "";
    $filter = LDAP_filter_escape($filter);

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (    # bind to a directory with dn and password
                dn       => "$bind_dn",
                password => "$ADMINpasswd"
                );
    
    if($mesg->code != 0) {
        $text = "bind $message->{failed}\n\n";
        $text .= &LDAPerror($mesg);
        $this->display_error($text,
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    
    my $result = $ldap->search ( # perform a search
                 base   => "cn=$group,$ldapbase",
                 scope  => "base",
                 filter => "objectclass=*",
                 attrs => ['description'],
                 );
    
    if($result->code == 0) {
        my ($entry) = $result->entries;
        if(defined $entry->get_value("description")) {
            $desc = join " ", $entry->get_value("description");
        }
    
    } else {
        $text = "search $message->{failed}\n\n";
        $text .= &LDAPerror($result);
        $ldap->unbind;   # take down session
        $this->display_error($text,
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $filterlist = Filterlist->new($q, $session, $ldap, $this);
    my $right_label = $q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"}).
                           $q->Tr(
                              $q->td({-align=>"center"},
                                 "$message->{desc}")
                              ).
                           $q->Tr(
                              $q->td({-align=>"center"},
                                 $q->textfield(-name=>'desc',
                                           -default=>$desc,
                                           -size=>20))
                              ).
                      $q->end_table();
    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->start_Tr({}).
            $q->start_td({}).
               $q->start_table({-cellspacing=>0, -cellpadding=>5, -width=>"100%"}).
                     $q->start_form(-action=>$confParam->{cgi_path}."/browse_groups.pl").
                     $q->Tr(
                        $this->help_th(3, "$message->{select_members} $group", "EDIT_GROUP")
                        ).
                     $filterlist->make_filter_view({"textfield"    => "filter",
                                              "textlabel"    => $message->{'filter'},
                                              "classname"   => $message->{'classe'},
                                              "listname"     => "uids",
                                              "listlabel_left" => $message->{'user'},
                                              "listlabel_right" => $right_label,
                                              "buttonlabel"  => $message->{'apply_filter'},
                                              "nofilter-msg" => $message->{'set_filter_first'},
                                              "listfilter"  => "(&".
                                                                   "(objectclass=posixAccount)(objectclass=SuSEeMailObject)".
                                                                   "(!(uid=mailadmin))(!(uid=cyrus))(!(gidnumber=103))".
                                                                   "(|(sn=$filter)(givenname=$filter)(uid=$filter))".
                                                                ")",
					      "listattrs"   => ['uid','cn'],
                                              "selectedfilter" => "(&(objectclass=posixGroup)(cn=$group))",
					      "selectedregex" => ["memberuid", "(.*)"],
                                              "multiple"     => "MULTIPLE",
                                              "error-url"    => "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID",
                                              "labelexp"     => ["\$\$ ","uid","(\$\$) ","cn"],
                                              "width"        => "30",
                                              "height"       => "10"
                                             }
                     ).
                     $q->start_Tr().
                        $q->start_td({-colspan=>3, -align=>"center"}).
			   $q->submit(-class=>"button",-name=>"update",
				      -value=>"$message->{update}");
    $html .=               $q->hidden(-name=>'sessionID', -value=>$sessionID).
                           $q->hidden(-name=>'group', -value=>$group).
                           $q->hidden(-name=>'doit', -value=>"g_update",
                                  -override=>1).
                           $q->hidden(-name=>'tab', -value=>"$tab").
                           $q->hidden(-name=>'stab', -value=>"$stab").
                           $q->end_form().
                        $q->end_td().
                     $q->start_Tr().   
                 $q->end_table().
               $q->end_td().
            $q->end_Tr();

    $html .= $q->end_table();

    $ldap->unbind;   # take down session
    $this->SUPER::display($html);
}

sub group_update {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"BROWSE_GROUPS", "CREATE_GROUP"))->data();

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $filter    = $q->param('filter');

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
   
    my $imaphost      = $confParam->{IMAPserver};
    my $imapport      = $confParam->{IMAPport};
    
    my $groupware   = read_rcconfig("SCHOOL_GROUPWARE") || 'all';

    my $group = $q->param('group');
    my @selected_uids = $q->param('uids');

    my @uids;
    my @muids;
    my @realy_add = ();

    my $SORTED_HOME = read_rcconfig("SCHOOL_SORTED_HOME");

#    print STDERR "GAGA\n";
#    foreach my $uid ( @selected_uids ) {
#	print STDERR "<$uid>\n";
#    }

    #print STDERR "selected: <". join " ", @selected_uids.">\n";

    my $desc  = $q->param('desc');
    my $text = "";

    if(!defined $group || $group eq "") {
    $this->display_error($message->{must_select_group},
              "$confParam->{cgi_path}/browse_groups.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
              2);
    exit;
    }

    my $ldap= Net::LDAP->new($ldaphost, port=>$ldapport, version => 3);
    
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (    # bind to a directory with dn and password
                dn       => "$bind_dn",
                password => "$ADMINpasswd"
                );
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my %replace = ( "description" => ($desc eq "" ? [] : $desc) );
    my %add; 
    my %delete;
    
    my @delete_uids;
    my @add_uids;
    my $gidNumber;
    if( defined $q->param('filtered') ){
        # changes to the attribute memberuid are only need if the hiddenfield 
        # 'filtered' is present 
        
                               
        # Search group members
        $mesg = $ldap->search( "base" => "cn=".$group.",".$ldapbase,
                       "scope" => "base",
                       "filter" => "objectclass=*",
                       "attrs" => ["memberUid", "gidNumber"]
                     );  
        if($mesg->code != 0){
            $text = "search $message->{failed}\n\n";
            $text .= &LDAPerror($mesg);
            $text .= $ldap->version;
            $ldap->unbind;   # take down session
            $this->display_error($text,
                      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
            exit;
        }
        my $entry = $mesg->entry(0);
        if( defined $entry ){
            @muids = $entry->get_value('memberUid');
	    $gidNumber = $entry->get_value('gidNumber');
        }else{
            @muids =();
        }
        my %m_uid_hash;

        foreach my $m_uid (@muids){
            $m_uid_hash{$m_uid} = 1;    
#            print STDERR "$m_uid\n";
        }

        if( scalar( @muids) == 0 ){
            @add_uids = @selected_uids;
            #print STDERR "no members\n";
        }else{
            foreach my $m_uid (@muids){
                $m_uid_hash{$m_uid} = 1;
            }
            foreach my $sel_uid (@selected_uids){
                if( !defined $m_uid_hash{$sel_uid} ){
                    push @add_uids,($sel_uid);
                    #print STDERR "add: $sel_uid \n";
                }
            }
        }
        if( scalar(@selected_uids) == 0){
            @delete_uids = @muids;
        }else{
            my %sel_uid_hash;
            foreach my $sel_uid (@selected_uids){
                $sel_uid_hash{$sel_uid} = 1;
#                print STDERR "selected $sel_uid\n";
            }
            foreach my $m_uid (@muids){
                if( !defined $sel_uid_hash{$m_uid} ){
                    push @delete_uids, ($m_uid);
#                    print STDERR "del: $m_uid \n";
                }
            }
        }
    }
# SCHULSERVER I
# update mail system if objectClass=SuSESchoolClass or objectClass=SuSESchoolGroup
    $mesg = $ldap->search( scope => "one",
			   base  => $ldapbase,
			   filter => "(&(cn=$group)(|(objectClass=SuSESchoolGroup)(objectClass=SuSESchoolClass)))",
			);
    if($mesg->count > 0){
	    my $imap = new Net::IMAP($imaphost, Debug => 0) or die ("can't connect to $imaphost: $!\n");
	    my $resp = $imap->login($ADMINuid, $ADMINpasswd);
	    foreach ( @add_uids ){
		$mesg = $ldap->search ( scope => "one",
					base  => $ldapbase,
					filter => "uid=$_",
					attrs => ["gidNumber"]
				);
		if($mesg->entry(0)->get_value("gidNumber") == 102) {
			$resp = $imap->setacl($group,$_,"lrswipcd");
		}
	    }
	    foreach ( @delete_uids ){
		$resp = $imap->deleteacl($group,$_);
	    }
	    $resp = $imap->logout();
    }
# SCHULSERVER I
   if( $groupware eq 'phpgw' || $groupware eq 'all' ) {
	$mesg = $ldap->search(
			    scope => "one",
			    base  => $ldapbase,
			    filter => "(&(cn=$group)(objectClass=SuSESchoolClass))",
			);
	my $is_a_class = 0;
	if($mesg->count > 0){
		$is_a_class = 1;
	}
	foreach ( @add_uids ){
		$mesg = $ldap->search (
				    scope => "one",
				    base  => $ldapbase,
				    filter => "uid=$_",
				    attrs => ["gidNumber","uidNumber"]
				);
		my $uidNumber = $mesg->entry(0)->get_value("uidNumber");
		my $GID = $mesg->entry(0)->get_value("gidNumber");
		 # Schler drfen nur in einer Klasse sein
		if($SORTED_HOME ne 'no' &&  $is_a_class == 1 && $GID == 100) {
			$mesg = $ldap->search ( scope => "one",
					base  => $ldapbase,
					filter => "(&(memberuid=$_)(objectClass=SuSESchoolClass))"
					);

			if($mesg->count > 0){ 
				$text .= $_." is member in an other class"."<br>\n";
				next;
			}
		}
		push @realy_add, $_;
	}
	foreach ( @delete_uids ){
		$mesg = $ldap->search (
				    scope => "one",
				    base  => $ldapbase,
				    filter => "uid=$_",
				    attrs => ["uidNumber"]
				);
		my $uidNumber = $mesg->entry(0)->get_value("uidNumber");
	}
    }
# SCHULSERVER E
# SCHULSERVER C
    if( scalar(@realy_add) > 0){
        #print STDERR "some values to add ". scalar(@add_uids) ."\n";
        $add{'memberuid'} = \@realy_add;
    }
    if( scalar(@delete_uids) > 0){
        #print STDERR "some values to del ". scalar(@delete_uids) ."\n";
        $delete{'memberuid'} = \@delete_uids;
    }

    $mesg = $ldap->modify( "cn=".$group.",".$ldapbase,
                           'replace' =>  \%replace,
                           'add' =>  ( (keys %add) ? \%add : {} ),
                           'delete' => ( (keys %delete) ? \%delete : {} )
                         );

    if($mesg->code != 0) {
            $text .= "modification $message->{failed}\n\n";
            $text .= &LDAPerror($mesg);
            $text .= $ldap->version;
            $ldap->unbind;   # take down session
            $this->display_error($text,
                      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
            exit;
    }
    foreach my $act_uid ( @realy_add ){
	my $dn = "uid=".$act_uid.",".$ldapbase;
	$mesg = $ldap->search( 'base' => "$dn", 
			       'scope' => 'base',
			       'filter' => "(gidNumber=$gidNumber)",
			       'attrs' => [ "gidNumber" ]
			    );
	if($mesg->code != 0){
    	        $text = "search $message->{failed}\n\n";
    	        $text .= &LDAPerror($mesg);
    	        $ldap->unbind;   # take down session
    	        $this->display_error($text,
    	                  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    	        exit;
	}
	#only add "comfireGroupID" if user isn't already primary Member of the Group
	if( $groupware eq 'comfire' || $groupware eq 'all' ) {
		if($mesg->count == 0){
		    $mesg = $ldap->modify( $dn,
					'add' => { "comfireGroupID" => $gidNumber }
				    );
#		    if($mesg->code != 0) {
#			    $text = "modification $message->{failed}\n\n";
#			    $text .= &LDAPerror($mesg);
#			    $ldap->unbind;   # take down session
#			    $this->display_error($text,
#				      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
#			    exit;
#		    }
		}
	} 
    }
    foreach my $act_uid ( @delete_uids ){
	my $dn = "uid=".$act_uid.",".$ldapbase;
	$mesg = $ldap->search( 'base' => "$dn", 
			       'scope' => 'base',
			       'filter' => "(gidNumber=$gidNumber)",
			       'attrs' => [ "gidNumber" ]
			    );
	if($mesg->code != 0){
    	        $text = "search $message->{failed}\n\n";
    	        $text .= &LDAPerror($mesg);
    	        $ldap->unbind;   # take down session
    	        $this->display_error($text,
    	                  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    	        exit;
	}
	#only delete "comfireGroupID" if user isn't primary Member of the Group
	if( $groupware eq 'comfire' || $groupware eq 'all' ) {
		if($mesg->count == 0){
		    $mesg = $ldap->modify( $dn,
					'delete' => { "comfireGroupID" => $gidNumber }
				    );
#		    if($mesg->code != 0) {
#			    $text = "modification $message->{failed}\n\n";
#			    $text .= &LDAPerror($mesg);
#			    $ldap->unbind;   # take down session
#			    $this->display_error($text,
#				      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
#			    exit;
#		    }
		}
	}
    }
    $ldap->unbind;   # take down session
    $text .= "$message->{edit_success_txt}\n";

    $q->param(-name=>'doit', -value=>"");
    $q->param(-name=>'group', -value=>"");
    $q->param(-name=>'uids', -value=>"");

    $this->display_msg($text,
        "$confParam->{cgi_path}/browse_groups.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
        2);
}

1;
