#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: change_lang.pm,v 1.2 2003/10/29 10:35:39 varkoly Exp $
# 
package change_lang;
use strict;

use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    my $setlang = $cgi->param("setlang");

    if( defined $setlang && $setlang ne "" ) {
	$this->setlanguage();
    } else {
	$this->startup();
    }
}

sub setlanguage {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"CHANGE_LANG"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};
    my $user_dn       = "uid=".$LOGINuid.",".$ldapbase;

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');

    my $newlang = $q->param("lang_grp");
    
    my %temp=("LANG" => $newlang);

    $session->modify(\%temp);
    $session->load($sessionID);
    $message = ($session->getLanguage($session->{confParam}->{LANG},"CHANGE_LANG"))->data();

    my $url = "$confParam->{cgi_path}/change_lang.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>$user_dn,
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }

    # find all resources
    $result = $ldap->search(base=>$user_dn,
			    scope=>"base",
			    filter=>"uid=$LOGINuid",
			    attrs=>[ 'preferredLanguage' ]);
    if($result->code != 0) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }

    my $entry = $result->entry(0);
    $entry->replace( preferredlanguage => "$newlang" );
    $result = $entry->update($ldap);

    if($result->code != 0) {
        my $errs = "update $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }

    $this->display_msg("$message->{lang_changed}",
		       $confParam->{cgi_path}."/change_lang.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		       1);
    exit;
}

sub startup {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"CHANGE_LANG"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');

    my @all_lang;
    foreach my $k (keys %$::lmsg) {
	push @all_lang, $k;
	#print STDERR "LANG=$k, ".$::lmsg->{$k}."\n";
    }

    my $html = "";
    $html .=	$q->start_table({-cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=		$q->start_Tr();
    $html .=	    	$q->start_td();
    $html .=				$q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=					$q->start_form(-action=>$confParam->{cgi_path}."/change_lang.pl");
    $html .=		    		$q->Tr( $this->help_th(2,  "$message->{title}", "CHANGE_LANG") );

    $q->autoEscape(undef);
    my $lang_imgs;
    foreach my $l ( @all_lang ) {#{{{
	my $tmpmsg = ($session->getLanguage($l,"CHANGE_LANG"))->data();
	$lang_imgs->{$l} = "&nbsp;&nbsp;&nbsp;".
	    $q->img({-alt=>"$l",
		     -src=>"$confParam->{image_path}/flag_".lc($l).".gif",
		     -border=>0}
		    )."&nbsp;&nbsp;&nbsp;$tmpmsg->{lang}";
    }#}}}
    
    $html .=            		$q->start_Tr();
    $html .=                		$q->td({-width=>"40%"}, "&nbsp;");
    $html .=                		$q->td( $q->radio_group(-name=>'lang_grp',
						   									-values=>\@all_lang,
														   	-labels=>$lang_imgs,
												   			-default=>$confParam->{LANG},
														   	-linebreak=>'true'));
    $html .=		            $q->end_Tr();
    $html .=            		$q->start_Tr();
    $html .=               			$q->td({-align=>"center", -colspan=>2},
				  						$q->submit(-class=>"button",-name=>"setlang", -value=>"$message->{set_lang}").
									  	$q->hidden(-name=>"sessionID", -value=>$sessionID).
									  	$q->hidden(-name=>"tab", -value=>$tab).
									  	$q->hidden(-name=>"stab", -value=>$stab));
    $html .=            		$q->end_Tr();
    $html .=        		$q->end_table();
	$html .=					$q->end_form();
    $html .=        	$q->end_td();
    $html .=     	$q->end_Tr();
    $html .= 	$q->end_table();

    $this->SUPER::display($html);
}

1;
