#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: create_resource.pm,v 1.3 2003/10/29 10:35:39 varkoly Exp $
#
package create_resource;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use Session;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };

@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    my $commit=$message->{"commit"};
    
    my $url = "$confParam->{cgi_path}/create_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    if ((!defined $cgi->param("run")) or ($cgi->param("run") eq "menu")) {

	my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server", $url);
	    exit;
	}
	my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
				 password=>$confParam->{"passwd"});
	if($result->code != 0) {
	    my $errs = "bind $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    exit;
	}

	# search for SuSEResourceObject
	$result = $ldap->search(base=>$confParam->{"baseDN"},
				scope=>"one",
				filter=>"(&(resourceName=*)(objectclass=SuSEResourceObject))",
				attrs=> [ 'resourceName' ]);
	if($result->code != 0) {
	    my $errs = "search $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    $ldap->unbind;
	    exit;
	}
	$ldap->unbind;
	
	my $resources;
	my $all = $result->count;
	for(my $i=0; $i<$all; $i++) {
	    my $entry = $result->entry($i);
	    my $r = join("", $entry->get_value('resourceName'));
	    push @$resources, $r;
	}
	@$resources = sort @$resources if defined $resources;
	
	my $html = "";

	$html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
	$html .=   $cgi->start_Tr();
	$html .=     $cgi->start_td();
	$html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
	$html .=         $cgi->Tr( $this->help_th(2, $message->{"res_create_title"}, "CREATE_RESOURCE") );
	$html .=         $cgi->Tr( $cgi->td({-colspan=>2, -class=>'AdminSubHeader'},
					    $message->{"res_howto"}) );
	$html .=         $cgi->start_Tr();
	$html .=           $cgi->td({-valign=>"middle", -align=>"left"},
				    $cgi->start_form().
				    $message->{"res_name"}.": ");
	$html .=           $cgi->td({}, $cgi->textfield(-name=>"grpname", -size=>26));
	$html .=         $cgi->end_Tr();
#      $html .=         $cgi->start_Tr();
#      $html .=           $cgi->td({-valign=>"middle", -align=>"center"}, "---Description---: ");
#      $html .=           $cgi->td({}, $cgi->textfield(-name=>"description", -size=>25));
#      $html .=         $cgi->end_Tr();
	$html .=         $cgi->start_Tr();
	$html .=           $cgi->td({-valign=>"top", -align=>"left"},
				    $message->{"res_resources"}.": ");
	if( (defined($resources)) && (@{$resources} > 0) ){
	    $html .=         $cgi->td({},
				  $cgi->scrolling_list(-name   => "resource_sel",
						       -multiple => 'true',
						       -values => $resources,
						       -width  => 200,
						       -size =>15 ));
	}else{
	    $html .=	   $cgi->td({},
				    "<i>".$message->{'res_no_entries'}."</i>"
				    );
	}
	$html .=         $cgi->end_Tr();
	$html .=         $cgi->start_Tr();
	$html .=           $cgi->td({-colspan=>2, -align=>"center"},
				    $cgi->submit(-class=>"button",-name=>"run",-value=>$message->{"commit"}).
				    $cgi->hidden(-name=>"sessionID", -value=>$sessionID).
				    $cgi->hidden(-name=>"tab", -value=>$tab).
				    $cgi->hidden(-name=>"stab", -value=>$stab).
				    $cgi->end_form() );
	$html .=         $cgi->end_Tr();
	$html .=       $cgi->end_table();
	$html .=     $cgi->end_td();
	$html .=   $cgi->end_Tr();
	$html .= $cgi->end_table();

	$this->SUPER::display($html);
    }
    elsif ($cgi->param("run") eq $commit) {
	my @res_selected = $cgi->param("resource_sel");
	my $uc_sel;
	foreach my $sel ( @res_selected ) {
	    push @$uc_sel, $sel;
	}
	my $resgroup  = $cgi->param("grpname");

#      my $descr     = $cgi->param("description");


	my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server", $url);
	    exit;
	}
	my $mesg = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			       password=>$confParam->{passwd});
	if($mesg->code != 0) {
	    my $errs = "bind $message->{failed}\n\n";
	    $errs .= &LDAPerror($mesg);
	    $this->display_error($errs, $url);
	    exit;
	}
	
	my $errs = "";
	my %attributes;
	$attributes{"objectclass"}         = "SuSEResourceGroupObject";
	$attributes{"resourceGroupName"}   = $resgroup;
#      if (defined $description) {
#        $attributes{""} = $description;
#      }

	if( defined $uc_sel ) {
	    $attributes{"resourceGroupMember"} = $uc_sel;
	}

	$mesg = $ldap->add(dn   => "resourceGroupName=$resgroup".",".$confParam->{baseDN},
			   attr => [%attributes]);
	if ($mesg->code != 0) {
	    $errs .= &LDAPerror($mesg);
	    $this->display_error($errs,$url);
	    $ldap->unbind;   # take down session
	    exit;
	}
	
	$this->display_msg($message->{"succ_completed"}, $url,1);
	$ldap->unbind;
    }
}


1;

