#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: create_vuser.pm,v 1.5 2003/10/29 10:35:39 varkoly Exp $
#
package create_vuser;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };

use GroupFilterlist;
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $message   = ($session->getLanguage($confParam->{LANG},"CREATE_VUSER"))->data();
    
    my $create = $cgi->param($message->{'vuser_create'});

    if(defined $create) {
        $this->add_vuser();
    } else {
        $this->startup();
    }
}

sub startup {
    my $this = shift;

    my $q         = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},"CREATE_VUSER","BROWSE_USER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $tab        = $q->param('tab');
    my $stab       = $q->param('stab');
    my $filter     = $q->param('filter') || "";
    $filter = LDAP_filter_escape($filter);

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    my $mesg = $ldap->bind (   # bind to a directory with dn and password
			       dn       => "uid=$ADMINuid,$ldapbase",
			       password => "$ADMINpasswd"
			       );
    
    if($mesg->code != 0) {
	my $text .= "LDAP bind failed\n";
	$text .=  &LDAPerror($mesg);
	$this->display_error($text,
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

#-- search for virtual domains
    
    $mesg = $ldap->search( base => 'o=DNS,'.$confParam->{baseDN},
               scope => 'sub',
               attrs => [ 'zoneName' ],
               filter => 'relativeDomainName=@' );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }
    

    my @vdoms;
    foreach my $i ($mesg->entries) {
        my @zones = $i->get_value('zoneName');
        foreach my $zn ( @zones ) {
            if( ! ( $zn =~ /IN-ADDR\.ARPA/i ) && ! ( $zn eq $confParam->{mailDomain} ) ) {
                push @vdoms, $zn;
            }
        }
    }


    if( ! @vdoms || ! defined $vdoms[0] ) {
	my $vdstab = ( ! defined $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ) ||
	    $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ne "true" ? "dns" : "vdom";
	$this->display_error($message->{'vuser_create_vdom_first'}."!",
			     "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=ho_dom\&stab=$vdstab",
			     3);
	exit;
    }

    my $filterlist = GroupFilterlist->new($q, $session, $ldap, $this);
    
    my $virt_email = $q->param('virt_email');
    if(! defined($virt_email) ){
        $virt_email = "";
    }
    my $vdom = $q->param('vdom');
    if( (! defined($vdom)) || ($vdom eq "") ){
	$vdom = $vdoms[0];
    }
    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->start_Tr().
             $q->start_td().
		$q->start_form(-action=>$confParam->{cgi_path}."/create_vuser.pl", -name=>"form").
                $q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"}).
                    $q->start_Tr().
                        $this->help_th(3, $message->{'vuser_add'},"CREATE_VUSER").
                    $q->end_Tr().
		       $q->start_Tr().
                         $q->td({-colspan=>3, -class=>'AdminSubHeader'},
                            $message->{'vuser_enter_email_and_select'}
                         ).
                       $q->end_Tr(). 
                    $q->start_Tr().
                        $q->td({-width => "30%"}, $message->{'vuser_email_address'}).
                        $q->td({-align=>"center"},
                               $q->textfield(-name=>'virt_email',
                                             -size=>10, 
                                             -default=>$virt_email,
                                             -override=>1).
			       " @ ".
			       $q->popup_menu(-name=>"vdom", 
					      -values=>\@vdoms,
					      -default=>$vdom)
                        ).
                        $q->td(" &nbsp;").
                    $filterlist->make_filter_view({"textfield"    => "filter",
                                             "textlabel"    => $message->{'filter'},
                                             "listname"     => "vuserid",
					     "classname"    => $message->{'classe'},
                                             "listlabel_left" => $message->{'user'},
                                             "buttonlabel"  => $message->{'apply_filter'},
                                             "nofilter-msg" => $message->{'set_filter_first'},
                                             "listfilter"   => "(&(objectclass=posixAccount)(objectclass=SuSEeMailObject)(!(gidNumber=103))(|(sn=$filter)(givenname=$filter)(uid=$filter)))",
					     "listattrs"    => ['uid','cn'],
                                             "multiple"     => "MULTIPLE",
                                             "error-url"    => "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID",
                                             "labelexp"     => ["\$\$ ","uid","(\$\$) ","cn"],
                                             "width"        => "30",
                                             "height"       => "10"
                                            }
                    ).
                    $q->start_Tr().
                        $q->td({-colspan=>3, -align=>"center"},
                           $q->submit(-class=>"button",-name=>$message->{'vuser_create'}).
                           $q->hidden(-name=>"sessionID", -value=>$sessionID).
                           $q->hidden(-name=>"tab", -value=>$tab).
                           $q->hidden(-name=>"stab", -value=>$stab)
                        ).
                    $q->end_Tr().
                $q->end_table().
                $q->end_form().
            $q->end_td().
        $q->end_Tr().
     $q->end_table();
    $this->SUPER::display($html);
}

sub add_vuser{
    my $this = shift;

    my $q         = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},"CREATE_VUSER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $tab        = $q->param('tab');
    my $stab       = $q->param('stab');
    my $filter     = $q->param('filter');
    my @uids = $q->param('vuserid');

    my $localpart = $q->param('virt_email');
    my $domain    = $q->param('vdom');

    #my $v_email = "$localpart\@$domain";
    my $v_email;
    ($v_email  = $localpart) =~ s|@.*||g;
    $v_email .= "\@$domain";
    print STDERR "TEST <".$::CONF_GLOBAL{'GENERAL'}{'AllowVirtualDomainCatchAll'}.">\n";
    if($::CONF_GLOBAL{'GENERAL'}{'AllowVirtualDomainCatchAll'} ne "true"){
	if( (! defined($localpart)) || ($localpart eq "") ){
	    $this->display_error($message->{'vuser_no_email_given'},
		"$confParam->{cgi_path}/create_vuser.pl?sessionID=$sessionID\&filter=$filter\&tab=$tab\&stab=$stab");
	    exit;
	}elsif( $localpart !~ /^[a-zA-Z0-9._-]+$/ ){
	    $this->display_error($message->{'vuser_email_invalid'},
		"$confParam->{cgi_path}/create_vuser.pl?sessionID=$sessionID\&filter=$filter".
			    "\&tab=$tab\&stab=$stab\&virt_email=$localpart&vdom=$domain");
	    exit;
	}
    }
    if( @uids == 0 ){
        $this->display_error($message->{'vuser_no_uid_selected'},
              "$confParam->{cgi_path}/create_vuser.pl?sessionID=$sessionID\&filter=$filter".
			    "\&tab=$tab\&stab=$stab\&virt_email=$localpart&vdom=$domain");
        exit;
    }
	my $ldap = Net::LDAP->new("$ldaphost", port=>$ldapport, version => 3);

	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server",
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    my $text;
	my $mesg = $ldap->bind (   # bind to a directory with dn and password
				   dn       => "uid=$ADMINuid,$ldapbase",
				   password => "$ADMINpasswd"
				   );
	
	if($mesg->code != 0) {
	    $text .= "LDAP bind failed\n";
	    $text .=  &LDAPerror($mesg);
	    $this->display_error($text,
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    ## check if virtual address is already assigned 
    $mesg = $ldap->search( base => $ldapbase,
                           scope => "one",
                           filter => "(&(objectClass=SuSEVirtUserObject)(vaddress=$v_email))",
                           attrs => ['uid']
                          );
	if($mesg->code == 0 && $mesg->count > 0) {
#        my $entry = $mesg->entry(0);
#        my @values = $entry->get_value('uid');
#        my $users = join "<br>", (@values);
            
	    $this->display_error("\"$v_email\" $message->{'vuser_email_already_assigned'}",
	   	                     "$confParam->{cgi_path}/create_vuser.pl?".
					"sessionID=$sessionID\&filter=$filter\&tab=$tab".
					"\&stab=$stab\&virt_email=$localpart&vdom=$domain");
	    exit;
    } elsif($mesg->code != 0){
	    $text .= "LDAP search failed\n";
	    $text .=  &LDAPerror($mesg);
	    $this->display_error($text,
	    	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    foreach my $uid (@uids){
        #check if user has objectClass SuSEVirtUserObject already
        my %mod = ( "vaddress" => $v_email);
        $mesg = $ldap->search( base => "uid=".$uid.",".$ldapbase,
                               scope => "base",
                               filter => "(objectClass=SuSEVirtUserObject)",
                               attrs => []
                             );
        if( ($mesg->code == 0) && ($mesg->count == 0) ){
            $mod{'objectClass'} = "SuSEVirtUserObject";
        }elsif( $mesg->code != 0 ){
            $text .= "LDAP search failed\n";
            $text .=  &LDAPerror($mesg);
            $this->display_error($text,
                      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
            exit;
        }
        $mesg = $ldap->modify( "uid=".$uid.",".$ldapbase ,
                               add => \%mod
                             );
        if( $mesg->code != 0 ){
            $text .= "LDAP modification failed\n";
            $text .=  &LDAPerror($mesg);
            $this->display_error($text,
                      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
            exit;
        }
    }
    $this->display_msg($message->{'vuser_created'},
                      "$confParam->{cgi_path}/create_vuser.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",1);
}

1;
