#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: dhcp.pm,v 1.13 2004/05/13 11:11:19 varkoly Exp $
#
package dhcp;
use strict;
use Storable qw(thaw freeze);
use Net::LDAP;
use Net::Netmask;
use LDAPUtils;
use Utils;
use Display;
use Session;
use cfreader;
use POSIX;
use URI::Escape;
#use CGI::Pretty qw( :html3 );
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = (qw(Display));

my $JavaScript=<<EOF;
function open_help_win( win_location ) {
    parem='width=400,height=600,directories=no,menubar=no,toolbar=no,resizable=yes';   contacts_window=window.open(win_location, 'ImapHelpWin', parem);
    if (!contacts_window.opener) contacts_window.opener=self;
}
function close_help_win() {
    document.all['ImapHelpWin'].close();
}
EOF


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $delete    = $cgi->param("delete");
    my $export    = $cgi->param("export");
    my $doit      = $cgi->param("doit");
    my $action    = $cgi->param("action");
    my $insert    = $cgi->param("insert");
    $this->{"ldap"} = Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $this->{"ldap"}) {
       $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
       exit;
    }
    my $mesg=$this->{"ldap"}->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    if( defined $doit && $doit ne "" ) {
        if($action eq 'delete' ) {
                $this->delete_dhcp();
        } else {
            $this->add($action);
        }
    } elsif ( defined $insert && $insert ne "") {
        $this->add_realy();
    } elsif( defined $export && $export ne "" ) {
        $this->create_dhcp_setup();
    } else {
        $this->startup();
    }
    $this->{"ldap"}->unbind;
}

sub startup {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $act     = $cgi->param("act");

    my $colls   = 2;
    my $dhcp    = "";
    my $mesg    = "";
    my $treelink=$confParam->{cgi_path}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab&act=edit";

    my $ldap      = $this->{"ldap"};
    my $TITLE   = $message->{dhcp_title};
    my $SECTION = 'DHCP';
    if( defined $act && $act eq  "edit" ) {
        $dhcp = $this->attributes("output");
        $TITLE = $message->{dhcp_edit_entry};
        $SECTION = 'DHCP_edit_entry';
    } elsif ( defined $cgi->param("subattr")) {
        $dhcp = $this->attributes("change");
        $colls=5;
    } else {
        my $base = "cn=DHCP,o=DHCP,".$confParam->{baseDN};
        #($colls,$dhcp) = $this->show_dhcp_entry($base);
        $dhcp = '<br>'.$this->show_dhcp_entry($base,0).'<br>';
        $colls = 1;
    }

    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({ -cellspacing=>0, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->Tr(
              [
               $this->help_th($colls, $TITLE, $SECTION),
               $cgi->td({-class=>'AdminSubHeader',-colspan=>$colls
                 }, $message->{title_help})
               ]
              );
    if( defined $act && $act ne "" ) {
        $html .= $cgi->start_form(-name=>"form",-action=>$confParam->{cgi_path}."/dhcp.pl");
        $html .= $cgi->Tr( $cgi->td( $dhcp ) );
    } else {
        $html .= $cgi->Tr( $cgi->td( $dhcp ) );
        $html .= $cgi->start_form(-name=>"form",-action=>$confParam->{cgi_path}."/dhcp.pl");
    }
    $html .= $cgi->Tr(
              $cgi->td({-align=>"center",-colspan=>$colls},
                   $cgi->submit(-class=>"button",-name=>"export", -value=>"$message->{export}").
                   $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
                   $cgi->hidden(-name=>"tab", -value=>"$tab").
                   $cgi->hidden(-name=>"stab", -value=>"$stab").
                   $cgi->end_form()
                   )
              );
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

sub delete_dhcp {
    my $this   = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $objectclass    = $cgi->param('objectClass')."DN";
    my $text    = "";
    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    my $ldapbase  = $confParam->{baseDN};


    my $dn   = $cgi->param("searchdn");
    my $ldap = $this->{"ldap"};
    my $mesg = "";

    $text    = $this->delete_entry($dn);

    my $result = $ldap->search(
                                base    => "cn=DHCP,o=DHCP,".$confParam->{baseDN},
                                  filter  => "$objectclass=$dn"
                            );
    my @entries=$result->entries;
    foreach my $i (@entries) {
        $mesg = $ldap->modify( dn=>$i->dn, delete=>{$objectclass=>$dn} );
        if($mesg->code != 0) { $text .= &LDAPerror($mesg);}
    }
    my $result = $ldap->search ( base   => $dn,
                                     scope  => 'base',
                                     filter => 'objectClass=dhcpHost',
                                     attrs  => ['dNSZoneDN','cn']
                                   );
    if($result->count > 0) {
        # It is a dhcpHost. The link in dNSZone  must be deleted to
            foreach my $i ($result->entries) {
                my $hostname = $i->get_value('cn');
                if( $hostname =~ /(.*?)\./ ) {
                    $hostname = $1;
                }
                foreach my $DN ($i->get_value('dNSZoneDN')) {
                    $mesg = $ldap->search ( base   => $DN,
                                 scope  => 'base',
                                 filter => 'objectClass=dNSZone',
                                 attrs  => ['relativeDomainName','zoneName']
                                );
                    my $host   = $mesg->entry(0)->get_value('relativeDomainName');
                    my $domain = $mesg->entry(0)->get_value('zoneName');
                       $ldap->delete($DN);
                        my $msg = $ldap->search ( base   => 'o=DNS,'.$ldapbase,
                                       filter => "pTRRecord=$host.$domain.",
                                       attrs  => []
                                          );
                        if($msg->count > 0) {
                       $msg = $ldap->delete($msg->entry(0)->dn);
                        }
                }
                if($mesg->code != 0) { $text .= &LDAPerror($mesg);}
                $session->suad_smbdeluser($hostname."\$");
                $session->suad_smbdeluser($hostname);
                my $command = $confParam->{bin_path}."/del_user -";
                my $cmd_args  = "LH=$ldaphost\nLP=$ldapport\nal=$ADMINuid\n";
                $cmd_args .= "ap=$ADMINpasswd\nuid=$hostname\nldap\nbDN=$ldapbase\n";
                my $errs = cmd_pipe($command, $cmd_args);
                $ldap->delete("uid=".$hostname."\$,".$ldapbase);
                # Das Homeverzeiniss wird archiviert
                $session->suad_move('/home/workstations/'.$hostname,'/home/archiv/'.$hostname);
                system("sudo /usr/sbin/archiv_user");
        }
    }
    $result = $ldap->search ( base    => $dn,
                                  scope    => 'base',
                  filter => 'objectClass=dhcpPool',
                  attrs    => ['cn']
                );
    if($result->count > 0) {
        # It is a dhcpPool. The DNS Entries must be deleted to
        my $result = $ldap->search ( base   => 'o=DNS,'.$confParam->{baseDN},
                                    scope  => 'sub',
                                    filter => "dhcpPoolDN=$dn",
                                    attrs  => []
                                  );
        foreach my $i ($result->entries) {
            $ldap->delete($i->dn);
        }
    }
    $ldap->delete($dn);
    if( $text ne "" ) {
        $this->display_error('Error by deleting:'.$dn."\n".$text,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
    } else {
        $this->display_msg('Deleted:'.$dn."\n".$text,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
    }    

}

sub delete_entry {
    my $this   = shift;
    my $dn     = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    my $ldapbase  = $confParam->{baseDN};
    my $ldap      = $this->{"ldap"};

    my $result = $ldap->search(base=>$dn,scope=>"one",filter=>"objectclass=*");
    my @entries=$result->entries;
    my $mesg = "";
    my $text = "";

    foreach my $i (@entries) {
        my $dn = $i->dn;
        my $result = $ldap->search ( base    => $dn,
                     filter => 'objectClass=dhcpHost',
                                     attrs  => ['dNSZoneDN','cn']
               );
        if($result->count > 0) {
            # It is a dhcpHost. The link in dNSZone  must be deleted to
            foreach my $i ($result->entries) {
                my $hostname = $i->get_value('cn');
                if( $hostname =~ /(.*?)\./ ) {
                    $hostname = $1;
                }
                foreach my $DN ($i->get_value('dNSZoneDN')) {
                   $mesg = $ldap->search ( base   => $DN,
                                 scope  => 'base',
                                 filter => 'objectClass=dNSZone',
                                 attrs  => ['relativeDomainName','zoneName']
                                );
                    my $host   = $mesg->entry(0)->get_value('relativeDomainName');
                    my $domain = $mesg->entry(0)->get_value('zoneName');
                       $ldap->delete($DN);
                        my $msg = $ldap->search ( base   => 'o=DNS,'.$ldapbase,
                                       filter => "pTRRecord=$host.$domain.",
                                       attrs  => []
                                          );
                        if($msg->count > 0) {
                       $msg = $ldap->delete($msg->entry(0)->dn);
                        }
                }
                if($mesg->code != 0) { $text .= &LDAPerror($mesg);}
                $session->suad_smbdeluser($hostname."\$");
                $session->suad_smbdeluser($hostname);
                my $command = $confParam->{bin_path}."/del_user -";
                my $cmd_args  = "LH=$ldaphost\nLP=$ldapport\nal=$ADMINuid\n";
                $cmd_args .= "ap=$ADMINpasswd\nuid=$hostname\nldap\nbDN=$ldapbase\n";
                my $errs = cmd_pipe($command, $cmd_args);
                $ldap->delete("uid=".$hostname."\$,".$ldapbase);
                # Das Homeverzeiniss wird archiviert
                $session->suad_move('/home/workstations/'.$hostname,'/home/archiv/'.$hostname);
                system("sudo /usr/sbin/archiv_user");
            }
        }
        $result = $ldap->search ( base    => $dn,
                                     scope    => 'base',
                                     filter => 'objectClass=dhcpPool',
                                     attrs    => ['cn']
                                   );
        if($result->count > 0) {
            # It is a dhcpPool. The DNS Entries must be deleted to
            my $result = $ldap->search ( base   => 'o=DNS'.$confParam->{baseDN},
                                     scope  => 'sub',
                                     filter => "dhcpPoolDN=$dn",
                                     attrs  => []
                                   );
                    foreach my $i ($result->entries) {
                $mesg = $ldap->delete($i->dn);
                if($mesg->code != 0) { $text .= 'Error by deleting:'.$i->dn.'<br>'.&LDAPerror($mesg);}
            }
        }
        $text .= $this->delete_entry($dn);
        $mesg = $ldap->delete($dn);
        if($mesg->code != 0) { $text .= 'Error by deleting:'.$dn.'<br>'.&LDAPerror($mesg);}
        return $text;
    }
}

sub choose_domain {
    my $this   = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $ldapbase  = $confParam->{baseDN};
    my $searchdn  = $cgi->param("searchdn");

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my @domains   = ();
    my $ldap      = $this->{'ldap'};
    my $mesg      = $ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
       $mesg      = $ldap->search (
                                       base    => "o=DNS,$ldapbase",
                                    scope    => 'sub',
                                    filter    => 'relativeDomainName=@',
                                    attrs    => ['zoneName']
                                     );
    if ( $mesg->code == 0 )    {
        foreach my $i ($mesg->entries) {
            my @zones = $i->get_value('zoneName');
            foreach my $zn ( @zones ) {
                if( ! ( $zn =~ /IN-ADDR.ARPA$/i ) ) {
                    push @domains, $zn;
                }
            }
        }
    } else {
        $this->display_error('No domains found<br>'.&LDAPerror($mesg),$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
        exit;
    }
    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({ -cellspacing=>0, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->Tr(
              [
               $this->help_th(1, "$message->{dhcp_title}", "DHCP_CHOOSE_DOMAIN")
               ]
              );
    $html .= $cgi->start_form(-name=>"form",-action=>$confParam->{cgi_path}."/dhcp.pl");
    $html .= $cgi->Tr(
                        $cgi->td({-align=>"center"}, $message->{choose_domain}.":".
                                 $cgi->popup_menu(-name=>'domain',-values=>\@domains))
                    );
    $html .= $cgi->Tr(
              $cgi->td({-align=>"center"},
                   $cgi->submit(-class=>'button',-name=>'doit', -value=>"$message->{next}").
                   $cgi->hidden(-name=>'action', -value=>'addhost').
                   $cgi->hidden(-name=>'searchdn', -value=>$searchdn).
                   $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
                   $cgi->hidden(-name=>'tab', -value=>$tab).
                   $cgi->hidden(-name=>'stab', -value=>$stab)
                   )
              );
    $html .= $cgi->end_form();
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

sub add  {
    my $this   = shift;
    my $what   = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('domain');

    if ( ($what eq 'addhost' || $what eq 'addpool') && (! defined $domain || $domain eq "" ) ) {
        $this->choose_domain();
        return;
    }

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();
    my $dhcpRange  =  '';
    my $dhcpSubnet =  '';

    my @attribute = ();
    my @a_hosts      = ();
    my %h_hosts      = ();
    my $searchdn  = $cgi->param('searchdn');
    my %attrs      = ();

    my $ldapbase = "o=DNS,".$confParam->{baseDN};
    my $ldap     = $this->{"ldap"};
    my $mesg     = $ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
       $mesg      = $ldap->search( base    => $ldapbase,
                                  scope => 'sub',
                                  filter => '(&(relativeDomainName=@)(zoneName=*IN-ADDR.ARPA))',
                                  attrs => ['zoneName'] 
                                );
    my $net_class = 0;
    my $net_address = 0;
    my $rev_net = 0;
    my @ip_parts = ();
    foreach my $i ($mesg->entry(0)->get_value('zoneName')) {
        if( ! ( $i =~ /IN-ADDR\.ARPA/i ) ) {
                    next;
        }else{
            $i =~ /^((\d+\.)+)IN-ADDR\.ARPA/i ;
            if(defined $1){
                @ip_parts = split /\./, $1;
                $net_class = scalar(@ip_parts);
                $net_address = join( ".", reverse(@ip_parts) );
                $rev_net = join( ".", @ip_parts );
            }else{
                print STDERR "Something is wrong here\n";
            }
        }
    }

    if( $what eq 'addgroup') {
        push @attribute, 'cn';
        $attrs{cn} = 'Group01';

    } elsif( $what eq 'addhost') {
        my $mesg      = $ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
        my $ldapbase  = "o=DNS,".$confParam->{baseDN};
        my $hostname  = `hostname`;
        chomp $hostname;
        my $result      = $ldap->search (
                                        base    => $ldapbase,
                                        filter    => "(&(zoneName=$domain)(!(relativeDomainName=@))".
                                                    "(!(relativeDomainName=$hostname))(!(objectClass=SuSEdhcpHost)))",
                                        attrs    => ['relativeDomainName','ARecord']
                                    );
        foreach my $i ($result->entries) {
            my $dn        = $i->dn;
            my $ip        = $i->get_value('ARecord');
            my $hostname= $i->get_value('relativeDomainName');
            push @a_hosts, $dn;
            $h_hosts{$dn} = "$hostname.$domain: $ip";
        }
        if(scalar (@a_hosts) == 0) {
            $this->display_msg($message->{no_host},
                        $confParam->{"cgi_path"}."/dns_managehost.pl?sessionID=$sessionID&tab=$tab&stab=dns_addhost&domain=$domain",3);
            return;            
        }
        push @attribute, ('dhcpHWAddress');
        $attrs{dhcpHWAddress}    = "00:A0:CC:73:30:66";
    } elsif( $what eq 'addpool') {
        push @attribute, ('cn', 'host_prefix');
        $attrs{cn}                 = 'Pool02';
        $attrs{host_prefix}     = 'pc';

        $dhcpRange =  $cgi->start_Tr().$cgi->start_td({-align=>'right'}).$message->{dhcpRange}.":";
        $dhcpRange .= $cgi->start_td();
        my @net_address = split /\./, $net_address;
        for( my $i=0; $i < 4; $i++ ){
            if( $i >= $net_class ) {
                $dhcpRange .= $cgi->textfield(-name => "from_ip_part_$i", -size => 3, -maxlength => 3);
                $dhcpRange .= ( $i < 3 ? "&nbsp;<b>.</b>&nbsp;" : "" );
            } else {
                $dhcpRange .= $cgi->textfield(-name => "from_ip_part_$i", -size => 3, 
                                -readonly => undef, -value=> $net_address[$i]);
                $dhcpRange .= "&nbsp;<b>.</b>&nbsp;";    
            }
        }
        $dhcpRange .= "&nbsp;&nbsp;&nbsp;<b>:</b>&nbsp;&nbsp;&nbsp;";
        for( my $i=0; $i < 4; $i++ ){
            if( $i >= $net_class ) {
                $dhcpRange .= $cgi->textfield(-name => "to_ip_part_$i", -size => 3, -maxlength => 3);
                $dhcpRange .= ( $i < 3 ? "&nbsp;<b>.</b>&nbsp;" : "" );
            } else {
                $dhcpRange .= $cgi->textfield(-name => "to_ip_part_$i", -size => 3, 
                                -readonly => undef, -value=> $net_address[$i]);
                $dhcpRange .= "&nbsp;<b>.</b>&nbsp;";    
            }

        }
        $dhcpRange .= $cgi->hidden(-name=>"net_address", -value=>"$net_address").
                         $cgi->hidden(-name=>"net_class", -value=>"$net_class").
                         $cgi->hidden(-name=>"rev_net", -value=>"$rev_net");
    } elsif( $what eq 'addsubnet') {
        push @attribute, ('dhcpNetMask');
        $dhcpSubnet    = $cgi->start_Tr().$cgi->start_td({-align=>'right'}).$message->{dhcpSubnet}.":".$cgi->start_td();
        my @net_address = split /\./, $net_address;
        for( my $i=0; $i < 4; $i++ ){
            if( $i >= $net_class ) {
                $dhcpSubnet .= $cgi->textfield(-name => "ip_part_$i", -size => 3, -maxlength => 3);
                $dhcpSubnet .= ( $i < 3 ? "&nbsp;<b>.</b>&nbsp;" : "" );
            } else {
                $dhcpSubnet .= $cgi->textfield(-name => "ip_part_$i", -size => 3, 
                                -readonly => undef, -value=> $net_address[$i]);
                $dhcpSubnet .= "&nbsp;<b>.</b>&nbsp;";    
            }
        }
        $dhcpSubnet .= $cgi->hidden(-name=>"net_address", -value=>"$net_address").
                         $cgi->hidden(-name=>"net_class", -value=>"$net_class").
                         $cgi->hidden(-name=>"rev_net", -value=>"$rev_net").
                      $cgi->end_td.$cgi->end_Tr;

        $attrs{dhcpNetMask}     = "24";
    } 

    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({ -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->Tr(
              [
               $this->help_th(2, "$message->{$what}", "DHCP_$what")
               ]
              );
    $html .= $cgi->start_form(-name=>"form",-action=>$confParam->{cgi_path}."/dhcp.pl");
    if( $what eq 'addhost') {
        $html .= $cgi->Tr(
                    $cgi->td({-align=>"right"}, $message->{choose_a_host}.":"),
                    $cgi->td($cgi->scrolling_list(-name    =>    'host',
                                         -value    =>    \@a_hosts,
                                         -labels=>    \%h_hosts,
                                         -size    =>    5,
                                         -width    =>    60
                                        ))
                );
    } elsif ( $what eq 'addsubnet') {
        $html .= $dhcpSubnet;
    }
    foreach my $i (@attribute) {
        $html .= $cgi->start_Tr().
                        $cgi->td({-align=>"right",-width=>'40%'}, $message->{$i}.":").
                        $cgi->start_td(-nowrap=>undef).
                        $cgi->textfield(-name=>"$i",-size=>20);
        if( $attrs{$i} ne "" ) {
            $html .=  '&nbsp;&nbsp;'.$message->{example}.'&nbsp;&nbsp;<b>:</b>&nbsp;&nbsp;'.$attrs{$i};
        }
        $html .= $cgi->end_td().$cgi->end_Tr()."\n";
    }
    if( $what eq 'addpool') {
        $html .= $dhcpRange.$cgi->end_td.$cgi->end_Tr;
    }
    $html .= $cgi->Tr(
              $cgi->td({-align=>"center",-colspan=>2},
                   $cgi->submit(-class=>"button",-name=>"insert", -value=>"$message->{insert}").
                   $cgi->hidden(-name=>"action", -value=>"$what").
                   $cgi->hidden(-name=>"searchdn", -value=>$searchdn).
                   $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
                   $cgi->hidden(-name=>"domain", -value=>"$domain").
                   $cgi->hidden(-name=>"tab", -value=>"$tab").
                   $cgi->hidden(-name=>"stab", -value=>"$stab").
                   $cgi->end_form()
                   )
              );
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

sub add_realy  {
    my $this   = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $ldap      = $this->{"ldap"};
    my $mesg      = '';

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $dn        = $cgi->param("searchdn");
    my $what      = $cgi->param("action");
    my $domain    = $cgi->param("domain");
    my $newdn ;
    my $mesg = "";

    my $text;
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    if( $what eq "addgroup") {
        $newdn = "cn=".$cgi->param("cn").",".$dn;
        $mesg = $ldap->add( dn => $newdn,
                            attr => [ 
                                objectClass => [ 'top','dhcpOptions', 'dhcpGroup' ],
                                cn => $cgi->param("cn")
                        ]);
        if ($mesg->code == 0 ) {
            $mesg = $ldap->modify ( $dn , add=>{ dhcpGroupDN => $newdn } ); 
            if($mesg->code != 0) { $text .= 'Error by modifying '.$dn.'<br>'.&LDAPerror($mesg); }
        } 
    } elsif( $what eq "addhost") {
            my $hostdn   = $cgi->param('host');
            if( !defined $hostdn || $hostdn eq '' ) { 
              $this->display_error($message->{choose_a_host}, $confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
              exit();
            }  
            my $dhcpHWAddress = $cgi->param("dhcpHWAddress");
            if( ! ($dhcpHWAddress =~ /^\w\w:\w\w:\w\w:\w\w:\w\w:\w\w$/)  ) {
                $this->display_error($message->{invalid_hwa}.': "'.$dhcpHWAddress.'"',
                          $confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
                exit();
            }
            my $result   = $ldap->search ( base=>$hostdn, scope=>'base', filter=>'objectClass=*', attrs=>['ARecord','relativeDomainName'] );
            if($result->code != 0)  { 
              $this->display_error('Error by searching '.$hostdn.'<br>'.&LDAPerror($result),
                          $confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab"); 
              exit(); 
            }
            if($result->count != 1) { 
              $this->display_error($message->{choose_a_host}, $confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
              exit();
            }  
            my $ipaddr   = $result->entry(0)->get_value('ARecord');
            my $hostname = $result->entry(0)->get_value('relativeDomainName');
                $newdn = "cn=$hostname.$domain,$dn";
                $mesg = $ldap->add( dn => $newdn,
                                   attr => [ 
                                        objectClass        => [ 'top', 'dhcpHost','dhcpOptions' ,'SuSEdhcpHost' ],
                                        dhcpHWAddress    => "ethernet ".$dhcpHWAddress,
                                        cn                => "$hostname.$domain",
                                        dhcpStatements  => "fixed-address ".$ipaddr,
                                        dnsZoneDN        => "$hostdn"
                                    ]    
                );
            if ($mesg->code == 0 ) {
                $mesg = $ldap->modify ( $dn ,     add=>{ dhcpHostDN => $newdn } );
                if($mesg->code != 0) { $text .= 'Error by modifying '.$dn.'<br>'.&LDAPerror($mesg); }
                $mesg = $ldap->modify ( $hostdn , add=>{ objectClass=> 'SuSEdhcpHost' , dhcpHostDN => $newdn } );
                if($mesg->code != 0) { $text .= 'Error by modifying '.$hostdn.'<br>'.&LDAPerror($mesg); }
            }
    } elsif( $what eq "addpool") {
        my $net_class      = $cgi->param('net_class');
        my $net_address    = $cgi->param('net_address');
        my $rev_net        = $cgi->param('rev_net');
        my @fromaddr       = split /\./, $net_address;
        my @toaddr           = split /\./, $net_address;
        for( my $i=$net_class; $i < 4 ; $i++ ){
            push @fromaddr, $cgi->param("from_ip_part_$i");
            push @toaddr  , $cgi->param("to_ip_part_$i");
        }
        if($fromaddr[1] != $toaddr[1]) {
            $this->display_error('Range to big:'.join('.',@fromaddr).'-'.join('.',@toaddr),$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
            exit();
        }
        if($fromaddr[2] > $toaddr[2] || $fromaddr[3] >= $toaddr[3] ||
                $fromaddr[1] > 255 || $fromaddr[1] < 0 || $toaddr[1] > 255 || $toaddr[1] < 0 ||
                $fromaddr[2] > 255 || $fromaddr[2] < 0 || $toaddr[2] > 255 || $toaddr[2] < 0 ||
                $fromaddr[3] > 254 || $fromaddr[3] < 1 || $toaddr[3] > 254 || $toaddr[3] < 1 
            ) {
            $this->display_error('Invalid Range:'.join('.',@fromaddr).'-'.join('.',@toaddr),$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
            exit();
        }
        my $count    = 0;
        if($fromaddr[2] != $toaddr[2]) {
            $count = ($toaddr[2] - $fromaddr[2]) * 255;
        } else {
            $count = $toaddr[3] - $fromaddr[3];
        }
        if        ($count < 10 )    { $count = 1; }
        elsif    ($count < 100 ) { $count = 2; }
        elsif    ($count < 1000 ){ $count = 3; }
        else    { $count = 5; }
        my $counter = 0;
        my $prefix    = $cgi->param('host_prefix');
        $newdn = "cn=".$cgi->param("cn").",".$dn;
        for(my $i=$fromaddr[2];  $i <= $toaddr[2]; $i++){
            for(my $j=$fromaddr[3];  $j <= $toaddr[3]; $j++){
                my @IP = ();
                push @IP, ($fromaddr[0], $fromaddr[1], $i,$j);
                my $hostname = sprintf "%s%0${count}d",$prefix,$counter;
                $counter++;
                my @rev_rel_ip = ();
                for( my $k=3; $k >= $net_class ; $k-- ){
                        push @rev_rel_ip, @IP[$k];
                }
                my $rev_rel_ip = join('.',@rev_rel_ip);
                my $hostdn = 'relativeDomainName='.$hostname.',ou='.$domain.',o=DNS,'.$confParam->{baseDN};
                $mesg = $ldap->add( dn    => $hostdn,
                            attr=> [
                                    relativeDomainName  => $hostname,
                                    aRecord             => join('.',@IP),
                                    dNSTTL              => '172800',
                                    zoneName            => $domain,
                                    dhcpPoolDN            => $newdn,
                                    objectClass         => ['top', 'dNSZone', 'SuSEdhcpHost']
                                ]
                   );
                if($mesg->code != 0) { $text .= 'Error by adding '.$hostdn.'<br>'.&LDAPerror($mesg); } 
                $hostdn = 'relativeDomainName='.$rev_rel_ip.',ou='.$rev_net.'.IN-ADDR.ARPA,o=DNS,'.$confParam->{baseDN};
                $mesg = $ldap->add( dn    => $hostdn,
                            attr=> [
                                    relativeDomainName  => $rev_rel_ip,
                                    pTRRecord           => "$hostname.$domain.",
                                    dNSTTL              => '172800',
                                    zoneName            => $rev_net.'.IN-ADDR.ARPA',
                                    dhcpPoolDN            => $newdn,
                                    objectClass         => ['top', 'dNSZone', 'SuSEdhcpHost']
                                ]
                );
                if($mesg->code != 0) { #this entry seems to already exist, we modify try to modify it
                    $mesg = $ldap->modify ( $hostdn, add=>{ 
                                                            pTRRecord   => "$hostname.$domain.",
                                                            objectClass => 'SuSEdhcpHost'
                                                        }
                                            );            
                }
                if($mesg->code != 0) { $text .= 'Error by adding '.$hostdn.'<br>'.&LDAPerror($mesg); } 
            }
        }
        $mesg = $ldap->add( dn => $newdn,
                            attr => [ 
                                objectClass    => [ 'top','dhcpOptions', 'dhcpPool' ],
                                cn            => $cgi->param("cn"),
                                dhcpRange    => join('.',@fromaddr).' '.join('.',@toaddr)
                        ]);
        if ($mesg->code == 0 ) {
            $mesg = $ldap->modify ( $dn , add=>{ dhcpPoolDN => $newdn } ); 
            if($mesg->code != 0) { $text .= 'Error by modifying '.$dn.'<br>'.&LDAPerror($mesg); }
        }
    } elsif( $what eq "addsubnet") {
        my @net_address = ();
        my $OK = 1;
        for( my $i=0; $i < 4; $i++ ){
            my $ip = $cgi->param("ip_part_$i");
            if($ip < 0 || $ip > 255 ) { $OK = 0; }
            push @net_address, $ip;
        }
        my $net_address = join( ".", @net_address );
        if($OK == 0) {
            $this->display_error($message->{invalid_dhcpSubnet}.':'.$net_address,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
            exit();
        }
        my $block = new Net::Netmask($net_address.'/'.$cgi->param('dhcpNetMask'));
        if ( $block->{'ERROR'} ne '' ) {
            $this->display_error($block->{'ERROR'},$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab",3);
            exit();
        }
        $newdn = "cn=".$block->base().",".$dn;
        $mesg = $ldap->add( dn => $newdn,
                            attr => [ 
                                objectClass => [ 'top','dhcpOptions', 'dhcpSubnet' ],
                                cn          => $block->base(),
                                dhcpNetMask => $block->bits()
                        ]);
        if ($mesg->code == 0 ) {
            $mesg = $ldap->modify ( $dn , add=>{ dhcpSubnetDN => $newdn } ); 
            if($mesg->code != 0) { $text .= 'Error by modifying '.$dn.'<br>'.&LDAPerror($mesg); }
        }   
    }
    if($mesg->code != 0) { $text .= 'Error by addnig '.$newdn.'<br>'.&LDAPerror($mesg); }
    if( $text ne "" ) {
        $this->display_error($text,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
    } else {
        $this->display_msg('Created:'.$newdn."\n",$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
    }    
}

sub show_dhcp_entry {
    my $this        = shift;
    my $akt_basedn    = shift;
    my $deepth        = shift;
    $deepth++;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $ldapbase  = $confParam->{baseDN};
    my $base_depth = ($ldapbase=~s/,/,/g);

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my %labels    = (  
            delete    => "$message->{delete}",
            addpool   => "$message->{addpool}",
            addgroup  => "$message->{addgroup}",
            addhost   => "$message->{addhost}",
            addsubnet => "$message->{addsubnet}"
            );

    my $html    = "";
    my $treelink=$confParam->{cgi_path}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab&act=edit";

    my $ldap  = $this->{"ldap"};

    my $result = $ldap->search(base=>$akt_basedn,scope=>"one",filter=>"objectclass=*");
    my @entries=$result->entries;

    $html .= $cgi->start_table({-class=>'dhcpSubTable', -cellspacing=>0, -cellpadding=>0, -border=>0, -width=>'100%'});
    foreach my $i (@entries) {
        my $dn = $i->dn;
        my $uri_dn = uri_escape($dn);
        my $was_bin_i = "";
        my $is_dhcpOptions = 0;
        my @actions = ();
        my $was_kan_i  = $cgi->hidden(-name=>"searchdn", -value=>$dn);
        foreach my $objectClass ($i->get_value("objectClass")) {
            if( $objectClass eq "dhcpSubnet" ) {
                $was_bin_i = "Subnet:&nbsp;".$i->get_value("cn")."/".$i->get_value("dhcpNetMask");
                @actions = ( 'addpool' , 'addgroup' , 'addhost' , 'delete' );
                $was_kan_i .= "&nbsp;".$cgi->popup_menu(-name=>'action', -values=>\@actions, -labels=>\%labels, -width=>1).
                              $cgi->hidden(-name=>"objectClass", -value=>"dhcpSubnet").
                               "&nbsp;".$cgi->submit(-class=>"button",-name=>'doit',-value=>$message->{doit});
            } elsif ( $objectClass eq "dhcpGroup" ) {
                $was_bin_i = "Group:&nbsp;".$i->get_value("description").' : '.$i->get_value("cn");
                @actions = ( 'addhost' , 'delete' );
                $was_kan_i .= "&nbsp;".$cgi->popup_menu(-name=>'action', -values=>\@actions, -labels=>\%labels, -width=>1).
                              $cgi->hidden(-name=>"objectClass", -value=>"dhcpGroup").
                               "&nbsp;".$cgi->submit(-class=>"button",-name=>'doit',-value=>$message->{doit});
            } elsif ( $objectClass eq "dhcpPool" ) {
                my $tmp        = $i->get_value('dhcpRange');
                $tmp =~ s/ /&nbsp;<b>:<\/b>&nbsp;/;
                $was_bin_i = "Pool:&nbsp;".$tmp;
                $was_kan_i .= $cgi->hidden(-name=>'action', -value=>'delete').
                              $cgi->hidden(-name=>"objectClass", -value=>"dhcpPool").
                               "&nbsp;".$cgi->submit(-class=>"button",-name=>'doit',-value=>$message->{delete});
            } elsif ( $objectClass eq "dhcpHost" ) {
                $was_bin_i = "Host:&nbsp;".$i->get_value("cn");
                $was_kan_i .= $cgi->hidden(-name=>'action', -value=>'delete').
                              $cgi->hidden(-name=>"objectClass", -value=>"dhcpHost").
                               "&nbsp;".$cgi->submit(-class=>"button",-name=>'doit',-value=>$message->{delete});
            } elsif ( $objectClass eq "dhcpOptions" ) {
                $is_dhcpOptions = 1;
            }
        }
        if( $was_bin_i eq ""  && $is_dhcpOptions) {
                $was_bin_i = "Options:&nbsp;".$i->get_value("cn");
                if($i->get_value("cn") eq 'Global') {
                    @actions = ( 'addsubnet' , 'addgroup' , 'addhost' );
                       $was_kan_i .= "&nbsp;".$cgi->popup_menu(-name=>'action', -values=>\@actions, -labels=>\%labels, -width=>1).
                                  $cgi->hidden(-name=>"objectClass", -value=>"dhcpOptions").
                                   "&nbsp;".$cgi->submit(-class=>"button",-name=>'doit',-value=>$message->{doit});
                }
        }
        if( $was_bin_i ne "" ) {
                my $width = '100%';
                if ($deepth > 2) {
                    $width = '65%';
                }
                $html .= $cgi->Tr(
                                $cgi->td({-class=>"dhcpSubTable$deepth", -nowrap=>undef, -valign=>'center'},
                                        $cgi->a({-href=>"$treelink\&searchdn=$uri_dn"}, '&nbsp;'.$was_bin_i)).
                                        $cgi->td({-class=>"dhcpSubTable$deepth", -nowrap=>undef, -valign=>'center', -width=>$width },
                                                     $cgi->start_form(-name=>"form",-action=>$confParam->{cgi_path}."/dhcp.pl").
                                                     $was_kan_i.
                                                     $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
                                                     $cgi->hidden(-name=>"tab", -value=>"$tab").
                                                     $cgi->hidden(-name=>"stab", -value=>"$stab").
                                                     $cgi->end_form()
                                        )
                                    );
        }
        $html .= $cgi->Tr(    $cgi->td().  $cgi->td($this->show_dhcp_entry($dn,$deepth) ) 
                        );
    }
    $html .=     $cgi->end_table."\n";


    return $html;
}

sub create_dhcp_setup {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $act     = $cgi->param("act");

    my $ldap      = $this->{"ldap"};
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server",
              "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    my @dhcp_conf = $this->create_dhcp_setup_rec("cn=DHCP,o=DHCP,".$confParam->{baseDN},-1);

    my $ret = $session->suad_save(\@dhcp_conf, '/etc/dhcpd.conf');
    my $errors = "";
    if( $ret->code ne "OK" ){
        $errors .= "saving dhcpd.conf: failed<br>";
        $errors .= $ret->data()."<br>";
    }else{
        $ret = $session->suad_service_exec('dhcpd', 'restart');
        $ret->code() =~ /OK:\s(\d+?)\s(.*)/ms;
        if( $1 != 0 ){
            $errors .= "restarting dhcpd: failed<br>$2";
        }
    }
    if($errors ne ""){
        $this->display_error($errors,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
    }else{
        $errors = $this->create_dns_setup();
        if($errors ne ""){
            $this->display_error($errors,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
        }
        $this->display_msg($message->{export_success},$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
    }
}

sub create_dhcp_setup_rec {
    my $this   = shift;
    my $akt_basedn = shift;
    my $deepth     = shift;
    my $akt_depth  = ($akt_basedn=~s/,/,/g);
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $ldapbase  = $confParam->{baseDN};
    my $base_depth = ($ldapbase=~s/,/,/g);

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $treelink=$confParam->{cgi_path}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab&act=edit";

    my $ldap  = $this->{"ldap"};

    my @dhcp_conf = ();

    my $result = $ldap->search(base=>$akt_basedn,scope=>"one",filter=>"objectclass=*");
    my @entries=$result->entries;
    my $left = "";
    for( my $i=0; $i < $deepth; $i++) {$left .= '  ';}
    $deepth++;

    foreach my $i (@entries) {
        my $dn = $i->dn;
        my $block = "";
        my @actions = ();
        foreach my $objectClass ($i->get_value('objectClass')) {
            if( $objectClass eq 'dhcpSubnet' ) {
                $block = '  ';
                my $network = new Net::Netmask($i->get_value('cn').'/'.$i->get_value('dhcpNetMask'));
                push @dhcp_conf, $left.'subnet '.$i->get_value('cn').' netmask '.$network->mask().' {';
            } elsif ( $objectClass eq "dhcpGroup" ) {
                $block = '  ';
                push @dhcp_conf, $left.'group { #Group: '.$i->get_value('description').' : '.$i->get_value('cn');
            } elsif ( $objectClass eq 'dhcpPool' ) {
                $block = '  '; 
                push @dhcp_conf, $left.'pool { #Pool: '.$i->get_value('cn');
                push @dhcp_conf, $left.'  range '.$i->get_value('dhcpRange').';';
            } elsif ( $objectClass eq 'dhcpHost' ) {
                $block = '  ';
                push @dhcp_conf, $left.'host '.$i->get_value('cn').' {';
                push @dhcp_conf, $left.$block.'hardware '.$i->get_value('dhcpHWAddress').';';
            }
        }
        foreach my $statement ($i->get_value('dhcpStatements')){
            if( $statement =~ /\}\s*$/ ) {
               push @dhcp_conf, $left.$block.$statement;
            } else {
               push @dhcp_conf, $left.$block.$statement.';';
            }
        }
        foreach my $option    ($i->get_value('dhcpOption')){
            push @dhcp_conf, $left.$block.'option '.$option.';';
        }
        push @dhcp_conf, $this->create_dhcp_setup_rec($dn,$deepth);
        if( $block ne "" ) {
            push @dhcp_conf, $left.'}';
        }
    }


    return @dhcp_conf;
}


sub attributes {

    use Data::Dumper;

    my $this      = shift;
    my $mode      = shift; #show entries: mode="output" OR change entries: mode="change"
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},'LDAP','DHCP'))->data();
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $ldap      = $this->{"ldap"};
    my $mesg      = "";

    my $html="";
    my $errs=""; #errormessage
    my $mesg=""; #dummy for the object returned by ldap->...
    my $searchdn=$cgi->param("searchdn");

    my $utf8val; #holds UTF8 value
   
    my $result = $ldap->search(base=>"$searchdn",scope=>"base",filter=>"objectclass=*");
    #search for the attributes of the dn
    if ($result->code != 0) {
        $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs,$this->{"url"});
        $ldap->unbind;   # take down session
        exit;
    }
    my $entry = $result->entry(0); #object found in search - contains all attributes
    my $value;
    my $changes=0;
    my %deletes=();
    
    if ($mode eq "change") {
        $html .=              $cgi->Tr({},$cgi->td({-colspan=>"5"},"&nbsp;"));
    }
    
    $searchdn=~/[\w=]+,(.+)/; #extract the old search DN, from the last query
    my $link = $confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab";
    my $link2= $link."&act=edit&searchdn=".uri_escape($searchdn)."&master=attributes";

    $link.= "&searchdn=".uri_escape($1);
    
    #to omit javascript history.back, the links are generated

    foreach my $attr ($entry->attributes) {
    #parse all attributes
        my $i=0; #value counter
        my $act_pos=0;
        my @values=(); #values of the attributes
        my @newvalues=(); #values from the form
        foreach $utf8val ($entry->get_value($attr)) {
            #parse all values of the attribute
            $value = $utf8val;
            if ($mode eq "output") {
                #show all attributes and their values
                $html .=        $cgi->start_Tr({});
                #show only we realy need
                if ( $attr !~/^dhcp/ && $attr !~/^cn/ ) {
                    $html .= $cgi->td({},
                                 $cgi->hidden(-name=>"$attr$act_pos", -value=>"$value"),
                                 $cgi->hidden(-name=>"DEL_--BINARY_$act_pos--:$attr",-value=>"N")
                             );
                } else {
                        $html .=          $cgi->td({-align=>"left",-valign=>"middle"},$message->{"$attr"} ? $message->{"$attr"} : $attr);
                        if (&check_bin($utf8val,$attr)) {
                            #binary value => no delete!
                            $html .=        $cgi->td({},
                                         "<input type=\"text\" name=\"$attr$act_pos\" size=\"50\" value='--BINARY_$act_pos--' readonly>",
                                         $cgi->hidden(-name=>"DEL_--BINARY_$act_pos--:$attr",-value=>"N")
                                         );
                        } elsif ( $attr !~/^dhcpStatements/ && $attr !~/^dhcpOption/ ) {
                            $html .=        $cgi->td({},
                                         "<input type=\"text\" name=\"$attr$act_pos\" size=\"50\" value='$value' readonly>",
                                         $cgi->hidden(-name=>"DEL_--BINARY_$act_pos--:$attr",-value=>"N")
                                         );
                        } else {
                            $value =~ s/'/\\'/g; #escape single-quotes
                            my $delval="DEL_$value:$attr";
                            #$delval=~s/'/\\'/g;
                            $html .=        $cgi->td({},
                                         "<input type=\"text\" name=\"$attr$act_pos\" size=\"50\" value='$value'>",
                                         #unfortunately CGI produces "" around each argument so we cant escape " and cant use CGI neither
                                         #$cgi->checkbox(-name=>$delval,-value=>"Y",-label=>$message->{"delete"})
                                         "<input type=\"checkbox\" name='$delval' value=\"Y\" label=\"".$message->{"delete"}."\">".$message->{"delete"}
                                         );
                        }
                }
                $html .=        $cgi->end_Tr;
            } elsif ($mode eq "change") {
                #change values
                if ((defined $cgi->param("DEL_$value:$attr")) and ($cgi->param("DEL_$value:$attr") eq "Y")) {
                    #check whether this attribute and value should be deleted
                    if ($deletes{$attr}) {
                        push @{$deletes{$attr}},$utf8val;
                    } else {
                        $deletes{$attr}=[$utf8val];
                    }
                    $html .=        $cgi->start_Tr({});
                    $html .=          $cgi->td({-align=>"left"},$attr.":");
                    $html .=          $cgi->td({-align=>"left"},$value);
                    $html .=          $cgi->td({-align=>"center",-colspan=>"2"},$message->{"ldap_deleted"});
                    $html .=          $cgi->td({-align=>"center"},$message->{"succ_completed"});
                    $html .=        $cgi->end_Tr;
                    $changes++; #did anything change?
                    $i--;
                } else {
                    #otherwise put old and possibly new values of the attribute in arrays
                    push @values,$utf8val;
                    push @newvalues,$cgi->param($attr.$act_pos);
                }
            }
            $i++; #value counter
            $act_pos++;
        } #end foreach value of one attribute

        if ($mode eq "change") {
            @values=sort(@values); 
            @newvalues=sort(@newvalues);
            for (my $j=0; $j<$i; $j++) { 
                #foreach value of the attribute
                $utf8val = $value = $values[$j];
                if (($value ne $newvalues[$j]) and (!&check_bin($values[$j],$attr))) {
                    #compare sorted arrays exclude binary fields
                    if ($i == 1) {
                        #ldap replace for attributes with only one value
                        $utf8val=$newvalues[$j];
                        $mesg=$ldap->modify($entry, replace=>{$attr=>$utf8val});
                        #then create the attribute again with its new value
                        if ($mesg->code != 0) {
                            $errs = "modify (replace) $message->{failed}\n\n";
                            $errs .= &LDAPerror($mesg);
                            $this->display_error($errs,$link2);
                            $ldap->unbind;   # take down session
                            exit;
                        }
                    } else {
                        #ldap delete and add for multiple values
                        $mesg=$ldap->modify($entry, delete=>{$attr=>$utf8val});
                        #first delete the attribute and value
                        if ($mesg->code != 0) {
                            $errs = "modify (delete) $message->{failed}\n\n";
                            $errs .= &LDAPerror($mesg);
                            $this->display_error($errs,$link2);
                            $ldap->unbind;   # take down session
                            exit;
                        }
                        $utf8val = $newvalues[$j];
                        $mesg=$ldap->modify($entry, add=>{$attr=>$utf8val});
                        #then create the attribute again with its new value
                        if ($mesg->code != 0) {
                            $errs = "modify (add) $message->{failed}\n\n";
                            $errs .= &LDAPerror($mesg);
                            $this->display_error($errs,$link2);
                            $ldap->unbind;   # take down session
                            exit;
                        }
                    }
                    $html .=        $cgi->start_Tr({});
                    $html .=          $cgi->td({-align=>"left"},$attr.":");
                    $html .=          $cgi->td({-align=>"left"},$value);
                    $html .=          $cgi->td({-align=>"center"},"=>");
                    $html .=          $cgi->td({-align=>"left"},$newvalues[$j]);
                    $html .=          $cgi->td({-align=>"center"},$message->{"succ_completed"});
                    $html .=        $cgi->end_Tr;
                    $changes++; #did anything change?
                }
            }
        }
    } #end foreach attribute

    if ($mode eq "output") {
        #a new valid attribute with a value can be entered here
        $html .=            $cgi->start_Tr({});
        $html .=            $cgi->td({-align=>"left"}, $message->{"new"}.'&nbsp;&nbsp;&nbsp;'.$cgi->a({-href=>"javascript:void(0);",
                                              -onclick=>"$JavaScript; open_help_win('$confParam->{cgi_path}/dhcp_options.pl?sessionID=$sessionID')"},
                                               $cgi->img({-border=>0,
                                                        -src=>"$confParam->{image_path}/help1.gif",
                                                        -width=>18,
                                                        -height=>18,
                                                        -alt=>"Help"
                                                        })
                                               ).'&nbsp;:');
#        $html .=            $cgi->td({-align=>"left",-valign=>"middle"},"<input type=\"text\" name=\"ADDITIONALATTR\" size=\"15\" value=\"\">");
        $html .=            $cgi->td({-align=>"left",-valign=>"middle"},"<input type=\"text\" name=\"ADDITIONALVALUE\" size=\"50\" value=\"\">");
        $html .=            $cgi->end_Tr({});
        $html .=            $cgi->Tr({},
                         $cgi->td({},"&nbsp;"),
                         $cgi->td({-align=>"left"},
#                          $cgi->button(-class=>"button",-value=>$message->{"ldap_tree"},-onClick=>"location.href='$link'"),
                          $cgi->reset(-class=>"button",-name=>$message->{"reset"}),
                          $cgi->submit(-class=>"button",-name=>"subattr",-value=>$message->{"ldap_submit_attrib"}),
                          $cgi->hidden(-name=>"searchdn",-value=>"$searchdn")
                          )
                     );
    } elsif ($mode eq "change") {
        if ($changes>0 and values(%deletes)) {
            #marked values are going to be deleted
            $mesg=$ldap->modify($entry, delete=>\%deletes);
            if ($mesg->code != 0) {
                $errs = "modify (delete) $message->{failed}\n\n";
                $errs .= &LDAPerror($mesg);
                $this->display_error($errs,$link2);
                $ldap->unbind;   # take down session
                exit;
            }
        }
        if ((defined $cgi->param("ADDITIONALVALUE")) and ($cgi->param("ADDITIONALVALUE") ne "")) {
            my %dhcp_options_h = ();
            open (FILE,"<".$confParam->{lib_path}."/dhcp_options.lib") or die "no libary file";
            while (<FILE>) {
                /([-\w]+)\s+(\w+)/;
                $dhcp_options_h{$1} = "$2";
            }
            close FILE;
            #the additional values from above are going to be added
            $utf8val = $cgi->param("ADDITIONALVALUE");
            $utf8val =~ /([-\w]+)/;
            if(! defined $dhcp_options_h{$1} || $dhcp_options_h{$1} eq "" ){
                $dhcp_options_h{$1} = 'dhcpOption';
            }    
            $mesg=$ldap->modify($entry, add=>{$dhcp_options_h{$1}=>$utf8val});
            if ($mesg->code != 0) {
                $errs = "modify (add) $message->{failed}\n\n";
                $errs .= &LDAPerror($mesg);
                $this->display_error($errs,$link2);
                $ldap->unbind;   # take down session
                exit;
            }
            $html .=          $cgi->start_Tr({});
            $html .=            $cgi->td({-align=>"left"},$dhcp_options_h{$1}.":");
            $html .=            $cgi->td({-align=>"left"},$cgi->param("ADDITIONALVALUE"));
            $html .=            $cgi->td({-align=>"center",colspan=>"2"},$message->{"ldap_added"});
            $html .=            $cgi->td({-align=>"center"},$message->{"succ_completed"});
            $html .=          $cgi->end_Tr({});
            $changes++;
        }
        if ($changes==0) {
            #produce an output even if there were no changes
            $html .=          $cgi->start_Tr({});
            $html .=             $cgi->td({-align=>"center",-colspan=>"5"},$message->{"ldap_nochange"});
            $html .=          $cgi->end_Tr({});
        }
    
        $html .=    $cgi->Tr({},$cgi->td({-colspan=>"5"},"&nbsp;"));
        $html .=    $cgi->start_Tr({});
        $html .=    $cgi->td({-align=>"center",-colspan=>"5"},
                        $cgi->button(-class=>"button",-value=>$message->{"dhcp"},-onClick=>"location.href='$link'"),
                        $cgi->button(-class=>"button",-value=>$message->{"ldap_back"},-onClick=>"location.href='$link2'")
                    );
        $html .=    $cgi->end_Tr({});
    }
    
    return $html;
}

sub check_bin {

    if ((defined $_[1]) and ($_[1]=~/.+;binary/)) {
        #attribute name indicates binary value
        return 1;
    #String contains a character between 1 and 31 except HT,LF,VT,FF,CR 
    #} elsif ($_[0]=~/([\x01-\x1f])/ and $1=~/[^\x09-\x0d]/) {
    } elsif ($_[0]=~/([\x01-\x1f])/) {
        return 1;
    } else {
        return 0;
    }
}

sub entrycomp {
    $a->dn cmp $b->dn;
}

sub create_dns_setup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= $this->{"ldap"};
    my $mesg = "";

    my $base = "o=DNS,".$confParam->{baseDN};
    my $mesg = $ldap->search(    base    => $base,
                scope    => 'sub',
                filter    => "objectclass=dNSZone" );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    $ldap->unbind;
        exit;
    }

    my $nsdn = $mesg->as_struct;

    my $namedtempl = "/etc/named.conf";
    my @namedconf = ();
    my $lastgen = localtime;
    my @pfmydest;

    push @namedconf,
    "//
// THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
// BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
//
// to change some options manually, edit $namedtempl
//
// Last generated at \"$lastgen\"
//
";

    # deassemble LDAP and reassemble DNS zonefiles

    $mesg = $ldap->search(    base => $base,
                scope => 'sub',
                filter => '(&(relativeDomainName=@)(objectclass=dNSZone))'
                );
    if( $mesg->code != 0 ){
        my $text = "LDAP: search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }elsif( $mesg->count == 0) {
        my $text = "search failed. No DNS zones found\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }    
    foreach my $zone_entry ( $mesg->entries ) {
    my $zone = $zone_entry->get_value('zoneName');
    my $soaent = $nsdn->{$zone_entry->dn};

    # disassemble NS Forward maps
    if( ! ( $zone =~ /IN-ADDR.ARPA/i ) ) {

        if( $zone ne $confParam->{mailDomain} ) {
        push @pfmydest,$zone;
        }

        my $zonefile = "/var/named/$zone.zone";
        my @soa = split(/ /,$soaent->{soarecord}->[0]);
        my @outf = ();
        push @outf,
        ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
        push @outf, "\$TTL $soaent->{dnsttl}->[0]";
        push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
        push @outf, "\t\t\t".$soa[2];
        push @outf, "\t\t\t".$soa[3];
        push @outf, "\t\t\t".$soa[4];
        push @outf, "\t\t\t".$soa[5];
        push @outf, "\t\t\t".$soa[6].")";
        foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
        push @outf, "\tIN\tNS\t".$nsrr;
        }
        foreach my $arr ( @{$soaent->{arecord}} ) {
        push @outf, "\tIN\tA\t".$arr;
        }
        foreach my $mxrr ( @{$soaent->{mxrecord}} ) {
        push @outf, "\tIN\tMX\t".$mxrr;
        }

        # now disassemble host entries
        foreach my $dn (sort ( keys %$nsdn )) {
        if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
            my $hent = $nsdn->{$dn};
            foreach my $hzone ( @{$hent->{zonename}} ) {
            if( $hzone eq $zone ) {
                push @outf, $hent->{relativedomainname}->[0]."\tIN\tA\t".$hent->{arecord}->[0];
            }
            }
        }
        }
        my $ret = $session->suad_save(\@outf, $zonefile);
        if(  $ret->code() ne "OK" ) {
        $this->display_error($ret->code(),
                     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
        exit;
        }

        # add zone to named.conf
        push @namedconf, "zone \"$zone\" {";
        push @namedconf, "\ttype master;";
        push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            if( $zone eq $confParam->{mailDomain} ) {
#                push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
#            }
# E SCHULSERVER 
        push @namedconf, "};";
        
    } else { # disassemble NS Reverse maps
        my $zonefile = "/var/named/$zone.zone";
        my @soa = split(/ /,$soaent->{soarecord}->[0]);
        my @outf = ();
        push @outf,
        ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
        push @outf, "\$TTL $soaent->{dnsttl}->[0]";
        push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
        push @outf, "\t\t\t".$soa[2];
        push @outf, "\t\t\t".$soa[3];
        push @outf, "\t\t\t".$soa[4];
        push @outf, "\t\t\t".$soa[5];
        push @outf, "\t\t\t".$soa[6].")";
        foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
        push @outf, "\tIN\tNS\t".$nsrr;
        }

        # now disassemble host entries
        foreach my $dn ( sort ( keys %$nsdn )) {
        if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
            my $hent = $nsdn->{$dn};
            foreach my $hzone ( @{$hent->{zonename}} ) {
            if( $hzone eq $zone ) {
                foreach my $arr ( @{$hent->{ptrrecord}} ) {
                push @outf, $hent->{relativedomainname}->[0]."\tIN\tPTR\t".$arr;
                }
            }
            }
        }
        }
        my $ret = $session->suad_save(\@outf, $zonefile);
        if(  $ret->code() ne "OK" ) {
        $this->display_error($ret->code(),
                     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
        exit;
        }

        # add zone to named.conf
        push @namedconf, "zone \"$zone\" {";
        push @namedconf, "\ttype master;";
        push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
# E SCHULSERVER 
        push @namedconf, "};";
    }
    }

    my $errors = "";

    my $ret = $session->suad_save(\@namedconf, "/etc/named.d/ldap.dump");
    if(  $ret->code() ne "OK" ) {
    $errors .= "named: ".$ret->code()."\n";
    #$this->display_error($ret->code(),
    #             $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
    #exit;
    }
    
    $ret = $session->suad_exec("nsreload");
    if(  $ret->code() ne "OK" ) {
    $errors .= "named: ".$ret->code()."\n";
    #$this->display_error($ret->code(),
    #             $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
    #exit;
    }

    
    $ret = $session->suad_exec("fetchdreload");
    if(  $ret->code() ne "OK" ) {
    $errors .= "fetchd: ".$ret->code()."\n";
    #$this->display_error($ret->code(),
    #             $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
    #exit;
    }
    return $errors;
}

1;
