#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: dns_deletehost.pm,v 1.1.1.1 2003/03/25 10:31:39 ray Exp $
#
package dns_deletehost;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP qw( :all );
use LDAPUtils;
use Display;
use Session;
use POSIX;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $delhost = $cgi->param('delhost');
    if( (defined $delhost) && ($delhost ne "") ){
	$this->del_host();
    }else{
        $this->startup();
    }
}

sub startup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"DNS_HOST"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != LDAP_SUCCESS) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $maildomain;
    my $ldap_result = $ldap->search( 'base' => "uid=cyrus,".$confParam->{'baseDN'},
				    'scope' => "base",
				    'attrs' => ['maildomain'],
				    'filter' => "(objectClass=*)" 
				   );
    if( $ldap_result->code != LDAP_SUCCESS ){
        my $text = "search failed\n".&LDAPerror($ldap_result);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    } elsif ($ldap_result->count == 0) {
        my $text = "search failed\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }else{
	$maildomain = $ldap_result->entry(0)->get_value('maildomain');
    }

    my $base = "o=DNS,".$confParam->{baseDN};
    my @exclude_hosts = ();

    $ldap_result = $ldap->search( base => "relativeDomainName=\@,$base",
				     scope => 'base',
				     attrs => [ 'mXRecord', 'nSRecord' ],
				     filter => "(objectclass=*)" );
    if ($ldap_result->code != LDAP_SUCCESS){
        my $text = "search failed\n".&LDAPerror($ldap_result);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }elsif( $ldap_result->count == 0) {
        my $text = "search failed\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }else{
	my $entry = $ldap_result->entry(0);
	my @mx_recs = $entry->get_value("mXRecord");
	foreach my $mx_rec (@mx_recs) {
	    if( $mx_rec =~ /^[0-9]+\s(.*)$/ ){
		push @exclude_hosts, $1;
	    }
	}
	my @ns_recs = $entry->get_value("nSRecord");
	push( @exclude_hosts, (@ns_recs) );
    }
    my %exclude_hosts = ();
    foreach( @exclude_hosts ){
	$exclude_hosts{$_} = 1;
#	print STDERR "exclude: $_ \n";
    }

    $mesg = $ldap->search( base => $base,
			   scope => 'one',
			   attrs => [ 'relativeDomainName' ],
			   filter => "(&(objectclass=dNSZone)(aRecord=*))" );
    if( $mesg->code != LDAP_SUCCESS ){
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    } elsif ($mesg->count == 0) {
        my $text = "search failed\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    $ldap->unbind;
    my @entries = $mesg->entries();
    my @hosts = ();
    foreach my $entry (@entries){
	my $rel_dom = $entry->get_value('relativeDomainName');
	if( ($rel_dom ne "@" ) && (! ( defined $exclude_hosts{$rel_dom.".".$maildomain."."} ) ) ){
	    push(@hosts, $rel_dom);
	}
    }
    if(scalar(@hosts)==0){
	$this->display_msg($message->{'dns_no_host_defined'},
			   $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID&tab=$tab",3);
	exit;
    }
    @hosts = sort(@hosts);
    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=	$cgi->start_Tr();
    $html .=	    $cgi->start_td();
    $html .=		$cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=		    $cgi->start_Tr();
    $html .=			    $this->help_th(1, $message->{'dns_deltitle'}, "DNS_DEL_HOST");
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td({-class=>'AdminSubHeaderRed', -colspan=>2});
    $html .=			     $message->{'dns_subtitle'};
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td({-align => "center", 
						-width => "50%"});
    $html .=			$cgi->start_form(-action=>$confParam->{cgi_path}."/dns_deletehost.pl");
    $html .=			    $cgi->scrolling_list(-name => "hostname",
							 -values => \@hosts,
							 -size => 10
							);
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td({-align => "center"});
    $html .=			    $cgi->submit(-class=>"button",-name => "delhost",
						 -value => $message->{'dns_del'});
    $html .=			    $cgi->hidden(-name => "sessionID",
						 -value => $sessionID);
    $html .=			    $cgi->hidden(-name => "tab",
						 -value => $tab);
    $html .=			    $cgi->hidden(-name => "stab",
						 -value => $stab);
    $html .=			$cgi->end_form();
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		$cgi->end_table();
    $html .=	    $cgi->end_td();
    $html .=	$cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
    exit;
}

sub del_host{
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $hostname = $cgi->param('hostname');
    my $really = $cgi->param('really');
    my @host_address = ();

    if( (! defined $hostname) || ($hostname eq "") ){
	$this->display_error($message->{'dns_no_host_selected'},
			     $confParam->{'cgi_path'}."/dns_deletehost.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
	exit;
    }

    if( (defined $really) && ($really ne "") ){
	my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server",
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
	my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			      $confParam->{baseDN}, password=>$confParam->{passwd});
	if($mesg->code != 0) {
	    my $text = "bind failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}

	my $maildomain;
	my $ldap_result = $ldap->search('base' => "uid=cyrus,".$confParam->{'baseDN'},
					'scope' => "base",
					'attrs' => ['maildomain'],
					'filter' => "(objectClass=*)" 
				       );
	if ($ldap_result->count == 0) {
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}else{
	    $maildomain = $ldap_result->entry(0)->get_value('maildomain');
	}
	$ldap_result = $ldap->search( 'base' => "o=DNS,".$confParam->{'baseDN'},
				      'scope' => 'one',
				      'attrs' => ['pTRRecord'],
				      'filter' => "(&(objectClass=dNSZone)(pTRRecord=$hostname.$maildomain.))"
				    );
	if($ldap_result->code() < 0){
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}elsif( $ldap_result->count() == 0 ){
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}
	my @ptr_rec = $ldap_result->entry(0)->get_value('pTRRecord');
	my $rev_dn = $ldap_result->entry(0)->dn();
	if( scalar(@ptr_rec) > 1 ){
	    # There are other host using the same IP-Address
	    $mesg = $ldap->modify($rev_dn,
			  "delete" => {"pTRRecord" => "$hostname.$maildomain."}
			 );
	}else{
	    # This was the last host using the IP-Address delete the entry for reverse lookup
	    $mesg = $ldap->delete($rev_dn);
	}
	
	if($mesg->code != 0) {
	    my $text = "LDAP: delete failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}
	$mesg = $ldap->delete("relativeDomainName=$hostname,o=DNS,".$confParam->{'baseDN'});
	if($mesg->code != 0) {
	    my $text = "LDAP: delete failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}

	#    print STDERR "successfully updated\n";
	my $base = "relativeDomainName=@,o=DNS,".$confParam->{baseDN};
	
	$mesg = $ldap->search( base => $base,
			       scope => 'base',
			       #attrs => [ 'zoneName' ],
			       filter => "objectclass=dNSZone" );
	if( $mesg->code != LDAP_SUCCESS){
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}elsif( $mesg->count == 0) {
	    my $text = "search failed\n";
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}

	my $zone_entry = $mesg->entry(0);
	
	# get out SOA and increase serial number
	my $soa = $zone_entry->get_value("sOARecord");
	my @soa = split(/ /,$soa);
	my $timestamp = $soa[2];
	my $sernr  = substr($timestamp, 8, 2);
	my $timenr = substr($timestamp, 0, 8);
	my $timenow = strftime("%Y%m%d",localtime);
	
	if( $timenr eq $timenow ) {
	    $sernr++;
	} else {
	    $timenr = $timenow;
	}
	
	$zone_entry->replace( sOARecord =>
			      $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".
			      $soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6]);
	
	$mesg = $zone_entry->update( $ldap );
	if($mesg->code != 0) {
	    my $text = "update failed\n".&LDAPerror($mesg);
	    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    $ldap->unbind;
	    exit;
	}
	
	$ldap->unbind;
	$this->display_msg($message->{'dns_del_done'}, 
			   $confParam->{'cgi_path'}."/dns_deletehost.pl?sessionID=$sessionID".
			   "&tab=$tab&stab=$stab",3);
	exit; 
    }else{
        #Ask for confirmation
        my $html = "";
        $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
        $html .= $cgi->start_Tr().
                $cgi->start_td().
                $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"}).
                    $cgi->start_Tr().
                    $cgi->td({-class=>'AdminHeader', -colspan=>2 },
                             "<b>$message->{dns_del_really}</b>"
                        ).
                    $cgi->end_Tr().
                    $cgi->start_Tr().
                    $cgi->start_td({-class=>'AdminSubHeaderRed', -colspan=>2, -align=>"center"});
                        $html .= $message->{'dns_del_should'}." ".$hostname." ".$message->{'dns_del_be_deleted'}.
                    $cgi->end_td().
                    $cgi->end_Tr().   
                    $cgi->start_Tr().
                    $cgi->td({-align=>"center"},
                        $cgi->start_form(-action=>$confParam->{cgi_path}."/dns_deletehost.pl").
                            $cgi->submit(-class=>"button",-name=>$message->{'no'}).
                            $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
                            $cgi->hidden(-name=>'tab', -value=>$tab).
                            $cgi->hidden(-name=>'stab', -value=>$stab).
                        $cgi->end_form()    
                    ).
                    $cgi->td({-align=>"center"},
                        $cgi->start_form(-action=>$confParam->{cgi_path}."/dns_deletehost.pl").
                            $cgi->submit(-class=>"button",-name=>$message->{'yes'}).
                            $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
                            $cgi->hidden(-name=>'tab', -value=>$tab).
                            $cgi->hidden(-name=>'stab', -value=>$stab).
                            $cgi->hidden(-name=>'really', -value=>1).
                            $cgi->hidden(-name=>'delhost', -value=>1).
                            $cgi->hidden(-name=>'hostname', -value=>$hostname).
                        $cgi->end_form()    
                    ).
                    $cgi->end_Tr().
                $cgi->end_table().
                $cgi->end_td().
            $cgi->end_Tr().    
            $cgi->end_table();
        
        $this->SUPER::display($html);
    }
    
}
1;

