#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: dns_managehost.pm,v 1.3 2003/09/29 06:45:28 varkoly Exp $
#
package dns_managehost;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP qw( :all );
use LDAPUtils;
use URI::Escape;
use Display;
use Session;
use POSIX;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $addhost = $cgi->param('addhost');
    if( (defined $addhost) && ($addhost ne "") ){
		$this->add_host();
    }else{
        $this->startup();
    }
}

sub startup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"DNS_HOST","MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('domain');
    my $hostname = $cgi->param('hostname') || "";
	$domain     =~ s/\*//g;

	if(! defined $domain || $domain eq '' ){
		$this->display_error($message->{select_domain},"$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",3);
		exit;
	}

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != LDAP_SUCCESS) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $base = "o=DNS,".$confParam->{baseDN};

    $mesg = $ldap->search( base => $base,
			   scope => 'sub',
			   attrs => [ 'zoneName' ],
			   filter => "objectclass=dNSZone" );

    my $net_class = 0;
    my $net_address = 0;
	foreach my $i ($mesg->entries) {
		foreach my $zn ( $i->get_value('zoneName') ) {
		if( ! ( $zn =~ /IN-ADDR\.ARPA/i ) ) {
			next;
		}else{
			$zn =~ /^((\d+\.)+)IN-ADDR\.ARPA/i ;
			if(defined $1){
				my @ip_parts = split /\./, $1;
					$net_class = scalar(@ip_parts);
					$net_address = join( '.', reverse(@ip_parts) );
				}else{
					print STDERR "Something is wrong here\n";
				}
			}
		}
	}

	my @net_address = split /\./, $net_address;
    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=	$cgi->start_Tr();
    $html .=	    $cgi->start_td();
    $html .=		$cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=		    $cgi->start_Tr();
    $html .=			$this->help_th(2, $message->{'dns_addtitle'}, "DNS_MANAGE_HOST");
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->td({-class=>'AdminSubHeaderRed', -colspan => 2}, $message->{'dns_subtitle'});
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td({-width => "50%"});
    $html .=			$cgi->start_form(-action=>$confParam->{cgi_path}."/dns_managehost.pl");
    $html .=			    $message->{'dns_hostname'};
    $html .=			$cgi->end_td();
    $html .=			$cgi->start_td();
    $html .=			    $cgi->textfield(-name => "hostname",
						    -default => $hostname,
						    -size => 20,
						    -maxlength => 255
						    );
    $html .=			    " .$domain";
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td();
    $html .=			    $message->{'dns_ipaddress'};
    $html .=			$cgi->end_td();
    $html .=			$cgi->start_td();
    	for( my $i=0; $i < 4; $i++ ){
		if( $i >= $net_class ) {
			$html .= $cgi->textfield(-name => "ip_part_$i",
						 -size => 3,
						 -maxlength => 3);
			$html .= ( $i < 3 ? "&nbsp;<b>.</b>&nbsp;" : "" );
		} else {
			$html .= $cgi->textfield(-name => "ip_part_$i",
						 -size => 3,
						 -value=> $net_address[$i]);
			$html .= "&nbsp;<b>.</b>&nbsp;";
		}
    	}
    $html .=			    $cgi->hidden(-name => "parts",
						 -value => 4-$net_class);
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		    $cgi->start_Tr();
    $html .=			$cgi->start_td({-align => "center", -colspan => 2});
    $html .=			    $cgi->submit(-class=>"button",-name => "addhost",
						 -value => $message->{'dns_add'});
    $html .=			    $cgi->hidden(-name => "sessionID", -value => $sessionID);
    $html .=			    $cgi->hidden(-name => "domain", -value => $domain);
    $html .=			    $cgi->hidden(-name => "tab", -value => $tab);
    $html .=			    $cgi->hidden(-name => "stab", -value => $stab);
    $html .=			$cgi->end_form();
    $html .=			$cgi->end_td();
    $html .=		    $cgi->end_Tr();
    $html .=		$cgi->end_table();
    $html .=	    $cgi->end_td();
    $html .=	$cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

sub add_host{
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('domain');
	$domain     =~ s/\*//g;

    my $hostname = $cgi->param('hostname');
    my $ip_parts = $cgi->param('parts');
    my @host_address 	  = ();
    my @full_host_address = ();
    my @wanted_net	  = ();
    my $in_own_net	  = 1;


    if( $hostname !~ /^[A-Za-z\-\d]+$/ ){
	my $uri = "$confParam->{cgi_path}/dns_managehost.pl?sessionID=$sessionID&tab=$tab&stab=$stab&".
		    "hostname=".uri_escape($hostname);
        $this->display_error($message->{'dns_invalidhostname'},$uri);
        exit;
    }

# --print STDERR "<$ip_parts>\n";

    for( my $i=3; $i >= 0; $i--){
	push @full_host_address, $cgi->param("ip_part_$i");
	if( $i >= 4-$ip_parts ) {
		push @host_address, $cgi->param("ip_part_$i");
	} else {
		push @wanted_net, $cgi->param("ip_part_$i");
	}
    }
    @full_host_address	= reverse(@full_host_address);
    @host_address	= reverse(@host_address);
    @wanted_net		= reverse(@wanted_net);

    
    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
		$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
   	 if($mesg->code != LDAP_SUCCESS) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }


    # Searching for Revers Zone Entry
    my $base = 'o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base		=> $base,
			   filter	=> "(&(relativeDomainName=@)(zoneName=*.IN-ADDR.ARPA))"
			);
    if( $mesg->code != LDAP_SUCCESS){
        my $text = "LDAP: search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }elsif( $mesg->count == 0) {
        my $text = "search failed: there is no revers zone\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $rev_zone_entry	= $mesg->entry(0);
    my @zones		= $mesg->entry(0)->get_value('zoneName');
    my $net_class	= 0;
    my $net_address	= "";
    my $rev_net		= "";
    foreach my $zn ( @zones ) {
		if( ! ( $zn =~ /IN-ADDR\.ARPA/i ) ) {
		    next;
		}else{
		    $zn =~ /^((\d+\.)+)IN-ADDR\.ARPA/i ;
		    if(defined $1){
				my @ip_parts = split /\./, $1;
				$net_class = scalar(@ip_parts);
				$net_address = join( ".", reverse(@ip_parts) );
				$rev_net = join( ".", @ip_parts );
		    }else{
				print STDERR "Something is wrong here\n";
		    }
		}
    }
    # is the wanted net in our DNS area ?
    if($net_address ne join( ".", @wanted_net)){
	$in_own_net = 0;
    } 

    # Searching for Zone Entry
    my $base = 'ou='.$domain.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search(	base	=> $base,
				scope	=> 'one',
				filter	=> 'relativeDomainName=@'
			);
    if( $mesg->code != LDAP_SUCCESS){
        my $text = "LDAP: search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		$ldap->unbind;
        exit;
    }elsif( $mesg->count == 0) {
        my $text = "search failed\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		$ldap->unbind;
        exit;
    }

    my $zone_entry = $mesg->entry(0);

    # check for valid numbers in @full_host_address
    foreach my $part (@full_host_address){
		if(( $part !~ /\d+/ ) || ( $part < 0 ) || ( $part > 254 ) ){
		    my $uri = "$confParam->{cgi_path}/dns_managehost.pl?sessionID=$sessionID&tab=$tab&stab=$stab&".
				"hostname=".uri_escape($hostname);
		    $this->display_error($message->{'dns_invalidip'}, $uri );
		    $ldap->unbind;
		    exit;
		}
    }

    # check if this host entry not exist soon
    $base = 'relativeDomainName='.$hostname.','.$base;
    my $ldap_result = $ldap->search(	base	=> $base,
					scope	=> 'base',
					attrs	=> ['aRecord'],
					filter	=> '(objectClass=*)'
				);
    if( $ldap_result->code < 0 ){
        my $text = "search failed\n".&LDAPerror($ldap_result);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		$ldap->unbind;
        exit;
    }elsif( $ldap_result->count > 0 ){
		# There is already a host by this name
		my $uri = "$confParam->{cgi_path}/dns_managehost.pl?sessionID=$sessionID&tab=$tab&stab=$stab&".
		    "hostname=".uri_escape($hostname)."&domain=$domain";
		$this->display_error($message->{'dns_host_already_defined'},$uri);
		$ldap->unbind;
		exit;
    }
    my $aRecord = join(".", @full_host_address );
    my $hostentry = Net::LDAP::Entry->new();
    $hostentry->dn( $base );
    $hostentry->add( 	aRecord			=> $aRecord,
		     	relativeDomainName	=> $hostname,
		     	dNSTTL			=> '172800',
			zoneName		=> $domain,
			objectClass		=> ['top', 'dNSZone']
		   );
    $mesg = $ldap->add($hostentry);
    if($mesg->code != 0) {
        my $text = "add failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		$ldap->unbind;
        exit;
    }

    if( $in_own_net ) {
	# we make the entry if it is in own Zone
	    my $rev_relDN .= join (".", reverse(@host_address) );
	    my $reverse_dn= 'relativeDomainName='.$rev_relDN.',ou='.$rev_net.'.IN-ADDR.ARPA,o=DNS,'.$confParam->{'baseDN'};
	    $ldap_result = $ldap->search(	base	=> $reverse_dn,
						scope	=> 'base',
						attrs	=> ['pTRRecord'],
						filter	=> '(objectClass=*)'
					   );
	    if( $ldap_result->code < 0 ){
		my $text = "search failed\n".&LDAPerror($ldap_result);
		$this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
			## try rollback
			$ldap->delete("relativeDomainName=$hostname,o=DNS,".$confParam->{'baseDN'});
			$ldap->unbind;
		exit;
	    }elsif( $ldap_result->count == 0 ){
		# The entry does not exist yet a new on has to be created
			my $reverse_entry = Net::LDAP::Entry->new();
			$reverse_entry->dn($reverse_dn);
			$reverse_entry->add(	relativeDomainName	=> $rev_relDN,
						pTRRecord		=> "$hostname.$domain.",
						dNSTTL			=> '172800',
						zoneName		=> $rev_net.'.IN-ADDR.ARPA',
						objectClass		=> ['top', 'dNSZone']
				    );
			$mesg = $ldap->add($reverse_entry);
			if($mesg->code != 0) {
				my $text = "add failed\n".&LDAPerror($mesg);
				$this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
				## try rollback
				$ldap->delete("relativeDomainName=$hostname,o=DNS,".$confParam->{'baseDN'});
				$ldap->unbind;
				exit;
			}
	    }else{
			# The entry is already defined just a new value to pTRRecord
			my $reverse_entry = Net::LDAP::Entry->new();
			$mesg = $ldap->modify( $reverse_dn, 
							"add" => { "pTRRecord" => "$hostname.$domain."}
					);
			if($mesg->code != LDAP_SUCCESS) {
			my $text = "modify failed\n".&LDAPerror($mesg);
			    $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
			    ## try rollback
			$ldap->delete("relativeDomainName=$hostname,o=DNS,".$confParam->{'baseDN'});
			    $ldap->unbind;
			    exit;
			}
	    }
	    # get out SOA for rev Zone and increase serial number
	    my $soa = $rev_zone_entry->get_value("sOARecord");
	    my @soa = split(/ /,$soa);
	    my $timestamp = $soa[2];
	    my $sernr  = substr($timestamp, 8, 2);
	    my $timenr = substr($timestamp, 0, 8);
	    my $timenow = strftime("%Y%m%d",localtime);

	    if( $timenr eq $timenow ) {
			$sernr++;
	    } else {
			$timenr = $timenow;
	    }

	    $rev_zone_entry->replace( sOARecord =>
				  $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".
				  $soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6]);
	    
	    $mesg = $rev_zone_entry->update( $ldap );
	    if($mesg->code != 0) {
		my $text = "update failed\n".&LDAPerror($mesg);
		$this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
			$ldap->unbind;
			exit;
	    }
    }

    # get out SOA for Zone and increase serial number
    my $soa = $zone_entry->get_value("sOARecord");
    my @soa = split(/ /,$soa);
    my $timestamp = $soa[2];
    my $sernr  = substr($timestamp, 8, 2);
    my $timenr = substr($timestamp, 0, 8);
    my $timenow = strftime("%Y%m%d",localtime);

    if( $timenr eq $timenow ) {
		$sernr++;
    } else {
		$timenr = $timenow;
    }

    $zone_entry->replace( sOARecord =>
			  $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".
			  $soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6]);
    
    $mesg = $zone_entry->update( $ldap );
    if($mesg->code != 0) {
        my $text = "update failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		$ldap->unbind;
		exit;
    }


#    print STDERR "successfully updated\n";
    
    $ldap->unbind;
    $this->display_msg($message->{'dns_add_done'}, 
		       $confParam->{'cgi_path'}."/dns_managehost.pl?sessionID=$sessionID".
				   "&tab=$tab&stab=$stab&domain=$domain",3);
    exit; 
    
}
1;

