#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: edit_resource.pm,v 1.3 2003/10/29 10:35:39 varkoly Exp $
#
package edit_resource;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP qw(:all);
use LDAPUtils;
use Display;
use Session;
use Filterlist;
use URI::Escape;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $new = $message->{"new"};

    my $save   = $cgi->param("save");
    my $delete = $cgi->param("delete_res");
    my $run = $cgi->param("run");

    if( defined $delete && $delete ne "" ) {
	#delete resource
	$this->delete;
    }
    elsif (defined $save && $save ne "" ) {
	#save new resource
	$this->save_resource;
    }
    elsif( defined $run && $run ne "" ) {
	#add new resource
	$this->add_resource;
    } else {
	$this->selection;
    }

}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub selection {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    my $url = "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver},port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }

    # find all resources
    $result = $ldap->search(base=>$confParam->{"baseDN"},
			    scope=>"one",
			    filter=>"(&(resourceName=*)(objectclass=SuSEResourceObject))");
    if($result->code != 0) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }

    my @entries=$result->entries;

    # now search for resources assigned to groups
    $result = $ldap->search(base=>$confParam->{"baseDN"},
			    scope=>"one",
			    filter=>"(&(resourceGroupMember=*)(objectclass=SuSEResourceGroupObject))",
			    attrs=> [ 'resourceGroupMember' , 'resourceGroupName' ]);
    if($result->code != 0) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }
    $ldap->unbind;

    my @gmembers = $result->entries;
    my $resource_href;

    foreach my $entry ( @gmembers ) {
	foreach my $attr ( $entry->get_value("resourceGroupMember") ) {
	    foreach my $rgn ( $entry->get_value("resourceGroupName") ) {
		push @{$resource_href->{$attr}}, $rgn;
		#print STDERR "ATTR=$rgn\n";
	    }
	}
    }

    my $all_res;
    foreach my $i (@entries) {
	my $e = $i->get_value("resourceName"); 
	push @$all_res, $e;
    }

    @$all_res = sort @$all_res if defined $all_res;

    my $html = "";

    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3,
				      -width=>"100%"});
    $html .=         $cgi->start_form();
    $html .=         $cgi->Tr( $this->help_th(1, $message->{"res_edit_title"}, "EDIT_RESOURCE") );
    $html .=         $cgi->Tr(
			      $cgi->td({-class=>'AdminSubHeader'},
				       $message->{res_del_howto})
			      );
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({-border=>0, -cellspacing=>0, -cellpadding=>3,
				-width=>"100%" });

    if( ! defined $all_res ) {
	$html .= $cgi->Tr(
			  $cgi->td(
				   $message->{res_no_entries}
				   )
			  );
    }
    
    foreach my $res ( @$all_res ) {
	my $td = "<b>$res</b>";
	my $isused;
	$html .= $cgi->start_Tr();
	if( defined $resource_href->{$res} ) {
	    $isused = 1;
	    $td .= " <i>($message->{res_used_by} ";
	    foreach my $usedby ( @{$resource_href->{$res}} ) {
		$td .= "$usedby, ";
	    }
	    $td =~ s/(^.*)\,/$1\)<\/i>/;
	}
	$html .= $cgi->start_td({-height => 25});
	$html .=    $td;
	$html .= $cgi->end_td();
	$html .= $cgi->start_td({-height => 25, -align => "center"});
	if( defined $isused ){ 
	    $html .=$cgi->img({src=>'/images/comment.gif', border=>0});
	}else{
	    my $url = $confParam->{'cgi_path'}."/edit_resource.pl?sessionID=$sessionID&tab=$tab&stab=$stab&".
			"delete_res=".uri_escape($res);
	    $html .= $cgi->a({href=> $url}); 
	    $html .= $cgi->img({src=>'/images/trashcan.gif', border=>0, alt => $message->{'delete'} });
	}
	$html .= $cgi->end_td();
	$html .= $cgi->end_Tr();
        $html .= $cgi->start_Tr();
        $html .=    $cgi->td({-colspan=>6 , -align=>'center'},[ 
			$cgi->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
		    ]);
	$html .= $cgi->end_Tr();  
    }
    
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();

    $html .=         $cgi->start_Tr();
    $html .=         $cgi->start_form();
    $html .=           $cgi->td({-align=>"center"},
                                #$cgi->submit(-class=>"button",-name=>"run",-value=>$message->{"edit"})." &nbsp;". 
                                $cgi->submit(-class=>"button",-name=>"run",-value=>$message->{"new"}).
                                $cgi->hidden(-name=>"sessionID", -value=>$sessionID).
                                $cgi->hidden(-name=>"tab", -value=>$tab).
                                $cgi->hidden(-name=>"stab", -value=>$stab).
				$cgi->end_form() );
    $html .=         $cgi->end_Tr();
    $html .=       $cgi->end_table();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


sub add_resource {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    my $filter    = $cgi->param("filter");
    $filter = LDAP_filter_escape($filter);

    my $url = "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != LDAP_SUCCESS) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }
    my $filterlist = Filterlist->new($cgi, $session, $ldap);

    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=	$cgi->start_Tr();
    $html .=	    $cgi->start_td();
    $html .=		$cgi->start_form();
    $html .=		    $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=			$cgi->Tr( $this->help_th(3, $message->{"res_edit_new"}, "EDIT_RESOURCE") );
    $html .=			$cgi->Tr(
				    $cgi->td({-colspan => 3},
					    "$message->{resource_name}",
					    $cgi->textfield(-name=>'new_res', -size=>50)
					    )
				    );
    $html .=			$cgi->start_Tr();
    $html .=			    $cgi->td({-colspan=>3, -align=>"center"},  
		                            $cgi->hr({-noshade => '1', -size => '1', -width=>'95%'})
		                        );
    $html .=			$cgi->end_Tr();
    $html .=			$cgi->start_Tr();
    $html .=			    $cgi->td({-colspan=>3, -align=>"left"},
		                            "<b>".$message->{'res_select_groups'}.":</b>");
    $html .=			$cgi->end_Tr();
    $html .=			$filterlist->make_filter_view({"textfield"    => "filter",
                                             "textlabel"    => $message->{'filter'},
                                             "listname"     => "res_grp",
                                             "listlabel_left" => $message->{'res_groups'},
                                             "buttonlabel"  => $message->{'apply_filter'},
                                             "nofilter-msg" => $message->{'set_filter_first'},
                                             "listfilter"   => "(&(objectclass=SuSEResourceGroupObject)(resourceGroupName=$filter))",
					     "listattrs"    => ['resourceGroupName'],
                                             "labelexp"     => ["\$\$ ","resourceGroupName"],
                                             "multiple"     => "MULTIPLE",
                                             "error-url"    => "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID",
                                             "width"        => "30",
                                             "height"       => "10",
                                             "charset"      => $this->{'charset'}
                                            });
    $html .=			$cgi->start_Tr();
    $html .=			    $cgi->td({-colspan=>3, -align=>"center"},
						$cgi->submit(-class=>"button",-name=>"save",-value=>$message->{"commit"}).
						$cgi->hidden(-name=>"run", -value=>$message->{"new"}).
						$cgi->hidden(-name=>"sessionID", -value=>$sessionID).
						$cgi->hidden(-name=>"tab", -value=>$tab).
						$cgi->hidden(-name=>"stab", -value=>$stab)
					    );
    $html .=			$cgi->end_Tr();
    $html .=		    $cgi->end_table();
    $html .=		$cgi->end_form();
    $html .=	    $cgi->end_td();
    $html .=	$cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}


#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

sub save_resource {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $newres    = $cgi->param("new_res");
    my @res_grp   = $cgi->param("res_grp");


    my $url = "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }

    $result = $ldap->add(dn  => "resourceName=".$newres.",".$confParam->{baseDN},
			 attr => [ 'resourceName' => $newres,
				   'objectclass' => 'SuSEResourceObject'
				   ]);
    
    if($result->code != 0) {
        my $errs = "add $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }
    #add to resourcegroups
    foreach my $group (@res_grp){
	my $dn = "resourceGroupName=".$group.",".$confParam->{'baseDN'};
	print STDERR $dn."\n";
	$result = $ldap->modify(dn => $dn,
				add => {resourceGroupMember => $newres}
			       );
	if($result->code != LDAP_SUCCESS){
            my $errs = "modify $message->{failed}\n\n";
            $errs .= &LDAPerror($result);
            $this->display_error($errs, $url);
	    $ldap->unbind;
            exit;
	}
    }
    $ldap->unbind;
    $this->display_msg($message->{"succ_completed"},
		       "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",1);
}

sub delete {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"RESOURCES"))->data();

    my $cgi = $this->{"cgi"};

    my $tab    = $cgi->param("tab");
    my $stab   = $cgi->param("stab");
    my $delete = $cgi->param("delete_res");
    #$delete =~ s/(^.*)\s+\(.*/$1/;

    my $url = "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }
    $result = $ldap->delete( dn => "resourceName=".$delete.",".$confParam->{baseDN} );
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
	$ldap->unbind;
        exit;
    }
    $ldap->unbind;
    
    $this->display_msg($message->{"succ_completed"},
		       "$confParam->{cgi_path}/edit_resource.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",1);
}


1;
