#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: edit_sfolder.pm,v 1.4 2003/10/29 10:35:39 varkoly Exp $
#

package edit_sfolder;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $nob  = $cgi->param('nob') || '';
    my $doit = $cgi->param("doit");


    if(defined $doit && $doit eq "g_select" && $nob eq "") {
	$this->sfolder_selected();
    } elsif(defined $doit && $doit eq "g_update") {
	$this->sfolder_update();
    } else {
	$this->startup();
    }
}

sub startup {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');

    my @fn = ();
    my $text = "";
    my $button_width = 150;

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (	# bind to a directory with dn and password
				dn       => "$bind_dn",
				password => "$ADMINpasswd"
				);
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $result = $ldap->search ( # perform a search
				 base   => "$ldapbase",
				 scope  => "one",
				 attrs => ['fn'],
				 filter => "(&(fn=*)(objectclass=SuSEIMAPFolderObject))"
				 );
    
    if($result->code == 0) {
	foreach my $entry ($result->all_entries) {
	    push @fn, $entry->get_value("fn");
	}
    } else {
	$text = "search $message->{failed}\n\n";
	$text .= &LDAPerror($result);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $size = 11;
    #if($#fn < 17) {
    #$size = $#fn + 3; 
    #}

    $ldap->unbind;   # take down session

    my $html = "";
    $html .= $q->start_form(-action=>$confParam->{cgi_path}."/edit_sfolder.pl");
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=     $q->start_Tr();
    $html .=         $q->start_td();
    $html .=             $q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});

    $html .=                 $q->Tr(
				    $this->help_th(2, $message->{edit_sfolder},"SELECT_SFOLDER")
				    );
    $html .=                 $q->Tr(
				    $q->td({-colspan => 2, -class=>'AdminSubHeader'},
					   $message->{select_sfolder})
				    );

    $html .=                     $q->start_Tr();
    $html .=                     $q->td({-align=>"center",-width=>"30%"},
					$q->scrolling_list(-name=>"sfolder",
							   -value=>[@fn],
							   -size=>$size,
							   #-default=>$fn[0]
							   )
					);

    $html .=                     $q->start_td();
    $html .=                     $q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    
    $html .=                     $q->Tr(
					[
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"change",
							   -value=>"$message->{edit}",
							   -width=>$button_width)
						),
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"rights",
							   -value=>"$message->{set_rights}",
							   -width=>$button_width)
						),
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"delete",
							   -value=>"$message->{b_delete}",
							   -width=>$button_width).
						$q->hidden(-name=>"sessionID", -value=>"$sessionID").
						$q->hidden(-name=>"doit", -value=>"g_select").
						$q->hidden(-name=>"tab", -value=>"$tab").
						$q->hidden(-name=>"stab", -value=>"$stab")
						),
					 #$q->td("<p>"),
					 $q->td({-class=>'AdminSubHeader'},
						"$message->{mail_dist_subtitle}"),
					 #$q->td("<p>"),
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"dist_usr",
							   -value=>"$message->{mail_dist_user}",
							   -width=>$button_width),
						),
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"dist_grp",
							   -value=>"$message->{mail_dist_group}",
							   -width=>$button_width)
						),
					 $q->td({-align=>"center"},
						$q->submit(-class=>"BlockButton",-name=>"dist_ext",
							   -value=>"$message->{mail_dist_ext}",
							   -width=>$button_width)
						)
					 ]
					);

    $html .=                     $q->end_table();
    $html .=                     $q->end_td();
    $html .=                     $q->end_Tr();


    $html .=             $q->end_table();
    $html .=         $q->end_td();
    $html .=     $q->end_Tr();
    $html .= $q->end_table();
    $html .= $q->end_form();


    $this->SUPER::display($html);
}

sub confirm {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $Folder    = $q->param('sfolder');
    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');

    my $button_width =100;

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->Tr({},
		    $q->td(
			$q->start_form(-action=>$confParam->{cgi_path}."/edit_sfolder.pl").
			   $q->table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"},
				     $q->Tr(
					    [
					     $q->td({-class=>'AdminHeader', -colspan=>2 },
							     "<b>$message->{confirm_delete_head}&nbsp;$Folder</b>"
						    ),
					     $q->td({-colspan=>2,
						     -class=>'AdminSubHeaderRed'},
						    $message->{confirm_delete_warn}),
					     ]),
				     $q->Tr(
					    $q->td({-align=>"center"},
						   $q->submit(-class=>"button",-name=>"nob",
							      -value=>"$message->{no}",
							      -width=>$button_width).
						   $q->hidden(-name=>'sessionID', -value=>$sessionID).
						   $q->hidden(-name=>'Folder',  -value=>$Folder).
						   $q->hidden(-name=>'sfolder', -value=>$Folder).
						   $q->hidden(-name=>'delete', -value=>$message->{b_delete}).
						   $q->hidden(-name=>'doit', -value=>"g_select").
						   $q->hidden(-name=>'tab', -value=>"$tab").
						   $q->hidden(-name=>'stab', -value=>"$stab")
						   ),
					    $q->td({ -align=>"center"},
						   $q->submit(-class=>"button",-name=>"yesb",
							      -value=>"$message->{yes}",
							      -width=>$button_width) )
					    )
				     ).
		       $q->end_form()
			   )
		    );
    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub sfolder_selected {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $sfolder   = $q->param('sfolder');
    my $change    = $q->param('change');
    my $rights    = $q->param('rights');
    my $delete    = $q->param('delete');
    my $yesb      = $q->param('yesb') || '';
    my $nob       = $q->param('nob') || '';

    if(!defined $sfolder || $sfolder eq "") {
	$this->display_error($message->{must_select_folder},
		      "$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		      3);
	exit;
    }

    if(defined $change && $change eq $message->{edit}) {
	$this->edit_sfolder($sfolder);
    } elsif(defined $rights && $rights eq $message->{set_rights}) {
	my $msg = $message->{manage_permissions_msg}." <b>$sfolder</b>";
	$this->display_msg($msg,
		    "$confParam->{cgi_path}/manage_folder_permissions.pl?sessionID=$sessionID\&Folder=$sfolder\&tab=$tab\&stab=$stab",
		    1);
	exit;

#####################
    } elsif(defined $q->param('dist_usr')) {
	my $msg = $message->{manage_permissions_msg}." <b>$sfolder</b>";
	$this->display_msg("",
		    "$confParam->{cgi_path}/mail_dist_user.pl?sessionID=$sessionID\&Folder=$sfolder\&tab=$tab\&stab=$stab",
		    1);
	exit;
    } elsif(defined $q->param('dist_grp')) {
	my $msg = $message->{manage_permissions_msg}." <b>$sfolder</b>";
	$this->display_msg("",
		    "$confParam->{cgi_path}/mail_dist_group.pl?sessionID=$sessionID\&Folder=$sfolder\&tab=$tab\&stab=$stab",
		    1);
	exit;
#####################
    
    } elsif(defined $q->param('dist_ext')) {
	my $msg = $message->{manage_permissions_msg}." <b>$sfolder</b>";
	$this->display_msg("",
			   "$confParam->{cgi_path}/mail_dist_ext.pl?sessionID=$sessionID\&Folder=$sfolder\&tab=$tab\&stab=$stab",
			   1);
	exit;
#####################
    
    } elsif(defined $delete && $delete eq $message->{b_delete} && $yesb eq "" && $nob eq "" ) {
	$this->confirm();
    } elsif(defined $delete && $delete eq $message->{b_delete} && $yesb ne "") {
	$this->delete_sfolder($sfolder);
    }
}


sub delete_sfolder() {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    my $imaphost      = $confParam->{IMAPserver}; 
    my $imapport      = $confParam->{IMAPport};

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $sfolder   = $q->param('sfolder');

    my $text = "";

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);

    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (	# bind to a directory with dn and password
				dn       => "$bind_dn",
				password => "$ADMINpasswd"
				);
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    $mesg = $ldap->search( base => "$ldapbase",
			   scope => "one",
			   filter=> "(&(objectclass=posixAccount)(alias=owner-$sfolder))",
			   attrs=> [ "alias" ] );
    if( $mesg->code != 0 ) {
	$text = "search $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$ldap->unbind;   # take down session
	$this->display_error($text,
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    if( $mesg->count > 0 ) {
	foreach my $e ( $mesg->all_entries ) {
	    $e->delete( alias => [ "owner-$sfolder" ] );
	    $e->update($ldap);
	}
    }
    
    $mesg = $ldap->delete("fn=".$sfolder.",".$ldapbase);
    if($mesg->code != 0) {
	$text = "delete $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    $ldap->unbind;   # take down session

    my $imap = new Net::IMAP($imaphost, Debug => 0);
    
    if(!defined $imap) {
	$this->display_error("can't connect to $imaphost: $!",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    # --- login as user
    my $resp = $imap->login($ADMINuid, $ADMINpasswd);
    if($$resp{Status} ne "ok") {
	$text = "login failed: Serverresponse:$$resp{Status} => $$resp{Text}\n";
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }    
    
    # --- create new Mailbox
    $resp = $imap->delete($sfolder);
    
    if($$resp{Status} ne "ok") {
	$text =  "delete failed: Serverresponse:$$resp{Status} => $$resp{Text}\n";
	$imap->logout();
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    $imap->logout();

    $text = "$sfolder: $message->{sfolder_delete_txt}\n";
    $q->param(-name=>'doit', -value=>"");
    $q->param(-name=>'sfolder', -value=>"");

    $this->display_msg($text,
		"$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		1);
    exit;
}

sub edit_sfolder() {
    my $this = shift;
    my $sfolder = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');

#    my @muid = ();
#    my @uid = ();
    my $desc = "";
    my $mailenabled = "";
    my $mailDeliveryProgram = "";
    my $text = "";

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (	# bind to a directory with dn and password
				dn       => "$bind_dn",
				password => "$ADMINpasswd"
				);
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $result = $ldap->search ( # perform a search
				 base   => "$ldapbase",
				 scope  => "one",
				 attrs => ['description', 'mailenabled'],
				 filter => "(&(fn=$sfolder)(objectclass=SuSEIMAPFolderObject))"
				 );
    
    if($result->code == 0) {
	if($result->count != 1) {
	    exit;
	}
	my ($entry) = $result->entries;
	if(defined $entry->get_value("description")) {
	    $desc = $entry->get_value("description");
	}
	if(defined $entry->get_value("mailenabled")) {
	    $mailenabled = $entry->get_value("mailenabled");
	}
	
    } else {
	$text = "search $message->{failed}\n\n";
	$text .= &LDAPerror($result);
	$ldap->unbind;   # take down session
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    $ldap->unbind;   # take down session
    
    my $yn = [ $message->{yes}, $message->{no} ];
    my $mdefault;
    if(defined $mailenabled && $mailenabled eq 'OK') {
	$mdefault = $message->{yes};
    } else {
	$mdefault = $message->{no};
    }

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->Tr({},
		    $q->td(
			$q->start_form(-action=>$confParam->{cgi_path}."/edit_sfolder.pl").
			   $q->table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"},
				     $q->Tr(
					    $this->help_th(2, $message->{edit_sfolder}, "EDIT_SFOLDER")
					    ),
				     $q->Tr(
					    $q->td({-colspan => 2, -class=>'AdminSubHeader'},
						$message->{'selected_sfolder'}.": <i>$sfolder</i>")
					    ),
				     $q->Tr(
					    $q->td("$message->{desc}"),
					    $q->td(
						   $q->textfield(-name=>'desc',
								 -default=>$desc, -size=>20))
					    ),
				     $q->Tr(
					    $q->td("$message->{with_email}"),
					    $q->td(
						   $q->popup_menu(-name=>'mailenabled',
								  -values=>$yn,
								  -default=>$mdefault))
					    ),
				     $q->Tr(
					    $q->td({-colspan=>2, -align => "center"}, 
						   $q->submit(-class=>"button",-name=>"update", -value=>"$message->{update}").
						   $q->hidden(-name=>"doit", -value=>"g_update",
							      -override=>1).
						   $q->hidden(-name=>"sessionID", -value=>"$sessionID").
						   $q->hidden(-name=>"sfolder", -value=>"$sfolder").
						   $q->hidden(-name=>"tab", -value=>"$tab").
						   $q->hidden(-name=>"stab", -value=>"$stab")
						   )
					    )
				     ).
		       $q->end_form()
			   )
		    );
    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub sfolder_update {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    
    my $message = ($session->getLanguage($confParam->{LANG},"CREATE_SFOLDER", "EDIT_SFOLDER"))->data();

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};

    my $tab         = $q->param('tab');
    my $stab        = $q->param('stab');
    my $sfolder     = $q->param('sfolder');
    my $desc        = $q->param('desc');
    my $mailenabled = $q->param('mailenabled');

    my $mailDeliveryProgram = "";
    my $text = "";

    if(!defined $sfolder || $sfolder eq "") {
	$this->display_error($message->{must_select_folder},
		      "$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		      3);
	exit;
    }

    my $ldap= Net::LDAP->new($ldaphost, port => $ldapport, version => 3);
    
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $bind_dn = "uid=".$ADMINuid.",".$ldapbase;
    
    my $mesg = $ldap->bind (	# bind to a directory with dn and password
				dn       => "$bind_dn",
				password => "$ADMINpasswd"
				);
    
    if($mesg->code != 0) {
	$text = "bind $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    if(defined $mailenabled && $mailenabled eq $message->{yes}) {
	my $domain = "";
	# check if mailenabled is already set. If it is set there is no need to check
	# for uniqueness of the address again

	
	my $result = $ldap->search( base => "fn=$sfolder,$ldapbase",
				    scope => "base",
				    attrs => ["mailenabled"],
				    filter => "(objectClass=*)"
				   );
	if($result->code != 0) {
	    $text = "search $message->{failed}\n\n";
	    $text .= &LDAPerror($result);
	    $ldap->unbind;   # take down session
	    $this->display_error($text,
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	} else {
	    my $entry = $result->entry(0);
	    if($entry->get_value("mailenabled") ne "OK"){
		$result = $ldap->search ( # perform a search
					     base   => "uid=cyrus,$ldapbase",
					     scope  => "base",
					     attrs => ['mailDomain'],
					     filter => "(objectclass=*)"
					     );
		
		if($result->code == 0) {
		    my $entry = $result->entry(0);
		    if(defined $entry->get_value("maildomain")) {
			$domain = $entry->get_value("maildomain");
		    }
		    
		} else {
		    $text = "search $message->{failed}\n\n";
		    $text .= &LDAPerror($result);
		    $ldap->unbind;   # take down session
		    $this->display_error($text,
				  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		    exit;
		}
		$result = $ldap->search( base => $ldapbase,
					    scope => "one",
					    attrs => ['objectclass'],
					    filter => "(| (alias=$sfolder) (uid=$sfolder))" );
		if($result->code == 0) {
		    if($result->count != 0){
			my $text = $sfolder."@".$domain.": ".$message->{'email_not_unique'}."\n\n";
			$ldap->unbind;   # take down session
			$this->display_error($text,
				      "$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
			exit;
		    }
		} else {
		    $text = "search $message->{failed}\n\n";
		    $text .= &LDAPerror($result);
		    $ldap->unbind;   # take down session
		    $this->display_error($text,
				  "$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		    exit;
		}
	    }
	    $mailenabled = "OK";
	    $mailDeliveryProgram = "\"|/usr/bin/formail -I \\\"From \\\" |/usr/lib/cyrus/bin/deliver -e -a cyrus -m $sfolder\"";
	}
    } else {
	$mailenabled = "REJECT";
	$mailDeliveryProgram = "";
    }

    $mesg = $ldap->modify( "fn=".$sfolder.",".$ldapbase,
			   changes => [
				       replace => 
				       [ description =>
					 ( $desc eq "" ? [] : [$desc] )],
				       replace => [ mailenabled => [$mailenabled] ],
				       replace =>
				       [ mailDeliveryProgram =>
					 ( $mailDeliveryProgram eq "" ?
					   [] : [$mailDeliveryProgram] ) ]
				       ]
			   );
    
    if($mesg->code != 0) {
	$text = "modify $message->{failed}\n\n";
	$text .= &LDAPerror($mesg);
	$text .= $ldap->version;
	$ldap->unbind;   # take down session
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my $search_res = $ldap->search( base => "$ldapbase",
			   scope => "one",
			   filter=> "(&(objectclass=posixAccount)(alias=owner-$sfolder))",
			   attrs=> [ "alias" ] );
    if( $search_res->code != 0 ) {
	$text = "search $message->{failed}\n\n";
	$text .= &LDAPerror($search_res);
	$ldap->unbind;   # take down session
	$this->display_error($text,
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $owner_alias_exists = 0;
    if( $search_res->count > 0) {
	$owner_alias_exists = 1;
    }
    if(defined $mailenabled && $mailenabled eq "OK" ) {
	if( ! $owner_alias_exists ){	
	    $mesg = $ldap->modify( "uid=mailadmin,$ldapbase",
				   changes => [ add =>  [ alias => "owner-$sfolder" ]]
				   );
	    if ($mesg->code != 0) {
		$text = "failed to add owner-$sfolder alias to mailadmin:";
		$text .= &LDAPerror($mesg);
		$ldap->unbind;   # take down session
		$this->display_error($text,
				     "$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		exit ;
	    }
	}
    } else {
	if( $search_res->count > 0 ) {
	    foreach my $e ( $search_res->all_entries ) {
		$e->delete( alias => [ "owner-$sfolder" ] );
		$e->update($ldap);
	    }
	}
    }

    $ldap->unbind;   # take down session

    $text = "$message->{edit_success_txt}\n";
    
    $q->param(-name=>'doit', -value=>"");
    $q->param(-name=>'sfolder', -value=>"");
    $q->param(-name=>'uids', -value=>"");

    $this->display_msg($text,
		"$confParam->{cgi_path}/edit_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		1);
    exit;
}

1;
