#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: global_conf.pm,v 1.5 2004/07/28 07:40:18 varkoly Exp $
#
package global_conf;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Config::IniFiles;
use File::Temp qw(tempfile);
use strict;
use Display;
use Utils;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

my @SUPPORTED_FORMATS = split /\s+/ , read_rcconfig("SCHOOL_SUPPORTED_IMPORT_FORMATS");

my $global_template = { 
	"GENERAL" => { "ImportFileFormat" => \@SUPPORTED_FORMATS,
		       "EnableSamba" => ["true", "false"],
		       "EnableUserSpamFrontend" => ["true", "false"],
		       "EnableFaxParamFrontend" => ["true", "false"],
		       "EnableSieveEditor" => ["true", "false"],
		       "EnableDynDns" => ["true", "false"],
		       "MonitorResolveAddr" => ["true", "false"],
		       "NewUserChangePassword" => ["true", "false"],
		       "DelUserNoAcl" => ["false", "true"],
		       "DisableDNSManagement" => [ "false", "true" ],
		       "DisableDHCPManagement" => [ "false", "true" ],
		       "DisableWebmailLink" => ["false", "true" ],
		       "DisableGroupwareLink" => [ "false", "true" ],
		       "DisableGroupwareLogin" => [ "false", "true" ]
		       },
	"FETCHD"  => { "keeponserver" => ["false", "true"],
		       "debug" => [ "0", "1", "2", "3", "4" ],
		       "append_fetch_header" => [ "true", "false" ],
		       "drop_undeliverable_mail" => [ "true", "false" ]
		       },
	"SECURITY" => { "DefaultPasswordHash" => ["CRYPT", "SMD5" ],
			"CheckClientIP" => [ "true", "false" ],
			"UseCookie" => [ "false", "true" ]
		      },
	"SESSIOND" => {},
	"SUAD" => {}
};

my $filename = "/etc/imap/globals.conf";
	

sub new {
    my $this = shift;
    my $self = new Display(@_);
    return bless $self, $this;
}


sub display {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $save = $cgi->param('save');
    if($save){
	$this->write_changes();
    }
    $this->startup();
}

sub startup {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $cgi       = $this->{"cgi"};

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"GLOBAL_CONF"))->data();
    my @r = ();
    my $msg = $session->save_store2session($filename, $filename, \@r);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($filename.": ".$message->{'cg_file_changed'},
				 $confParam->{cgi_path}."/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
				 $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }

    my $ini = new Config::IniFiles( -file => $filename );
    if( ! $ini ){
	$this->display_error($message->{'cg_unable_to_open'},
			     $confParam->{cgi_path}."/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	exit;
    }

    my $html =  $cgi->start_table({-class=>'AdminBorder', -cellspacing=>3, -cellpadding=>0, -width=>"100%"});
    $html .=        $cgi->start_Tr();
    $html .=            $cgi->start_td();
    $html .=                $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=                    $cgi->Tr( $this->help_th(3, $message->{'cg_title'}, "GLOBAL_CONF" ));
    $html .=                    $cgi->Tr( $cgi->td({-class=>'AdminSubHeader', -colspan => 3}, $message->{'cg_subtitle'}));
    $html .=                    $cgi->start_Tr();
    $html .=                        $cgi->start_td({-align => "center"});
    $html .=			    $cgi->start_multipart_form(-action=>$confParam->{cgi_path}."/global_conf.pl");
    $html .=                            $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    foreach my $section ( $ini->Sections()){
	$html .=			    $cgi->start_Tr();
	$html .=				$cgi->start_td({-colspan => "3" });
	$html .=				    $section;
	$html .=				$cgi->end_td();
        $html .=			    $cgi->end_Tr();
	foreach my $key ( $ini->Parameters($section) ){
	    $html .=			    $cgi->start_Tr();
	    $html .=				$cgi->start_td({-width => "20"});
	    $html .=				$cgi->end_td();
	    $html .=				$cgi->start_td();
	    $html .=				    $key;
	    $html .=				$cgi->end_td();
	    $html .=				$cgi->start_td();
	    my $possible_val = $global_template->{$section}->{$key};
	    if( $possible_val ){
		$html .=			    $cgi->popup_menu( -name => $section."+".$key,
								      -values => $possible_val,
								      -default => $ini->val($section, $key) );
	    }else{
		$html .=			    $cgi->textfield( -name => $section."+".$key,
								     -value => $ini->val($section, $key),
								     -size => 45 );
								      
	    }
	    $html .=				$cgi->end_td();
	    $html .=			    $cgi->end_Tr();
	}
    }
    $html .=				    $cgi->start_Tr();
    $html .=					$cgi->start_td({-colspan => 3, -align => "center"});
    $html .=					    $cgi->hidden(-name => "sessionID", -value => $sessionID);
    $html .=					    $cgi->hidden(-name => "tab", -value => $tab);
    $html .=					    $cgi->hidden(-name => "stab", -value => $stab);
    $html .=					    $cgi->submit(-class=>"button",-name => "save", -value => $message->{'save'});
    $html .=					$cgi->end_td();
    $html .=				    $cgi->end_Tr();
    $html .=                            $cgi->end_table();
    $html .=			    $cgi->end_form();
    $html .=                        $cgi->end_td();
    $html .=                    $cgi->end_Tr();
    $html .=                $cgi->end_table;
    $html .=            $cgi->end_td;
    $html .=        $cgi->end_Tr;
    $html .=    $cgi->end_table;

    $this->SUPER::display($html);
}

sub write_changes {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $cgi       = $this->{"cgi"};

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"GLOBAL_CONF"))->data();

    my $ini = new Config::IniFiles( -file => "/etc/imap/globals.conf" );
    my $params = $cgi->Vars();

    # map back the hidden field names (section+key) to a hash of hashes
    foreach my $complete_key (keys( %$params )){
	my ($section, $key) = split /\+/, $complete_key;
	if( defined $global_template->{$section} ){
	    $ini->setval($section, $key, $params->{$complete_key} );
	}
    }

    # save new config to a temporary file
    my ($fh, $file) = tempfile(DIR => "/tmp/");
    if(! $ini->WriteConfig( $file )){
	$this->display_error($message->{'cg_write_error'},
			     $confParam->{cgi_path}."/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	exit;
    }
    close $fh;

    # write contents of the tempfile to /etc/globals.conf
    if( ! open $fh, $file){
	$this->display_error($message->{'cg_write_error'},
			     $confParam->{cgi_path}."/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	exit;
    }
    my @file = <$fh>;
    chomp(@file);
    my $msg = $session->save_read2session($filename, $filename);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($filename.": ".$message->{'cg_file_changed'},
				 $confParam->{cgi_path}."/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
				 $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }

    my $ret = $session->suad_save(\@file, $filename);
    if(  $ret->code() ne "OK" ) {
	$this->display_error($message->{'cg_write_error'}.$ret->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    unlink($file);
    close $fh;
    my $status = "";
    
    my $ret = $session->suad_exec("apachereload");
    if(  $ret->code() ne "OK" ) {
	$status .= "Apache reload failed: ".$ret->code()."\n";
    }else{
        $status .= "Reloading Apache configuration: ".$ret->code()."\n";
    }
    my $ret = $session->suad_exec("fetchdreload");
    if(  $ret->code() ne "OK" ) {
	$status .= "Fetchd reload failed: ".$ret->code()."\n";
    }else{
        $status .= "Reloading Fetchd configuration: ".$ret->code()."\n";
    }
    $msg = $session->save_clear2session($filename);
    if( $msg->code() ne "OK" ) {
	$this->display_error($msg->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    $this->display_msg($message->{'cg_saved_ok'}."\n\n".$status,
	    "$confParam->{cgi_path}/global_conf.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab", 3);
    exit;
}

1;
