#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: internet.pm,v 1.1.1.1 2003/03/25 10:31:39 ray Exp $
#
package internet;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Storable qw(thaw freeze);
use Net::LDAP;
use LDAPUtils;
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $doit = $cgi->param('doit');

    if(defined $doit && $doit eq "deaktivate") {
	$this->deaktivate_him();
    } elsif(defined $doit && $doit eq "aktivate") {
	$this->aktivate_him();
    } elsif(defined $doit && $doit eq "process") {
	$this->process_users();
    } else {
	$this->startup();
    }
}

sub startup {
    my $this = shift;

    my $q = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $message = ($session->getLanguage($confParam->{LANG},"INTERNET"))->data();
    my $sessionID = $confParam->{"sessionID"};

    my $doit      = $q->param('doit');
    my $filter    = $q->param('filter');
    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $user      = $q->param('USER');

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    #load list of users from session
    my $frozen = $confParam->{"usersToMod"};
    if( (! defined $frozen) || ($frozen eq "") ){
	$this->display_error($message->{'error_reading_session'},
	    	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    $frozen = pack('H*',$frozen); # convert back to binary
    my $user_ref = thaw($frozen);     # melt down... :-)
    my @users    = @{$user_ref};
    #delete userlist from $confparm
    $session->modify( {'usersToMod' => undef} );

    my $text = "";
    
    my $button_width=100;
    my $shadowExpire = "";
    my $ldap = Net::LDAP->new("$ldaphost", port => $ldapport, version => 3);

    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    my $mesg = $ldap->bind (   # bind to a directory with dn and password
			       dn       => "uid=$ADMINuid,$ldapbase",
			       password => "$ADMINpasswd"
			       );

    if($mesg->code != 0) {
        $text .= "bind failed\n";
        $text .=  &LDAPerror($mesg);
	$this->display_error($text,
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my @users_to_act;
    my @users_to_deact;
    my $act_user="";
    foreach $act_user (@users){
	my $internetdisabled ="";
	$mesg = $ldap->search (  # perform a search
	    		     base   => "$ldapbase",
	    		     scope  => "one",
	    		     attrs => ['internetdisabled'],
	    		     filter => "(&(uid=$act_user)(objectclass=SuSEeMailObject))"
	    		     );
	
	if($mesg->code == 0 && $mesg->count == 1) {
	
	    my $result = $mesg->as_struct;
	    my %result = %$result;
	    
	    
	    if( defined @{$result{"uid=".$act_user.",".$ldapbase}{internetdisabled}} ) {
	        $internetdisabled  = join "", @{$result{"uid=".$act_user.",".$ldapbase}{internetdisabled}};
	    }
	    if( (defined $internetdisabled ) && ($internetdisabled  eq "true") ){
			# User is actually deactivate, push him into the
			# users_to_activcate array
			push @users_to_act, $act_user;
	    }else{
			# User is actually deactivate, push him into the
			# users_to_activcate array
			push @users_to_deact, $act_user;
	    }
	} else {
	    $text .= "search failed\n";
	    $text .=  &LDAPerror($mesg);
	    $this->display_error($text,
	    	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }
    $ldap->unbind;

    # write the two lists of users into the session data
    if(@users_to_act){
		my $frozen = freeze(\@users_to_act);      #freeze data
		$frozen = unpack('H*',$frozen);   #convert to ASCII
		my %tmp = ( "usersToAct" => $frozen );
		my $s_reply=$session->modify(\%tmp);
		if( $s_reply->code() ne "OK" ) {
		    $this->display_error($message->{'write2session_failed'},
	    		     "$confParam->{'cgi_path'}/menu.pl?sessionID=$sessionID");
		}
    }
    if(@users_to_deact){
		my $frozen = freeze(\@users_to_deact);      #freeze data
		$frozen = unpack('H*',$frozen);   #convert to ASCII
		my %tmp = ( "usersToDeact" => $frozen );
		my $s_reply=$session->modify(\%tmp);
		if( $s_reply->code() ne "OK" ) {
		    $this->display_error($message->{'write2session_failed'},
	    		     "$confParam->{'cgi_path'}/menu.pl?sessionID=$sessionID");
		}
    }

    my $html = "";
		$html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"}).
	    $q->start_Tr().
		$q->start_td().
		    $q->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"}).
		    $q->start_form(-action=>$confParam->{cgi_path}."/internet.pl");
    if( @users_to_act > 0 ){
		$html .= $q->start_Tr().
		$q->td({-colspan=>2, -class=>'AdminTdRed'}, $message->{'aktivate_user_really'}).
	    $q->end_Tr().
	    $q->start_Tr().
		$q->start_td({-colspan => 2, -align=>'center'});
		foreach $act_user (@users_to_act){
		    $html .= $act_user."<br>";
		}
		$html .= $q->end_td().
		    $q->end_Tr();
    }
    if( @users_to_deact > 0){
		$html .= $q->start_Tr().
		$q->td({-colspan=>2, -class=>'AdminTdRed'}, $message->{'deaktivate_user_really'}).
	    $q->end_Tr().
	    $q->start_Tr().
		$q->start_td({-colspan => 2, -align=>'center'});
		foreach $act_user (@users_to_deact){
		    $html .= $act_user."<br>";
		}
		$html .= $q->end_td().
	    $q->end_Tr();
    }
    $html .=		$q->start_Tr().
			    $q->start_td({-align=>"center"}).
			       $q->submit(-class=>"button",-name=>"buttons",
					  -value=>"$message->{no}",
					  -width=>$button_width).
			       $q->hidden(-name=>"sessionID", -value=>"$sessionID").
			       $q->hidden(-name=>"doit", -value=>"process").
			       $q->hidden(-name=>"filter", -value=>"$filter").
			       $q->hidden(-name=>"filtered", -value=>1).
			       $q->hidden(-name=>"USER", -value=>"$user").
			       $q->hidden(-name=>"tab", -value=>"$tab").
			       $q->hidden(-name=>"stab", -value=>"$stab").
			    $q->end_td().
			    $q->start_td({-align=>"center"}).
			       $q->submit(-class=>"button",-name=>"buttons",
					  -value=>"$message->{yes}",
					  -width=>$button_width ).
			    $q->end_td().
			$q->end_Tr().
		    $q->end_form().
		$q->end_table().
	    $q->end_td().
	$q->end_Tr().$q->end_table();
	
    $this->back_url("$confParam->{cgi_path}/browse_user.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&filter=$filter\&filtered=1");
    $this->SUPER::display($html);
}

sub process_users{
    my $this = shift;

    my $q = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $message = ($session->getLanguage($confParam->{LANG},"INTERNET"))->data();
    my $sessionID = $confParam->{"sessionID"};

    my $doit      = $q->param('doit');
    my $filter    = $q->param('filter');
    my $tab       = $q->param('tab');
    my $stab      = $q->param('stab');
    my $user      = $q->param('USER');
    my $b         = $q->param('buttons');

    my $ADMINuid      = $confParam->{uid};
    my $ADMINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    
    my @users_to_act;
    my @users_to_deact;

#    print STDERR "processing\n";
    #load list of users from session
    my $frozen_act = $confParam->{"usersToAct"};
    my $frozen_deact = $confParam->{"usersToDeact"};
    if( ((! defined $frozen_act) || ($frozen_act eq "")) && ((!defined $frozen_deact) || ($frozen_deact eq "")) ){
    	print STDERR "missing Data\n";
		$this->display_error($message->{'error_reading_session'},
	    	      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		exit;
    }
    if( (defined $frozen_act) && ($frozen_act ne "") ){
		$frozen_act = pack('H*',$frozen_act); # convert back to binary
		my $user_ref = thaw($frozen_act);     # melt down... :-)
		@users_to_act    = @{$user_ref};
		#delete userlist from $confparm
		$session->modify( {'usersToAct' => undef} );
    }

    if( (defined $frozen_deact) && ($frozen_deact ne "") ){ 
		$frozen_deact = pack('H*',$frozen_deact); # convert back to binary
		my $user_ref = thaw($frozen_deact);     # melt down... :-)
		@users_to_deact    = @{$user_ref};
		#delete userlist from $confparm
		$session->modify( {'usersToDeact' => undef} );
    }

    if(defined $b && $b eq $message->{yes}) {

	my $text = "";

	my $ldap = Net::LDAP->new("$ldaphost", port => $ldapport, version => 3);

	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server",
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}

	my $mesg = $ldap->bind (   # bind to a directory with dn and password
				   dn       => "uid=$ADMINuid,$ldapbase",
				   password => "$ADMINpasswd"
				   );
	
	if($mesg->code != 0) {
	    $text .= "bind failed\n";
	    $text .=  &LDAPerror($mesg);
	    $this->display_error($text,
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
	my $failure = 0;
	my $act_user="";
	foreach $act_user (@users_to_act){
	    my @mod_op;
	    push @mod_op, "replace", [ 'internetdisabled', [ 'false' ]];
	    $mesg = $ldap->modify( "uid=".$act_user.",".$ldapbase, changes => \@mod_op);
	    
	    if($mesg->code != 0 ) {
			$failure = 1;
	        $text .= "<font color='red'>\"$act_user\" $message->{'activation_failed'}</font>\n";
	        $text .=  &LDAPerror($mesg)."<br>";
	    } else {
			$text .= "\"$act_user\" $message->{'successfully_activated'}\n";
	    }
	}
	$text .= "<br>";
	foreach $act_user (@users_to_deact){
	    my @mod_op;
	    push @mod_op, "replace", [ 'internetdisabled', [ 'true' ]];
	    $mesg = $ldap->modify( "uid=".$act_user.",".$ldapbase, changes => \@mod_op  );
	    
	    if($mesg->code != 0 ) {
			$failure = 1;
	        $text .= "<font color='red'>\"$act_user\" $message->{'deactivation_failed'}</font>\n";
	        $text .=  &LDAPerror($mesg)."<br>";
	    } else {
			$text .= "\"$act_user\" $message->{'successfully_deactivated'}\n";
	    }
	}
        $ldap->unbind;
	if($failure){
	    $this->display_error($text,
	 	  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}else{
	    $this->display_msg($text, 
			       "$confParam->{cgi_path}/browse_user.pl?sessionID=$sessionID&".
			       "filter=$filter&filtered=1\&tab=$tab\&stab=$stab" ,
			       1);
	}
    } else {
	$this->display_msg($message->{abort}, 
		    "$confParam->{cgi_path}/browse_user.pl?sessionID=$sessionID&".
		    "filter=$filter&filtered=1\&tab=$tab\&stab=$stab" ,
		    1);
    }
}


1;
