#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: ldap_browser.pm,v 1.3 2003/10/29 10:35:40 varkoly Exp $
#
package ldap_browser;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP; #LDAP modules
use cfreader;
use LDAPUtils;
use Display; #display class for depicting everything
use URI::Escape;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

sub new {
    my $this = shift;
    my $self = new Display(@_);

    return bless $self, $this;
}

sub display {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},"LDAP"))->data();

    my $cgi       = $this->{"cgi"};

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    $this->{"url"} = $confParam->{cgi_path}."/ldap_browser.pl?sessionID=$sessionID&tab=$tab&stab=$stab"; 

    $this->{"ldap"} = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);
    my $ldap      = $this->{"ldap"};
    
    if (!defined $ldap) {
	$this->display_error($message->{"ldap_connect_error"},$this->{"url"});
	exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},
			 password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    my $basedn=$confParam->{"baseDN"}; #Parameter of current baseDN. It contains the treedepth

    my $numofentries;
    my @dn_elems; #array of the elements of the actual DN. This DN will be used for queries
    if (defined $cgi->param("searchdn")) {
	@dn_elems=split /,/,$cgi->param("searchdn");
	$this->{"url"}.="&searchdn=".$cgi->param("searchdn");
    }
    else {
	@dn_elems=split /,/,$basedn;
	$this->{"url"}.="&searchdn=$basedn";
    }
    my @bdn_parts = split(/,/,$basedn);
    my $basedn_cnt = scalar(@bdn_parts) ; #Number of elements of the baseDN
    my $treedepth=$#dn_elems+1; # depth of the LDAP tree. This equals the number of
                                # branches plus basedn_cnt
    
    my $filter; #filter for LDAP queries. It contains CGI-params filter1 and filter2
    my $filterset; #has a filter been set

    if (defined $cgi->param("filter1") and defined $cgi->param("filter2")) {
	if ($cgi->param("filter1") eq "objectclass") {
	    $filter="objectclass=".$cgi->param("filter2");
	}
	elsif ($cgi->param("filter1") eq "fn") {
	    $filter="(&(fn=".$cgi->param("filter2").")(objectclass=SuSEImapFolderObject))";
	}
	elsif ($cgi->param("filter1") eq "grp") {
	    $filter="(&(cn=".$cgi->param("filter2").")(objectclass=PosixGroup))";
	}
	elsif ($cgi->param("filter1") eq "uid") {
	    $filter="(&(uid=".$cgi->param("filter2").")(objectclass=PosixAccount))";
	}
	$this->{"url"}.="&filter1=".$cgi->param("filter1")."&filter2=".$cgi->param("filter2");
	$filterset=1;
    }
    else {
	$filter="objectclass=*";
	$this->{"url"}.="&filter1=objectclass&filter2=*";
	$filterset=0;
    }

    my $html="";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=   $cgi->start_form(-action=>$confParam->{cgi_path}."/ldap_browser.pl",-method=>"post");
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_table({-cellspacing=>"0",-cellpadding=>"3",-width=>"100%"});
    $html .=         $cgi->Tr( $this->help_th(1, $message->{"ldap_title"}, "LDAP_BROWSER") );
    $html .=         $cgi->Tr( $cgi->td({-class=>'AdminSubHeaderRed'},
				$message->{ldap_warn}));
    $html .=         $cgi->start_Tr({});
    $html .=           $cgi->start_td({-align=>"center"});
    $html .=             $cgi->start_table({-border=>"0",-cellspacing=>"0",-cellpadding=>"0",-width=>"100%"});
    
    if (!defined $cgi->param("master") or $cgi->param("master") eq "tree") {
	#create the tree
	$this->{"url"}.="&master=tree";
	$html .=             $cgi->start_Tr();
	for (my $i=1; $i<$treedepth; $i++) {
	    $html .=             $cgi->td({-width=>"15"},$cgi->img({-src=>"/images/spacer.gif",-height=>"1",-width=>"20"}));
	}
	$html .=               $cgi->td({},$cgi->img({-src=>"/images/spacer.gif",-height=>"1",-width=>700-25*($treedepth+1)}));
        #unfortunately NN seems not to care about col width. Thus I create 1 pixel pictures which define the width
	$html .=             $cgi->end_Tr();
	$html .=             $cgi->Tr({},
				      $cgi->td(&colspan("hash",$treedepth-1+1,{}),
					       $cgi->popup_menu("filter1",
								["objectclass","fn","grp","uid"],
								"objectclass",
								{"objectclass"=>$message->{ldap_objectclass},"fn"=>$message->{ldap_folder},"grp"=>$message->{ldap_group},"uid"=>$message->{ldap_user}}),
					       $cgi->textfield(-name=>"filter2",-default=>"*",-size=>"20"),
					       $cgi->submit(-class=>"button",-value=>$message->{"ldap_submit_tree"},-name=>"subtree")
					       )
				      );
	$html .=             $cgi->Tr({},$cgi->td(&colspan("hash",$treedepth+1,{}),"&nbsp;"));
	$html .=             $cgi->Tr({},$cgi->td(&colspan("hash",$treedepth-1+1,{}),$basedn));
	if ($filterset==1) {
	    #try to create a tree if a filter has been set
	    my $dummy =          $this->create_tree(\@dn_elems,$treedepth,$basedn_cnt,$basedn_cnt,$filter,[]);
	    if ($dummy ne "-1") {
		$html .= $dummy;
	    }
	    else {
		$html .=         $cgi->Tr({},$cgi->td({},""),$cgi->td({-height=>"40",-align=>"left"},$message->{ldap_filter_first}));
	    }
	}
	else {
	    #print message if no filter has been set so far
	    $html .=           $cgi->Tr({},$cgi->td({},""),$cgi->td({-height=>"40",-align=>"left"},$message->{ldap_filter_first}));
	}
	$html .=           $cgi->end_table();
	$html .=           $cgi->hidden(-name=>"master", -value=>"tree");
    }
    elsif ($cgi->param("master") eq "attributes") {
	#create the site with attributes or change them
	if (! defined $cgi->param("subattr")) {
	    #if the button for submitting attribute changes was _not_ pressed
	    $html .=           $this->attributes("output",$filter);
	}
	else {
	    $html .=           $this->attributes("change",$filter);
	}
	$html .=           $cgi->end_table();
	$html .=           $cgi->hidden(-name=>"master", -value=>"attributes");
    }
    $html .=             $cgi->hidden(-name=>"tab", -value=>$tab);
    $html .=             $cgi->hidden(-name=>"stab", -value=>$stab);           
    $html .=             $cgi->hidden(-name=>"sessionID", -value=>$sessionID);
    $html .=           $cgi->end_td();
    $html .=         $cgi->end_Tr();
    $html .=       $cgi->end_table;
    $html .=     $cgi->end_td();
    $html .=     $cgi->end_form;
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table;

    $ldap->unbind;

    $this->SUPER::display($html);
}

#---------------------------------------------------------------------------------------------------

sub create_tree { #create the tree (recursive)
    my $this       = shift;
    my $dn_elems   = shift; #array of the elements of the complete DN
    my $treedepth  = shift; #depth of the whole tree
    my $basedn_cnt = shift; #depth of baseDN
    my $depth      = shift; #local depth used right now
    my $html       = "";
    my $ldap       = $this->{"ldap"};
    my $cgi        = $this->{"cgi"};

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},"LDAP"))->data();
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    my $filter    = shift; #search filter for LDAP
    my @filter    = ($cgi->param("filter1"),$cgi->param("filter2")); #the elements of the filter

    my $final     = shift; #array whether this is this the last entry of the actual depth

    my $depthbase = $confParam->{"baseDN"}; #DN used right now
    for (my $i=$basedn_cnt+1; $i<=$depth; $i++) {
	$depthbase=$$dn_elems[$treedepth-$i].",".$depthbase;
    }

    my $result = $ldap->search(base=>$depthbase,scope=>"one",filter=>$filter);
    if ($result->code != 0) {
	my $errs = "search $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs,$this->{"url"});
	$ldap->unbind;   # take down session
	exit;
    }

    my $numofentries=$result->count; #this is a result count
    my @entries=$result->entries;
    @entries=sort entrycomp @entries; #sort by dn

    for (my $i=0; $i<$numofentries; $i++) {
	my $icon="middle";
        if ($i==$numofentries-1) {
	    #choose proper icon
	    $icon="last";
        }
	my $entry = $entries[$i];
	$html .=          "<TR>";

	for (my $j=$basedn_cnt; $j<$depth; $j++) {
	    if ($$final[$j] ne "last") {
		#usually a single line is printed as spacer
		$html .=          "<TD valign=\"middle\" ><IMG src=\"/images/folder_spacer.gif\" width=\"18\" height=\"20\"></TD>";
	    }
	    else {
		#print a blank for the last entries
		$html .=          "<TD valign=\"middle\" ><IMG src=\"/images/spacer.gif\" width=\"18\" height=\"20\"></TD>";
	    }
	}
	
	$html .=              "<TD valign=\"middle\" >";
	my $treelink=$confParam->{cgi_path}."/ldap_browser.pl?sessionID=$sessionID&tab=$tab&stab=$stab&filter1=$filter[0]&filter2=$filter[1]";
	my $temp;
	$entry->dn=~/^([^=]+=[^,]+),/;
	my $utf8attr=$1;
	my $attribute=$utf8attr;
	if (($depth<$treedepth) and (${$dn_elems}[@$dn_elems-$depth-1] eq $utf8attr)) {
	#recursion for next deeper level: if there can be a deeper one _and_ if we are at the right height
        $$final[$depth]=$icon;
        $temp  = $this->create_tree($dn_elems,$treedepth,$basedn_cnt,$depth+1,$filter,$final);
        if ($temp ne -1) {
	    #the leading pic is created here - if there is no deeper nesting its not a link
	    $html .=          "<A href=\"$treelink&searchdn=".
		uri_escape(join(",",@{$dn_elems}[$treedepth-$depth..@$dn_elems-1]))."\">";
	    $html .=            "<IMG src=\"/images/folder_opened_$icon.gif\" border=\"0\" width=\"18\" height=\"20\">";
	    $html .=          "</A>";
        }
        else {
	    $html .=            "<IMG src=\"/images/folder_nothing_$icon.gif\" border=\"0\" width=\"18\" height=\"20\">";
        }
    }
    else {
	#otherwise the closed foldericon is shown
        $temp  = -1; #no string to be added
        $html .=            "<A href=\"$treelink&searchdn=".uri_escape("$utf8attr,$depthbase")."\">";
        $html .=              "<IMG src=\"/images/folder_closed_$icon.gif\" border=\"0\" width=\"18\" height=\"20\">";
        $html .=            "</A>";
    }
    $html .=            "</TD>";
    $html .=            "<TD ".&colspan("html",$treedepth-$depth+1).">";
    $html .=              "<A href=\"$treelink&searchdn=".uri_escape("$utf8attr,$depthbase")."&master=attributes\">$attribute";
    $html .=            "</TD>";
    $html .=          "</TR>";
    if ($temp ne -1) {
        $html .= $temp; #string from the recursion
    }
}

if ($numofentries==0) { #in the case of no results -1 is returned instead of the html-string
    return -1;
}

return $html;
}

#---------------------------------------------------------------------------------------------------

sub attributes {

    use Data::Dumper;

    my $this      = shift;
    my $mode      = shift; #show entries: mode="output" OR change entries: mode="change"
    my $ldap      = $this->{"ldap"};
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message   = ($session->getLanguage($confParam->{LANG},"LDAP"))->data();
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    my $html="";
    my $errs=""; #errormessage
    my $mesg=""; #dummy for the object returned by ldap->...
    my $searchdn=$cgi->param("searchdn");

    my $utf8val; #holds UTF8 value
    
    my $result = $ldap->search(base=>"$searchdn",scope=>"base",filter=>"objectclass=*");
    #search for the attributes of the dn
    if ($result->code != 0) {
	$errs = "search $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs,$this->{"url"});
	$ldap->unbind;   # take down session
	exit;
    }
    my $entry = $result->entry(0); #object found in search - contains all attributes
    my $value;
    my $changes=0;
    my %deletes=();
    
    if ($mode eq "change") {
	$html .=              $cgi->Tr({},$cgi->td({-colspan=>"5"},"&nbsp;"));
    }
    
    $searchdn=~/[\w=]+,(.+)/; #extract the old search DN, from the last query
    my $link = $confParam->{"cgi_path"}."/ldap_browser.pl?sessionID=$sessionID&tab=$tab&stab=$stab&filter1=".
		uri_escape($cgi->param("filter1"))."&filter2=".
		uri_escape($cgi->param("filter2"));

    my $link2= $link."&searchdn=".uri_escape($searchdn)."&master=attributes";

    $link.= "&searchdn=".uri_escape($1);
    
    #to omit javascript history.back, the links are generated

    foreach my $attr ($entry->attributes) {
	#parse all attributes
	my $i=0; #value counter
	my $act_pos=0;
	my @values=(); #values of the attributes
	my @newvalues=(); #values from the form
	foreach $utf8val ($entry->get_value($attr)) {
	    #parse all values of the attribute
	    $value = $utf8val;
	    if ($mode eq "output") {
		#show all attributes and their values
		$html .=        $cgi->start_Tr({});
		$html .=          $cgi->td({-align=>"left",-valign=>"middle"},$attr);
		if (&check_bin($utf8val,$attr)) {
		    #binary value => no delete!
		    $html .=        $cgi->td({},
					     "<input type=\"text\" name=\"$attr$act_pos\" size=\"50\" value='--BINARY_$act_pos--' readonly>",
					     $cgi->hidden(-name=>"DEL_--BINARY_$act_pos--:$attr",-value=>"N")
					     );
		}
		else {
		    $value =~ s/'/\\'/g; #escape single-quotes
		    my $delval="DEL_$value:$attr";
		    #$delval=~s/'/\\'/g;
		    $html .=        $cgi->td({},
					     "<input type=\"text\" name=\"$attr$act_pos\" size=\"50\" value='$value'>",
					     #unfortunately CGI produces "" around each argument so we cant escape " and cant use CGI neither
					     #$cgi->checkbox(-name=>$delval,-value=>"Y",-label=>$message->{"delete"})
					     "<input type=\"checkbox\" name='$delval' value=\"Y\" label=\"".$message->{"delete"}."\">".$message->{"delete"}
					     );
		}
		$html .=        $cgi->end_Tr;
	    }
	    elsif ($mode eq "change") {
		#change values
		if ((defined $cgi->param("DEL_$value:$attr")) and ($cgi->param("DEL_$value:$attr") eq "Y")) {
		    #check whether this attribute and value should be deleted
		    if ($deletes{$attr}) {
			push @{$deletes{$attr}},$utf8val;
		    }
		    else {
			$deletes{$attr}=[$utf8val];
		    }
		    $html .=        $cgi->start_Tr({});
		    $html .=          $cgi->td({-align=>"left"},$attr.":");
		    $html .=          $cgi->td({-align=>"left"},$value);
		    $html .=          $cgi->td({-align=>"center",-colspan=>"2"},$message->{"ldap_deleted"});
		    $html .=          $cgi->td({-align=>"center"},$message->{"succ_completed"});
		    $html .=        $cgi->end_Tr;
		    $changes++; #did anything change?
		    $i--;
		}
		else {
		    #otherwise put old and possibly new values of the attribute in arrays
		    push @values,$utf8val;
		    push @newvalues,$cgi->param($attr.$act_pos);
		}
	    }
	    $i++; #value counter
	    $act_pos++;
	} #end foreach value of one attribute

	if ($mode eq "change") {
	    @values=sort(@values); 
	    @newvalues=sort(@newvalues);
	    for (my $j=0; $j<$i; $j++) { 
		#foreach value of the attribute
		$utf8val = $value = $values[$j];
		if (($value ne $newvalues[$j]) and (!&check_bin($values[$j],$attr))) {
		    #compare sorted arrays exclude binary fields
		    if ($i == 1) {
			#ldap replace for attributes with only one value
			$utf8val=$newvalues[$j];
			$mesg=$ldap->modify($entry, replace=>{$attr=>$utf8val});
			#then create the attribute again with its new value
			if ($mesg->code != 0) {
			    $errs = "modify (replace) $message->{failed}\n\n";
			    $errs .= &LDAPerror($mesg);
			    $this->display_error($errs,$link2);
			    $ldap->unbind;   # take down session
			    exit;
			}
		    }
		    else {
			#ldap delete and add for multiple values
			$mesg=$ldap->modify($entry, delete=>{$attr=>$utf8val});
			#first delete the attribute and value
			if ($mesg->code != 0) {
			    $errs = "modify (delete) $message->{failed}\n\n";
			    $errs .= &LDAPerror($mesg);
			    $this->display_error($errs,$link2);
			    $ldap->unbind;   # take down session
			    exit;
			}
			$utf8val = $newvalues[$j];
			$mesg=$ldap->modify($entry, add=>{$attr=>$utf8val});
			#then create the attribute again with its new value
			if ($mesg->code != 0) {
			    $errs = "modify (add) $message->{failed}\n\n";
			    $errs .= &LDAPerror($mesg);
			    $this->display_error($errs,$link2);
			    $ldap->unbind;   # take down session
			    exit;
			}
		    }
		    $html .=        $cgi->start_Tr({});
		    $html .=          $cgi->td({-align=>"left"},$attr.":");
		    $html .=          $cgi->td({-align=>"left"},$value);
		    $html .=          $cgi->td({-align=>"center"},"=>");
		    $html .=          $cgi->td({-align=>"left"},$newvalues[$j]);
		    $html .=          $cgi->td({-align=>"center"},$message->{"succ_completed"});
		    $html .=        $cgi->end_Tr;
		    $changes++; #did anything change?
		}
	    }
	}

    } #end foreach attribute

    if ($mode eq "output") {
	#a new valid attribute with a value can be entered here
	$html .=            $cgi->Tr({},$cgi->td({-align=>"left",-colspan=>"2"},$message->{"new"}.":"));
	$html .=            $cgi->start_Tr({});
	$html .=              $cgi->td({-align=>"left",-valign=>"middle"},"<input type=\"text\" name=\"ADDITIONALATTR\" size=\"15\" value=\"\">");
	$html .=              $cgi->td({-align=>"left",-valign=>"middle"},"<input type=\"text\" name=\"ADDITIONALVALUE\" size=\"50\" value=\"\">");
	$html .=            $cgi->end_Tr({});
	$html .=            $cgi->Tr({},
				     $cgi->td({},"&nbsp;"),
				     $cgi->td({-align=>"left"},
					      $cgi->button(-class=>"button",-value=>$message->{"ldap_tree"},-onClick=>"location.href='$link'"),
					      $cgi->reset(-class=>"button",-name=>$message->{"reset"}),
					      $cgi->submit(-class=>"button",-name=>"subattr",-value=>$message->{"ldap_submit_attrib"}),
					      $cgi->hidden(-name=>"searchdn",-value=>"$searchdn"),
					      $cgi->hidden(-name=>"filter1",-value=>$cgi->param("filter1")),
					      $cgi->hidden(-name=>"filter2",-value=>$cgi->param("filter2"))
					      )
				     );
    }
    elsif ($mode eq "change") {
	if ($changes>0 and values(%deletes)) {
	    #marked values are going to be deleted
	    $mesg=$ldap->modify($entry, delete=>\%deletes);
	    if ($mesg->code != 0) {
		$errs = "modify (delete) $message->{failed}\n\n";
		$errs .= &LDAPerror($mesg);
		$this->display_error($errs,$link2);
		$ldap->unbind;   # take down session
		exit;
	    }
	}
	if ((defined $cgi->param("ADDITIONALATTR")) and ($cgi->param("ADDITIONALATTR") ne "")) {
	    #the additional values from above are going to be added
	    $utf8val = $cgi->param("ADDITIONALVALUE");
	    $mesg=$ldap->modify($entry, add=>{$cgi->param("ADDITIONALATTR")=>$utf8val});
	    if ($mesg->code != 0) {
		$errs = "modify (add) $message->{failed}\n\n";
		$errs .= &LDAPerror($mesg);
		$this->display_error($errs,$link2);
		$ldap->unbind;   # take down session
		exit;
	    }
	    $html .=          $cgi->start_Tr({});
	    $html .=            $cgi->td({-align=>"left"},$cgi->param("ADDITIONALATTR").":");
	    $html .=            $cgi->td({-align=>"left"},$cgi->param("ADDITIONALVALUE"));
	    $html .=            $cgi->td({-align=>"center",colspan=>"2"},$message->{"ldap_added"});
	    $html .=            $cgi->td({-align=>"center"},$message->{"succ_completed"});
	    $html .=          $cgi->end_Tr({});
	    $changes++;
	}
	if ($changes==0) {
	    #produce an output even if there were no changes
	    $html .=          $cgi->start_Tr({});
	    $html .=             $cgi->td({-align=>"center",-colspan=>"5"},$message->{"ldap_nochange"});
	    $html .=          $cgi->end_Tr({});
	}

	$html .=            $cgi->Tr({},$cgi->td({-colspan=>"5"},"&nbsp;"));
	$html .=            $cgi->start_Tr({});
	$html .=               $cgi->td({-align=>"center",-colspan=>"5"},
					$cgi->button(-class=>"button",-value=>$message->{"ldap_tree"},-onClick=>"location.href='$link'"),
					$cgi->button(-class=>"button",-value=>$message->{"ldap_back"},-onClick=>"location.href='$link2'")
					);
	$html .=            $cgi->end_Tr({});
    }
    
    return $html;
}

####################################################################################################
####################################################################################################
####################################################################################################

sub colspan {
    #a simple help function, that checks whether or not the colspan tag should be added
    my $type=shift; #either a hash or a html-string can be created
    my $colspan=shift; #number of columns to cover
    if ($type eq "hash") {
	my $hashref=shift;
	if ($colspan!=0 && $colspan!=1) {
	    #if a hash should be returned, the rest of the hash can be passed as parameter
	    $hashref->{"-colspan"}=$colspan;
	}
	return $hashref;
    }
    elsif ($type eq "html") {
	if ($colspan!=0 && $colspan!=1) {
	    #colspan is only sensible if 2 or more columns are covered
	    return " colspan=\"$colspan\" ";
	}
	else {
	    return "";
	}
    }
}


sub check_bin {

    if ((defined $_[1]) and ($_[1]=~/.+;binary/)) {
	#attribute name indicates binary value
	return 1;
    #String contains a character between 1 and 31 except HT,LF,VT,FF,CR 
    #} elsif ($_[0]=~/([\x01-\x1f])/ and $1=~/[^\x09-\x0d]/) {
    } elsif ($_[0]=~/([\x01-\x1f])/) {
	return 1;
    } else {
	return 0;
    }
}


sub entrycomp {
    $a->dn cmp $b->dn;
}

1;


=pod
    
    =head1 NAME
    
    ldap_browser.pm - Browser for any LDAP structure

    =head1 ABSTRACT
    At first, the tree of the base DN is shown. You can easily
    dive into subdirectories by clicking on the plus icon.
    Attributes can be changed by clicking on the corresponding
    link.

    =head2 Subtrees
    The browser is able only to show one nested structure at
    once. So if you branch several times in one tree all
    this branches will be hidden when you click at another
    root.  If the tree doesn't contain any subfolders, the
  plus will be replaced by a single vertical line. If
  subfolders are shown, the plus will be replaced by a
  minus. If the minus is clicked, the expanded tree will
  shrink again.

=head2 Attributes
  If you have clicked on one link, all belonging
  attributes are shown. If one attribute has more than
  one values, it's shown multiple times. Each textfield
    contains one value. After each textfield there is a
    checkbox, with which the entry can be deleted.
    The last two textfields are for additional values. Every
    valid attribute and value can be entered. If an
    attribute is entered, which already exists, an
    an additional entry for this attribute is created.

    =head1 SYNOPSIS
    
    use ldap_browser;

my $menu = new ldap_browser($cgi, $session)

    $menu->display();

=head1 AUTHOR

    Thomas Barthel, <shad@suse.de>

    =head1 SEE ALSO

    =cut
