#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: mail_dist_ext.pm,v 1.1.1.1 2003/03/25 10:31:39 ray Exp $
#
package mail_dist_ext;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use Session;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };

@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);

    return bless $self, $this;
}

sub display {
    my $this = shift;
    my $cgi = $this->{"cgi"};

    my $add     = $cgi->param("add");
    my $edit    = $cgi->param("edit");
    my $delete  = $cgi->param("delete");
    my $addsubs = $cgi->param("add_subs");
    my $modsubs = $cgi->param("mod_subs");

    if( defined $add && $add ne "" ) {
	$this->add();
    } elsif( defined $addsubs && $addsubs ne "" ) {
	$this->add_subs();
    } elsif( defined $edit && $edit ne "" ) {
	$this->edit();
    } elsif( defined $modsubs && $modsubs ne "" ) {
	$this->mod_subs();
    } elsif( defined $delete && $delete ne "" ) {
	$this->delete();
    } else {
	$this->startup();
    }
}

sub startup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    my $ldap = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);

    if (!defined $ldap) {
	$this->display_error($message->{"ldap_connect_error"},$this->{"url"});
	exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    my $result=$ldap->search(base=>$confParam->{"baseDN"},
			     scope=>"one",
			     filter=>"(&(fn=".$cgi->param("Folder").")(objectclass=SuSEIMAPFolderObject)(mailenabled=REJECT))");

    #check if sfolder can receive mail
    if ($result->code != 0) {
	my $errs = "search: $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs,$this->{"url"});
	$ldap->unbind;   # take down session
	exit;
    }
    if ($result->count>0) {
	$this->display_error($message->{dist_cannot_receive},
			     $confParam->{cgi_path}."/edit_sfolder.pl?sessionID=$sessionID&tab=$tab&stab=$stab"
			     );
	$ldap->unbind;   # take down session
	exit;
    }
    
    $result=$ldap->search(base=>$confParam->{"baseDN"},
			  scope=>"one",
			  filter=>"(&(fn=".$cgi->param("Folder").")(objectclass=SuSEIMAPFolderObject))",
			  attrs=> [ 'deliverToUID' ]);
    
    if ($result->code != 0) {
	my $errs = "search: $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs,$this->{"url"});
	$ldap->unbind;   # take down session
	exit;
    }

    my @ext_list;
    foreach my $entry ($result->all_entries) {
	my @attrs = $entry->get_value("deliverToUID");
	foreach my $val ( @attrs ) {
	    #print STDERR "<$val>\n";
	    next if $val !~ /\@/;
	    push @ext_list, $val;
	}
    }

    my $html = "";
    my $folder = $cgi->param('Folder');
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/mail_dist_ext.pl");
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=         $cgi->Tr($this->help_th(3, "$message->{dist_ext}", "MAIL_DIST_EXT"));
    $html .=         $cgi->Tr( $cgi->td({-class=>'AdminSubHeader',-colspan=>"3"},
                                        $message->{"dist_affected_folder"}.": <i>".$cgi->param("Folder")."</i>"));
    $html .=         $cgi->Tr( $cgi->td({-align=>"center", -colspan=>"3"},
					$cgi->scrolling_list(-name=>"ext_subscriber",
							     -value=>[@ext_list],
							     -multiple=>'true',
							     -override=>1,
							     -size=>20
							     )
					));
    $html .=         $cgi->Tr(
			      $cgi->td({-align=>"center"},
				       $cgi->submit(-class=>"button",-name=>"add",
						    -value=>"$message->{dist_add}",
						    -width=>10)
				       ),
			      $cgi->td({-align=>"center"},
				       $cgi->submit(-class=>"button",-name=>"edit",
						    -value=>"$message->{dist_edit}",
						    -width=>10)
				       ),
			      $cgi->td({-align=>"center"},
				       $cgi->submit(-class=>"button",-name=>"delete",
						    -value=>"$message->{dist_delete}",
						    -width=>10)
				       )
			      );
    $html .=         $cgi->hidden(-name=>"Folder",-value=>$cgi->param("Folder"));
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");
    $html .=         $cgi->hidden(-name=>"lang", -value=>$confParam->{LANG});
    
    $html .=       $cgi->end_table();
    $html .=       $cgi->end_form();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}


sub add {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();
    
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    
    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    my $html = "";
    my $folder = $cgi->param('Folder');
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/mail_dist_ext.pl");
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=         $cgi->Tr($this->help_th(1, "$message->{dist_add_new}", "MAIL_DIST_EXT"));
    $html .=         $cgi->Tr( $cgi->td({-class=>'AdminSubHeader',-colspan=>"1"},
                                        $message->{"dist_affected_folder"}.": <i>".$cgi->param("Folder")."</i>"));
    $html .=         $cgi->Tr( $cgi->td({-align=>"center", -colspan=>"1"},
					$cgi->textfield(-name=>"new_subscriber",
							-value=>"",
							-override=>1,
							-size=>40
							)
					));
    $html .=         $cgi->Tr(
			      $cgi->td({-align=>"center"},
				       $cgi->submit(-class=>"button",-name=>"add_subs",
						    -value=>"$message->{dist_add}",
						    -width=>20)
				       )
			      );
    $html .=         $cgi->hidden(-name=>"Folder",-value=>$cgi->param("Folder"));
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");
    $html .=         $cgi->hidden(-name=>"lang", -value=>$confParam->{LANG});
    
    $html .=       $cgi->end_table();
    $html .=       $cgi->end_form();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
} 

sub add_subs {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();
    
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $Folder    = $cgi->param("Folder");
    my $newsubs   = $cgi->param("new_subscriber");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    if( ! defined $newsubs || $newsubs eq "" ) {
        $this->display_error("$message->{dist_add_empty}",$this->{"url"});
        exit;
    }

    if( $newsubs !~ /\@/ ) {
        $this->display_error("$message->{dist_add_invalid}",$this->{"url"});
        exit;
    }
    
    my $ldap = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);
    
    if (!defined $ldap) {
	$this->display_error($message->{"ldap_connect_error"},$this->{"url"});
	exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    $mesg = $ldap->modify("fn=$Folder,".$confParam->{"baseDN"},
			  changes=> [ add => [ deliverToUID => "$newsubs" ]]
			  );

    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    } else {
	$this->display_msg("$newsubs $message->{dist_ext_added}",$this->{"url"},1);
	exit;
    }

} 

sub delete {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();
    
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $Folder    = $cgi->param("Folder");
    my @todelete  = $cgi->param("ext_subscriber");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    if( ! defined @todelete ) {
        $this->display_error("$message->{dist_nothing_selected}",$this->{"url"});
        exit;
    }
    
    my $ldap = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);
    
    if (!defined $ldap) {
	$this->display_error($message->{"ldap_connect_error"},$this->{"url"});
	exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    my @modop;
    foreach my $subs ( @todelete ) {
	push @modop, "delete", [ 'deliverToUID', [ "$subs" ] ];
    }
    
    $mesg = $ldap->modify("fn=$Folder,".$confParam->{"baseDN"},
			  changes=> \@modop
			  );
    
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    } else {
	$this->display_msg("$message->{dist_deleted}",$this->{"url"},1);
	exit;
    }
    
} 

sub edit {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();
    
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $Folder    = $cgi->param("Folder");
    my @toedit    = $cgi->param("ext_subscriber");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    if( ! defined @toedit || ! defined $toedit[0] ) {
        $this->display_error("$message->{dist_nothing_selected}",$this->{"url"});
        exit;
    }
    
    my $html = "";
    my $folder = $cgi->param('Folder');
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/mail_dist_ext.pl");
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>1, -width=>"100%"});
    $html .=         $cgi->Tr($this->help_th(2, "$message->{dist_edit_selected}", "MAIL_DIST_EXT"));
    $html .=         $cgi->Tr( $cgi->td({-class=>'AdminSubHeader',-colspan=>"2"},
                                        $message->{"dist_affected_folder"}.": <i>".$cgi->param("Folder")."</i>"));

    foreach my $addr ( @toedit ) {

	$html .=         $cgi->Tr(
				  $cgi->td({-align=>"left", -width=>"30%"}, "$addr"),
				  $cgi->td({-align=>"left", -width=>"70%"},
					   $cgi->textfield(-name=>"modify_$addr",
							   -value=>"$addr",
							   -override=>1,
							   -size=>"50"
							   )
					   )
				  );
    }

    $html .=         $cgi->Tr(
			      $cgi->td({-align=>"center", -colspan=>"2"},
				       $cgi->submit(-class=>"button",-name=>"mod_subs",
						    -value=>"$message->{dist_submit}",
						    -width=>20)
				       )
			      );
    $html .=         $cgi->hidden(-name=>"Folder",-value=>$cgi->param("Folder"));
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");
    $html .=         $cgi->hidden(-name=>"lang", -value=>$confParam->{LANG});
    
    $html .=       $cgi->end_table();
    $html .=       $cgi->end_form();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
    
} 

sub mod_subs {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();
    
    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");
    my $Folder    = $cgi->param("Folder");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_ext.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    my $ldap = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);
    
    if (!defined $ldap) {
	$this->display_error($message->{"ldap_connect_error"},$this->{"url"});
	exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    my @modop;
    foreach my $param ( keys %{$cgi->Vars()} ) {
	if( $param =~ /^modify_(.*)/ ) {
	    my $oldval = $1;
	    my $newval = $cgi->param("$param");
	    #print STDERR "<$oldval> -> <$newval>\n";
	    push @modop, "delete", [ 'deliverToUID', [ "$oldval" ] ];
	    push @modop, "add",    [ 'deliverToUID', [ "$newval" ] ];
	}
    }
    
    $mesg = $ldap->modify("fn=$Folder,".$confParam->{"baseDN"},
			  changes=> \@modop
			  );
    
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    } else {
	$this->display_msg("$message->{dist_modified}",$this->{"url"},1);
	exit;
    }

} 

1;
