#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: mail_dist_group.pm,v 1.3 2004/09/30 10:15:40 varkoly Exp $
#
package mail_dist_group;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use Session;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };

use Filterlist;
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message    = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    $this->{"ldap"} = Net::LDAP->new($confParam->{"LDAPserver"}, port=>$confParam->{"LDAPport"}, version => 3);
    my $ldap      = $this->{"ldap"};
    $ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"});

    my $filter="";
    if ((defined $cgi->param("FILTER")) and ($cgi->param("FILTER") ne "")) {
      $filter=$cgi->param("FILTER");
      $filter = LDAP_filter_escape($filter);
    }
    my $url=$confParam->{cgi_path}."/mail_dist_group.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    $this->{"url"}=$url;
    my $width=30;

    
    my $result=$ldap->search(base => $confParam->{"baseDN"},
			     scope => "one",
			     filter => "(&(fn=".$cgi->param("Folder").
				       ")(objectclass=SuSEIMAPFolderObject)(mailenabled=REJECT))"
			    );
    #check if sfolder can receive mail
    if ($result->code != 0) {
	my $errs = "search: $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs,$this->{"url"});
	$ldap->unbind;   # take down session
	exit;
    }
    if ($result->count>0) {
	$this->display_error($message->{dist_cannot_receive},
			     $confParam->{cgi_path}."/edit_sfolder.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
	$ldap->unbind;   # take down session
	exit;
    }

###### apply changes ##
    if ((defined $cgi->param("UN_SUBSCRIBE")) and ($cgi->param("GROUPS") ne $message->{"dist_filter_first"})) {
    #"apply changes" was pressed and its a real selection (not "set filter")
	my $oldfilter=$cgi->param("oldfilter"); #last filter
	my @defaults=(); #old setting
        my @newbies=$cgi->param("GROUPS"); #new setting

	# make a hash from @newbies, so it's easier to check 
	# if some group is in @newbies
	my %newbies_hash = ();
	foreach my $newbie (@newbies){
	    $newbies_hash{$newbie} = 1;
	}
 
	$result=$ldap->search(base=>$confParam->{"baseDN"},
      	  		    scope=>"one",
      	  		    filter=>"(&(fn=".$cgi->param("Folder").
      	  			    ")(objectclass=SuSEIMAPFolderObject)".
      	  			    "(mailDeliveryProgram=*grpdeliver*-g*))",
      	  		    attrs=>['mailDeliveryProgram']
      	  		   );
      	if ($result->code != 0) {
	    my $errs = "search: $message->{failed}\n\n";
      	    $errs .= &LDAPerror($result);
      	    $this->display_error($errs,$this->{"url"});
      	    $ldap->unbind;   # take down session
      	    exit;
      	}
      	if ($result->count>0) {
	    my @dummy=$result->entries;
      	    foreach my $j ($dummy[0]->get_value("mailDeliveryProgram")) {
      	    #find groups that are subscribed to the folder
		if ($j=~/-g ([-\.\w]+)/) {
		    push @defaults,$1;
      	      	}
      	    }
      	} else {
	    @defaults=();
      	}
      	#get all groups that were displayed in the filterlist
      	my $res_grps = $ldap->search( base => $confParam->{'baseDN'},
      	  			   scope => "one",
      	  			   filter => "(&(objectClass=posixGroup)(cn=$oldfilter))",
				   attrs => ['cn']
      	  			 );
      	if ($res_grps->code != 0) {
	    my $errs = "search: $message->{failed}\n\n";
      	    $errs .= &LDAPerror($result);
      	    $this->display_error($errs,$this->{"url"});
      	    $ldap->unbind;   # take down session
      	    exit;
      	}
	my %displayed_groups = ();
	my @entries = $res_grps->entries();
	foreach my $entry (@entries){
	    $displayed_groups{$entry->get_value('cn')} = 1;
	    #push @displayed_groups, $entry->get_value("cn");  
	}
	# now, from the list of subscribed groups, and the list of the display groups
	# get the displayed groups, that were subscribed to the folder
	my @displayed_subs = ();
	my %disp_subs_hash = ();
	foreach my $subscribed (@defaults){
	    if( defined $displayed_groups{$subscribed} ){
		push @displayed_subs, $subscribed;
		$disp_subs_hash{$subscribed}=1;
	    }
	}
	# all groups which are in @displayed_subs but not in @newbies have to be 
	# unsubscribed from the shared folder
	my $baseDN = $confParam->{'baseDN'};

	foreach my $i (@displayed_subs) {
	    if (! defined $newbies_hash{$i} ){
		$result=$ldap->modify("fn=".$cgi->param("Folder").",".$confParam->{"baseDN"},
				       delete=>{ "mailDeliveryProgram" => "\"|/usr/bin/grpdeliver ".
									  "-l ldap -b \\\"$baseDN\\\" -g $i\""}
				     );
		if ($result->code != 0) {
		    my $errs = "modify: $message->{failed}\n\n";
		    $errs .= &LDAPerror($result);
		    $this->display_error($errs,$this->{"url"});
		    $ldap->unbind;   # take down session
		    exit;
		}
	    }
	}
	# all groups which are in @newbies but not in @displayed_sub have to be 
	# subscribed to the shared folder
	foreach my $i (@newbies) {
	    if(! defined $disp_subs_hash{$i} ) {
            #a new one is not member of the old set => add him
		if (!$this->member_chk($i)) {
		    $result=$ldap->modify("fn=".$cgi->param("Folder").",".$confParam->{"baseDN"},
					  add=>{"mailDeliveryProgram"=>"\"|/usr/bin/grpdeliver -l ldap ".
								       "-b \\\"$baseDN\\\" -g $i\""}
					 );
		    if ($result->code != 0) {
	                my $errs = "modify: $message->{failed}\n\n";
			$errs .= &LDAPerror($result);
	                $this->display_error($errs,$this->{"url"});
			$ldap->unbind;   # take down session
			exit;
		    }
		} else {
		    $this->display_error($message->{"dist_member_grp"},$this->{"url"});
		    $ldap->unbind;   # take down session
		    exit;
		}
	    }
	}
    }
    elsif ((defined $cgi->param("UN_SUBSCRIBE")) and (!defined $cgi->param("FILTERBUTTON"))) {
    #delete filter variable if "UN_SUBSCRIBE" instead of "filter" was pressed
      $filter="";
    }
#######################
    my $filterlist = Filterlist->new($cgi, $session, $ldap);
    
    my $actions = $cgi->start_table({-border=>0,-width=>"100%"}).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->submit(-class=>"button",-name=>"UN_SUBSCRIBE",-value=>$message->{"commit"},-width=>($width * 7))
                )
            ).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->reset(-class=>"button",-name=>$message->{"reset"},-width=>($width * 7))
                )    
            ).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->button(-class=>"button",-value=>$message->{"dist_back"},-width=>($width * 7),
                                 -onclick=>"location.href='".$confParam->{cgi_path}.
                                           "/edit_sfolder.pl?sessionID=$sessionID&tab=$tab&stab=$stab'")
                        )
            ).
            $cgi->end_table();
    
    my $html = "";
    my $folder = $cgi->param("Folder");
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/mail_dist_group.pl");
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=         $cgi->Tr($this->help_th(3, $message->{"dist_grp"}, "MAIL_DIST_GROUP"));
    $html .=         $cgi->Tr($cgi->td({-class=>'AdminSubHeader', -colspan=>"3"},
                                       $message->{"dist_affected_folder"}.": <i>".$cgi->param("Folder")."</i>"));
    $html .=         $filterlist->make_filter_view({"textfield"    => "FILTER",   #name of textfield
                                                    "textlabel"    => $message->{'dist_filter'},
                                                    "buttonlabel"  => $message->{'dist_use_filter'},
                                                    "listname"     => "GROUPS",   #name of selectbox
                                                    "listlabel_left" => $message->{'user'},
                                                    "listlabel_right" => $actions,
                                                    "nofilter-msg" => $message->{"dist_filter_first"},
                                                                   #message if no filter has been set
                                                    "listfilter"   => "(&(objectclass=posixGroup)(!(gidnumber=103))(cn=$filter))",
						    "listattrs"	   => ['cn'],
                                                                   #ldap-filter for entries in selectbox
                                                    "width"        => $width,     #width
                                                    "height"       => 20,         #height
                                                    "multiple"     => "MULTIPLE", #multiple
                                                    "selectedfilter" => "(&(objectclass=SuSEIMAPFolderObject)".
								      "(fn=$folder)".
								      "(mailDeliveryProgram=*grpdeliver*-g*))",
                                                                  #ldap-rule for selected in selectbox
                                                    "error-url"    => "$url",     #url to return after errors
                                                    "selectedregex"=> ['mailDeliveryProgram','.*grpdeliver.*\-g ([-\.\w]+)'],
                                                    "charset"      => $this->{'charset'}
                     });
    $html .=         $cgi->hidden(-name=>"oldfilter",-value=>"$filter", -override=>1);
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"Folder",-value=>$cgi->param("Folder"));
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");
    $html .=         $cgi->hidden(-name=>"lang", -value=>$confParam->{LANG});
 
    $html .=       $cgi->end_table();
    $html .=       $cgi->end_form();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

####################################################################################################
####################################################################################################

sub member_chk {
    my $this = shift;

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $message    = ($session->getLanguage($confParam->{LANG},"MAIL_DIST_GROUP"))->data();

    my $ldap = $this->{"ldap"};
    
    my $group   = shift;
    my $return  = 0;

    my $folder  = ($this->{"cgi"})->param("Folder");
    
    my $result = $ldap->search(base=>$confParam->{"baseDN"},scope=>"one",filter=>"(&(cn=$group)(objectclass=PosixGroup))");
    if ($result->code != 0) {
      my $errs = "search: $message->{failed}\n\n";
      $errs .= &LDAPerror($result);
      $this->display_error($errs,$this->{"url"});
      $ldap->unbind;   # take down session
      exit;
    }
    my @dummy=$result->entries;

    foreach my $member ($dummy[0]->get_value("MemberUID")) {
      my $result2 = $ldap->search(base=>$confParam->{"baseDN"},scope=>"one",filter=>"(&(uid=$member)(objectClass=PosixAccount)(Alias=$folder))");
      if ($result2->code != 0) {
        my $errs = "search: $message->{failed}\n\n";
        $errs .= &LDAPerror($result2);
        $this->display_error($errs,$this->{"url"});
        $ldap->unbind;   # take down session
        exit;
      }
      if ($result2->count>0) {
        $return+=1;
      }
    }

    return $return;
}

1;
