#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: mail_dist_user.pm,v 1.2 2003/05/09 13:17:13 varkoly Exp $
#
package mail_dist_user;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use LDAPUtils;
use Display;
use Session;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };

use Filterlist;
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $sessionID  = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_DIST"))->data();

    my $tab       = $cgi->param("tab");
    my $stab      = $cgi->param("stab");

    $this->{"url"} = $confParam->{cgi_path}."/mail_dist_user.pl?sessionID=$sessionID&tab=$tab&stab=$stab&Folder=".$cgi->param("Folder");
    my $url = $this->{"url"}; #follow this url on errors

    my $ldap = Net::LDAP->new($confParam->{"LDAPserver"}, port => $confParam->{"LDAPport"}, version => 3);
    $this->{'ldap'} = $ldap;

    if (!defined $ldap) {
      $this->display_error($message->{"ldap_connect_error"},$this->{"url"});
      exit;
    }
    my $mesg=$ldap->bind("uid=".$confParam->{"uid"}.",".$confParam->{"baseDN"},password=>$confParam->{"passwd"}) ;
    if($mesg->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($mesg);
        $this->display_error($errs,$this->{"url"});
        exit;
    }

    my $filter="";
    if ((defined $cgi->param("filter")) and ($cgi->param("filter") ne "")) {
      $filter=$cgi->param("filter");
      $filter = LDAP_filter_escape($filter);
    }
    my $default;


    my $result=$ldap->search(base=>$confParam->{"baseDN"},
			     scope=>"one",
			     filter=>"(&(fn=".$cgi->param("Folder").")(objectclass=SuSEIMAPFolderObject)(mailenabled=REJECT))");

    #check if sfolder can receive mail
    if ($result->code != 0) {
      my $errs = "search: $message->{failed}\n\n";
      $errs .= &LDAPerror($result);
      $this->display_error($errs,$this->{"url"});
      $ldap->unbind;   # take down session
      exit;
    }
    if ($result->count>0) {
      $this->display_error($message->{dist_cannot_receive},
			   $confParam->{cgi_path}."/edit_sfolder.pl?sessionID=$sessionID&tab=$tab&stab=$stab"
			   );
      $ldap->unbind;   # take down session
      exit;
    }
    
###### apply changes ##
    if ((defined $cgi->param("UN_SUBSCRIBE")) and ($cgi->param("SUBSCRIBED") ne $message->{"dist_filter_first"})) {
    #"apply changes" was pressed and its a real selection (not "set filter")
	my $oldfilter=$cgi->param("oldfilter"); #last filter
      	my @defaults=(); #old setting
      	my @newbies=$cgi->param("SUBSCRIBED"); #new setting
      	
      	$result = $ldap->search(base => $confParam->{"baseDN"},
				scope => "one",
      	  		      	filter => "(&(objectclass=SuSEIMAPFolderObject)(deliverToUID=$oldfilter)(fn=".$cgi->param("Folder")."))",
			      	attrs => ['deliverToUID']
			       );
      	if ($result->code != 0) {
	    my $errs = "search: $message->{failed}\n\n";
      	    $errs .= &LDAPerror($result);
      	    $this->display_error($errs,$this->{"url"});
      	    $ldap->unbind;   # take down session
      	    exit;
      	}
      	my @dummy=$result->entries;
      	foreach my $i(@dummy){
	    push @defaults, $i->get_value("deliverToUID");
      	}
      	foreach my $i (@defaults) {
	    if ($i !~ /\@/ && !grep(/^$i$/,@newbies)) {
		#an old one is not member of the new set => delete him
		$result=$ldap->modify("fn=".$cgi->param("Folder").",".$confParam->{"baseDN"},
				      delete=>{"deliverToUID"=>$i});
		if ($result->code != 0) {
		    my $errs = "modify: $message->{failed}\n\n";
		    $errs .= &LDAPerror($result);
		    $this->display_error($errs,$this->{"url"});
		    $ldap->unbind;   # take down session
		    exit;
		}
	    }
      	}
      	foreach my $i (@newbies) {
	    if (!grep(/^$i$/,@defaults)) {
		#a new one is not member of the old set => add him
		if (!$this->member_chk($i)) {
		    $result=$ldap->modify("fn=".$cgi->param("Folder").",".$confParam->{"baseDN"},
					  add=>{"deliverToUID"=>$i});
		    if ($result->code != 0) {
			my $errs = "modify: $message->{failed}\n\n";
			$errs .= &LDAPerror($result);
			$this->display_error($errs,$this->{"url"});
			$ldap->unbind;   # take down session
			exit;
		    }
		}
		else {
		    $this->display_error($message->{"dist_member_usr"},$this->{"url"});
		    $ldap->unbind;   # take down session
		    exit;
		}
	    }
      	}
    }
    elsif ((defined $cgi->param("UN_SUBSCRIBE")) and (!defined $cgi->param("FILTERBUTTON"))) {
    #delete filter variable if "UN_SUBSCRIBE" instead of "filter" was pressed
      $filter="";
    }
#######################
    my $filterlist = Filterlist->new($cgi, $session, $ldap);
    my $actions = $cgi->start_table({-border=>0,-width=>"100%"}).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->submit(-class=>"button",-name=>"UN_SUBSCRIBE",-value=>$message->{"dist_submit"},-width=>"200"))
            ).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->reset(-class=>"button",-name=>$message->{"reset"},-width=>"200"))
            ).
            $cgi->Tr(
                $cgi->td({-valign=>"top",-align=>"center"},
                    $cgi->button(-class=>"button",-value=>$message->{"dist_back"},-width=>"200",
                                 -onclick=>"location.href='".$confParam->{cgi_path}.
                                           "/edit_sfolder.pl?sessionID=$sessionID&tab=$tab&stab=$stab'")
                        )
            ).
            $cgi->end_table();

    my $html = "";
    my $folder = $cgi->param('Folder');
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/mail_dist_user.pl");
    $html .=       $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=         $cgi->Tr($this->help_th(3, $message->{"dist_usr"}, "MAIL_DIST_USER"));
    $html .=         $cgi->Tr( $cgi->td({-class=>'AdminSubHeader',-colspan=>"3"},
                                        $message->{"dist_affected_folder"}.": <i>".$cgi->param("Folder")."</i>"));
    $html .=         $filterlist->make_filter_view({"textfield"    => "filter", #name textfield
                                                    "textlabel"    => $message->{'dist_filter'},
                                                    "buttonlabel"  => $message->{'dist_use_filter'},
                                                    "listname"     => "SUBSCRIBED", #name list
                                                    "listlabel_left" => $message->{'user'},
                                                    "listlabel_right" => $actions,
                                                    "nofilter-msg" => $message->{"dist_filter_first"}, #message for no set filter
                                                    "listfilter"   => "(&(objectclass=posixAccount)(!(gidNumber=103))(|(sn=$filter)(givenname=$filter)(uid=$filter)))",
						    "listattrs"	   => ['uid', 'cn'],
                                                    "width"        => 30, #width of textfield
                                                    "height"       => 20, #length of list
                                                    "multiple"     => "MULTIPLE",
                                                    "selectedfilter"=> "(&(objectclass=SuSEIMAPFolderObject)(!(gidNumber=103))(fn=$folder)(deliverToUID=$filter))",
						    "selectedregex" => ['deliverToUID', '.*'],
                                                    "labelexp"     => ["\$\$ ","uid","(\$\$) ","cn"],
                                                                      #ldap rule for defaults
                                                    "error-url"    => $this->{"url"}, #url for errors,
                                                    "charset"      => $this->{'charset'}
                     });
    $html .=         $cgi->hidden(-name=>"Folder",-value=>$cgi->param("Folder"));
    $html .=         $cgi->hidden(-name=>"oldfilter",-value=>"$filter", -override=>1);
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");
    $html .=         $cgi->hidden(-name=>"lang", -value=>$confParam->{LANG});
    
    $html .=       $cgi->end_table();
    $html .=       $cgi->end_form();
    $html .=     $cgi->end_td();
    $html .=   $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}


####################################################################################################
 
sub member_chk {
    my $this = shift;
 
    my $session    = $this->{"session"};
    my $confParam  = $session->{"confParam"};
    my $message    = ($session->getLanguage($confParam->{LANG},"MAIL_DIST_GROUP"))->data();
 
    my $ldap = $this->{"ldap"};

    my $folder = ($this->{"cgi"})->param("Folder");
    my $user   = shift;
    my $return  = 0;
 
    my $result = $ldap->search(base=>$confParam->{"baseDN"},
			       scope=>"one",
			       filter=>"(&(memberUid=$user)(objectclass=PosixGroup))");
    if ($result->code != 0) {
      my $errs = "search: $message->{failed}\n\n";
      $errs .= &LDAPerror($result);
      $this->display_error($errs,$this->{"url"});
      $ldap->unbind;   # take down session
      exit;
    }

    if ($result->count>0) {
      foreach my $dummy ($result->entries) {
        my $group=$dummy->get_value("cn");
        my $result2 = $ldap->search(base=>$confParam->{"baseDN"},
				    scope=>"one",
				    filter=>"(&(fn=$folder)(mailDeliveryProgram=*grpdeliver*-g ".
					    "$group*)(objectClass=SuSEIMAPFolderObject))");
        if ($result2->code != 0) {
          my $errs = "search: $message->{failed}\n\n";
          $errs .= &LDAPerror($result2);
          $this->display_error($errs,$this->{"url"});
          $ldap->unbind;   # take down session
          exit;
        }
        if ($result2->count>0) {
          $return+=1;
        }
      }
    }
 
    return $return;
}


1;
