# 
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: mail_filter.pm,v 1.2 2003/05/06 11:30:44 varkoly Exp $
#

package mail_filter;

use strict;
use Storable qw(thaw freeze);
use CGI qw/:standard *table start_ul -no_xhtml/;
use CGI::Carp qw(fatalsToBrowser);
use Net::IMAP;
use Net::LDAP;
use LDAPUtils;
use IMAPUtils;
use Display;
use ManageSieve;
use sieve;
use vars qw(@ISA $confParam);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };


@ISA = qw(Display);

sub new{
    my $this = shift;
    my $self = new Display(@_);

    return bless $self, $this;
}

####################################
# display
sub display{
    my $this = shift;
    $this->{'filtername'}= "filter.sieve";
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $sessionID = $confParam->{'sessionID'};

    my $username= $confParam->{"uid"};
    my $passwd=$confParam->{"passwd"};
    my $imapserver=$confParam->{"IMAPserver"};

    $this->{'HTMLOUT'} = "" ;
    my $mode = $q->param('mode');
    if (! defined $mode ){
        $mode = 'display';
    }

    if( ($mode eq 'display') || ($mode eq 'forget') ){
        $this->display_script();
    } elsif($mode eq 'modify') {
	$this->create_rule();
    } elsif($mode eq 'append'){
	$this->create_rule();
    } elsif ($mode eq 'delete'){
	my $sure = $q->param('sure');
	if( (defined $sure) && ($sure ne "" )){
	    my $rule_num = $q->param('rule_num');
	    my $filters = $this->read_filter_from_session();
	    splice @{$filters}, $rule_num, 1;
	    $this->write_filter_to_session($filters);
	    $this->submit_changes();
	}else{
	    $this->ask_del_confirm();
	}
    } elsif ($mode eq 'comment'){
        my $rule_num = $q->param('rule_num');
        my $filters = $this->read_filter_from_session();
        $filters->[$rule_num]->[5] = 1;
        $this->write_filter_to_session($filters);
        $this->submit_changes();
    } elsif ($mode eq 'uncomment'){
        my $rule_num = $q->param('rule_num');
        my $filters = $this->read_filter_from_session();
        $filters->[$rule_num]->[5] = 0;
        $this->write_filter_to_session($filters);
        $this->submit_changes();
    } elsif ($mode eq 'move'){
#        print STDERR "Move Rule!\n";
        $this->move_rule();
    } elsif ($mode eq 'submit'){
#        print STDERR "Mode:  SUBMIT\n";
        my $rule = $this->read_filter_from_form();
        my $filters = $this->read_filter_from_session();
        my $rule_num = $q->param('rule_num');
#        print STDERR "rule_num $rule_num\n";
    
        $filters->[$rule_num] = $rule;
       
#        print STDERR "$filters";
        $this->write_filter_to_session($filters);
        $this->check_filter($rule_num);
        $this->submit_changes(); 
    }

}
# end display
###################################

###################################
#
sub display_script(){
    my $this = shift;
    my $q = $this->{'cgi'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    if( $this->load_script_to_session() < 0){
	$this->display_msg($message->{'procmail_unable_to_parse'}, $confParam->{'cgi_path'}.'/edit_sieve.pl?'.
			   "sessionID=$sessionID&tab=sieve&stab=sieve_edit",3);
	exit;
    }
    $session->load($sessionID);
    my $filter_ref = $this->read_filter_from_session();
    my @filterrules = @{$filter_ref};
    my $HTMLOUT ="";
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');

    #No rules defined at the moment -> display default screen
    if( @filterrules == 0){
        $HTMLOUT .= "\n".$q->start_table({-class=>'AdminBorder', -cellspacing => 2, -width=>"100%", -cellpadding=>0}).
            $q->start_Tr().
                $q->start_td().
                    $q->start_table({-cellspacing=>0, -width=>"100%",-cellpadding=>3}).
                        $q->Tr([$this->help_th(1, $message->{'procmail_no_filter_rules'},'MAILFILTER' )]).
                        $q->start_Tr().
                            $q->start_td({-class=>'AdminSubHeader'}).
                                $message->{procmail_no_rule_defined}.
                            $q->end_td().
                        $q->end_Tr().
                        $q->start_Tr().
                            $q->start_td({-align=>"right", -valign=>"middle"}).
                                $q->start_form(-action=>$confParam->{cgi_path}.'/mail_filter.pl').
                                    hidden(-name=>'sessionID',
                                            -value=>$sessionID,
                                            -override=>1).
                                    hidden(-name=>'rule_num',
                                            -value => 0,
                                            -override => 1).
                                    hidden(-name=>'tab',
                                            -value=>$tab,
                                            -override=>1).
                                    hidden(-name=>'stab',
                                            -value=>$stab,
                                            -override=>1).
                                    hidden(-name=>'mode',
                                            -value => 'append',
                                            -override => 1).
                                    submit(-value => $message->{'procmail_rule_append'}).
                                $q->end_form.
                            $q->end_td.
                        $q->end_Tr.
                    $q->end_table.
                $q->end_td.
            $q->end_Tr.
        $q->end_table;
	$this->SUPER::display($HTMLOUT);
        exit;
    }

    # Display the filterrules
    my $count=0;
    $HTMLOUT .= "\n".$q->start_table({-class=>'AdminBorder', -cellspacing => 2, -width=>"100%", -cellpadding=>0});
    $HTMLOUT .= $q->start_Tr().
        $q->start_td().
            $q->start_table({-cellspacing => 0, -width=>"100%", -cellpadding=>3}).
                $q->Tr( [ $this->help_th(6,$message->{'procmail_filter_rules'},'MAILFILTER') ] ).    
                $q->start_Tr().
                    $q->td({-class=>'AdminSubHeader', -colspan => 6}, [
                        $message->{'procmail_disp_inactive'}
                    ]).
                $q->end_Tr().    
            $q->start_Tr.
                $q->td({-colspan=>6 , -align=>'center'},[ 
                    $q->hr({-noshade => '1', -size => '1', -width=>'0%'}) 
                ]).
            $q->end_Tr;    

    foreach my $rule (@filterrules){
        my $conj_op =  ($rule->[3] eq 'anyof') ? 
                            $message->{'procmail_rule_conj_anyof'} : 
                            $message->{'procmail_rule_conj_allof'};
        my $bgcolor = ($rule->[5] ? '#c0c0c0' : '#e9e9e9');
        my $fgcolor = ($rule->[5] ? '#666666' : '#000000');
        $HTMLOUT .= $q->start_Tr().$q->td({-align=>'center', -valign=>'middle'},
                    "<b>".($count+1).". </b>").
                    $q->start_td();
            my $sizerule =  $$rule[0];
            my $filter = $$rule[1];
            $HTMLOUT .= "<font color=$fgcolor>\n";
            if (keys %{$sizerule}){
                $HTMLOUT .= $message->{'procmail_if_message'}." ".(($sizerule->{'op'} eq 'over') ?
                                                            $message->{'procmail_over'}:
                                                            $message->{'procmail_under'}). " ".
                   ($sizerule->{'size'})." ".
                   ($message->{'procmail_unit_'.$sizerule->{'unit'}})." ".$message->{'procmail_is'}." ";
                if( @{$filter} > 0 ){
                    $HTMLOUT .= $conj_op;
                }else{
                    $HTMLOUT .= ",";
                }
                $HTMLOUT .= $q->br;
            }    
            for(my $i = 0; $i <= $#$filter; $i++){
                my $filterrule=$$filter[$i];
                $HTMLOUT .= $message->{'procmail_header_'.$filterrule->{'type'}}." ";
                my $headers=($filterrule->{'headers'});
                for(my $j=0; $j < (@{$headers}); $j++){
                    $HTMLOUT .= "\"<i>".$headers->[$j]."</i>\"";
                    if($j == (@{$headers})-1){
                        $HTMLOUT .= " ";
                    } elsif($j == (@{$headers})-2){
                        $HTMLOUT .= " $message->{'procmail_rule_conj_anyof'} ";
                    } else {
                       $HTMLOUT .= ", ";
                    }   
                }    
                $HTMLOUT .= $message->{'procmail_match_'.($filterrule->{'not'} ? "not_".$filterrule->{'match'} : $filterrule->{'match'}) }.
                    ' "<i>'.$q->escapeHTML($filterrule->{'content'}).'</i>"';
                if($i < (@{$filter}-1)){
                    $HTMLOUT .= " $conj_op";
                }else{
                    $HTMLOUT .= ",";
                }
                $HTMLOUT .= $q->br;
            }    
            my %action = %{$$rule[2]};
            $HTMLOUT .= "$message->{'procmail_then'} ";
            if ($action{'action'} eq 'fileinto'){
                $HTMLOUT .= $message->{'procmail_fileinto'}.": <b>".$q->escapeHTML(IMAPUtils::utf7_decode($action{'param'})).br."</b>\n";
            }elsif($action{'action'} eq 'redirect'){ 
                $HTMLOUT .= $message->{'procmail_redirect'}.": <b>".$q->escapeHTML($action{'param'}).br."</b>\n";
            }elsif($action{'action'} eq 'reject'){
                my $tmp_param = $q->escapeHTML($action{'param'});
                $tmp_param =~ s/\n/<BR>/g; 
                $HTMLOUT .= "$message->{'procmail_reject'}:<BR>\"<i>".$tmp_param." </i>\"".br."\n";
            }elsif($action{'action'} eq 'discard'){
                $HTMLOUT .= "$message->{'procmail_discard'}.".br."\n";
            }
            if(! $rule->[4]){
                $HTMLOUT .= "<br><i>$message->{'procmail_continue_filter'}</i>\n";
            }
            $HTMLOUT .= "</font> ".$q->end_td;    
            #
            #display the toolbox icons (delete, comment, move)
            #
            $HTMLOUT .= $q->start_td({-align=>"center",-width=>'7%'});
                        if($count != 0){ 
                            $HTMLOUT .= $q->a({href=>"$confParam->{cgi_path}/mail_filter.pl?".
                                                "sessionID=$sessionID".
                                                "&rule_num=$count".
                                                "&mode=move".
                                                "&new_pos=up".
                                                "&tab=$tab".
						"&stab=$stab"}, 
                                            $q->img({src=>'/images/arrow_up.gif', border=>0}) );
                            $HTMLOUT .= $q->a({href=>"$confParam->{cgi_path}/mail_filter.pl?".
                                                "sessionID=$sessionID".
                                                "&rule_num=$count".
                                                "&mode=move".
                                                "&new_pos=first".
                                                "&tab=$tab".
						"&stab=$stab"}, 
                                            $q->img({src=>'/images/arrow_top.gif', border=>0}) );
                        }
                        if ($count != (@filterrules-1) ){
                            $HTMLOUT .= $q->br.$q->a({href=>"$confParam->{cgi_path}/mail_filter.pl?".
                                                "sessionID=$sessionID".
                                                "&rule_num=$count".
                                                "&mode=move".
                                                "&new_pos=down".
                                                "&tab=$tab".
						"&stab=$stab"}, 
                                            $q->img({src=>'/images/arrow_down.gif', border=>0}) );
                            $HTMLOUT .= $q->a({href=>"$confParam->{cgi_path}/mail_filter.pl?".
                                                "sessionID=$sessionID".
                                                "&rule_num=$count".
                                                "&mode=move".
                                                "&new_pos=last".
                                                "&tab=$tab".
						"&stab=$stab"}, 
                                            $q->img({src=>'/images/arrow_bottom.gif', border=>0}) );
                        }
                        if (@filterrules == 1){
                            $HTMLOUT .= "&nbsp;";
                        }
            $HTMLOUT .= $q->end_td();            
            if($rule->[5]){
                $HTMLOUT .= $q->start_form(-action =>$confParam->{cgi_path}.'/mail_filter.pl').
                            $q->td({-valign=>"middle", -align=>"center",-width=>'5%'},[
                            $q->image_button(-name=>$message->{'procmail_rule_uncomment'}, -class=>"image",
                                -src=>'/images/uncomment.gif',
                                -align=>"middle",
                                -border=>"no").br.
                            $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override => 1).
                            $q->hidden(-name=>'rule_num',
                                    -value => "$count",
                                    -override => 1).
                            $q->hidden(-name=>'mode',
                                    -value => "uncomment",
                                    -override => 1).
			    $q->hidden(-name => 'tab',
				    -value => $tab,
				    -override => 1).
			    $q->hidden(-name => 'stab',
				    -value => $stab,
				    -override => 1)
                       ]). $q->end_form;

            }else{
                $HTMLOUT .= $q->start_form(-action =>$confParam->{cgi_path}.'/mail_filter.pl').
                            $q->td({-valign=>"middle" ,-align=>"center",-width=>'5%'},[
                            $q->image_button(-name=>$message->{'procmail_rule_comment'}, -class=>"image",
                                -src=>'/images/comment.gif',
                                -align=>"middle",
                                -border=>"no").br.
                            $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override => 1).
                            $q->hidden(-name=>'rule_num',
                                    -value => "$count",
                                    -override => 1).
                            $q->hidden(-name=>'mode',
                                    -value => "comment",
                                    -override => 1).
			    $q->hidden(-name => 'tab',
				    -value => $tab,
				    -override => 1).
			    $q->hidden(-name => 'stab',
				    -value => $stab,
				    -override => 1)
                       ]). $q->end_form;
            }       
            $HTMLOUT .= $q->start_form(-action =>$confParam->{cgi_path}.'/mail_filter.pl').
                        $q->td({-valign=>"middle" ,-align=>"center",-width=>'5%'},[
                        $q->image_button(-name=>$message->{'procmail_rule_modify'}, -class=>"image",
                                -src=>'/images/edit.gif',
                                -align=>"middle",
                                -border=>"no").br.
                        $q->hidden(-name=>'sessionID',
                                -value=>$sessionID,
                                -override => 1).
                        $q->hidden(-name=>'rule_num',
                                -value => "$count",
                                -override => 1).
                        $q->hidden(-name=>'mode',
                                -value => "modify",
                                -override => 1).
                        $q->hidden(-name => 'tab',
                                -value => $tab,
                                -override => 1).
                        $q->hidden(-name => 'stab',
                                -value => $stab,
                                -override => 1)
                   ]). $q->end_form;
            $HTMLOUT .= $q->start_form(-action =>$confParam->{cgi_path}.'/mail_filter.pl').
                        $q->td({-align=>"center",-width=>'5%'},[
                        $q->image_button(-name=>$message->{'procmail_rule_delete'}, -class=>"image",
                                    -src=>'/images/trashcan.gif',
                                    -align=>"middle",
                                    -border=>"no").br.
                        $q->hidden(-name=>'sessionID',
                                -value=>$sessionID,
                                -override => 1).
                        $q->hidden(-name=>'rule_num',
                                -value => "$count",
                                -override => 1).
                        $q->hidden(-name=>'mode',
                                -value => "delete",
                                -override => 1).
			$q->hidden(-name => 'tab',
				    -value => $tab,
				    -override => 1).
			$q->hidden(-name => 'stab',
				    -value => $stab,
				    -override => 1)
                    ]).$q->end_form;
        $HTMLOUT .= $q->end_Tr.
            $q->start_Tr.
                $q->td({-colspan=>6 , -align=>'center'},[ 
                    $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
                ]).
            $q->end_Tr;    
        $count++            
    }
    $HTMLOUT .= $q->start_Tr.
                $q->td({-colspan=>6 , -align=>'center'},[ 
                    $message->{'procmail_remaining_inbox'} 
                ]).
            $q->end_Tr.
            $q->start_Tr.
                $q->td({-colspan=>6 , -align=>'center'},[ 
                    $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
                ]).
            $q->end_Tr;    
    my @values = (0..$count);
    my %labels;
    foreach(@values){
        $labels{$_} = $_ +1;
    }
    $HTMLOUT .= $q->start_Tr().$q->td(["&nbsp;"]).
            $q->start_form(-action=>$confParam->{cgi_path}.'/mail_filter.pl').
        $q->td({-align=>'right', -colspan=>5},[
            $q->submit(-class=>"button",-value => $message->{'procmail_rule_append'}).
            " ".$message->{'procmail_at_position'}.": ".
            $q->popup_menu(-name=>'rule_num',
                      -values=> \@values,
                      -labels=> \%labels,
                      -default => $#values).
	        $q->hidden(-name=>'sessionID',
                -value=>$sessionID,
                    -override => 1).
            $q->hidden(-name=>'rule_num',
                    -value => $count+1,
                    -override => 1).
            $q->hidden(-name=>'append_mode',
                    -value=>1,
                    -override=>1).
            $q->hidden(-name => 'mode',
                    -value=> 'append',
                    -override => 1).
	    $q->hidden(-name => 'tab',
			-value => $tab,
			-override => 1).
	    $q->hidden(-name => 'stab',
			-value => $stab,
			-override => 1)
            
        ]).
            $q->end_form().          
    $q->end_Tr().
    $q->end_td().$q->end_Tr().$q->end_table().
    $q->end_table;
    $this->SUPER::display($HTMLOUT);
}
#
######################################

#########################################
#
sub check_filter{
    my $this = shift;
    my $q = $this->{'cgi'};
    my $rule_num = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    $session->load($sessionID);
    my $filters = $this->read_filter_from_session();
    my $rule = $filters->[$rule_num];
    my $size_rule = $rule->[0];
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');

    if ( (defined $size_rule->{'size'}) && (!($size_rule->{'size'} =~ /^\d+$/)) ){
        $this->display_error($message->{'procmail_size_not_a_number'}, 
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID.'&mode=modify&tab='.$tab."&stab=".$stab.'&rule_num='.$q->param('rule_num'), 3);
        exit;            
    }

    my $action = $rule->[2];
    if( ($action->{'action'} eq 'redirect') && ( (! defined $action->{'param'}) || ($action->{'param'} eq "") ) ){
        $this->display_error($message->{'procmail_redirect_but_no_email'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID.'&mode=modify&tab='.$tab."&stab=".$stab.'&rule_num='.$q->param('rule_num'), 3);
        exit;            
    }
    if( ($action->{'action'} eq 'redirect') && (! ($action->{'param'} =~ /.+\@.+/)) ){
        $this->display_error($message->{'procmail_redirect_no_valid_email'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID.'&mode=modify&tab='.$tab."&stab=".$stab.'&rule_num='.$q->param('rule_num'), 3);
        exit;            
    }
    if( ($action->{'action'} eq 'reject') && ((! defined $action->{'param'}) || ($action->{'param'} eq "") )){
        $this->display_error($message->{'procmail_reject_no_reason'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID.'&mode=modify&tab='.$tab."&stab=".$stab.'&rule_num='.$q->param('rule_num'), 3);
        exit;            
    }
}
#
#########################################

#########################################
#
sub submit_changes(){
#    print STDERR "submit_changes()\n";
    my $this = shift;
    my $confParam = $this->{'session'}->{'confParam'};
    my $frozen = $confParam->{'sieveScript'};
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);     # melt down... :-)
    
    my $SCRIPT = struct_to_script($script_ref);
#    print STDERR $SCRIPT;
    $this->save_script($SCRIPT);
    exit;
}
#
#########################################

#########################################
#
sub save_script($){
    my $this =shift;
    my $script = shift;
    my $q = $this->{'cgi'};
    my $filtername = $this->{'filtername'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $uid = $confParam->{"uid"};
    my $passwd = $confParam->{"passwd"};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $imapserver = $confParam->{'IMAPserver'};
    my $sieveport = $confParam->{'SIEVEport'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    
    my $sieve_obj = ManageSieve->new($imapserver, $sieveport);
    my ($res, $msg) = $sieve_obj->connect();
    if ($res ne "OK"){
        $this->display_error("connection failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->authplain($uid,$uid,$passwd);
    if ($res ne "OK"){
        $this->display_error("authentication failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->putScript($filtername, $script);
    if ($res ne "OK"){
        $this->display_error("Sieve put failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->setActive($filtername);
    if ($res ne "OK"){
        $this->display_error("Sieve activate failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    $sieve_obj->logout();
    $this->display_msg($message->{'procmail_saved'}, $confParam->{'cgi_path'}."/mail_filter.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
}

#
########################################

########################################
#
sub move_rule(){
    my $this = shift;
    my $q = $this->{'cgi'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $imapserver = $confParam->{'IMAPserver'};
    my $sieveport = $confParam->{'SIEVEport'};
    my $uid = $confParam->{"uid"};
    my $passwd = $confParam->{"passwd"};
    my $rule_num = $q->param('rule_num');
    my $new_pos = $q->param('new_pos');
    my $sessionID = $confParam->{'sessionID'};
    my $script = $this->read_filter_from_session();
   
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $filtername = $this->{'filtername'};

    if ($new_pos eq 'up'){
#        print STDERR "\n MOVE $rule_num $new_pos \n";
        ($script->[$rule_num], $script->[$rule_num-1]) = ($script->[$rule_num-1], $script->[$rule_num]);

    }elsif ($new_pos eq 'down'){
#        print STDERR "\n MOVE $rule_num $new_pos \n";
        ($script->[$rule_num], $script->[$rule_num+1]) = ($script->[$rule_num+1], $script->[$rule_num]);
    
    }elsif ($new_pos eq 'first'){
#        print STDERR "\n MOVE $rule_num $new_pos \n";
        my $temp = splice @{$script}, $rule_num,1;
        unshift @{$script}, $temp;
    
    }elsif ($new_pos eq 'last'){
#        print STDERR "\n MOVE $rule_num $new_pos \n";
        my $temp = splice @{$script}, $rule_num, 1;
        push @{$script}, $temp;

    }
    $this->write_filter_to_session($script); 
    $session->load($sessionID); 
    $confParam = $session->{'confParam'};
    my $frozen = $confParam->{'sieveScript'};
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);     # melt down... :-)

    my $sieve = struct_to_script($script_ref);
    
    my $sieve_obj = ManageSieve->new($imapserver, $sieveport);
    my ($res, $msg) = $sieve_obj->connect();
    if ($res ne "OK"){
        $this->display_error("connection failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->authplain($uid,$uid,$passwd);
    if ($res ne "OK"){
        $this->display_error("authentication failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->putScript($filtername, $sieve);
    if ($res ne "OK"){
        $this->display_error("Sieve put failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->setActive($filtername);
    if ($res ne "OK"){
        $this->display_error("Sieve activate failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    $sieve_obj->logout();
    $this->display_script();
}

#
#######################################

#######################################
#
sub load_script_to_session(){
    my $this = shift;
    my $q = $this->{'cgi'};
    my $filtername = $this->{'filtername'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $imapserver = $confParam->{'IMAPserver'};
    my $sieveport = $confParam->{'SIEVEport'};
    my $uid = $confParam->{"uid"};
    my $passwd = $confParam->{"passwd"};
    my $sessionID = $confParam->{'sessionID'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');

    my $sieve="";
    my $sieve_obj = ManageSieve->new($imapserver, $sieveport);
    my ($res, $msg) = $sieve_obj->connect();
    if ($res ne "OK"){
        $this->display_error("connection failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->authplain($uid,$uid,$passwd);
    if ($res ne "OK"){
        $this->display_error("authentication failed: $msg", $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.$sessionID.
                    "&mode=display&tab=$tab&stab=$stab", 3);
        exit;
    }
    ($res, $msg) = $sieve_obj->getScript($filtername);
    if ($res ne "OK"){
	$sieve="";
    }else{
	$sieve=$msg;
#	print STDERR "SCRIPT: $msg\n";
    }
    $sieve_obj->logout();
    
    my $script_ref=parse_script($sieve);
    if(defined $script_ref){
#        print STDERR "Script parsed!\n";
        my $frozen = freeze $script_ref;      #freeze data
        $frozen = unpack('H*',$frozen);   #convert to ASCII
        my %tmp = ( "sieveScript" => $frozen );
        my $s_reply=$session->modify(\%tmp);
        if( $s_reply->code() ne "OK" ) {
#            print STDERR "unable to store data in session\n";
            exit;
        }else {
	    return 0;
#            print STDERR "Data saved\n" ;
        }
    }else{
#        print STDERR "Unable to parse script!\n";
	return -1;
    }
}
#
#######################################

#######################################
#
sub write_filter_to_session(){
#    print STDERR "write_filter_to_session\n";
    my $this = shift;
    my $filterrules = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $frozen = $confParam->{"sieveScript"};
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);     # melt down... :-)
    $script_ref->[2]=$filterrules;
    undef $frozen;
    
    $frozen = freeze $script_ref;      #freeze data
    $frozen = unpack('H*',$frozen);   #convert to ASCII

    my %tmp = ( "sieveScript" => $frozen );
    my $s_reply=$session->modify(\%tmp);
    if( $s_reply->code() ne "OK" ) {
#        print STDERR "unable to store data in session\n";
        exit;
    }else {
#        print STDERR "Data saved\n" ;
    }
    $session->load($confParam->{'sessionID'});
}
#
#######################################

#######################################
#
sub read_filter_from_session(){
#    print STDERR "read_filter_from_session()\n";
    my $this = shift;
    my $script_ref;
    my $confParam = $this->{'session'}->{'confParam'};

    my $frozen = $confParam->{"sieveScript"};
    $frozen = pack('H*',$frozen); # convert back to binary
    $script_ref = thaw($frozen);     # melt down... :-)
   
    my $filters = $script_ref->[2];
    return $filters;
}
#
#######################################

#######################################
#
sub create_rule{
    my $this = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $mode = $q->param('mode');
    my $rule_num = $q->param('rule_num');
    my $filters = $this->read_filter_from_session();
    my $create_step = $q->param('create_step');
    my $cancelsub_but = $q->param('cancelsub'); 
    my $cancel_but = $q->param('cancel'); 
#    print STDERR "MODE:   $mode\n";
    $q->delete('mode');
    if( ($mode eq "append") ){
#    if( (! defined $filters->[$rule_num]) ){
#	print STDERR "created empty rule\n";
	my %sizerule;
	#my %headerrule;
	my @headerrules;
	#$headerrules[0] = \%headerrule;
	my %action;
	my $conj_op;
	my $new_filter=[\%sizerule,\@headerrules,\%action, $conj_op, 1];
	splice @{$filters}, $rule_num, 0, $new_filter; 
	$this->write_filter_to_session($filters);
    }elsif( (! defined $cancel_but) && (! defined $cancelsub_but) && (defined $create_step) ){ 
	$this->read_from_form($create_step, $filters);
	$this->write_filter_to_session($filters);
    }

    my $size_but = $q->param('create_size_filter'); 
    my $delete_size_but = $q->param('delete_size_filter');
    my $header_but = $q->param('create_header_filter'); 
    my $delete_header_but =  $q->param('delete_header_filter');
    my $envelope_but = $q->param('create_envelope_filter'); 
    my $action_but = $q->param('create_action'); 
    my $filter_num = $q->param('filter_num');
    my $save_but = $q->param('save');

    if( defined $cancel_but){
	$this->display_script();
	exit;
    }
    if( defined $save_but ){
	$confParam = ($session->load($sessionID))->data();
	my $frozen = $confParam->{"sieveScript"};
	$frozen = pack('H*',$frozen); # convert back to binary
	my $script_ref = thaw($frozen);     # melt down... :-)
	my $script = struct_to_script($script_ref);
#	print STDERR "$script\n";
	$this->save_script($script);
	$session->modify({"sieveScript" => ""});
	exit;
    }elsif( defined $size_but ){
	$this->create_size_rule($filters);
    }elsif( defined $delete_size_but ){
	$this->delete_rule($filters,"size");
	$this->write_filter_to_session($filters);
	$this->create_rule_base($filters);
    }elsif( defined $header_but){
	$this->create_header_rule($filters, "header", $filter_num);
    }elsif( defined $delete_header_but ){
	$this->delete_rule( $filters, "header" );
	$this->write_filter_to_session($filters);
	$this->create_rule_base($filters);
    }elsif( defined $envelope_but){
	$this->create_header_rule($filters, "envelope", $filter_num);
    }elsif( defined $action_but){
	$this->create_action($filters);
    }else{
	$this->create_rule_base($filters);
    }
	
}

sub read_from_form(){
#    print STDERR "read_from_form()\n";
    my ($this, $step, $filters) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $rule_num = $q->param('rule_num');
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    

    if($step eq "conj_op"){
	my $conj_op = $q->param('conj_op');
	if( (defined $conj_op) && ( ($conj_op eq "allof") || ($conj_op eq "anyof") ) ){
#	    print STDERR "CONJ_OP:         $conj_op\n";
	    $filters->[$rule_num]->[3] = $conj_op;
	}else{
#	    print STDERR "CONJ_OP $conj_op\n";
	    $this->display_error("--- INVALID RULE CONJUNCTION ---",
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
	    exit;            
	}
    }elsif( $step eq "size_rule"){
	my $op = $q->param('op');
	my $size = $q->param('size');
	my $unit = $q->param('unit');
	if( (!defined $op) || ( ($op ne "over") && ($op ne "under") ) ){
	    $this->display_error("--- INVALID SIZE OP ---",
				$confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_size_filter=1'.
					   "&op=$op&size=$size&unit=$unit",
				3);
	    exit;            
	}
	if( (!defined $unit) || ( $unit !~ /^[BKM]$/ ) ){
	    $this->display_error("--- INVALID SIZE UNIT ---",
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_size_filter=1'.
					   "&op=$op&size=$size&unit=$unit",
                      3);
	    exit;            
	}
	if( (!defined $size) || ( $size !~ /^[0-9]+$/ ) ){
	    $this->display_error($message->{'procmail_size_is_nan'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_size_filter=1'.
					   "&op=$op&size=$size&unit=$unit",
                        3);
	    exit;            
	}
#	print STDERR "SIZERULE: $op $size $unit\n";
	if( (defined $size) && ( $size ne "") ){
	    $filters->[$rule_num]->[0] = { "op" => $op,
					   "size" => $size,
					   "unit" => $unit };
	}
    }elsif( $step eq "header_rule"){
	my $type = $q->param('header_type');
	my @headers = $q->param('header');
	my $match =  $q->param('match');
	my $content = $q->param('content');
	my $filter_num = $q->param('filter_num');
	my $free_headers = $q->param('free_headers');
	my $not = 0;
	if((! @headers) && ( (! defined $free_headers) || ($free_headers eq "") ) ){
	    $this->display_error($message->{"procmail_no_header_selected"},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_header_filter=1'.
					   (($filter_num ne "") ? "&filter_num=$filter_num" : ""),
                        3);
	    exit;            
	}
	if( (! defined $match) || ( $match eq "") ){
	    $this->display_error("--- NO MATCH TYPE ---",
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_header_filter=1'.
					   (($filter_num ne "") ? "&filter_num=$filter_num" : ""),
                        3);
	    exit;            
	}else{
	    if( $match =~ /^(not_)?(.+)$/ ){
		if( defined $1){
		    $not = 1;
		    $match = $2;
		}
	    }
	}
	if( (! defined $content) || ( $content eq "" ) ){
	    $this->display_error($message->{'procmail_no_content'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
				$sessionID.'&mode=modify'.
					   "&tab=$tab".
					   "&stab=$stab".
					   '&rule_num='.$q->param('rule_num').
					   '&create_header_filter=1'.
					   (($filter_num ne "") ? "&filter_num=$filter_num" : ""),
                        3);
	    exit;            
	}
	my @free = split /\s/, $free_headers;
	push @headers, (@free);
	my $headerrules = $filters->[$rule_num]->[1];
	my $rule = { 'type' => $type,
		     'headers' => \@headers,
		     'match' => $match,
		     'content' => $content,
		     'not' =>  $not };
#	print STDERR "HEADER: $not $content $match\n";
	if( (!defined $filter_num) || ($filter_num !~ /^[0-9]+$/ ) ){
	    push @$headerrules, $rule;
	}else{
	    splice(@$headerrules, $filter_num, 1, $rule);
	}

    }elsif( $step eq "action_rule"){
	my $action = $q->param('action');
	my $continue = $q->param('continue');
	if( (! defined $action) || ( $action !~ /^(fileinto|reject|redirect|discard)$/ ) ){
	    $this->display_error($message->{'procmail_no_action'},
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
	    exit;            
	}
	my $param="";
	if( $action eq "fileinto" ){
	    $param = $q->param('param_file');
	    my $username= $confParam->{"uid"};
	    my $passwd=$confParam->{"passwd"};
	    my $imaphost=$confParam->{IMAPserver};
	    my $folders = IMAPUtils::getFolder($username, "p", $passwd, $imaphost);
	    $param = $folders->{$param};
	    if( (! defined $param) || ($param eq "") ){
		$this->display_error("--- NO MAILBOX ---",
			$confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
			    $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
		exit;            
	    }
	}elsif( $action eq "redirect" ){
	    my $localpart = $q->param('param_localpart');
	    my $dompart   = $q->param('param_domainpart');
	    if( defined $dompart && $dompart ne "" ) {
		$param = $localpart."\@".$dompart;
		if( (! defined $localpart) || ($localpart =~ /\@/) ){
		    $this->display_error($message->{'procmail_redirect_no_valid_email'},
					 $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
					 $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
		    exit;            
		}
	    } else {
		$param = $q->param('param_redirect');
		if( (! defined $param) || ($param !~ /^.+\@.+$/) ){
		    $this->display_error($message->{'procmail_redirect_no_valid_email'},
					 $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
					 $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
		    exit;            
		}
	    }
	}elsif( $action eq "reject" ){
	    $param = $q->param('param_reject');
	    if( (! defined $param) || ($param eq "") ){
		$this->display_error($message->{'procmail_reject_no_reason'},
			$confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
			    $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
		exit;            
	    }
	}
	if( (!defined $continue) || ( $continue ne "1" ) ){
	    $filters->[$rule_num]->[4] = 1;
	}else{
	    $filters->[$rule_num]->[4] = 0;
	}
	$filters->[$rule_num]->[2] = { "action" => $action,
				       "param" => $param};
#	print STDERR "ACTION $action $param\n";
    }
    
}
sub delete_rule(){
    my ($this, $filters, $which) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $rule_num=$q->param('rule_num');
    my $filter = $filters->[$rule_num];
    my $conj_op = $filter->[3] || "and";
    my $size_rule = $filter->[0];
    my $header_rules = $filter->[1];
    my $action_rule = $filter->[2];
    if( $which eq "size" ){
        $filter->[0] = {};
    }elsif( $which eq "header" ){
	my $filter_num =  $q->param('filter_num');
	if( (! defined $filter_num) || ( $filter_num !~ /^[0-9]+$/ ) ){
	    $this->display_error("--- INVALID FILTER NO ---",
                    $confParam->{'cgi_path'}.'/mail_filter.pl?sessionID='.
                        $sessionID."&mode=modify&tab=$tab&stab=$stab&rule_num=".$q->param('rule_num'), 3);
	    exit;
	}else{
	    splice( @$header_rules, $filter_num, 1 );
	}
    }
}

sub create_rule_base{
#    print STDERR "create_rule_base\n";
    my ($this, $filters) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $rule_num=$q->param('rule_num');
    my $filter = $filters->[$rule_num];
    my $conj_op = $filter->[3] || "and";
    my $size_rule = $filter->[0];
    my $header_rules = $filter->[1];
    my $action_rule = $filter->[2];
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');

    my $html="";
    $html .= $q->start_table({-class=>'AdminBorder', -border=>0, -width=>"100%", -cellspacing=>1});
    $html .=    $q->start_Tr();
    $html .=	$q->start_td();
    $html .=	    $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>3});
    $html .=		$q->Tr( [$this->help_th(3,$message->{'procmail_create_rule'},'MAILFILTER_CREATE_RULE') ] );
    $html .=		$q->Tr( [$q->td( {-colspan => 3, -class=>'AdminSubHeader'},
					     $message->{"procmail_edit_filter_conditions"} 
					   )
				    ]);
    $html .=		$q->Tr( [$q->td( { -colspan => 3},
					     ["&nbsp;".$message->{'procmail_choose_conjunction'}.":"]
					   )
				    ]);
    my %labels = ( 'allof' => $message->{'procmail_rule_conj_allof'},
		   'anyof' => $message->{'procmail_rule_conj_anyof'}
		 );
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-colspan => 3});
    $html .=			$q->start_form(-action => $confParam->{'cgi_path'}.'/mail_filter.pl');
    $html .=			"&nbsp;";
    $html .=			$q->popup_menu( -name => 'conj_op',
						 -values => ['allof', 'anyof'],
						 -default => $conj_op ,
						 -linebreak => 'true',
						 -labels => \%labels,
						 -overide => 1
						);
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->td({-colspan=>3 , -align=>'center'}, 
				    $q->hr({-noshade => '1', -size => '1', -width=>'90%'}) );
    $html .=		$q->end_Tr();    
    if( ( (! defined $size_rule) || (! keys %$size_rule) ) &&	
	( (! defined $header_rules) || ( @$header_rules == 0) ) ){
	$html .=	$q->start_Tr();
	$html .=	    $q->td({-colspan => 3}, 
				    "&nbsp;".$message->{'procmail_no_conditions'});
	$html .=	$q->end_Tr();
    }
    if( (defined $size_rule) && (keys %$size_rule) ){	
	my $op = $size_rule->{'op'};
	my $unit = $size_rule->{'unit'};
	my $size = $size_rule->{'size'};
#	print STDERR "DISP SIZE RULE $op $unit $size\n";
	if( (defined $op) && (defined $size) && (defined $unit) ){
	    $html .=	$q->start_Tr();
	    $html .=	    $q->start_td({-height => 30});
	    $html .=		"&nbsp;";
	    $html .=		$message->{'procmail_if_message'}." ".
				$message->{"procmail_$op"}." $size ".$message->{"procmail_unit_$unit"}.
				" ".$message->{'procmail_is'};
	    if( (defined $header_rules) && ( @$header_rules > 0) ){
		$html .=	" ".$message->{"procmail_rule_conj_$conj_op"};
	    }
	    $html .=	    $q->end_td();
	    $html .=	    $q->start_td({ -align => "center"});
	    $html .=		$q->a({ href => $confParam->{cgi_path}."/mail_filter.pl?".
						"mode=modify".
						"&sessionID=$sessionID".
						"&tab=$tab&".
						"&stab=$stab&".
						"rule_num=$rule_num".
						"&create_size_filter=1" 
				      },
				 	$q->img({ src => "/images/edit.gif", 
						  alt => $message->{'procmail_rule_modify'},
						  border => 0})
				     );
	    $html .=	    $q->end_td();
	    $html .=	    $q->start_td({ -align => "center"});
	    $html .=		$q->a({ href => $confParam->{cgi_path}."/mail_filter.pl?".
						"mode=modify".
						"&sessionID=$sessionID".
						"&tab=$tab&".
						"&stab=$stab&".
						"rule_num=$rule_num".
						"&delete_size_filter=1" 
				      },
				 	$q->img({ src => "/images/trashcan.gif", 
						  alt => $message->{'procmail_rule_delete'},
						  border => 0})
				     );
	    $html .=	    $q->end_td();
	    $html .=	$q->end_Tr();
	}
    }
    if( (defined $header_rules) && ( @$header_rules > 0) ){
	my $i = 0;
	foreach my $rule (@$header_rules){
	    my $type =  $rule->{'type'};
	    my $headers = $rule->{'headers'};
	    my $match = $rule->{'match'};
	    my $content = $rule->{'content'};
	    my $not =  $rule->{'not'};
	    $html .=	$q->start_Tr();
	    $html .=	    $q->start_td({-height => 30});
	    $html .=		"&nbsp;";
	    $html .=		$message->{"procmail_header_$type"}." ";
	    for(my $i=0; $i < scalar(@$headers); $i++){
		$html .=	"\"".$headers->[$i]."\"";
		if( defined $headers->[$i+2] ){
		    $html .= ", ";
		}elsif( defined $headers->[$i+1] ){
		    $html .= " ".$message->{'procmail_rule_conj_anyof'}." ";
		}else{
		    $html .= " ";
		}
	    }
	    $html .=		$message->{'procmail_match_'.($not ? "not_" : "" ).$match}." <i>\"".$q->escapeHTML($content)."\"</i>";
	    if( (defined $header_rules->[$i+1]) ){
		$html .=	" ".$message->{"procmail_rule_conj_$conj_op"};
	    }
	    $html .=	    $q->end_td();
	    $html .=	    $q->start_td({ -align => "center" });
	    $html .=		$q->a({ href => $confParam->{cgi_path}."/mail_filter.pl?".
						"mode=modify".
						"&sessionID=$sessionID".
						"&tab=$tab&".
						"&stab=$stab&".
						"rule_num=$rule_num".
						"&create_$type"."_filter=1".
						"&filter_num=$i"
				      },
				 	$q->img({ src => "/images/edit.gif", 
						  alt => $message->{'procmail_rule_modify'}, 
						  border => 0})
				     );
	    $html .=	    $q->end_td();
	    $html .=	    $q->start_td({ -align=> "center"});
	    $html .=		$q->a({ href => $confParam->{cgi_path}."/mail_filter.pl?".
						"mode=modify".
						"&sessionID=$sessionID".
						"&tab=$tab&".
						"&stab=$stab&".
						"rule_num=$rule_num".
						"&delete_header_filter=1".
						"&filter_num=$i"
				      },
				 	$q->img({ src => "/images/trashcan.gif", 
						  alt => $message->{'procmail_rule_delete'}, 
						  border => 0})
				     );
	    $html .=	    $q->end_td();
	    $html .=	$q->end_Tr();
	    $i++;
	}
    }
    $html .=		$q->start_Tr();
    $html .=		    $q->td({-colspan=>3 , -align=>'center'}, 
				    $q->hr({-noshade => '1', -size => '1', -width=>'90%'}) );
    $html .=		$q->end_Tr();    
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({ -align => "left", -colspan => 3});
    $html .=			"&nbsp;";
    $html .=			$message->{'procmail_create_filter_condition'};
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-colspan=>3, -align=>"right", -width=>"30%"});
    if( (!defined $size_rule) || (! keys %$size_rule) ){	
        $html .=		$q->submit(-class=>"button", -name => "create_size_filter",
					    -value => $message->{'procmail_size_restriction'})." ";
    }
    $html .=			$q->submit(-class=>"button", -name => "create_header_filter",
					    -value => $message->{'procmail_header_filter'})." ";
    $html .=			$q->submit(-class=>"button", -name => "create_envelope_filter",
					    -value => $message->{'procmail_envelope_filter'});
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({ -colspan => 3, -align => "right"});
    $html .=			$q->hidden( -name => "mode",
					    -value => "modify",
					    -overide => 1).
		                $q->hidden(-name=>"sessionID",
                                           -value=>$sessionID,
                                           -override=>1).
                                $q->hidden(-name=>"tab",
                                           -value=>$tab,
                                           -override=>1).
                                $q->hidden(-name=>"stab",
                                           -value=>$stab,
                                           -override=>1).
				$q->hidden(-name => "rule_num",
					   -value => $rule_num,
					   -override => 1).
				$q->hidden(-name => "create_step",
					   -value => "conj_op",
					   -override => 1);
    $html .=			$q->submit(-class=>"button", -name => "cancel",
					    -value => $message->{'procmail_cancel'});
    if( ( (defined $size_rule) && (keys %$size_rule) )  || 	
	( (defined $header_rules) && ( @$header_rules > 0) ) ){
	$html .=		" ".$q->submit(-class=>"button", -name => "create_action",
					    -value => $message->{'procmail_continue'});
    }
    $html .=			$q->end_form();
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=	    $q->end_table();
    $html .=	$q->end_td();
    $html .=    $q->end_Tr();
    $html .= $q->end_table();
    # create an empty rule and write it to the session
    $this->SUPER::display($html);
    
    return;
}

sub create_size_rule(){
#    print STDERR "create_size_rule\n";
    my ($this, $filters) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $create_step = $q->param('create_step');
    my $rule_num = $q->param('rule_num');
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $size_rule = $filters->[$rule_num]->[0];
    my $op = "over";
    my $size = "";
    my $unit = "B";
    if(defined $size_rule){
	$op = $size_rule->{'op'};
	$size = $size_rule->{'size'};
	$unit = $size_rule->{'unit'};
    }
    my $html="";
    $html .= $q->start_table({-class=>'AdminBorder', -width=>"100%", -cellspacing=>1});
    $html .=    $q->start_Tr();
    $html .=	$q->start_td();
    $html .=	    $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>3});
    $html .=		$q->Tr( [$this->help_th(1,$message->{'procmail_create_rule'},'MAILFILTER_CREATE_SIZERULE') ] );
    $html .=		$q->Tr( [$q->td( {-class=>'AdminSubHeader'},
					     $message->{'procmail_create_size_restriction'}
					   )
				    ]);
#    $html .=		$q->Tr( [$q->td( {-bgcolor=>$Display::__gray1},
#					     $message->{'procmail_create_size_restriction'}
#					   )
#				    ]);
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-align => "center"});
    $html .=			$q->start_form(-action => $confParam->{'cgi_path'}.'/mail_filter.pl');
    $html .=			$message->{'procmail_message_is'}." ";
    $html .=			$q->popup_menu( -name => 'op',
                                                -values => ["over","under"],
                                                -labels => {'over' => $message->{'procmail_over'},
                                                            'under' => $message->{'procmail_under'}},
                                                -default => $op)." ";
    $html .=			$q->textfield(-name => 'size',
                                              -default => $size,
                                               -size => 6);
    $html .=                    $q->popup_menu(-name => 'unit',
                                               -values => ['B','K','M'],
                                               -labels => {B => $message->{'procmail_unit_B'},
                                                           K => $message->{'procmail_unit_K'},
                                                           M => $message->{'procmail_unit_M'}},
                                               -default => $unit 
					       );
    $html .=			$q->hidden( -name => "create_step",
					    -value => "size_rule",
					    -override => 1);
    $html .=			$q->hidden( -name => "mode",
					    -value => "modify",
					    -overide => 1);
    $html .=                    $q->hidden( -name=>"sessionID",
                                            -value=>$sessionID,
                                            -override=>1).
                                $q->hidden( -name=>"tab",
                                            -value=>$tab,
                                            -override=>1).
                                $q->hidden( -name=>"stab",
                                            -value=>$stab,
                                            -override=>1).
				$q->hidden(-name => "rule_num",
					   -value => $rule_num,
					   -override => 1);
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-align=>"right"});
    $html .=			$q->submit(-class=>"button", -name => "cancelsub",
				    	    -value => $message->{'procmail_back'})." ";
    $html .=			$q->submit(-class=>"button", -name => "next",
					    -value => $message->{'procmail_ok'});
    $html .=			$q->end_form();
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=	    $q->end_table();
    $html .=	$q->end_td();
    $html .=    $q->end_Tr();
    $html .= $q->end_table();
    $this->SUPER::display($html);
    return;
}

sub create_header_rule(){
#    print STDERR "create_rule_step_header\n";
    my ($this, $filters, $type, $filter_num) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $create_step = $q->param('create_step');
    my $rule_num = $q->param('rule_num');
    my @fields;
    if( $type eq "header"){
	@fields =( 'Bcc', 'Cc', 'Date', 'From', 'Received','Reply-To',
		   'Return-Path',  'Sender', 'Subject', 'To', 'X-Spam-Flag'  );
    }elsif( $type eq "envelope"){
	@fields = ( 'From', 'To');
    }
    my $headers = [];
    my $match = "";
    my $content = "";
    if( defined $filter_num ){
#	print STDERR "edit rule $filter_num\n";
	my $edit_rule= $filters->[$rule_num]->[1]->[$filter_num];
	$headers = $edit_rule->{'headers'};
	$match = $edit_rule->{'match'};
	$content = $edit_rule->{'content'};
    }
    my $free_headers="";
    foreach my $header (@$headers){
	if( ! (grep /^$header$/, @fields) ){
	    $free_headers .= $header." ";
	}
    }
    my $html="";
    $html .= $q->start_table({-class=>'AdminBorder', -width=>"100%", -cellspacing=>2});
    $html .=    $q->start_Tr();
    $html .=	$q->start_td();
    $html .=	    $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>3});
    $html .=		$q->Tr( [$this->help_th(4,$message->{'procmail_create_rule'},'MAILFILTER_CREATE_HEADERRULE') ] );
    $html .=		$q->Tr( [$q->td( {-class=>'AdminSubHeader', -colspan => 4},
					     $message->{"procmail_".$type."_filter"}
					   )
				    ]);
#    $html .=		$q->Tr( [$q->td( {-bgcolor=>$Display::__gray1, -colspan => 4},
#					     "&nbsp;".$message->{"procmail_".$type."_filter"}
#					   )
#				    ]);
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td();
    $html .=			$message->{"procmail_one_".$type."_field"};
    $html .=			$q->start_form(-action => $confParam->{'cgi_path'}.'/mail_filter.pl');
    $html .=		    $q->end_td();
    $html .=		    $q->start_td();
    $html .=			$q->scrolling_list( -name => "header",
                                                    -values => \@fields,
						    -defaults => $headers,
						    -size => 5,
						    -width => 15,
						    -multiple => "true",
						    -override => 1);
    $html .=		    $q->end_td();
    $html .=		    $q->start_td();
    $html .=			$q->popup_menu(-name => "match",
                                                -values => ['is','contains','regex', 'not_is', 'not_contains', 'not_regex'],
                                                -labels => { "is" => $message->{'procmail_match_is'},
                                                            "contains" => $message->{'procmail_match_contains'},
                                                            "regex" => $message->{'procmail_match_regex'}, 
                                                            "not_is" => $message->{'procmail_match_not_is'},
                                                            "not_contains" => $message->{'procmail_match_not_contains'},
                                                            "not_regex" => $message->{'procmail_match_not_regex'}
                                                           },
						-default => $match,
						-override => 1);
    $html .=		    $q->end_td();
    $html .=		    $q->start_td();
    $html .=			$q->textfield(-name => "content",
					      -value => $content);
    $html .=			$q->hidden( -name => "create_step",
					    -value => "header_rule",
					    -override => 1);
    $html .=			$q->hidden( -name => "mode",
					    -value => "modify",
					    -overide => 1);
    $html .=                    $q->hidden( -name=>"sessionID",
                                            -value=>$sessionID,
                                            -override=>1).
                                $q->hidden( -name=>"tab",
                                            -value=> $tab,
                                            -override=>1).
                                $q->hidden( -name=>"stab",
                                            -value=> $stab,
                                            -override=>1).
				$q->hidden(-name => "rule_num",
					   -value => $rule_num,
					   -override => 1).
				$q->hidden( -name => "filter_num",
					    -value => $filter_num,
					    -overrive => 1).
				$q->hidden( -name => "header_type",
					    -value => $type,
					    -override => 1);
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td();
    $html .=			"&nbsp;";
    $html .=		    $q->end_td();
    $html .=		    $q->start_td();
    $html .=			$q->textfield( -name => "free_headers",
					       -value => $free_headers);
    $html .=		    $q->end_td();
    $html .=		    $q->start_td({-colspan => 2});
    $html .=			$message->{'procmail_more_'.$type.'_here'};
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-colspan => 4, -align=>"right"});
    $html .=			$q->submit(-class=>"button", -name => "cancelsub",
						-value => $message->{'procmail_back'})." ";
    $html .=			$q->submit(-class=>"button", -name => "next",
						-value => $message->{'procmail_ok'});
    $html .=			$q->end_form();
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=	    $q->end_table();
    $html .=	$q->end_td();
    $html .=    $q->end_Tr();
    $html .= $q->end_table();
    $this->SUPER::display($html);
    return;
}

sub create_action{
#    print STDERR "create_rule_step_header\n";
    my ($this, $filters) = @_;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $create_step = $q->param('create_step');
    my $rule_num = $q->param('rule_num');
    my $username= $confParam->{"uid"};
    my $passwd=$confParam->{"passwd"};
    my $imaphost=$confParam->{IMAPserver};
    my $folders = IMAPUtils::getFolder($username, "i", $passwd, $imaphost);
    my @dec_folders;
    if( !(defined $folders) && !(keys(%{$folders})) ){
	$folders = { 'INBOX' => 'INBOX' };
    }
    @dec_folders = keys %{$folders};
    @dec_folders = sort(@dec_folders);

    my $action_ref = $filters->[$rule_num]->[2];
    my $action = "";
    my $param = "";
    if( (defined $action_ref) && (keys %$action_ref) ){
	$action = $action_ref->{'action'};
	$param = $action_ref->{'param'};
    }
    if( ! $action ){
	$action = "fileinto";
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $redir;
    $mesg = $ldap->search(base => $confParam->{baseDN},
			  scope => "one", 
			  filter => "uid=cyrus",
			  attrs => [ "allowedFwdDomain" ]);
    if( $mesg->code ) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    my @fwddoms = $mesg->entry(0)->get_value("allowedFwdDomain");
    if( ! defined $fwddoms[0] ) {
	$redir = $q->td([textfield(-name =>'param_redirect',
				   -default => (($action eq "redirect") ? $param :"")).br]);
    } else {
	my ( $localpart, $dompart ) = split(/@/,$param);
	$redir = $q->td([textfield(-name =>'param_localpart', 
				   -default => (($action eq "redirect") ? $localpart :""))."\@".
			 popup_menu(-name =>'param_domainpart',
				    -values => \@fwddoms,
				    -default => $dompart ).br]);
    }

    my $html .= ""; 
    $html .= $q->start_table({-class=>'AdminBorder', -width=>"100%", -cellspacing=>2});
    $html .=    $q->start_Tr();
    $html .=	$q->start_td();
    $html .=	    $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>3});
    $html .=		$q->Tr( [$this->help_th(2,$message->{'procmail_create_rule'},'MAILFILTER_CREATE_ACTION') ] );
    $html .=		$q->Tr( [$q->td( {-class=>'AdminSubHeader', -colspan => 2},
					     [ $message->{'procmail_choose_action'} ]
					   )
				]);
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-width=>'50%' });
    $html .=			$q->start_form(-action => $confParam->{'cgi_path'}.'/mail_filter.pl');
    $html .=			"<input type=radio name=\"action\" value=\"fileinto\"".
				(($action eq "fileinto") ? "checked" :"")."> ".$message->{'procmail_fileinto'} ;
    $html .=		    $q->end_td();
    $html .=                $q->td([            
				$q->popup_menu(-name => 'param_file',
					       -values => \@dec_folders,
					       -default => (($action eq "fileinto") ? IMAPUtils::utf7_decode($param) :"")).br]);
    $html .=             $q->end_Tr;
    $html .=             $q->start_Tr();
    $html .=                 $q->td(
				    [ '<input type=radio name="action" value="redirect"'.
				    (($action eq "redirect") ? "checked" :"")."> ".$message->{'procmail_redirect'}
				    ]); 
    $html .=                 $redir;
    $html .=             $q->end_Tr;
    $html .=             $q->start_Tr();
    $html .=                  $q->td(['<input type=radio name="action" value="reject"'.
				    (($action eq "reject") ? "checked" :"")."> ".$message->{'procmail_reject'}
				    ]); 
    $html .=                  $q->td([$q->textarea(-name=>'param_reject',
						   -default => (($action eq "reject") ? $param :""),
                                                   -rows => 5,
                                                   -cols => 50)
                                     ]);
    $html .=             $q->end_Tr;
    $html .=             $q->start_Tr();
    $html .=                 $q->start_td({-colspan=>2});
    $html .=			'<input type=radio name="action" value="discard"'.     
				    (($action eq "discard") ? "checked" :"")."> ".$message->{'procmail_discard'};
    $html .=			$q->hidden( -name => "create_step",
					    -value => "action_rule",
					    -override => 1);
    $html .=			$q->hidden( -name => "mode",
					    -value => "modify",
					    -overide => 1);
    $html .=                    $q->hidden( -name=>"sessionID",
                                            -value=>$sessionID,
                                            -override=>1).
                                $q->hidden( -name=>"tab",
                                            -value=>$tab,
                                            -override=>1).
                                $q->hidden( -name=>"stab",
                                            -value=>$stab,
                                            -override=>1).
				$q->hidden(-name => "rule_num",
					   -value => $rule_num,
					   -override => 1);
    $html .=		    $q->end_td();
    $html .=		$q->end_Tr();
    $html .=		$q->start_Tr();
    $html .=		    $q->td({-colspan => 2 , -align=>'center'}, 
				    $q->hr({-noshade => '1', -size => '1', -width=>'90%'}) );
    $html .=		$q->end_Tr();    
    $html .=		$q->start_Tr({-valign => "middle"});
    $html .=		    $q->td({-colspan => 2, -valign => "middle"},[
                                    $q->checkbox(-name => 'continue',
                                                 -checked =>  (! $filters->[$rule_num]->[4]) ,
                                                 -value => '1',
                                                 -label => " ".$message->{'procmail_continue_filter'}
                                                )
                                   ]);                 
    $html .=            $q->end_Tr;
    $html .=		$q->start_Tr();
    $html .=		    $q->start_td({-colspan => 2, -align=>"right"});
    $html .=			$q->submit(-class=>"button", -name => "cancel",
					    -value => $message->{'procmail_cancel'})." ";
    $html .=			$q->submit(-class=>"button", -name => "cancelsub",
					    -value => $message->{'procmail_back'})." ";
    $html .=			$q->submit(-class=>"button", -name => "save",
					    -value => $message->{'procmail_save'});
    $html .=			$q->end_form();
    $html .=		    $q->end_td();
    $html .=	    $q->end_table();
    $html .=	$q->end_td();
    $html .=	$q->end_Tr();
    $html .= $q->end_table();
    $this->SUPER::display($html);
    return;
}

sub ask_del_confirm{
    my $this = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"PROCMAIL"))->data();
    my $rule_num = $q->param('rule_num');

    my $HTMLOUT .= $q->start_table({-class=>'AdminBorder', -width=>"100%",-cellpadding => 0, -cellspacing=>2 }).
	        $q->start_Tr().
	            $q->start_td().
	                $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>3}).
	                    $q->start_Tr().
				$q->td({-class=>'AdminHeader', -colspan=>2 },
					 "<b>".$message->{'procmail_rule_delete'}."</b>"
				).
	                    $q->end_Tr().
	                    $q->start_Tr().
	                        $q->td({-class=>'AdminSubHeaderRed', -colspan=>2},
	                                [$q->b($message->{'procmail_really_delete'})]).
	                    $q->end_Tr().
	                    $q->start_Tr.
	                        $q->td({-colspan=>3 , -align=>'center'},[ 
	                            $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
	                        ]).
	                    $q->end_Tr.    
	                    $q->start_Tr({-valign=>"middle"}).
	                        $q->start_td({-align=>"center",-width=>"50%"}).
				    $q->start_form((-action=>$confParam->{'cgi_path'}.'/mail_filter.pl')).
	                            $q->submit(-class=>"button",-value=> $message->{'procmail_cancel'}).
	                            $q->hidden(-name=>"mode",
	                                       -value=>"display",
	                                       -override=>1).
				    $q->hidden( -name => "tab",
					    -value => $tab,
					    -override => 1).
				    $q->hidden( -name => "stab",
					    -value => $stab,
					    -override => 1).
	                            $q->hidden(-name=>"sessionID",
	                                      -value=>$sessionID,
	                                      -override=>1).
	                        $q->end_form().        
	                        $q->end_td().    
	                        $q->start_td({-align => "center", -width=>"50%"}).
				    $q->start_form((-action=>$confParam->{'cgi_path'}.'/mail_filter.pl')).
	                            $q->submit(-class=>"button",-value=> $message->{'procmail_ok'}).
	                            $q->hidden(-name=>"mode",
	                                    -value=>"delete",
	                                    -override=>1).
				    $q->hidden( -name => "tab",
					    -value => $tab,
					    -override => 1).
				    $q->hidden( -name => "stab",
					    -value => $stab,
					    -override => 1).
	                            $q->hidden(-name=>"sure",
	                                    -value=>1,
	                                    -override=>1).
				    $q->hidden(-name => "rule_num",
					    -value => $rule_num,
					    -override => 1).
	                            $q->hidden(-name=>"sessionID",
	                                    -value=>$sessionID,
	                                    -override=>1).
				    $q->end_form().        
	                        $q->end_td().    
	                    $q->end_Tr().
	                $q->end_table().
	            $q->end_td().
	        $q->end_Tr().
	    $q->end_table();    
    $this->SUPER::display($HTMLOUT);
    exit;
}
1;

