#!/usr/bin/perl -w
#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: mail_graph.pm,v 1.3 2003/11/03 22:18:13 varkoly Exp $
#

# based on: mailgraph -- a postfix statistics rrdtool frontend
# copyright (c) 2000, 2001, 2002 David Schweikert <dws@ee.ethz.ch>
# released under the GNU General Public License

package mail_graph;

use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use File::Temp qw(tempfile);
use Display;
use RRDs;
use vars qw(@ISA);
use subs qw(exit);

# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    $self->{'xpoints'} = 540;
    $self->{'ypoints'} = 150;
    $self->{'ypoints_err'} = 80;
    $self->{'rrd'} = '/var/lib/mailgraph/mailgraph.rrd'; # path to where the RRD database is
    $self->{'tmp_dir'} = '/srv/www/htdocs/tmp/'; # temporary directory where to store the images
    $self->{'graphs'} = [
	{ title => 'mg_subtitle_day',   seconds => 3600*24,        },
	{ title => 'mg_subtitle_week',  seconds => 3600*24*7,      },
	{ title => 'mg_subtitle_month', seconds => 3600*24*31,     },
	{ title => 'mg_subtitle_year',  seconds => 3600*24*365, },
    ];
    return bless $self, $this;
}


sub display {
    my $this = shift;
    my $q = $this->{'cgi'};
    my $type = $q->param('type');
    if( defined $type && ($type eq "std" || $type eq "size" || $type eq "err") ){
        $this->draw_and_send($type);
    }else{
	$this->print_html();
    }
}

sub draw_and_send(){
    my $this = shift;
    my $type = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_GRAPH"))->data();
    my $q = $this->{'cgi'};
    my $interval = $q->param('interval');
    if( ! $interval || ( $interval < 0 ) || ( $interval > 3 ) ){
	$interval = 0;
    }
    my $range = $this->{'graphs'}->[$interval]->{'seconds'};
    my $title = $message->{$this->{'graphs'}->[$interval]->{'title'}};
    
    my ($fh, $file) = tempfile(DIR => "/tmp/");
    my $ret = undef;
    if( $type eq "size" ){
	$ret = $this->graph_size($range, $message->{'mg_data'}, $file, $message);
    }elsif( $type eq "err"){
	$ret = $this->graph_err($range, $message->{'mg_errors'}, $file, $message);
    }else{
	$ret = $this->graph($range, $message->{'mg_sent_received'}, $file, $message);
    }
    
    if( defined($ret) ){
	unlink($file);
        $this->display_error("--Unable_create_graph-- $ret", 
                    $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
        exit;            
    }else{
	print $q->header("image/png");
	binmode STDOUT;
	print <$fh>;
	$fh->close();
	unlink($file);
    }
}

sub graph($$$) {
    my $this = shift;
    my $range = shift;
    my $title = shift;
    my $file = shift;
    my $message = shift;
    Encode::from_to($title , 'utf8', 'iso-8859-2');
    my $mg_sent       = $message->{'mg_sent'};       Encode::from_to($mg_sent , 'utf8', 'iso-8859-2'); 
    my $mg_total      = $message->{'mg_total'};      Encode::from_to($mg_total , 'utf8', 'iso-8859-2'); 
    my $mg_received   = $message->{'mg_received'};   Encode::from_to($mg_received , 'utf8', 'iso-8859-2'); 
    my $mg_created_on = $message->{'mg_created_on'}; Encode::from_to($mg_created_on , 'utf8', 'iso-8859-2'); 
    my $rrdargs = "rrdtool graph $file ".
		"--imgformat PNG ".
		"--width $this->{'xpoints'} ".
		"--height $this->{'ypoints'} ".
		"--start -$range ".
		"--end -".int($range*0.01)." ".
		"--vertical-label msgs/min ".
		"--title \"$title\" ".
		"--lazy ".
		"\"DEF:sent=".$this->{'rrd'}.":sent:AVERAGE\" ".
		"\"DEF:recv=".$this->{'rrd'}.":recv:AVERAGE\" ".
		"\"DEF:msent=".$this->{'rrd'}.":sent:MAX\" ".
		"\"DEF:mrecv=".$this->{'rrd'}.":recv:MAX\" ".
		"\"CDEF:rsent=sent,60,*\" ".
		"\"CDEF:rrecv=recv,60,*\" ".
		"\"CDEF:rmsent=msent,60,*\" ".
		"\"CDEF:rmrecv=mrecv,60,*\" ".
		"\"CDEF:vsent=sent,UN,0,sent,IF,$range,*\" ".
		"\"CDEF:vrecv=recv,UN,0,recv,IF,$range,*\" ".
		"\"AREA:rsent#000099:$mg_sent\" ".
		"\"GPRINT:vsent:AVERAGE:$mg_total".'\: %.0lf msgs" '.
		'"GPRINT:rmsent:MAX:Max\: %.0lf msgs/min" '.
		"\"LINE2:rrecv#009900:.$mg_received\" ".
		"\"GPRINT:vrecv:AVERAGE:$mg_total".'\: %.0lf msgs" '.
		'"GPRINT:rmrecv:MAX:Max\: %.0lf msgs/min\l" '.
		'"HRULE:0#000000" '.
		'"COMMENT:\s" '.
		"\"COMMENT:$mg_created_on: ".localtime(time).'\r"';
    system("$rrdargs");
    return undef;
}

sub graph_size($$$)
{
    my $this = shift;
    my $range = shift;
    my $title = shift;
    my $file = shift;
    my $message = shift;
    Encode::from_to($title , 'utf8', 'iso-8859-2');
    my $mg_data       = $message->{'mg_data'};       Encode::from_to($mg_data , 'utf8', 'iso-8859-2'); 
    my $mg_total      = $message->{'mg_total'};      Encode::from_to($mg_total , 'utf8', 'iso-8859-2'); 
    my $mg_created_on = $message->{'mg_created_on'}; Encode::from_to($mg_created_on , 'utf8', 'iso-8859-2'); 

    my $rrdargs = "rrdtool graph $file ".
                "--imgformat PNG ".
                "--width $this->{'xpoints'} ".
                "--height $this->{'ypoints'} ".
                "--start -$range ".
                "--end -".int($range*0.01)." ".
                "--vertical-label msgs/min ".
                "--title \"$title\" ".
                "--lazy ".
	    	"\"DEF:data=".$this->{'rrd'}.':data:AVERAGE" '.
	    	"\"DEF:mdata=".$this->{'rrd'}.':data:MAX" '.
	    	'"CDEF:rdata=data,60,*" '.
	    	'"CDEF:rmdata=mdata,60,*" '.
	    	"\"CDEF:vdata=data,UN,0,data,IF,$range,*\" ".
	    	"\"LINE2:rdata#000099:$mg_data\" ".
	    	"\"GPRINT:vdata:AVERAGE:$mg_total".'\: %.0lf Bytes" '.
	    	'"GPRINT:rmdata:MAX:Max.\: %.0lf Bytes/min" '.
                '"HRULE:0#000000" '.
                '"COMMENT:\s" '.
                "\"COMMENT:$mg_created_on: ".localtime(time).'\r"';
    system("$rrdargs");
    return undef;
}



sub graph_err($$$)
{
    my $this = shift;
    my $range = shift;
    my $title = shift;
    my $file = shift;
    my $message = shift;
    Encode::from_to($title , 'utf8', 'iso-8859-2');
    my $mg_total      = $message->{'mg_total'};      Encode::from_to($mg_total , 'utf8', 'iso-8859-2'); 
    my $mg_created_on = $message->{'mg_created_on'}; Encode::from_to($mg_created_on , 'utf8', 'iso-8859-2'); 

    my $rrdargs = "rrdtool graph $file ".
                "--imgformat PNG ".
                "--width $this->{'xpoints'} ".
                "--height $this->{'ypoints'} ".
                "--start -$range ".
                "--end -".int($range*0.01)." ".
                "--vertical-label msgs/h ".
                "--title \"$title\" ".  
                "--lazy ".
                "\"DEF:bounced=".$this->{'rrd'}.":bounced:AVERAGE\" ".
                "\"DEF:rejected=".$this->{'rrd'}.":rejected:AVERAGE\" ".
                "\"DEF:mbounced=".$this->{'rrd'}.":bounced:MAX\" ".
                "\"DEF:mrejected=".$this->{'rrd'}.":rejected:MAX\" ".
                "\"CDEF:rbounced=bounced,3600,*\" ".
                "\"CDEF:rrejected=rejected,3600,*\" ".
                "\"CDEF:rmbounced=mbounced,60,*\" ".
                "\"CDEF:rmrejected=mrejected,60,*\" ".
                "\"CDEF:vbounced=bounced,UN,0,bounced,IF,$range,*\" ".
                "\"CDEF:vrejected=rejected,UN,0,rejected,IF,$range,*\" ".
                '"AREA:rbounced#000000:Bounced" '.
                "\"GPRINT:vbounced:AVERAGE:$mg_total".'\: %.0lf msgs" '.
                '"GPRINT:rmbounced:MAX:Max\: %.0lf msgs/min" '.
                '"LINE2:rrejected#BB0000:Rejected" '.
                "\"GPRINT:vrejected:AVERAGE:$mg_total".'\: %.0lf msgs" '.
                '"GPRINT:rmrejected:MAX:Max\: %.0lf msgs/min\l" '.
                '"HRULE:0#000000" '.
                '"COMMENT:\s" '.
                "\"COMMENT:$mg_created_on: ".localtime(time).'\r"';
    system("$rrdargs");
    return undef;
}

sub print_html()
{
    my $this = shift;
    my $q = $this->{'cgi'};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $message = ($session->getLanguage($confParam->{LANG},"MAIL_GRAPH"))->data();
    my $sessionID = $confParam->{"sessionID"};
    my $tab        = $q->param('tab');
    my $stab       = $q->param('stab');
    my $values = [ "0", "1", "2", "3" ];
    my $lables = { "0" => $message->{$this->{'graphs'}->[0]->{'title'}},
		   "1" => $message->{$this->{'graphs'}->[1]->{'title'}},
		   "2" => $message->{$this->{'graphs'}->[2]->{'title'}},
		   "3" => $message->{$this->{'graphs'}->[3]->{'title'}}
		 };
    my $interval = $q->param('interval');
    if( ! $interval || ( $interval < 0 ) || ( $interval > 3 ) ){
	$interval = 0;
    }
    my $subtitle = $message->{$this->{'graphs'}->[$interval]->{'title'}};
    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=	$q->start_Tr();
    $html .=	    $q->start_td();
    $html .=		$q->start_multipart_form(
				-action => $confParam->{'cgi_path'}."/mail_graph.pl",
				-name => "form");
    $html .=		    $q->start_table({-cellspacing => 0, -cellpadding=>3, -width=>"100%"});
    $html .=			$q->start_Tr();
    $html .=			    $this->help_th(2, $message->{'mg_title'}, "MAIL_GRAPH");
    $html .=			$q->end_Tr();
    $html .=			$q->start_Tr();
    $html .=			    $q->td({-class=>'AdminSubHeader', -colspan => 2}, $subtitle);
    $html .=			$q->end_Tr();

    $html .=			$q->start_Tr();
    $html .=			    $q->start_td({-colspan=>2});
    $html .=				$q->br();
    $html .=			    $q->end_td();
    $html .=			$q->end_Tr();
    $html .=			$q->start_Tr();
    $html .=			    $q->start_td({-align => "center", -valign => "middle"});
    $html .=				$message->{'mg_select_interval'}.": ";
    $html .=			    $q->end_td();
    $html .=			    $q->start_td({-valign => "middle"});
    $html .=				$q->popup_menu( -name => "interval",
							-values => $values,
							-labels => $lables,
							-default => "0");
    $html .=				$q->hidden( -name => "tab",
						    -value => $tab );
    $html .=				$q->hidden( -name => "stab",
						    -value => $stab );
    $html .=				$q->hidden( -name => "sessionID",
						    -value => $sessionID )." ";
    $html .=				$q->submit(-class=>"button", -name => $message->{'mg_display'});
    $html .=			    $q->end_td();
    $html .=			$q->end_Tr();
    $html .=			$q->start_Tr();
    $html .=			    $q->start_td({-colspan=>2});
    $html .=				$q->br();
    $html .=			    $q->end_td();
    $html .=			$q->end_Tr();
    
    $html .=			$q->start_Tr();
    $html .=			    $q->start_td({-align => "center", -colspan => 2});
    $html .=				$q->img({-src => $confParam->{'cgi_path'}."/mail_graph.pl?type=std&".
							 "sessionID=$sessionID&interval=$interval", 
						 -alt => "mailgraph"}).$q->br();;
    $html .=				$q->img({-src => $confParam->{'cgi_path'}."/mail_graph.pl?type=err&".
							 "sessionID=$sessionID&interval=$interval", 
						 -alt => "mailgraph"}).$q->br();;
    $html .=				$q->img({-src => $confParam->{'cgi_path'}."/mail_graph.pl?type=size&".
							 "sessionID=$sessionID&interval=$interval", 
						 -alt => "mailgraph"});
    $html .=			    $q->end_td();
    $html .=			$q->end_Tr();
    $html .=		    $q->end_table();
    $html .=		$q->end_form();
    $html .=	    $q->end_td();
    $html .=	$q->end_Tr();
    $html .= $q->end_table();
			
    $this->SUPER::display($html);
}

1;
