#
# Copyright (c) 2003 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: manage_aliases.pm,v 1.2 2003/05/09 13:17:13 varkoly Exp $
#
package manage_aliases;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use cfreader;
use Filterlist;
use strict;
use Display;
use LDAPUtils;
use Net::LDAP qw(:all);
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}


sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    
    my $doit = $cgi->param('doit');

    my $addremove = $cgi->param('addremove');
    my $save      = $cgi->param('save');
    my $add       = $cgi->param('add');
    my $remove    = $cgi->param('remove');
    my $alias     = $cgi->param('alias');


    if( ( defined $addremove && $addremove ne "" ) &&
	( defined $alias && $alias ne "" ) ) {
	$this->add_user_alias();
    } elsif( ( defined $save && $save ne "" ) &&
	     ( defined $alias && $alias ne "" ) ) {
	$this->rename_alias();
    } elsif( defined $add && $add ne "" ) {
	$this->add_alias();
    } elsif( defined $remove && $remove ne "" ) {
	$this->remove_alias();
    } else {
	$this->startup();
    }

}

sub startup {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $cgi       = $this->{"cgi"};

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"ALIASES"))->data();

    my $showby    = $cgi->param('showby') || "alias";
    my $filter    = $cgi->param('filter') || "*";
    my $setfilter = $cgi->param('setfilter');

    my $url = "$confParam->{cgi_path}/manage_aliases.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $displayhash;
    my $outer = "alias";
    my $inner = "uid";

    if( $showby eq "alias" ) {
	$outer = "alias";
	$inner = "uid";
    } elsif ( $showby eq "user" ) {
	$outer = "uid";
	$inner = "alias";
    }
    
    if( defined $setfilter && $setfilter ne "" ) {

	my $ldap= Net::LDAP->new($confParam->{LDAPserver},
				 port => $confParam->{LDAPport},
				 version => 3);
	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server", $url);
	    exit;
	}
	my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
				 password=>$confParam->{"passwd"});
	if($result->code != 0) {
	    my $errs = "bind $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    exit;
	}

	$result = $ldap->search(base=>$confParam->{"baseDN"},
				scope=>"one",
				filter=>"($outer=$filter)",
				attrs=>['alias','uid']);
	if($result->code != 0) {
	    my $errs = "search $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    $ldap->unbind;
	    exit;
	}
	
	my @aentries = $result->entries;

	foreach my $entry ( @aentries ) {
	    foreach my $o ( $entry->get_value("$outer") ) {
		foreach my $i ( $entry->get_value("$inner") ) {
		    push @{$displayhash->{"$o"}}, $i;
		    #print STDERR "$o -> $i\n";
		}
	    }
	}
    }


    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_table({-border=>0, -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    
    $html .=       $cgi->Tr( $this->help_th(1, "$message->{aliases_head}", "MANAGE_ALIASES") );
    
    $html .=       $cgi->start_Tr();
    $html .=         $cgi->start_td();
    $html .=           $cgi->start_table({-width=>"100%"});
    $html .=           $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_aliases.pl");
    $html .=           $cgi->Tr([
				 $cgi->td({-valign=>"center", -nowrap=>undef},
					  [
					   "$message->{aliases_sortby}",
					   $cgi->popup_menu(-name=>"showby",
							    -values=>[ 'user', 'alias'],
							    -default=>'alias',
							    -labels=>{ 'user' => $message->{aliases_users},
								       'alias' => $message->{aliases_alias}} ),
					   $cgi->textfield(-name=>"filter", -value=>$filter),
					   $cgi->submit(-name=>"setfilter",
							-class=>"BlockButton",
							-value=>"$message->{aliases_setfilter}").
					   $cgi->hidden(-name=>"sessionID",-value=>"$sessionID").
					   $cgi->hidden(-name=>"tab",-value=>"$tab").
					   $cgi->hidden(-name=>"stab",-value=>"$stab")
					   ]
					   ),
				 $cgi->td({-colspan=>4, -align=>"center"},
					  $cgi->hr({-noshade => '1', -size => '1', -width=>'98%'})
					  )
				 ]
				);
    $html .=           $cgi->end_form;
    $html .=           $cgi->end_table;
    $html .=         $cgi->end_td;
    $html .=       $cgi->end_Tr;


    $html .=       $cgi->start_Tr();
    $html .=         $cgi->start_td();
    $html .=           $cgi->start_table({-width=>"100%"});
    
    $html .=       $cgi->start_Tr();
    my $colspan;
    if( $outer eq "alias" ) {
	$colspan = 5;
	$html .=       $cgi->td({-nowrap=>undef},
				["<b>$message->{aliases_alias}</b>", "", "&nbsp;|&nbsp;", "<b>$message->{aliases_forward}</b>", ""]);
    } else {
	$colspan = 6;
	$html .=       $cgi->td({-nowrap=>undef},
				["<b>$message->{aliases_users}</b>", "<b>$message->{aliases_alias}</b>", "", "&nbsp;|&nbsp;", "<b>$message->{aliases_new}</b>", ""]);
    }
    $html .=       $cgi->end_Tr();
    $html .=       $cgi->start_Tr();
    $html .=       $cgi->td({-colspan=>$colspan, -align=>"center"},
			    $cgi->hr({-noshade => '1', -size => '1', -width=>'98%'}));
    $html .=       $cgi->end_Tr();

    if( defined $displayhash ) {
	
	foreach my $key ( sort keys %$displayhash ) {
	    if( $outer eq "alias" ) {
		# aliases on the right hand
		$html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_aliases.pl");
		$html .= $cgi->Tr($cgi->td({-width=>1},
					   [
					    $cgi->textfield(-name=>"alias",
							    -value=>"$key",
							    -override=>1
							    ),
					    $cgi->submit(-name=>"save",
							 #-class=>"BlockButton",
							 -value=>"$message->{aliases_save}"
							 )
					    ]
					   ),
				  $cgi->td({-width=>1},
					   "&nbsp;|&nbsp;"
					   ).
				  $cgi->td({-width=>1},
					   $cgi->scrolling_list(-name=>"list",
								-values=>$displayhash->{"$key"},
								-override => 1, 
								-multiple=>1
								)).
				  $cgi->td({-width=>1}, 
					   [
					    $cgi->submit(-name=>"addremove",
							 #-class=>"BlockButton",
							 -value=>"$message->{aliases_addremove}"
							 ).
					    $cgi->hidden(-name=>"sessionID",-value=>"$sessionID").
					    $cgi->hidden(-name=>"oldalias",-value=>"$key").
					    $cgi->hidden(-name=>"filter",-value=>"$filter").
					    $cgi->hidden(-name=>"tab",-value=>"$tab").
					    $cgi->hidden(-name=>"stab",-value=>"$stab")
					    ]
					   )
				  );
		#$html .= $cgi->Tr($cgi->td({-colspan=>5},$cgi->as_string));
		$html .= $cgi->end_form();
	    } else {
		# user on the right hand, must NOT be editable!
		$html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_aliases.pl");
		$html .= $cgi->Tr($cgi->td({-width=>1}, "$key"),
				  $cgi->td({-width=>1}, 
					   $cgi->scrolling_list(-name=>"list",
								-values=>$displayhash->{"$key"},
								-multiple=>1
								)),
				  $cgi->td({-width=>1}, 
					   [
					    $cgi->submit(-name=>"remove",
							 #-class=>"BlockButton",
							 -value=>"$message->{aliases_remove}"
							 ),
					    "&nbsp;|&nbsp;",
					    $cgi->textfield(-name=>"newalias", -value=>""),
					    $cgi->submit(-name=>"add",
							 #-class=>"BlockButton",
							 -value=>"$message->{aliases_add}"
							 ).
					    $cgi->hidden(-name=>"sessionID",-value=>"$sessionID").
					    $cgi->hidden(-name=>"filter",-value=>"$filter").
					    $cgi->hidden(-name=>"uid",-value=>"$key").
					    $cgi->hidden(-name=>"tab",-value=>"$tab").
					    $cgi->hidden(-name=>"stab",-value=>"$stab")
					    ]
					   )
				  );
		$html .= $cgi->end_form();
	    }
	}
    }

    $html .=           $cgi->end_table;
    $html .=         $cgi->end_td;
    $html .=       $cgi->end_Tr;

    $html .=     $cgi->end_table;
    $html .=     $cgi->end_td;
    $html .=   $cgi->end_Tr;
    $html .= $cgi->end_table;

    $this->SUPER::display($html);
}

sub add_user_alias {
    my $this = shift;
    
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"ALIASES"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $alias   = $cgi->param('alias');
    my $filter  = $cgi->param('filter') || "*";

    my $url = "$confParam->{cgi_path}/manage_aliases.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&filter=$filter\&setfilter=1\&showby=alias";
    $this->back_url($url);

    my $ldap= Net::LDAP->new($confParam->{LDAPserver},
			     port => $confParam->{LDAPport},
			     version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server", $url);
	exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
	my $errs = "bind $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	exit;
    }

    if( (defined $cgi->param("ALIAS_SAVE")) && ( $cgi->param("ALIAS") ne "NOFILTER" ) ) {
      	my @uids = $cgi->param("UID"); #new setting

	# add alias to users which don't have one
	foreach my $uid ( @uids ) {
	    
	    $result = $ldap->search(base=>"uid=$uid,".$confParam->{"baseDN"},
				    scope=>"base",
				    filter=>"(alias=$alias)",
				    attrs=>['uid']);
	    if($result->code != 0) {
		my $errs = "search $message->{failed}\n\n";
		$errs .= &LDAPerror($result);
		$this->display_error($errs, $url);
		$ldap->unbind;
		exit;
	    }

	    if( $result->count == 0 ) {
		$result = $ldap->modify(dn => "uid=$uid,".$confParam->{baseDN},
					add => {alias => $alias}
					);
		if($result->code != LDAP_SUCCESS){
		    my $errs = "modify $message->{failed}\n\n";
		    $errs .= &LDAPerror($result);
		    $this->display_error($errs, $url);
		    $ldap->unbind;
		    exit;
		}
	    }
	    
	}

	# remove aliases from users if they are no longer in the list
	$result = $ldap->search(base=>$confParam->{"baseDN"},
				scope=>"one",
				filter=>"(alias=$alias)",
				attrs=>['uid', 'alias']);
	if($result->code != 0) {
	    my $errs = "search $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    $ldap->unbind;
	    exit;
	}

	my @aentries = $result->entries;
	foreach my $entry ( @aentries ) {
	    my $euid = ($entry->get_value("uid"))[0];

	    my $found = 0;
	    foreach my $uid ( @uids ) {
		$found = 1 if $uid eq $euid;
	    }
	    if( ! $found ) {
		my @aliases = $entry->get_value("alias");
		my $newaliases;
		foreach my $a ( @aliases ) {
		    if( $a ne $alias ) {
			push @$newaliases, $a;
		    }
		}
		$entry->replace( "alias" => $newaliases );
		$result = $entry->update( $ldap );
		if($result->code != 0) {
		    my $errs = "update $message->{failed}\n\n";
		    $errs .= &LDAPerror($result);
		    $this->display_error($errs, $url);
		    $ldap->unbind;
		    exit;
		}
	    }
	}
    }

    my $filterlist = Filterlist->new($cgi, $session, $ldap, $this);
    my $actions = $cgi->start_table({-border=>0,-width=>"100%"});
    $actions .=   $cgi->Tr(
			   $cgi->td({-valign=>"top",-align=>"center"},
				    $cgi->submit(-class=>"button",
						 -name=>"ALIAS_SAVE",
						 -value=>"$message->{aliases_save}",
						 -width=>"200")));
    $actions .=   $cgi->Tr(
			   $cgi->td({-valign=>"top",-align=>"center"},
				    $cgi->button(-class=>"button",-value=>"$message->{aliases_back}",-width=>"200",
						 -onclick=>"location.href='".$url."'")
				    )
			   );
    $actions .=   $cgi->end_table();

    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=   $cgi->start_Tr();
    $html .=     $cgi->start_td();
    $html .=       $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_aliases.pl");
    $html .=       $cgi->start_table({-border=>0, -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    
    $html .=       $cgi->Tr( $this->help_th(3, "$message->{aliases_add_user} <i>$alias</i>", "MANAGE_ALIASES") );

    $html .=         $filterlist->make_filter_view({"textfield"    => "filter", #name textfield
                                                    "textlabel"    => "$message->{aliases_filter}",
                                                    "buttonlabel"  => "$message->{aliases_setfilter}",
                                                    "listname"     => "UID", #name list
                                                    "listlabel_left" => "$message->{aliases_users}",
                                                    "listlabel_right" => $actions,
                                                    "nofilter-msg" => "$message->{aliases_nofilter}",
						    "listfilter"   => "(&(objectclass=posixAccount)(!(gidNumber=103))(|(sn=$filter)(givenname=$filter)(uid=$filter)))",
						    "listattrs"	   => ['uid', 'cn'],
                                                    "width"        => 30, #width of textfield
                                                    "height"       => 20, #length of list
                                                    "multiple"     => "MULTIPLE",
                                                    "selectedfilter"=> "(&(objectclass=posixAccount)(!(gidNumber=103))(alias=$alias))",
						    "selectedregex" => ['uid', '.*'],
                                                    "labelexp"     => ["\$\$ ","uid","(\$\$) ","cn"],
                                                                      #ldap rule for defaults
                                                    "error-url"    => $this->{"url"} #url for errors
                     });
    $html .=         $cgi->hidden(-name=>"sessionID",-value=>"$sessionID");
    $html .=         $cgi->hidden(-name=>"addremove",-value=>"gaga");
    $html .=         $cgi->hidden(-name=>"alias",-value=>"$alias");
    $html .=         $cgi->hidden(-name=>"tab",-value=>"$tab");
    $html .=         $cgi->hidden(-name=>"stab",-value=>"$stab");


    $html .=     $cgi->end_table;
    $html .=     $cgi->end_form();
    $html .=     $cgi->end_td;
    $html .=   $cgi->end_Tr;
    $html .= $cgi->end_table;

    $this->SUPER::display($html);

    $ldap->unbind;
}

sub rename_alias {
    my $this = shift;
    
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"ALIASES", "CREATE_USER"))->data();

    my $tab      = $cgi->param('tab');
    my $stab     = $cgi->param('stab');
    my $alias    = $cgi->param('alias');
    my $oldalias = $cgi->param('oldalias');
    my $filter  = $cgi->param('filter') || "*";

    my $url = "$confParam->{cgi_path}/manage_aliases.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&filter=$filter\&setfilter=1\&showby=alias";
    $this->back_url($url);

    if($alias =~ /[^a-zA-Z0-9-_.&]+/ ) {  #   Match a non-word character
	$this->display_error($alias.": ".$message->{default_alias_not_set},
			     $url);
	exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver},
			     port => $confParam->{LDAPport},
			     version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server", $url);
	exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
	my $errs = "bind $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	exit;
    }

    $result = $ldap->search(base=>$confParam->{"baseDN"},
			    scope=>"one",
			    filter=>"(alias=$oldalias)",
			    attrs=>['alias']);
    if($result->code != 0) {
	my $errs = "search $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	$ldap->unbind;
	exit;
    }

    my @aentries = $result->entries;

    foreach my $entry ( @aentries ) {
	my @aliases = $entry->get_value("alias");
	my $newaliases;
	push @$newaliases, $alias;
	foreach my $a ( @aliases ) {
	    if( $a ne $oldalias ) {
		push @$newaliases, $a;
	    }
	}
	$entry->replace( "alias" => $newaliases );
	$result = $entry->update( $ldap );
	if($result->code != 0) {
	    my $errs = "update $message->{failed}\n\n";
	    $errs .= &LDAPerror($result);
	    $this->display_error($errs, $url);
	    $ldap->unbind;
	    exit;
	}
	
    }

    $ldap->unbind;
    $this->display_msg("$message->{aliases_renamed}", $url, 1);
}

sub add_alias {
    my $this = shift;
    
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"ALIASES", "CREATE_USER"))->data();

    my $tab      = $cgi->param('tab');
    my $stab     = $cgi->param('stab');
    my $uid      = $cgi->param('uid');
    my $newalias = $cgi->param('newalias');
    my $filter   = $cgi->param('filter') || "*";

    my $url = "$confParam->{cgi_path}/manage_aliases.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&filter=$filter\&setfilter=1\&showby=user";
    $this->back_url($url);

    if($newalias =~ /[^a-zA-Z0-9-_.&]+/ ) {  #   Match a non-word character
	$this->display_error($newalias.": ".$message->{default_alias_not_set},
			     $url);
	exit;
    }

    if( ! defined $newalias || $newalias eq "" ) {
	$this->display_error("$message->{aliases_invalid}",$url);
	exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver},
			     port => $confParam->{LDAPport},
			     version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server", $url);
	exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
	my $errs = "bind $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	exit;
    }

    $result = $ldap->modify(dn => "uid=$uid,".$confParam->{baseDN},
			    add => { alias => $newalias}
			    );
    if($result->code != LDAP_SUCCESS){
	my $errs = "modify $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	$ldap->unbind;
	exit;
    }

    $ldap->unbind;
    $this->display_msg("$message->{aliases_added}", $url, 1);
}

sub remove_alias {
    my $this = shift;
    
    my $cgi       = $this->{"cgi"};
    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"ALIASES"))->data();

    my $tab      = $cgi->param('tab');
    my $stab     = $cgi->param('stab');
    my $uid      = $cgi->param('uid');
    my @remove   = $cgi->param('list');
    my $filter  = $cgi->param('filter') || "*";

    my $url = "$confParam->{cgi_path}/manage_aliases.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&filter=$filter\&setfilter=1\&showby=user";
    $this->back_url($url);

    if( ! defined @remove ) {
	$this->display_error("$message->{aliases_nothing_selected}",$url);
	exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver},
			     port => $confParam->{LDAPport},
			     version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server", $url);
	exit;
    }
    my $result = $ldap->bind(dn=>"uid=".$confParam->{uid}.",".$confParam->{baseDN},
			     password=>$confParam->{"passwd"});
    if($result->code != 0) {
	my $errs = "bind $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	exit;
    }

    my @changes;
    foreach my $a ( @remove ) {
	push @changes, "delete", [ "alias", [ "$a" ] ];
    }
    $result = $ldap->modify(dn => "uid=$uid,".$confParam->{baseDN},
			    changes => \@changes
			    );
    if($result->code != LDAP_SUCCESS){
	my $errs = "modify $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	$ldap->unbind;
	exit;
    }

    $ldap->unbind;
    $this->display_msg("$message->{aliases_deleted}", $url, 1);
}


1;
