#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: manage_folder_permissions.pm,v 1.6 2003/11/06 10:49:28 varkoly Exp $
#

package manage_folder_permissions;
use strict;
use Encode;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use Net::IMAP;
use URI::Escape;
use LDAPUtils;
use IMAPUtils;
use Net::SMTP;
use POSIX qw(strftime setsid);
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_FOLDER",
					 "MANAGE_FOLDER_PERMISSIONS", "BROWSE_USER"))->data();

    my $tab       = $cgi->param('tab');
    my $stab      = $cgi->param('stab');
    my $Action    = $cgi->param('Action') || '';
    my $doit      = $cgi->param('doit') || '';
    my $Folder    = $cgi->param('Folder') || '';
    my $User      = $cgi->param('User') || '';
    my $Group     = $cgi->param('Group') || '';
    my $Filter    = $cgi->param('Filter') || '*';
    my $Permissions = join('', $cgi->param('Permission')) || '*';
    $cgi->delete('Permission');
    my %labels=("0"=>"-- ".$message->{set_filter_first}." --");
    my @user=("0");
    
    my $LOGINuid      = $confParam->{uid};
    my $LOGINpasswd   = $confParam->{passwd};
    
    my $ldapbase      = $confParam->{baseDN};
    my $ldaphost      = $confParam->{LDAPserver};
    my $ldapport      = $confParam->{LDAPport};
    my $imaphost      = $confParam->{IMAPserver};
    my $imapport      = $confParam->{IMAPport};
    
    #print STDERR "MANAGE PERM OF <$Folder>\n";
    my $DECFolder = IMAPUtils::utf7_decode($Folder);

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    my $imap = new Net::IMAP($imaphost, Debug => 0);
    if(!defined $imap) {
	$this->display_error("can't connect to $imaphost: $!",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }

    my $ret = $imap->login($LOGINuid, $LOGINpasswd);
    
    if($$ret{Status} ne "ok") {
	$this->display_error("login $message->{failed}: Serverresponse:$$ret{Status} => $$ret{Text}\n",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab");
    } else {
	$ldap->bind ("uid=".$confParam->{uid}.", ".$confParam->{baseDN},
		     password=>$confParam->{passwd});
	
	# User wants to SET permissions for an USER
	if($Action eq $message->{'set'} && $Folder ne '' && 
	   $User ne '' && $User ne "-- ".$message->{set_filter_first}." --") {
	    my $resp = $imap->setacl($Folder, $User, $Permissions);
	    if($$resp{Status} eq "ok") {
		$Folder = $cgi->escape("$Folder");
		if( defined $::CONF_GLOBAL{'GENERAL'}{'EnableFolderPermissionMail'} && 
		    $::CONF_GLOBAL{'GENERAL'}{'EnableFolderPermissionMail'} eq 'true' ) {
		    if( $confParam->{uid} eq "cyrus" ) {
			$this->mail2user($Folder, $User);
		    }
		}
		my $uri = "$confParam->{cgi_path}/manage_folder_permissions.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&Folder=$Folder";
		$this->display_msg($message->{'set_access_for'}." <i>$User</i> ".$message->{'set_access_to'}.
				   " <i>$Permissions</i>", $uri, 3);
		$imap->logout;
		$ldap->unbind;
		exit;
	    } else {
		$this->display_error("SETACL $message->{failed}! Folder:$DECFolder User:$User Permissions:$Permissions\n$$resp{Text}", 
				     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		$imap->logout;
		$ldap->unbind;
		exit;
	    }
	} elsif($Action eq $message->{'set'} && $Folder ne '' && $Group ne '') {
	    # User wants to SET permissions for a GROUP
	    
	    my $resp = $imap->setacl($Folder, "group:$Group", $Permissions);
	    if($$resp{Status} eq "ok") {
		$Folder = $cgi->escape("$Folder");
		if( defined $::CONF_GLOBAL{'GENERAL'}{'EnableFolderPermissionMail'} && 
		    $::CONF_GLOBAL{'GENERAL'}{'EnableFolderPermissionMail'} eq 'true' ) {
		    if( $confParam->{uid} eq "cyrus" ) {
			$this->mail2group($Folder, $Group);
		    }
		}
		my $uri = "$confParam->{cgi_path}/manage_folder_permissions.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&Folder=$Folder";
		$this->display_msg($message->{'set_access_for_group'}." <i>$Group</i> ".$message->{'set_access_to'}.
				   " <i>$Permissions</i>", $uri , 3);
		$imap->logout;
		$ldap->unbind;
		exit;
	    } else {
		$this->display_error( 
			      "SETACL $message->{failed}! Folder:$DECFolder Group:$Group Permissions:$Permissions\n$$resp{Text}", 
			      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		$imap->logout;
		$ldap->unbind;
		exit;
	    }
	} elsif($Action eq $message->{'b_delete'} && $Folder ne '' && $User ne '') {
	    # User wants to DELETE permissions for an user or a group
	    
	    my $resp = $imap->deleteacl($Folder, $User);
	    if($$resp{Status} eq "ok") {
		$Folder = $cgi->escape("$Folder");
		my $uri = "$confParam->{cgi_path}/manage_folder_permissions.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&Folder=$Folder";
		$this->display_msg($message->{'delete_access_for'}." $User$Group", $uri , 3);
		$imap->logout;
		$ldap->unbind;
		exit;
	    } else {
		$this->display_error(
			      "DELETEACL $message->{failed}! Folder:$DECFolder User/Group:$User Permissions:$Permissions\n$$resp{Text}", 
			      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		$imap->logout;
		$ldap->unbind;
		exit;
	    }
	} elsif($doit eq 'Filter' and $Filter ne '') {
	    # The userlist hat to be filled
	    my $exist;
	    my $entry;
	    my $uid;
	    my $cn;
	    my $i;
	    
	    $Filter =~ s/,.*$//;
	    
	    if($Filter ne "") {
		$exist=$ldap->search(base=>"$confParam->{baseDN}",
				     scope  => "one",
				     attrs => ['uid', 'cn'],
				     filter=>"(&(uid=$Filter)(objectclass=SuSEeMailObject)(objectclass=posixAccount)(!(gidnumber=103)))");
		
		if ($exist->count!=0) {
		    @user=('');
		    $labels{''} = '-';

		    for ($i=0; $i<$exist->count; $i++) {
			$entry=$exist->entry($i);
			$uid=$entry->get_value("uid");
			$cn=$entry->get_value("cn");
			Encode::from_to($cn , "utf8", "$this->{'charset'}");
			push @user,$uid;
			$labels{$uid} = "$uid ($cn)";
		    }
		    @user = sort @user;
		    # --- add anyone to set rights to everybody :-)
		    push @user, "anyone";
		    $labels{'anyone'} = "anyone ($message->{anyone})";
		}
	    }
	}
	
	
	########################################################################################
	#       Generating of the HTML-Code
	########################################################################################
	
	# get the group-list
	my $result = $ldap->search(base => $confParam->{baseDN},
				   scope  => "one",
				   attrs => ['cn'],
				   filter => "(&(cn=*)(objectclass=posixGroup))");
	my @cn = ('');
	
	if($result->code == 0) {
	    foreach my $entry ($result->all_entries) {
		push @cn, $entry->get_value("cn");
	    }
	    @cn = sort @cn;
	} else {
	    my $err = "$message->{err_grouplist}!\n".&LDAPerror($result);
	    $this->display_error("$err",
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    $ldap->unbind;
	    $imap->logout;
	    exit;
	}
	
	$ldap->unbind;
	
	# fetch all the permissions already given
	my $anof = sub {
	    my $self = shift;
	    my $resp = shift;
	    
	    foreach ($resp->identifiers) {
		$this->{"acl"}->{$_} = $resp->identifier($_);
		#print STDERR "KEY:$_\tVALUE:$acl{$_}\n";
	    }
	};
	
	$imap->set_untagged_callback('acl', $anof);
	$ret = $imap->getacl($Folder);
	
	if( $ret->status ne 'ok' ) {
	    if( $$ret{Text} =~ /Mailbox does not exist/ ) {

		my $text = "";
		
		# --- recreate Folder
		my $resp = $imap->create($Folder);
		
		if($$resp{Status} ne "ok") {
		    $text =  "create failed: Serverresponse:$$resp{Status} => $$resp{Text}\n";
		}
		
		# --- set all righs to admin
		$resp = $imap->setacl($Folder, $LOGINuid, "lrswipcda");
		if($$resp{Status} ne "ok") {
		    $text .=  "setacl failed: Serverresponse:$$resp{Status} => $$resp{Text}\n";
		    $text .= "I continue ...\n";
		} 
		
		$resp = $imap->deleteacl($Folder, "anyone");
		if($$resp{Status} ne "ok") {
		    $text .= "deleteacl failed: Serverresponse:$$resp{Status} => $$resp{Text}\n";
		    $text .= "I continue ...\n";
		} 
		
		if($text ne "") {
		    $this->display_error($text,
					 "$confParam->{cgi_path}/create_sfolder.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
		    $imap->logout;
		    exit;
		}

		if( $text eq "" ) {
		    $this->display_error( "<b>$DECFolder:</b> $message->{folder_recreated}",
					  "$confParam->{cgi_path}/manage_folder_permissions.pl?sessionID=$sessionID\&Folder=$Folder\&tab=$tab&stab=$stab");
		} else {
		    $this->display_error( "<b>$DECFolder:</b> $message->{folder_recreation_failed}\n$text",
					  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		}
	    } else {
		$this->display_error( 
				      "$message->{fetch_acl} $DECFolder $message->{failed}: Serverresponse:$$ret{Status} => $$ret{Text}",
				      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    }
	    $imap->logout;
	    exit;
	} else {
	    my $html = "";
	    $cgi->delete('User');
	    $cgi->delete('Action');

	    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
	    $html .= $cgi->start_Tr();
	    $html .= $cgi->start_td();
	    ####### existing permissions section #######
	    $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
	    $html .= $cgi->Tr(
			      [
			       $this->help_th(1, $message->{'existing'},
					      "PERMISSION_EXIST"),
			       $cgi->td({-class=>'AdminSubHeader'},
					"$message->{'folder'}: $DECFolder")
			       ]
			      );
	    $html .= $cgi->Tr(
			      $cgi->td({-align=>"center"},
				       $this->ExistingPermissionsList())
			      );
	    $html .= $cgi->end_table();
	    $html .= $cgi->end_td();
	    $html .= $cgi->end_Tr();
	    $html .= $cgi->end_table();

	    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
	    ########## add permissions section ##########
	    $html .= $cgi->start_Tr();
	    $html .= $cgi->start_td();
	    $html .= $cgi->start_table({-cellspacing=>0, -width => "100%", -cellpadding=>3});
	    $html .= $cgi->Tr(
			      [
			       $this->help_th(2, $message->{'add_permissions'},
					      "PERMISSION_ADD"),
			       $cgi->td({-class=>'AdminSubHeader', -colspan=>2
					 },
					"$message->{'folder'}: $DECFolder")
			       ]
			      );
	    $html .= $cgi->Tr(
			      $cgi->td({-colspan=>2,
					-align=>"center"},
				       $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_folder_permissions.pl"),
				       $cgi->textfield(-name=>'Filter',
						       -default=>'*')." ".
				       $cgi->submit(-class=>"button",-name=>'Action',
						    -value=>'Filter').
				       $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
				       $cgi->hidden(-name=>'Folder', -value=>$Folder).
				       $cgi->hidden(-name=>'doit', -value=>"Filter").
				       $cgi->hidden(-name=>"tab", -value=>$tab).
				       $cgi->hidden(-name=>"stab", -value=>$stab).
				       $cgi->end_form()
				       )
			      );
	    $html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_folder_permissions.pl");
	    $html .= $cgi->Tr(
			      $cgi->td({-align=>"center", -colspan=>2,
				    },
				       "<hr noshade size=\"2\"><br>$message->{'user'} ".
				       $cgi->scrolling_list(-name=>"User",
							    -value=>\@user,
							    -default=>$user[2],
							    -size=>1,
							    -labels=>\%labels,
							    -width=>200).
				       "<br><b>$message->{'or'}</b> $message->{'group'} ".
				       $cgi->scrolling_list(-name=>"Group",
							    -value=>\@cn,
							    -size=>1,
							    -width=>200).
				       " $message->{'may'}:<br><hr noshade size=\"2\"><br>")
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission',
						      -value=>'l',
						      -label=>$message->{'lookup'})), 
			      $cgi->td(
				       $message->{'desc_lookup'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission',
						      -value=>'r', 
						      -label=>$message->{'read'})), 
			      $cgi->td($message->{'desc_read'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'s', 
						      -label=>$message->{'store'})), 
			      $cgi->td($message->{'desc_store'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'w', 
						      -label=>$message->{'write'})), 
			      $cgi->td($message->{'desc_write'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'i', 
						      -label=>$message->{'insert'})),
			      $cgi->td(
				       $message->{'desc_insert'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'p', 
						      -label=>$message->{'post'})), 
			      $cgi->td($message->{'desc_post'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'c', 
						      -label=>$message->{'create'})), 
			      $cgi->td(
				       $message->{'desc_create'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission',
						      -value=>'d', 
						      -label=>$message->{'delete'})),
			      $cgi->td(
				       $message->{'desc_delete'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td(
				       $cgi->checkbox(-name=>'Permission', 
						      -value=>'a', 
						      -label=>$message->{'administer'})),
			      $cgi->td(
				       $message->{'desc_administer'})
			      );
	    $html .= $cgi->Tr(
			      $cgi->td({-align=>"center", -colspan => 2},
				       $cgi->submit(-class=>"button",-name=>'Action',
						    -value=>$message->{'set'}).
				       $cgi->hidden(-name=>'sessionID', -default=>$sessionID).
				       $cgi->hidden(-name=>'Folder', -default=>$Folder).
				       $cgi->hidden(-name=>'Filter', -default=>$Filter).
				       $cgi->hidden(-name=>"tab", -value=>$tab).
				       $cgi->hidden(-name=>"stab", -value=>$stab).
				       $cgi->end_form()
				       )
			      );
	    
	    $html .= $cgi->end_table();
	    $html .= $cgi->end_td();
	    $html .= $cgi->end_Tr();
	    $html .= $cgi->end_table();
	    
	    $this->SUPER::display($html);
	}
    }

    $imap->logout();
    
}

sub ExistingPermissionsList {
    my $this = shift;
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_FOLDER",
					 "MANAGE_FOLDER_PERMISSIONS", "BROWSE_USER"))->data();

    my $tab       = $cgi->param('tab');
    my $stab      = $cgi->param('stab');
    my $Folder    = $cgi->param('Folder') || '';
    my $User      = $cgi->param('User') || '';
    my $Group     = $cgi->param('Group') || '';

    my $LOGINuid      = $confParam->{uid};
    
    my $pers;
    my $HTMLList = $cgi->start_table({-cellspacing=>0, -cellpadding=>5, -width=>"100%"})."\n";
    
    my $acl = $this->{"acl"};
    foreach $pers (keys %$acl ) {
	unless($pers eq 'cyrus' && $LOGINuid ne 'cyrus') {
	    my @aclarray = mkarray( $this->{"acl"}->{$pers} );
	    
	    $HTMLList .= $cgi->Tr($cgi->td({-align=>"center",
					    -width=>60}, "<font >$pers</font>"),
				  $cgi->td({-align=>"center"},
					   $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_folder_permissions.pl")."\n".
					   $cgi->checkbox_group(-name=>'Permission', 
								-values=>[ 'l','r','s','w','i','p','c','d', 'a'],
								-labels=>{'l' => 'l ',
									  'r' => 'r ',
									  's' => 's ',
									  'w' => 'w ',
									  'i' => 'i ',
									  'p' => 'p ',
									  'c' => 'c ',
									  'd' => 'd ',
									  'a' => 'a '},
								-defaults=>\@aclarray)."\n<BR>".
					   $cgi->submit(-class=>"button",-name=>'Action', -value=>$message->{'b_delete'})." ".
					   $cgi->submit(-class=>"button",-name=>'Action', -value=>$message->{'set'}).
					   $cgi->hidden(-name=>'User', -default=>$pers).
					   $cgi->hidden(-name=>'sessionID', -default=>$sessionID).
					   $cgi->hidden(-name=>'Folder', -default=>$Folder).
					   $cgi->hidden(-name=>"tab", -value=>$tab).
					   $cgi->hidden(-name=>"stab", -value=>$stab).
					   $cgi->end_form()
					  )
				 );
	}
    }
    
    $HTMLList .= $cgi->end_table();
    
    return $HTMLList;
}

sub mkarray($) {
    my ($string) = @_;
    my @array;
    my $i;
    
    for($i = 0; $i < length($string); $i++) {
	push(@array, substr($string, $i, 1));
    }
    
    return @array;
}

sub mail2user {
    my $this = shift;

    my $folder = shift;
    my $user   = shift;

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $smtpserver = $confParam->{'SMTPserver'};
    my $message = ($session->getLanguage($confParam->{LANG}, "MANAGE_FOLDER_PERMISSIONS"))->data();

    my $msg = 
	"Subject: $message->{new_folder_access_subject} $folder\n".
	"To: $user\n".
	"\n".
	"$message->{new_folder_access_body}\n";

    # fork and detach mail sending process
    # see: "http://apachetoday.com/news_story.php3?ltsn=2001-02-27-001-06-NW-LF-PL" why all this
    # is done
    $SIG{'CHLD'} = 'IGNORE';
    my $pid;
    my $apache_req = Apache->request();
    if (! defined ($pid=fork())){
	print STDERR "fork() failed in mail2user\n";
    }elsif($pid == 0){
	#child
	chdir '/' or die "Can't chdir to /: $!";
	close STDIN;
	close STDOUT;
	setsid or die "Can't start a new session: $!";
	my $smtp = Net::SMTP->new("$smtpserver");
	$smtp->mail("mailadmin");
	$smtp->to($user);
	$smtp->data();
	$smtp->datasend($msg);
	$smtp->dataend;
	$smtp->quit;
	CORE::exit;
    }
}

sub mail2group {
    my $this = shift;

    my $folder = shift;
    my $group  = shift;

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $basedn    = $confParam->{"baseDN"};
    my $maildomain = $confParam->{'mailDomain'};
    my $date = strftime("%a, %d %b %Y %H:%M:%S %z\n",localtime);

    my $message = ($session->getLanguage($confParam->{LANG}, "MANAGE_FOLDER_PERMISSIONS"))->data();

    my $msg = 
	"\n".
	"From: mailadmin\@$maildomain\n".
	"To: $group\n".
	"Subject: $message->{new_folder_access_subject} $folder\n".
	"Date: $date\n".
	"\n".
	"$message->{new_folder_access_body}\n";
    
    # fork and detach mail sending process
    # see: "http://apachetoday.com/news_story.php3?ltsn=2001-02-27-001-06-NW-LF-PL" why all this
    # is done
    $SIG{'CHLD'} = 'IGNORE';
    my $pid;
    my $apache_req = Apache->request();
    if (! defined ($pid=fork())){
	print STDERR "fork() failed in mail2group\n";
    }elsif($pid == 0){
	#child
	chdir '/' or die "Can't chdir to /: $!";
	close STDIN;
	close STDOUT;
	setsid or die "Can't start a new session: $!";
	open(OUT, "|/usr/bin/grpdeliver -l localhost -b \"$basedn\" -g $group") or die "unable to exec grpdeliver";
	print OUT $msg;
	close(OUT);
	CORE::exit;
    }
}

1;
