#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: manage_vdom.pm,v 1.5 2003/09/29 06:45:28 varkoly Exp $
#
package manage_vdom;
use strict;
use Storable qw(thaw freeze);
use Net::LDAP;
use LDAPUtils;
use Display;
use Session;
use cfreader;
use POSIX;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = (qw(Display));


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{'cgi'};
    my $session = $this->{"session"};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};

    my $new    = $cgi->param('new');
    my $soa    = $cgi->param('soa');
    my $save_soa= $cgi->param('save_soa');
    my $delete = $cgi->param('delete');
    my $sdtype = $cgi->param('d_type');
    my $export = $cgi->param('export');
    my $expert = $cgi->param('expert');
    my $domain = $cgi->param('vdom_list');
    my $add_host = $cgi->param('add_host');
    my $del_host = $cgi->param('del_host');
    my $show_hosts = $cgi->param('show_hosts');
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    if( defined $new && $new ne '' ) {
	$this->newdomain();
    } elsif( defined $add_host && $add_host ne "" ) {
	print $cgi->header( -Refresh=>"0; URL=/suse/dns_managehost.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&domain=$domain" );
    } elsif( defined $del_host && $del_host ne "" ) {
	$this->del_host();
    } elsif( defined $show_hosts && $show_hosts ne "" ) {
	$this->show_hosts();
    } elsif( (defined $soa && $soa ne "") || (defined $expert && $expert ne "")) {
	$this->soaedit();
    } elsif( defined $save_soa && $save_soa ne "" ) {
	$this->save_soa();
    } elsif( defined $delete && $delete ne "" ) {
	$this->rmdomain();
    } elsif( defined $sdtype && $sdtype ne "" ) {
	$this->switch_domain_type();
    } elsif( defined $export && $export ne "" ) {
	$this->create_dns_setup();
    } else {
	$this->startup();
    }
}

sub startup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

#-- search for virtual domains

    $mesg = $ldap->search( base => 'o=DNS,'.$confParam->{baseDN},
			   scope => 'sub',
			   attrs => [ 'zoneName', 'MTALocaldomain' ],
			   filter => 'relativeDomainName=@' );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }
    $ldap->unbind;

    my %vdlabels;
    my @vdoms = ('*'.$confParam->{mailDomain}.'*');
    foreach my $i ($mesg->entries) {
    	foreach my $zn ( $i->get_value('zoneName') ) {
	    if( ! ( $zn =~ /IN-ADDR\.ARPA/i ) && ! ( $zn eq $confParam->{mailDomain} ) ) {
		my $dstyle = $i->get_value('MTALocaldomain') eq 'true' ? "[S]" : "[P]";
		push @vdoms, $zn;
		$vdlabels{$zn} = "$dstyle $zn";
	    }
    	}
    }	


    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->Tr( $this->help_th(2, "$message->{title_head}", "MANAGE_VDOM"));
    if( ( ! defined $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} )
	|| $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ne "true" ){
	$html .= $cgi->Tr( $cgi->td({-class=>'AdminSubHeader',-colspan=>2 }, $message->{title_help}));
    }		
    $html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl");
    my $rowspan = 2;
    if( ( ! defined $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} )
	|| $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ne "true" ){
	$rowspan = 4;
    }
    $html .= $cgi->Tr(
		      $cgi->td({-align=>"center",-width=>'50%',-rowspan=>$rowspan},
			       $cgi->scrolling_list(-name=>'vdom_list',
						    -value=>\@vdoms,
						    -labels=>\%vdlabels,
						    -size=>5,
						    -width=>200
						    )
			       ).
		      $cgi->td( {-align=>"center",-width=>'50%'},
				$cgi->submit(-class=>'BlockButton',-name=>'delete', -value=>"$message->{delete}")
				)
		      );
    $html .=  $cgi->Tr(
		       $cgi->td( {-align=>"center",-width=>'50%'},
				 $cgi->submit(-class=>'BlockButton',-name=>'d_type',
					      -value=>"$message->{switch_dom_type}")
				 )
		       );
    if( ( ! defined $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} )
	|| $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ne "true" ){
	$html .=  $cgi->Tr(
			   $cgi->td( {-align=>"center",-width=>'50%'},
				     $cgi->submit(-class=>'BlockButton',-name=>'show_hosts', -value=>"$message->{edit_hosts}")
				     )
			   );
	$html .=  $cgi->Tr(
			   $cgi->td( {-align=>"center",-width=>'50%'},
				     $cgi->submit(-class=>"BlockButton",-name=>"soa", -value=>"$message->{soa}")
				     )	
			   );
    }
    $html .=  	   		$cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
	$cgi->hidden(-name=>"tab", -value=>"$tab").
	$cgi->hidden(-name=>"stab", -value=>"$stab").
	$cgi->end_form();
    $html .= $cgi->Tr({},
		      $cgi->td( {-align=>"center",-colspan=>2 },
				$cgi->hr({-with=>"100%"})
				)
		      );
    $html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl");
    $html .= $cgi->Tr(
		      $cgi->td({-align=>"center",-colspan=>2 },
			       "$message->{new_dom}".
			       $cgi->textfield(-name=>"new_vdom",
					       -value=>"",
					       -size=>20
					       )."&nbsp".
			       $cgi->submit(-class=>"button",-name=>"new", -value=>"$message->{add}").
			       $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
			       $cgi->hidden(-name=>"tab", -value=>"$tab").
			       $cgi->hidden(-name=>"stab", -value=>"$stab").
			       $cgi->end_form()
			       )
		      );
    $html .= $cgi->Tr({},
		      $cgi->td( {-align=>"center",-colspan=>2 },
				$cgi->hr({-with=>"100%"})
				)
		      );
    if( ( ! defined $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} )
        || $::CONF_GLOBAL{'GENERAL'}{'DisableDNSManagement'} ne "true" ){
	$html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl");
	$html .= $cgi->Tr(
			  $cgi->td({-align=>"center",-colspan=>2},
				   $cgi->submit(-class=>"button",-name=>"export", -value=>"$message->{export}").
				   $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
				   $cgi->hidden(-name=>"tab", -value=>"$tab").
				   $cgi->hidden(-name=>"stab", -value=>"$stab").
				   $cgi->end_form()
				   )
			  );
    }
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);
}

sub rmdomain {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $dom  = $cgi->param('vdom_list');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";
    
    if( ! defined $dom ) {
	$this->display_error($message->{select_domain}, $url, 3);
	exit;
    }

    if( ($dom =~ s/\*//g) > 0 ) {
	$this->display_error($message->{no_delete_main_domain}, $url, 3);
	exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    $mesg = $ldap->search( base => $confParam->{baseDN},
			   scope => 'one',
			   filter => "vaddress=*$dom",
			   attrs => [ 'vaddress' ]);
    if ($mesg->count != 0) {
        $this->display_error($message->{delete_first},
			     "$confParam->{cgi_path}/edit_vuser.pl?sessionID=$sessionID\&tab=user\&stab=v_edit",
			     3);
	$ldap->unbind;
        exit;
    }

    my $base = 'ou='.$dom.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base => $base,
			   scope => 'one',
			   filter => "objectclass=*" );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $text = "";
    foreach my $i ($mesg->entries) {
	my $mesg = $ldap->delete( $i->dn );
	if($mesg->code != 0) {
	    $text .= 'Faild to delete DN: '.$i->dn.':'.&LDAPerror($mesg)."\n";
	}
    }
    $mesg = $ldap->delete( $base );
    if($mesg->code != 0) {
	$text .= 'Faild to delete DN: '.$base.':'.&LDAPerror($mesg)."\n";
    }
    
    $this->display_msg($text.$message->{'vdom_successfully_deleted'}, $url, 1);
    $ldap->unbind;
}

sub switch_domain_type {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $dom  = $cgi->param('vdom_list');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";
    
    if( ! defined $dom ) {
	$this->display_error($message->{select_domain}, $url, 3);
	exit;
    }
    
    if( ($dom =~ s/\*//g) > 0 ) {
	$this->display_error($message->{no_switch_main_domain}, $url, 3);
	exit;
    }
    
    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $base = 'relativedomainname=@,ou='.$dom.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base => $base,
			   attrs => [ 'MTALocaldomain' ],
			   filter => "objectclass=*" );
    if ($mesg->code) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $mtalocal;
    foreach my $e ( $mesg->all_entries ) {
	$mtalocal = $e->get_value('MTALocaldomain');
    }

    if( $mtalocal eq 'true' ) {
	$mesg = $ldap->modify( $base, replace => { 'MTALocaldomain' => "$dom" } );
    } else {
	$mesg = $ldap->modify( $base, replace => { 'MTALocaldomain' => "true" } );
    }

    if ($mesg->code) {
        my $text = "modify failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
    } else {
	$this->display_msg($message->{dom_type_changed}, $url, 1);
    }
    $ldap->unbind;
    exit;
}

sub newdomain {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $newdom  = $cgi->param('new_vdom');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    if( $newdom =~ /[^a-zA-Z0-9-.]+/ ) {
	$this->display_error($message->{vdom_wrong_domain}, "$url");
	exit;
    }
    $newdom =~ /(\w+)\.(\w+)/;
    if( $newdom eq "" || ! $1 || ! $2 ) {
	$this->display_error($message->{vdom_wrong_domain}, "$url");
	exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $base = 'relativeDomainName=@,ou='.$confParam->{mailDomain}.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base => $base,
			   scope => 'base',
			   filter => "objectclass=dNSZone" );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $zone_entry = $mesg->entry(0);

    # get out SOA and increase serial number
    my $soa = $zone_entry->get_value("sOARecord");
    my @soa = split(/ /,$soa);
    my $timestamp = $soa[2];
    my $sernr  = substr($timestamp, 8, 2);
    my $timenr = substr($timestamp, 0, 8);
    my $timenow = strftime("%Y%m%d",localtime);

#    if( $timenr eq $timenow ) {
#		$sernr++;
#    } else {
#		$timenr = $timenow;
#    }

    my $sernr	 = '00';
    my $timenr   = $timenow;
    my $sOARecord= $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".$soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6];
    my $dNSTTL   = $zone_entry->get_value('dNSTTL');
    my @mXRecord = $zone_entry->get_value('mXRecord');
    my @nSRecord = $zone_entry->get_value('nSRecord');

    # create new tree entry for the new domain

    $base = 'ou='.$newdom.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->add( dn 	=> $base,
			attr => [
				 objectClass	=> ['top','organizationalUnit'],
				 ou			=> "$newdom"
				 ]	
			);	

    $mesg = $ldap->add( dn     		=> 'relativeDomainName=@,'.$base,
			attr => [
				 objectClass 		=> [ 'top','dNSZone','SuSEVirtualDomainObject'],
				 relativeDomainName	=> '@',
				 zoneName			=> $newdom,
				 sOARecord			=> $sOARecord,
				 dNSTTL				=> $dNSTTL,
				 mXRecord			=> \@mXRecord,
				 nSRecord			=> \@nSRecord,
				 MTALocaldomain		=> 'true'
				 ]	
			);	
    
    if($mesg->code != 0) {
        my $text = "update failed\n".&LDAPerror($mesg);
        $this->display_error($text,$url);
	$ldap->unbind;
	exit;
    }

    $this->display_msg($message->{'vdom_successfully_created'}, $url, 1);
    $ldap->unbind;
}

sub show_hosts {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('vdom_list');
    $domain =~ s/\*//g;

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    if(! defined $domain || $domain eq '' ){
        $this->display_error($message->{select_domain},$url,3);
        exit;
    }

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server",
			     $url);
        exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,$url);
        exit;
    }
    my $maildomain;
    my $ldap_result = $ldap->search( 'base' => "uid=cyrus,".$confParam->{'baseDN'},
				     'scope' => "base",
				     'attrs' => ['maildomain'],
				     'filter' => "(objectClass=*)"
				     );
    if( $ldap_result->code != 0 ){
        my $text = "search failed\n".&LDAPerror($ldap_result);
        $this->display_error($text,$url);
        $ldap->unbind;
        exit;
    } elsif ($ldap_result->count == 0) {
        my $text = "search failed\n";
        $this->display_error($text,$url);
	$ldap->unbind;
        exit;
    }else{
        $maildomain = $ldap_result->entry(0)->get_value('maildomain');
    }

    my $base = 'ou='.$domain.',o=DNS,'.$confParam->{baseDN};

    my @exclude_hosts = ();

    $ldap_result = $ldap->search( base => "relativeDomainName=\@,$base",
				  scope => 'base',
				  attrs => [ 'mXRecord', 'nSRecord' ],
				  filter => "objectclass=*" );
    if ($ldap_result->code != 0){
        my $text = "search failed 1\n".&LDAPerror($ldap_result);
        $this->display_error($text,$url);
        $ldap->unbind;
        exit;
    }elsif( $ldap_result->count == 0) {
        my $text = "search failed 2\n";
        $this->display_error($text,$url);
        $ldap->unbind;
        exit;
    }else{
	my $entry = $ldap_result->entry(0);
	my @mx_recs = $entry->get_value("mXRecord");
	foreach my $mx_rec (@mx_recs) {
	    if( $mx_rec =~ /^[0-9]+\s(.*)$/ ){
		push @exclude_hosts, $1;
	    }
	}
	my @ns_recs = $entry->get_value("nSRecord");
        push( @exclude_hosts, (@ns_recs) );
    }
    my %exclude_hosts = ();
    foreach( @exclude_hosts ){
        $exclude_hosts{$_} = 1;
#   print STDERR "exclude: $_ \n";
    }

    $mesg = $ldap->search( base => $base,
			   scope => 'one',
			   attrs => [ 'relativeDomainName','aRecord' ],
			   filter => "(&(!(objectClass=SuSEdhcpHost))(objectclass=dNSZone))" );
    if( $mesg->code != 0 ){
        my $text = "search failed 3\n".&LDAPerror($mesg);
        $this->display_error($text,$url);
        $ldap->unbind;
        exit;
    }

    $ldap->unbind;
    my @entries = $mesg->entries();
    my @hosts = ();
    foreach my $entry (@entries){
        my $rel_dom = $entry->get_value('relativeDomainName');
        if( ($rel_dom ne "@" ) && (! ( defined $exclude_hosts{$rel_dom.".".$maildomain."."} ) ) ){
	    push @hosts, $rel_dom.'.'.$domain.' '.$entry->get_value('aRecord');
	}
    }
    if(scalar(@hosts)==0){
	push @hosts, $message->{'dns_no_host_defined'};
    }

    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->Tr( $this->help_th(2, "$message->{title_head}", "MANAGE_VDOM2"));

    $html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl");
    $html .= $cgi->Tr(
		      $cgi->td({-align=>"center",-width=>'50%',-rowspan=>3},
			       $cgi->scrolling_list(-name=>'hostname',
						    -value=>\@hosts,
						    -size=>10,
						    -width=>200
						    )
			       ).
		      $cgi->td( {-align=>"center",-width=>'50%'},
				$cgi->submit(-class=>'BlockButton',-name=>'add_host', -value=>"$message->{add_host}")
				)
		      );
    $html .=  $cgi->Tr(
		       $cgi->td( {-align=>"center",-width=>'50%'},
				 $cgi->submit(-class=>'BlockButton',-name=>'del_host', -value=>"$message->{del_host}")
				 ).
		       $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
		       $cgi->hidden(-name=>"vdom_list", -value=>"$domain").
		       $cgi->hidden(-name=>"tab", -value=>"$tab").
		       $cgi->hidden(-name=>"stab", -value=>"$stab")
		       );

    $html .= $cgi->end_form();
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();
    $this->SUPER::display($html);

}


sub del_host{
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('vdom_list');
    $domain		=~ s/\*//g;
    my $url	= "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $hostname = $cgi->param('hostname');
    if($hostname =~ /\./) {
	$hostname = (split /\./,$hostname)[0];
    }
    my $really = $cgi->param('really');
    my @host_address = ();

    if( (! defined $hostname) || ($hostname eq "") ){
	$this->display_error($message->{'dns_no_host_selected'},
			     $confParam->{'cgi_path'}."/manage_vdom.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
	exit;
    }

    if( (defined $really) && ($really ne "") ){
	my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port=>$confParam->{LDAPport}, version => 3);
	if(!defined $ldap) {
	    $this->display_error("can't contact LDAP Server",
				 $url);
	    exit;
	}
	my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			      $confParam->{baseDN}, password=>$confParam->{passwd});
	if($mesg->code != 0) {
	    my $text = "bind failed\n".&LDAPerror($mesg);
	    $this->display_error($text,$url);
	    exit;
	}
	
	my $ldap_result = $ldap->search(	base	=> 'o=DNS,'.$confParam->{'baseDN'},
						scope	=> 'sub',
						attrs	=> ['pTRRecord'],
						filter	=> "(&(objectClass=dNSZone)(pTRRecord=$hostname.$domain.))"
						);
	if($ldap_result->code() < 0){
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,$url);
	    $ldap->unbind;
	    exit;
	}
	if( $ldap_result->count() != 0 ) {
		my @ptr_rec = $ldap_result->entry(0)->get_value('pTRRecord');
		my $rev_dn = $ldap_result->entry(0)->dn();
		if( scalar(@ptr_rec) > 1 ){
		    # There are other host using the same IP-Address
		    $mesg = $ldap->modify($rev_dn,
					  "delete" => {"pTRRecord" => "$hostname.$domain."}
					  );
		}else{
		    # This was the last host using the IP-Address delete the entry for reverse lookup
		    $mesg = $ldap->delete($rev_dn);
		}
		
		if($mesg->code != 0) {
		    my $text = "LDAP: delete failed\n".&LDAPerror($mesg);
		    $this->display_error($text,$url);
		    $ldap->unbind;
		    exit;
		}
	}
	$mesg = $ldap->delete("relativeDomainName=$hostname,ou=$domain,o=DNS,".$confParam->{'baseDN'});
	if($mesg->code != 0) {
	    my $text = "LDAP: delete failed\n".&LDAPerror($mesg);
	    $this->display_error($text,$url);
	    $ldap->unbind;
	    exit;
	}
	
	#    print STDERR "successfully updated\n";
	my $base = "o=DNS,".$confParam->{baseDN};
	
	$mesg = $ldap->search( base => $base,
			       scope => 'sub',
			       filter => '(&(relativeDomainName=@)(objectclass=dNSZone))' );
	if( $mesg->code != 0 ){
	    my $text = "LDAP: search failed\n".&LDAPerror($mesg);
	    $this->display_error($text,$url);
	    $ldap->unbind;
	    exit;
	}elsif( $mesg->count == 0) {
	    my $text = "search failed\n";
	    $this->display_error($text,$url);
	    $ldap->unbind;
	    exit;
	}
	
	foreach my $zone_entry ($mesg->entries) {
	    my $zn = $zone_entry->get_value('zoneName');
	    if( $zn =~ /IN-ADDR\.ARPA/i || $zn eq $domain ) {			
		# get out SOA and increase serial number
		my $soa = $zone_entry->get_value("sOARecord");
		my @soa = split(/ /,$soa);
		my $timestamp = $soa[2];
		my $sernr  = substr($timestamp, 8, 2);
		my $timenr = substr($timestamp, 0, 8);
		my $timenow = strftime("%Y%m%d",localtime);
		
		if( $timenr eq $timenow ) {
		    $sernr++;
		} else {
		    $timenr = $timenow;
		}
		
		$zone_entry->replace( sOARecord =>
				      $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".
				      $soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6]);
		
		$mesg = $zone_entry->update( $ldap );
		if($mesg->code != 0) {
		    my $text = "update failed\n".&LDAPerror($mesg);
		    $this->display_error($text,$url);
		    $ldap->unbind;
		    exit;
		}
	    }
	}
	
	$ldap->unbind;
	$this->display_msg($message->{'dns_del_done'}, 
			   $confParam->{'cgi_path'}."/manage_vdom.pl?sessionID=$sessionID".
			   "&tab=$tab&stab=$stab&vdom_list=$domain",3);
	exit; 
    }else{
	#Ask for confirmation
        my $html = "";
        $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
        $html .= $cgi->start_Tr().
	    $cgi->start_td().
	    $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"}).
	    $cgi->start_Tr().
	    $cgi->td({-class=>'AdminHeader', -colspan=>2 },
		     "<b>$message->{dns_del_really}</b>"
		     ).
		     $cgi->end_Tr().
		     $cgi->start_Tr().
		     $cgi->start_td({-class=>'AdminSubHeaderRed', -colspan=>2, -align=>"center"});
	$html .= $message->{'dns_del_should'}.'"'.$hostname.'"'.$message->{'dns_del_be_deleted'}.
	    $cgi->end_td().
	    $cgi->end_Tr().   
	    $cgi->start_Tr().
	    $cgi->td({-align=>"center"},
		     $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl").
		     $cgi->submit(-class=>"button",-name=>$message->{'no'}).
		     $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
		     $cgi->hidden(-name=>'tab', -value=>$tab).
		     $cgi->hidden(-name=>'stab', -value=>$stab).
		     $cgi->end_form()    
		     ).
		     $cgi->td({-align=>"center"},
			      $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl").
			      $cgi->submit(-class=>"button",-name=>$message->{'yes'}).
			      $cgi->hidden(-name=>'sessionID', -value=>$sessionID).
			      $cgi->hidden(-name=>'tab', -value=>$tab).
			      $cgi->hidden(-name=>'stab', -value=>$stab).
			      $cgi->hidden(-name=>'really', -value=>1).
			      $cgi->hidden(-name=>'del_host', -value=>1).
			      $cgi->hidden(-name=>'hostname', -value=>$hostname).
			      $cgi->hidden(-name=>'vdom_list', -value=>$domain).
			      $cgi->end_form()    
			      ).
			      $cgi->end_Tr().
			      $cgi->end_table().
			      $cgi->end_td().
			      $cgi->end_Tr().    
			      $cgi->end_table();
        
        $this->SUPER::display($html);
    }
    
}
1;

sub save_soa {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $domain  = $cgi->param('domain');
    my $year	= $cgi->param('year');
    my $month	= $cgi->param('month');
    my $day	= $cgi->param('day');
    my $relno	= $cgi->param('relno');
    my $refresh	= $cgi->param('refresh');
    my $retry	= $cgi->param('retry');
    my $expiry	= $cgi->param('expiry');
    my $minimum	= $cgi->param('minimum');
    my $nsr_new	= $cgi->param('nsr_new');
    my $mxr_pri_new	= $cgi->param('mxr_pri_new');
    my $mxr_name_new	= $cgi->param('mxr_name_new');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab\&vdom_list=$domain\&soa=soa";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    
    my $base = 'relativeDomainName=@,ou='.$domain.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base => $base,
			   scope => 'base',
			   filter => "objectclass=dNSZone",
			   attrs => [ "sOARecord", "dNSTTL", "nSRecord", "mXRecord", "aRecord" ] );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $zone_entry     = $mesg->entry(0);
    my $soa = $zone_entry->get_value("sOARecord");
    my ($host, $postmaster) = split(/ /,$soa, 2);
    $zone_entry->replace(
			 sOARecord => "$host $postmaster $year$month$day$relno $refresh $retry $expiry $minimum"
			 );
    
    $zone_entry->replace( dNSTTL => $cgi->param('dnsttl') );
    
    my @nSRecords = $zone_entry->get_value('nSRecord');
    if( scalar(@nSRecords) ) { $zone_entry->delete('nSRecord'); }
    foreach my $nsr ( @nSRecords ) {
	my $NS = $cgi->param("nsr_$nsr");
	$NS =~ s/\s//g;
    	if( ! (defined $cgi->param("del_nsr_$nsr") && $cgi->param("del_nsr_$nsr") eq 'on') ) {
	    if( $NS =~ /\.$/ ) {
		$zone_entry->add(nSRecord	=>	$NS);
	    } else {
		$zone_entry->add(nSRecord       =>	$NS.'.');
	    }
    	}
    }
    if ( defined $nsr_new && $nsr_new ne '' ) {
	$nsr_new =~ s/\s//g;
	if( $nsr_new =~ /\.$/ ) {
	    $zone_entry->add(nSRecord	=>	$nsr_new);
	} else {
	    $zone_entry->add(nSRecord	=>	$nsr_new.'.');
	}
    }
    my @mXRecords = $zone_entry->get_value('mXRecord');
    if( scalar (@mXRecords) ) { $zone_entry->delete('mXRecord'); }
    foreach my $mxr ( @mXRecords ) {
	my $MX = $cgi->param("mxr_name_$mxr");
	$MX =~ s/\s//g;
    	if( ! (defined $cgi->param("del_mxr_$mxr")  && $cgi->param("del_mxr_$mxr") eq 'on') ) {
	    if( $MX =~ /\.$/ ) {
		$zone_entry->add(mXRecord	=>	$cgi->param("mxr_pri_$mxr").' '.$MX);
	    } else {
		$zone_entry->add(mXRecord	=>	$cgi->param("mxr_pri_$mxr").' '.$MX.'.');
	    }
    	}	
    }
    if ( defined $mxr_name_new && $mxr_name_new ne '' && defined $mxr_pri_new && $mxr_pri_new ne '') {
	$mxr_name_new =~ s/\s//g;
	if( $mxr_name_new !~ /\.$/ ) {
	    $zone_entry->add(mXRecord   =>  $mxr_pri_new.' '.$mxr_name_new);
	} else {
	    $zone_entry->add(mXRecord   =>  $mxr_pri_new.' '.$mxr_name_new.'.');
	}
    }
    
    $mesg = $zone_entry->update( $ldap );
    if($mesg->code != 0) {
        my $text = "update failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    } else {
    	$this->create_dns_setup();
    }
}

sub soaedit {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $expert  = $cgi->param('expert');
    my $akt_domain = $cgi->param('vdom_list') || $cgi->param('domain'); 

    $akt_domain =~ s/\*//g;

    if(! defined $akt_domain || $akt_domain eq '' ){
        $this->display_error($message->{select_domain},"$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",3);
        exit;
    }

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
    
    my $base = 'relativeDomainName=@,ou='.$akt_domain.',o=DNS,'.$confParam->{baseDN};
    $mesg = $ldap->search( base => $base,
			   scope => 'base',
			   filter => "objectclass=dNSZone",
			   attrs => [ "sOARecord", "dNSTTL", "nSRecord", "mXRecord", "aRecord" ] );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $zone_entry = $mesg->entry(0);

    my $soa = $zone_entry->get_value("sOARecord");
    my $dnsttl = $zone_entry->get_value("dNSTTL");
    my ($host, $postmaster, $serial, $refresh, $retry, $expiry, $minimum ) = split(/ /,$soa);

    my $year  = substr($serial,0,4);
    my $month = substr($serial,4,2);
    my $day   = substr($serial,6,2);
    my $relno = substr($serial,8,2);

    my @yearrange;
    for(my $s=strftime("%Y",localtime)-2; $s<=strftime("%Y",localtime)+2; $s++) {
	push @yearrange, $s;
    }
    my @monthrange;
    for(my $m=1; $m<=12; $m++) {
	push @monthrange, sprintf("%02s",$m);
    }
    my @dayrange;
    for(my $d=1; $d<=31; $d++) {
	push @dayrange, sprintf("%02s",$d);
    }

    my $sdata;
    $sdata->{'key'} = "_MANAGE_VDOM";
    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
    $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->Tr(
		      [
		       $this->help_th(2, $message->{'SOA_EDITOR'}.' :"'.$akt_domain.'"', "MANAGE_VDOM")
		       ]
		      );
    $html .= $cgi->start_form(-action=>$confParam->{cgi_path}."/manage_vdom.pl");
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td();
#    $html .= $cgi->start_table({ -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->start_table({-align=>'center', -cellspacing=>0, -cellpadding=>3});
    $html .= $cgi->Tr( $cgi->td( {-align=>'left',-colspan=>2}, $message->{ns_records}));
    
    foreach my $nsr ( $zone_entry->get_value("nSRecord") ) {
#	$nsr =~ s/\.$//;
	$html .= $cgi->Tr(
			  $cgi->td( {-align=>'right'}, $message->{name}),
			  $cgi->td( {-align=>'left'},  $cgi->textfield(-name=>"nsr_$nsr", -value=>$nsr, -width=>30)),
			  $cgi->td( {-align=>'left'},  "&nbsp;"),
			  $cgi->td( {-align=>'left'},  "&nbsp;"),
			  $cgi->td( {-align=>'left',-width=>80},  $cgi->checkbox(-name=>"del_nsr_$nsr", -label=>$message->{delete}))
			  );
    }
    $html .= $cgi->Tr( $cgi->td( {-align=>'left',-colspan=>2}, $message->{new_ns_record}));
    $html .= $cgi->Tr(
		      $cgi->td( {-align=>'right'}, $message->{name}), 
		      $cgi->td( {-align=>'left'},  $cgi->textfield(-name=>'nsr_new',-width=>30))
		      );

    $html .= $cgi->Tr( $cgi->td( {-align=>'left',-colspan=>2}, $message->{mx_records}));
    foreach my $mxr ( $zone_entry->get_value("mXRecord") ) {
	my ($pri, $mxhost) = split(/ /,$mxr);
#	$mxhost =~ s/\.$//;
	$html .= $cgi->Tr(
			  $cgi->td( {-align=>'right'}, $message->{name}),
			  $cgi->td( {-align=>'left'},  $cgi->textfield(-name=>"mxr_name_$mxr", -value=>$mxhost, -width=>30)),
			  $cgi->td( {-align=>'right'}, $message->{priority}),
			  $cgi->td( {-align=>'left'},  $cgi->textfield(-name=>"mxr_pri_$mxr", -value=>$pri, -size=>3, -maxlength=>2)),
			  $cgi->td( {-align=>'left',-width=>80},  $cgi->checkbox(-name=>"del_mxr_$mxr",    -label=>$message->{delete}))
			  );
    }
    $html .= $cgi->Tr( $cgi->td( {-align=>'left',-colspan=>2}, $message->{new_mx_record}));
    $html .= $cgi->Tr(
		      $cgi->td( {-align=>'right'}, $message->{name}),
		      $cgi->td( {-align=>'left'},  $cgi->textfield(-name=>'mxr_name_new',-width=>30)),
		      $cgi->td( {-align=>'right'},  $message->{priority}),
		      $cgi->td( {-align=>'left'}, $cgi->textfield(-name=>'mxr_pri_new', -size=>3, -maxlength=>2)), 
		      $cgi->td( {-align=>'left'},  $message->{pri_text})
		      );
    
    $html .= $cgi->end_table();
    $html .= $cgi->start_table({ -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    if( defined $expert && $expert ne "" ) {
	$html .= $cgi->Tr(
			  $cgi->td(
				   ['$TTL '.
				    $cgi->textfield(-name=>"dnsttl", -value=>$dnsttl,
						    -size=>10, -override=>1),
				    "&nbsp", "&nbsp", "&nbsp", "&nbsp"]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   ["@", "IN", "SOA", "$host", "$postmaster ("]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   [ "&nbsp", "&nbsp", "&nbsp",
				     $cgi->popup_menu(-name=>"year", -values=>\@yearrange,
						      -default=>strftime("%Y",localtime)).
				     $cgi->popup_menu(-name=>"month", -values=>\@monthrange,
						      -default=>sprintf("%02s",strftime("%m",localtime))).
				     $cgi->popup_menu(-name=>"day", -values=>\@dayrange,
						      -default=>sprintf("%02s",strftime("%d",localtime))).
				     "&nbsp;&nbsp;".
				     $cgi->textfield(-name=>"relno", -value=>$relno,
						     -maxlength=>10, -size=>4, -override=>1),
				     "<pre>Serial Number</pre>"]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   [ "&nbsp", "&nbsp", "&nbsp",
				     $cgi->textfield(-name=>"refresh", -value=>$refresh, -size=>5),
				     "<pre>Refresh</pre>"]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   [ "&nbsp", "&nbsp", "&nbsp",
				     $cgi->textfield(-name=>"retry", -value=>$retry, -size=>5),
				     "<pre>Retry</pre>" ]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   [ "&nbsp", "&nbsp", "&nbsp",
				     $cgi->textfield(-name=>"expiry", -value=>$expiry, -size=>5),
				     "<pre>Expiry</pre>" ]
				   )
			  );
	$html .= $cgi->Tr(
			  $cgi->td(
				   [ "&nbsp", "&nbsp", "&nbsp",
				     $cgi->textfield(-name=>"minimum", -value=>$minimum, -size=>5)." )",
				     "<pre>Minimum</pre>" ]
				   )
			  );
	$html .= $cgi->Tr(
		          $cgi->td( {-colspan=>5,-align=>'center' },
				    $cgi->submit(-name=>'save_soa',-value=>$message->{save_soa}).
				    $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
				    $cgi->hidden(-name=>"tab", -value=>"$tab").
				    $cgi->hidden(-name=>"stab", -value=>"$stab").
				    $cgi->hidden(-name=>"domain", -value=>"$akt_domain")
				    )
			  );
    } else {
	$html .= $cgi->Tr(
			  $cgi->td(
				   $cgi->hidden(-name=>"dnsttl",	-value=>"$dnsttl").		
				   $cgi->hidden(-name=>"year",	-value=>strftime("%Y",localtime)).		
				   $cgi->hidden(-name=>"month",	-value=>sprintf("%02s",strftime("%m",localtime))).		
				   $cgi->hidden(-name=>"day",	-value=>sprintf("%02s",strftime("%d",localtime))).		
				   $cgi->hidden(-name=>"relno",	-value=>"$relno").
				   $cgi->hidden(-name=>"refresh",	-value=>"$refresh").
				   $cgi->hidden(-name=>"retry",	-value=>"$retry").
				   $cgi->hidden(-name=>"expiry",	-value=>"$expiry").
				   $cgi->hidden(-name=>"minimum",	-value=>"$minimum")
				   )
			  );
	$html .= $cgi->Tr(
		          $cgi->td( {-colspan=>5,-align=>'center' },
				    $cgi->submit(-name=>'save_soa',-value=>$message->{save_soa}).
				    $cgi->submit(-name=>'expert',-value=>$message->{expert}).
				    $cgi->hidden(-name=>"sessionID", -value=>"$sessionID").
				    $cgi->hidden(-name=>"tab", -value=>"$tab").
				    $cgi->hidden(-name=>"stab", -value=>"$stab").
				    $cgi->hidden(-name=>"domain", -value=>"$akt_domain")
				    )
			  );
    }
    $html .= $cgi->end_table();
    $html .= $cgi->end_form();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();

    $this->SUPER::display($html);

    $ldap->unbind;
}

sub create_dns_setup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $url = "$confParam->{cgi_path}/manage_vdom.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $base = "o=DNS,".$confParam->{baseDN};
    $mesg = $ldap->search(	base	=> $base,
				scope	=> 'sub',
				filter	=> "objectclass=dNSZone" );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $nsdn = $mesg->as_struct;

    my $namedtempl = "/etc/named.conf";
    my @namedconf = ();
    my $lastgen = localtime;
    my @pfmydest;

    push @namedconf,
    "//
// THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
// BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
//
// to change some options manually, edit $namedtempl
//
// Last generated at \"$lastgen\"
//
";

    # deassemble LDAP and reassemble DNS zonefiles

    $mesg = $ldap->search(	base => $base,
				scope => 'sub',
				filter => '(&(relativeDomainName=@)(objectclass=dNSZone))'
				);
    if( $mesg->code != 0 ){
        my $text = "LDAP: search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }elsif( $mesg->count == 0) {
        my $text = "search failed. No DNS zones found\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }	
    $ldap->unbind;
    foreach my $zone_entry ( $mesg->entries ) {
	my $zone = $zone_entry->get_value('zoneName');
	my $soaent = $nsdn->{$zone_entry->dn};

	# disassemble NS Forward maps
	if( ! ( $zone =~ /IN-ADDR.ARPA/i ) ) {

	    if( $zone ne $confParam->{mailDomain} ) {
		push @pfmydest,$zone;
	    }

	    my $zonefile = "/var/named/$zone.zone";
	    my @soa = split(/ /,$soaent->{soarecord}->[0]);
	    my @outf = ();
	    push @outf,
	    ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
	    push @outf, "\$TTL $soaent->{dnsttl}->[0]";
	    push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
	    push @outf, "\t\t\t".$soa[2];
	    push @outf, "\t\t\t".$soa[3];
	    push @outf, "\t\t\t".$soa[4];
	    push @outf, "\t\t\t".$soa[5];
	    push @outf, "\t\t\t".$soa[6].")";
	    foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
		push @outf, "\tIN\tNS\t".$nsrr;
	    }
	    foreach my $arr ( @{$soaent->{arecord}} ) {
		push @outf, "\tIN\tA\t".$arr;
	    }
	    foreach my $mxrr ( @{$soaent->{mxrecord}} ) {
		push @outf, "\tIN\tMX\t".$mxrr;
	    }

	    # now disassemble host entries
	    foreach my $dn (sort ( keys %$nsdn )) {
		if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
		    my $hent = $nsdn->{$dn};
		    foreach my $hzone ( @{$hent->{zonename}} ) {
			if( $hzone eq $zone ) {
			    push @outf, $hent->{relativedomainname}->[0]."\tIN\tA\t".$hent->{arecord}->[0];
			}
		    }
		}
	    }
	    my $ret = $session->suad_save(\@outf, $zonefile);
	    if(  $ret->code() ne "OK" ) {
		$this->display_error($ret->code(),
				     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
		exit;
	    }

	    # add zone to named.conf
	    push @namedconf, "zone \"$zone\" {";
	    push @namedconf, "\ttype master;";
	    push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            if( $zone eq $confParam->{mailDomain} ) {
#                push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
#            }
# E SCHULSERVER 
	    push @namedconf, "};";
	    
	} else { # disassemble NS Reverse maps
	    my $zonefile = "/var/named/$zone.zone";
	    my @soa = split(/ /,$soaent->{soarecord}->[0]);
	    my @outf = ();
	    push @outf,
	    ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
	    push @outf, "\$TTL $soaent->{dnsttl}->[0]";
	    push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
	    push @outf, "\t\t\t".$soa[2];
	    push @outf, "\t\t\t".$soa[3];
	    push @outf, "\t\t\t".$soa[4];
	    push @outf, "\t\t\t".$soa[5];
	    push @outf, "\t\t\t".$soa[6].")";
	    foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
		push @outf, "\tIN\tNS\t".$nsrr;
	    }

	    # now disassemble host entries
	    foreach my $dn ( sort ( keys %$nsdn )) {
		if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
		    my $hent = $nsdn->{$dn};
		    foreach my $hzone ( @{$hent->{zonename}} ) {
			if( $hzone eq $zone ) {
			    foreach my $arr ( @{$hent->{ptrrecord}} ) {
				push @outf, $hent->{relativedomainname}->[0]."\tIN\tPTR\t".$arr;
			    }
			}
		    }
		}
	    }
	    my $ret = $session->suad_save(\@outf, $zonefile);
	    if(  $ret->code() ne "OK" ) {
		$this->display_error($ret->code(),
				     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
		exit;
	    }

	    # add zone to named.conf
	    push @namedconf, "zone \"$zone\" {";
	    push @namedconf, "\ttype master;";
	    push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
# E SCHULSERVER 
	    push @namedconf, "};";
	}
    }

    my $errors = "";

    my $ret = $session->suad_save(\@namedconf, "/etc/named.d/ldap.dump");
    if(  $ret->code() ne "OK" ) {
	$errors .= "named: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }
    
    $ret = $session->suad_exec("nsreload");
    if(  $ret->code() ne "OK" ) {
	$errors .= "named: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }

    
    $ret = $session->suad_exec("fetchdreload");
    if(  $ret->code() ne "OK" ) {
	$errors .= "fetchd: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }

    $this->display_msg($message->{export_success}.($errors eq "" ? "" : ":\nErrors:\n".$errors),
		       $url, ($errors ne "" ? undef : 1));
}

1;
