#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: postfix.pm,v 1.5 2003/10/02 09:22:29 varkoly Exp $
#
package postfix;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use cfreader;
use Utils;
use strict;
use Display;
use vars qw(@ISA);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);

    return bless $self, $this;
}


sub display {
    my $this = shift;
    
# these are all needed by this module
    $this->{"needed"} = { "relayhost" => "",
			  "disable_dns_lookups" => "no",
			  "defer_transports" => "",
			  "smtpd_use_tls" => "yes",
			  "relay_clientcerts" => "ldap:ldaprelcert",
			  "smtpd_tls_CAfile" => "/etc/ssl/CA/usedCA.pem",
			  "smtpd_tls_cert_file" => "/etc/ssl/certs/cert.pem",
			  "smtpd_tls_key_file" => "/etc/ssl/certs/skey.pem",
			  "smtpd_sasl_auth_enable" => "yes",
			  "smtpd_sasl_security_options" => "noanonymous",
			  "smtpd_client_restrictions" => "permit_sasl_authenticated",
			  "smtp_sasl_auth_enable"      => "yes",
                          "smtp_sasl_security_options" => "noanonymous",
                          "smtp_sasl_password_maps"    => "hash:/etc/postfix/saslpasswd" 
			};
    
    $this->{"tlsopt"} = [ "smtpd_use_tls",
			  "relay_clientcerts",
			  "smtpd_tls_CAfile",
			  "smtpd_tls_cert_file",
			  "smtpd_tls_key_file" ];
    $this->{"saslopt"} = [ "smtpd_sasl_auth_enable",
			   "smtpd_sasl_security_options",
			   "smtpd_client_restrictions" ];

    $this->{"relayhost_auth"} = [ "smtp_sasl_auth_enable",
			      "smtp_sasl_security_options",
			      "smtp_sasl_password_maps"
			    ];

    
    my $cgi = $this->{"cgi"};
    
    my $doit    = $cgi->param('doit');
    
    if(defined $doit && $doit eq "add") {
	$this->modify();
    } else {
	$this->startup();
    }

}

sub startup {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $q       = $this->{"cgi"};

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"POSTFIX"))->data();

    my $html = "";
    my $inbetween = "";
    my $spambutton = "";
    my $virusbutton = "";

    $this->readConfig();
    
    
    $q->param(-name=>'relayhost', -value=>$this->{"conf"}->{relayhost});
    
    my $tfsize = 40;


    if( -s "/etc/ssl/CA/usedCA.pem" ) {
	$inbetween .=  $q->td(
			      [$message->{postfix_cfg_tls},
			       $q->popup_menu(-name=>'tls',
					      -values=>[ "$message->{on}", "$message->{off}"],
					      -default=>$this->{"conf"}->{tls} )]
			      );
    }

    if( -s "/etc/imap/procmailrc" ) {
	$spambutton .= $q->td(
			      [$message->{postfix_cfg_spam},
			       $q->popup_menu(-name=>'spam',
					      -values=>[ "$message->{on}", "$message->{off}"],
					      -default=>$this->{"conf"}->{spam} )] );
    }
    
    system("/bin/rpm -q amavisd-postfix > /dev/null");
    my $installed = $? >> 8;
    my $ret = $session->suad_service_exec("amavis","status");
    $ret->code() =~ /OK:\s(\d)/;
    my $amavisdrunning = $1;
    if( $installed == 0 && $amavisdrunning == 0 ) {
	$virusbutton .= $q->td(
				  [$message->{postfix_cfg_virus},
				   $q->popup_menu(-name=>'virus',
						  -values=>[ "$message->{on}", "$message->{off}"],
						  -default=>$this->{"conf"}->{virus} )] );
    }
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $q->Tr({},
		    $q->td({},
			   $q->table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"},
				     $q->Tr(
					    [ 
					      $this->help_th(2, $message->{postfix_cfg_head},"POSTFIX"),
					      $q->td({-colspan=>2, -class=>'AdminSubHeaderRed',
						      },
						     ["$message->{postfix_cfg_warn}"]),
					      $q->start_form(-method=>"get",
							     -action=>$confParam->{cgi_path}."/postfix.pl"),
					      $q->td(
						     [$message->{postfix_cfg_relay},
						      $q->textfield(-name=>'relayhost',
								    -value=>$this->{"conf"}->{relayhost},
								    -size=>$tfsize)]  ),
					      $q->td(
                                                     [$message->{relayhost_need_auth},
                                                      $q->checkbox(-name   =>'relayhost_auth',
								   -checked=>$this->{"conf"}->{relayhost_auth},
                                                                   -label  =>"")]  ),
							$q->td(
                                                             ['&nbsp',$message->{user}.'&nbsp;:&nbsp;'.
                                                              $q->textfield(-name=>'relayhost_auth_user',
                                                                            -value=>$this->{"conf"}->{relayhost_auth_user},
                                                                            -size=>20)]  ),
							$q->td(
                                                             ['&nbsp',$message->{password}.'&nbsp;:&nbsp;'.
                                                              $q->textfield(-name=>'relayhost_auth_password',
                                                                            -value=>$this->{"conf"}->{relayhost_auth_password},
                                                                            -size=>20)]  ),
					      $q->td(
						     [$message->{postfix_cfg_dialup},
						      $q->popup_menu(-name=>'DOD',
								     -values=>[ "$message->{on}", "$message->{off}"],
								     -default=>$this->{"conf"}->{dialup} )] ),
					      $q->td(
						     [$message->{postfix_cfg_sasl},
						      $q->popup_menu(-name=>'sasl',
								     -values=>[ "$message->{on}", "$message->{off}"],
								     -default=>$this->{"conf"}->{sasl} )] ),
#
#  insert the above generated html code here:
#
					      $spambutton,
					      $virusbutton,
					      $inbetween,
					      $q->td({-colspan=>2,
						      -align=>"center"},
						     $q->submit(-class=>"button",-name=>$message->{postfix_doit}).
						     $q->hidden(-name=>"doit", -value=>"add").
						     $q->hidden(-name=>"sessionID", -value=>$sessionID).
						     $q->hidden(-name=>"tab", -value=>$tab).
						     $q->hidden(-name=>"stab", -value=>$stab).
						     $q->end_form()
						     )
					      ]
					    )
				     )
			   )
		    );

    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub modify() {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $q       = $this->{"cgi"};

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"POSTFIX"))->data();

    my $cf = "/etc/postfix/main.cf";
    my $mastercf = "/etc/postfix/master.cf";
    
    my $msg = $session->save_read2session("$mastercf", $mastercf);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($mastercf.": ".$message->{postfix_file_modified},
				 "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
				 "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }

    if( ! defined $q->param('virus') ) {
	$q->param(-name=>'virus', value=>$message->{off});
    }
    if( ! defined $q->param('spam') ) {
	$q->param(-name=>'spam', value=>$message->{off});
    }

    my $master = $msg->data();
    for(my $i=0; $i<scalar(@$master); $i++) {
	my $l = $master->[$i];

	if( $l =~ /^procmail\s+unix/ ) {
	    $master->[$i+1] = "  flags=R user=nobody argv=/usr/bin/procmail -t -m /etc/imap/procmailrc \${sender} \${recipient}";
	} 
	
	# VIRUS OFF, SPAM OFF
	if( ( defined $q->param('virus') && $q->param('virus') ne $message->{on} ) &&
	    ( defined $q->param('spam') && $q->param('spam') ne $message->{on} ) ) {
	# set smtpd lines to clear state:
	# smtp       inet  n       -       n       -       -   smtpd
	# smtps      inet  n       -       n       -       -   smtpd
	##localhost:10025 inet n   -       n       -       -   smtpd -o content_filter=procmail:spamfilter
	    #print STDERR "VIRUS OFF, SPAM OFF\n";
	    if( $l =~ /(^smtp[s]*\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1;
	    }
	    if( $l =~ /^[\#]*(localhost:10025\s+inet.*smtpd).*/ ) {
		$master->[$i] = "#".$1." -o content_filter=procmail:spamfilter";
	    }
	# VIRUS ON, SPAM OFF
	} elsif( ( defined $q->param('virus') && $q->param('virus') eq $message->{on} ) &&
		 ( defined $q->param('spam') && $q->param('spam') ne $message->{on} ) ) {
	# smtp       inet  n       -       n       -       -   smtpd -o content_filter=vscan:
	# smtps      inet  n       -       n       -       -   smtpd -o content_filter=vscan:
	# localhost:10025 inet n   -       n       -       -   smtpd -o content_filter=
	    #print STDERR "VIRUS ON, SPAM OFF\n";
	    if( $l =~ /(^smtp[s]*\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1." -o content_filter=vscan:";
	    }
	    if( $l =~ /^[\#]*(localhost:10025\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1." -o content_filter=";
	    }
	# VIRUS ON, SPAM ON
	} elsif( ( defined $q->param('virus') && $q->param('virus') eq $message->{on} ) &&
		 ( defined $q->param('spam') && $q->param('spam') eq $message->{on} ) ) {
	# smtp       inet  n       -       n       -       -   smtpd -o content_filter=vscan:
	# smtps      inet  n       -       n       -       -   smtpd -o content_filter=vscan:
	# localhost:10025 inet n   -       n       -       -   smtpd -o content_filter=procmail:spamfilter
	    #print STDERR "VIRUS ON, SPAM ON\n";
	    if( $l =~ /(^smtp[s]*\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1." -o content_filter=vscan:";
	    }
	    if( $l =~ /^[\#]*(localhost:10025\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1." -o content_filter=procmail:spamfilter";
	    }
	# VIRUS OFF, SPAM ON
	} elsif( ( defined $q->param('virus') && $q->param('virus') ne $message->{on} ) &&
		 ( defined $q->param('spam') && $q->param('spam') eq $message->{on} ) ) {
	# smtp       inet  n       -       n       -       -   smtpd -o content_filter=procmail:spamfilter
	# smtps      inet  n       -       n       -       -   smtpd -o content_filter=procmail:spamfilter
	##localhost:10025 inet n   -       n       -       -   smtpd -o content_filter=procmail:spamfilter
	    #print STDERR "VIRUS OFF, SPAM ON\n";
	    if( $l =~ /(^smtp[s]*\s+inet.*smtpd).*/ ) {
		$master->[$i] = $1." -o content_filter=procmail:spamfilter";
	    }
	    if( $l =~ /^[\#]*(localhost:10025\s+inet.*smtpd).*/ ) {
		$master->[$i] = "#".$1." -o content_filter=procmail:spamfilter";
	    }
	}
    }

    $msg = $session->save_read2session("$cf", $cf);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($cf.": ".$message->{postfix_file_modified},
				 "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
				 "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }
    my $r = $msg->data();

    my $cfr = new cfreader("=");

    # get all commands from main.cf
    my $all = $cfr->getAllCMD($r);

    # this commands must be active (remove comment)
    my @active = ( "disable_dns_lookups",
		   "defer_transports",
		   "myhostname",
		   "relayhost");
    my @inactive = ();

    $all->{"relayhost"}     = $q->param('relayhost');

    if( defined $q->param('DOD') && $q->param('DOD') eq $message->{on} ) {
	$all->{"disable_dns_lookups"} = "yes";
	$all->{"defer_transports"} = "smtp";
    } else {
	$all->{"disable_dns_lookups"} = "no";
	$all->{"defer_transports"} = "";
    }
    
    if( ( defined $q->param('DOD') && $q->param('DOD') eq $message->{on} )
	&& $all->{"relayhost"} eq "" ) {
	$this->display_error($message->{postfix_cfg_norelay},
			     "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	exit;
    } elsif( defined $q->param('DOD') && $q->param('DOD') eq $message->{on} ) {
	# try to find out, if there is already an entry for this host
	open(IN, "/usr/bin/grep ".$all->{"relayhost"}." /etc/hosts |");
	my @hp = <IN>;
	close(IN);

	if( $#hp < 0 ) {
	    $this->display_msg("",
			       $confParam->{cgi_path}."/hosts_edit.pl?host=".$all->{"relayhost"}."\&sessionID=$sessionID\&tab=$tab\&stab=$stab",
			       1);
	    exit;
	}
    }

    if( $q->param('relayhost_auth') ) {
	foreach my $opt ( @{$this->{"relayhost_auth"}} ) {
	    $all->{$opt} = $this->{"needed"}->{$opt};
	}
	push @active, @{$this->{"relayhost_auth"}};
	my $tmp = $all->{'relayhost'}." ".$q->param('relayhost_auth_user').":".$q->param('relayhost_auth_password');	
	my @password = ();
	push @password, $tmp;
	my $ret = $session->suad_save(\@password, '/etc/postfix/saslpasswd');
	   $ret = $session->suad_unlink('/etc/postfix/saslpasswd.orig');
	   $ret = $session->suad_chmod('/etc/postfix/saslpasswd','640');
           $ret = $session->suad_stdout_exec("/usr/sbin/postmap /etc/postfix/saslpasswd");
	   $ret = $session->suad_chmod('/etc/postfix/saslpasswd.db','644');
        my $output = $ret->code();   
           $output =~ /OK:\s+(\d+)/mg;  
           if( $1 != 0 ) {
                $this->display_error("failed to create postmap",
                                      $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID"
                                    );
                exit(1);
            }
    } else {
	my $ret = $session->suad_unlink('/etc/postfix/saslpasswd');
	   $ret = $session->suad_unlink('/etc/postfix/saslpasswd.db');
	push @inactive, @{$this->{"relayhost_auth"}};
    }

    if( defined $q->param('sasl') && $q->param('sasl') eq $message->{on} ) {
	foreach my $opt ( @{$this->{"saslopt"}} ) {
	    $all->{$opt} = $this->{"needed"}->{$opt};
	}

	push @active, @{$this->{"saslopt"}};
    } else {
	push @inactive, @{$this->{"saslopt"}};
    }

    if( defined $q->param('tls') && $q->param('tls') eq $message->{on} ) {
	foreach my $opt ( @{$this->{"tlsopt"}} ) {
	    $all->{$opt} = $this->{"needed"}->{$opt};
	}
	push @active, @{$this->{"tlsopt"}};
    } else {
	push @inactive, @{$this->{"tlsopt"}};
    }
    
    $cfr->setAllCMD( $all, $r, \@inactive, \@active );

    my $ar = $cfr->HASH2CF($r);

    my $ret = $session->suad_save($ar, $cf);
    if(  $ret->code() ne "OK" ) {
	$this->display_error($ret->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }

    $ret = $session->suad_save($master, $mastercf);
    if(  $ret->code() ne "OK" ) {
	$this->display_error($ret->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }

    $ret = $session->suad_exec("pfreload");
    if(  $ret->code() ne "OK" ) {
	$this->display_error($ret->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }

    $msg = $session->save_clear2session("$cf");
    if( $msg->code() ne "OK" ) {
	$this->display_error($msg->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    $msg = $session->save_clear2session("$mastercf");
    if( $msg->code() ne "OK" ) {
	$this->display_error($msg->code(),
			     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	exit;
    }

    
    $this->display_msg($message->{postfix_saved_ok},
		       "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab",
		       1);
}

sub readConfig() {
    my $this = shift;

    my $session   = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $q       = $this->{"cgi"};

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"POSTFIX"))->data();

    my $cf = "/etc/postfix/main.cf";
    my $mastercf = "/etc/postfix/master.cf";

    my $cfr = new cfreader("=");
    
    my $com = $cfr->getCommands("/usr/sbin/postconf");
    if( ! defined $com ) {
	$this->display_error($message->{postfix_exp_no_commands},
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    if( ! open(IN, "< $cf") ) {
	$this->display_error($message->{postfix_exp_not_readable}."$cf: $?",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my @raw = <IN>;
    chomp(@raw);
    close(IN);
    
    if( ! open(IN, "< $mastercf") ) {
	$this->display_error($message->{postfix_exp_not_readable}."$mastercf: $?",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    my @masterraw = <IN>;
    chomp(@masterraw);
    close(IN);

    # store data into session
    my $msg = $session->save_store2session("$mastercf", $mastercf, \@masterraw);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($mastercf.": ".$message->{postfix_file_modified},
				 "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
				 "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }

    $this->{"conf"}->{spam} = ( usesSPAMFilter(\@masterraw) ? $message->{on} : $message->{off} );

    # check for virus scanner (amavis)
    my $count=0;
    for(my $i=0; $i<scalar(@masterraw); $i++) {
	my $l = $masterraw[$i];
	$count++ if $l =~ /^[\w:]+\s+inet.*content_filter=vscan:/;
	if( $l =~ /^vscan\s+unix/ ) {
	    $count++ if $masterraw[$i+1] =~ /^.*argv=\/usr\/sbin\/amavis/;
	}
    }

    $this->{"conf"}->{virus} = ( $count > 1 ? $message->{on} : $message->{off} );

    #my $start = (times)[0];
    my $r = $cfr->CF2HASH(\@raw, $com);
    #my $delta = (times)[0] - $start;

    #print STDERR "Zeit=$delta\n";

    # get all commands from main.cf
    my $all  = $cfr->getAllCMD($r);

    # create needed cmd/value pairs if not existant
    foreach my $n ( keys %{$this->{"needed"}} ) {
	if( ! defined $all->{$n} ) {
	    # print STDERR "$n is not defined\n";
	    push @$r,
	    { status => "C_CMD",
	      cmd => $n,
	      value => $this->{"needed"}->{$n},
	      multi => 0 };
	}
    }

    # again get all commands from main.cf
    $all = $cfr->getAllCMD($r);
    # and we need the status of all the cmds
    my $stat = $cfr->getAllSTATUS($r);

    # store data into session
    my $msg = $session->save_store2session("$cf", $cf, $r);
    if( $msg->code() ne "OK" ) {
	if( $msg->code() eq "MD5SUM" ) {
	    $this->display_error($cf.": ".$message->{postfix_file_modified},
			  "$confParam->{cgi_path}/postfix.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
	    exit;
	} else {
	    $this->display_error($msg->code(),
			  "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	    exit;
	}
    }

    my $dialup = 0;
    my $tls = 0;

    $this->{"conf"}->{relayhost} = ( $stat->{"relayhost"} eq "CMD" ? $all->{"relayhost"} : "" );
    
    # if they are created new above, they must be also inactive!
    if( $all->{"disable_dns_lookups"} eq "yes" &&
	$all->{"defer_transports"} eq "smtp" ) {
	$this->{"conf"}->{dialup} = $message->{on};
    } else {
	$this->{"conf"}->{dialup} = $message->{off};
    }

    if( $stat->{"smtpd_use_tls"} eq "CMD" && $all->{"smtpd_use_tls"} eq "yes" &&
	$stat->{"relay_clientcerts"} eq "CMD" &&
	$stat->{"smtpd_tls_CAfile"} eq "CMD" &&
	$stat->{"smtpd_tls_cert_file"} eq "CMD" &&
	$stat->{"smtpd_tls_key_file"} eq "CMD" ) {
	$this->{"conf"}->{tls} = $message->{on};
    } else {
	$this->{"conf"}->{tls} = $message->{off};
    }

    if( $stat->{"smtpd_sasl_auth_enable"} eq "CMD" && $all->{"smtpd_sasl_auth_enable"} eq "yes" &&
	$stat->{"smtpd_sasl_security_options"} eq "CMD" &&
	$stat->{"smtpd_client_restrictions"} eq "CMD" ) {
	$this->{"conf"}->{sasl} = $message->{on};
    } else {
	$this->{"conf"}->{sasl} = $message->{off};
    }

    if( $stat->{"smtp_sasl_auth_enable"} eq "CMD" && $all->{"smtp_sasl_auth_enable"} eq "yes" &&
	$stat->{"smtp_sasl_security_options"} eq "CMD" &&
	$stat->{"smtp_sasl_password_maps"} eq "CMD" ) {
	$this->{"conf"}->{relayhost_auth} = 1;
	my $data = $session->suad_read('/etc/postfix/saslpasswd');
	my @raw = split(/\n/, $data->data());
	my $relayhost = $this->{"conf"}->{relayhost};
	foreach (@raw) {
		if(/$relayhost (.*):(.*)/) {
			$this->{"conf"}->{relayhost_auth_user}=$1;
			$this->{"conf"}->{relayhost_auth_password}=$2;
		}
	}
    } else {
	$this->{"conf"}->{relayhost_auth} = 0;
	$this->{"conf"}->{relayhost_auth_user}     = '';
	$this->{"conf"}->{relayhost_auth_password} = '';
    }

}

1;
