#
# Copyright (c) 2002 SuSE GmbH Nuernberg, Germany.  All rights reserved.
#
#
# $Id: register_pc.pm,v 1.22 2004/10/14 15:46:57 varkoly Exp $
# 
package register_pc;
use strict;

use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::LDAP;
use Net::Netmask;
use LDAPUtils;
use Utils;
use Display;
use cfreader;
use POSIX;
use Storable qw(thaw freeze);
use vars qw(@ISA);
use subs qw(exit);

# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);


sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};
    my $do_register        = $cgi->param("do_register");
    my $do_register_realy  = $cgi->param("do_register_realy");
    my $register_only_room = $cgi->param("register_only_room");

    if( defined $register_only_room && $register_only_room ne "" ) {
        $this->do_register_room();
    } elsif( defined $do_register && $do_register ne "" ) {
	    $this->do_register();
    } elsif( defined $do_register_realy && $do_register_realy ne "" ) {
	    $this->do_register_realy();
	    $this->create_dhcp_setup();
    } else {
	    $this->startup();
    }
}

sub do_register_room {
    my $this = shift;
    my $comeback = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"REGISTER"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};
    my $user_dn       = "uid=".$LOGINuid.",".$ldapbase;

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $newroom = $q->param('newroom');
    $newroom =~ s/[\s+|\n]//g;

    my $url = "$confParam->{cgi_path}/register_pc.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";
    my $ldap= Net::LDAP->new($confParam->{LDAPserver},port=>$confParam->{LDAPport},version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>$user_dn, password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }
    # ist it a jocker??
    $result = $ldap->search(base=>"o=DHCP,".$ldapbase,
                            filter=>"(&(Objectclass=SuSESchoolRoom)(description=$newroom))",
                            attrs=>[ 'cn' ]);
    if($result->count > 0) {
      if(!defined $comeback || $comeback ne 'comeback' ) {
        $this->display_error($message->{room_exist}.' : '.$newroom, $url);
        exit;
      } else {
        return $result->entry(0)->dn;  
      }    
    }    

    # find all not registered rooms
    $result = $ldap->search(base=>"o=DHCP,".$ldapbase,
                            filter=>"(&(Objectclass=SuSESchoolRoom)(!(description=*)))",
                            attrs=>[ 'cn' ]);
    if($result->code != 0) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        $ldap->unbind;
        exit;
    }

    if($result->count < 1) {
        $this->display_error($message->{no_enough_dhcp_range}, $url);
        exit;
    } 
    if(length($newroom) > 9) {
	    $this->display_error($message->{room_to_long}.":".$newroom, $url);
	    exit;
    }
    if( $newroom =~ /[^a-zA-Z0-9-]+/ ) {
	    $this->display_error($message->{room_invalid}.":".$newroom, $url);
	    exit;
    }
    my @rooms   = ();
    my %roomdns = ();
    foreach my $entry ($result->all_entries) {
       my $cn = $entry->get_value('cn');
       push @rooms,$cn;
       $roomdns{$cn} = $entry->dn();
    }   
    my $roomcn = @rooms[0];
    $result = $ldap->modify( $roomdns{$roomcn},
                             add => { description=> $newroom }
                           );
    if($result->code != 0) {
       my $errs = "modify $message->{failed}\n\n";
       $errs .= &LDAPerror($result);
       $this->display_error($errs, $url);
       $ldap->unbind;
       exit;
    }
    if(!defined $comeback || $comeback ne 'comeback' ) {
      $this->display_msg("$message->{register_room_success}", $url, 1);
      exit;
    }
    return $roomdns{$roomcn};
}

sub do_register {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"REGISTER","MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};
    my $user_dn       = "uid=".$LOGINuid.",".$ldapbase;

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $domain  = $q->param('domain');
    my $roomdn  = $q->param('rooms');
    my $newroom = $q->param('newroom');
    my $if_in_domain = $q->param('if_in_domain');
    my $remote_ip    = $q->remote_addr;
    my @dhcphosts = ();
    $newroom   =~ s/[\s+|\n]//g;
    if(defined $newroom && $newroom ne '') {
        $roomdn = $this->do_register_room('comeback');
    }    

    my $SCHOOL_SERVER_NET = read_rcconfig("SCHOOL_SERVER_NET");
    my $block = new Net::Netmask($SCHOOL_SERVER_NET);
    my $new_ip = '';
    my $hostname = '';
    my $dhcpHWAddress = '';
    my $url = "$confParam->{cgi_path}/register_pc.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    # Aus dem Servernetz sollte man nix Registrieren:
    if(  ! $block->match($remote_ip) ) {
        my $tmp = `/sbin/arp -a $remote_ip`;
        $tmp =~ /(\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)/;
        $dhcpHWAddress = $1;
    }
    # make an LDAP connection
    my $ldap= Net::LDAP->new($confParam->{LDAPserver},port=>$confParam->{LDAPport},version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>$user_dn, password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }
    # find all defined hosts in this room and
    # find all the Avaiable IP-Addresses in these room
    $result = $ldap->search( base   => $roomdn,
                             filter => 'objectClass=SuSESchoolRoom',
                             attrs  => ['dhcpHostDN','description','dhcpRange','dhcpNetMask']
                           );
    if($result->code != 0 || $result->count < 1 ) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result).':'.$result->count;
        $this->display_error($errs, $url);
        $ldap->unbind;
        exit;
    }
    my $roomnet   = $result->entry(0)->get_value('dhcpRange').'/'.$result->entry(0)->get_value('dhcpNetMask');
    my $roompref  = $result->entry(0)->get_value('description');
    my @dhcpHostDN = $result->entry(0)->get_value('dhcpHostDN');
    my %roomhosts  = ();
    foreach my $i (@dhcpHostDN) {
        $result = $ldap->search( base   => $i, filter => 'objectClass=dhcpHost' , attrs  => ['dnsZoneDN'] );
        if($result->code != 0 || $result->count < 1 ) {
            # something is wrong
            print STDERR "dhcpHostDN not exist $i\n";
            next;
        }
        my $dnsZoneDN = $result->entry(0)->get_value('dnsZoneDN');    
        $result = $ldap->search( base   => $dnsZoneDN, ,filter=> 'objectClass=dNSZone', attrs  => ['ARecord'] );
        if($result->code != 0 || $result->count < 1 ) {
            # something is wrong
            print STDERR "dnsZoneDN not exist $dnsZoneDN\n";
            next;
        }
        $roomhosts{$result->entry(0)->get_value('ARecord')} = 1;
    }
    $block = new Net::Netmask($roomnet);
    my @hosts      = ();
    my %lhosts     = ();
    my $base       = $block->base();
    my $broadcast  = $block->broadcast();
    my $counter    = 0;
    foreach my $i ($block->enumerate()) {
        if(  $i ne $base && $i ne $broadcast ) {
            $counter ++;
            if ( $roomhosts{$i} ) {
                next;
            }
            my $hostname = lc(sprintf("$roompref-pc%02d",$counter));
            push @hosts, $hostname.':'.$i;
            $lhosts{$hostname.':'.$i} = $hostname.'.'.$domain;
        }
    }
    #Saving avaiable hostnames
    my $fhosts = freeze(\@hosts);
    $fhosts = unpack('H*',$fhosts);
    my %tmp = ( "fhosts" => $fhosts );
    my $s_reply=$session->modify(\%tmp);

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>'100%'});
    $html .= $q->start_Tr();
    $html .= $q->start_td();
    $html .= $q->start_table({ -align=>"center", -cellspacing=>0, -cellpadding=>3, -width=>'100%'});
    $html .= $q->Tr( [ $this->help_th(2, "$message->{addhost}", 'DHCP_addhost') ]);
    $html .= $q->start_form(-name=>'form',-action=>$confParam->{cgi_path}.'register_pc.pl');
    $html .= $q->Tr($q->td({-align=>"center",-width=>'40%'}, $message->{choose_a_host}.":").
                    $q->td({-align=>"center",-width=>'40%'}, $message->{'dhcpHWAddress'}.':')
                   );
    $html .= $q->Tr( $q->td({-align=>"center"},$q->scrolling_list( -name    => 'host',
                                                -value   => \@hosts,
                                                -labels  => \%lhosts,
                                                -size    => 8,
                                                -default => $hosts[0]
                                             )
                      ).
                      $q->td({-align=>"center"},
                          $q->textarea(-name=>'dhcpHWAddress',-default=>$dhcpHWAddress, -rows=>10,-cols=>20)
                      )
                   );
    $html .= $q->Tr($q->td({-align=>"center",-width=>'40%'}, $message->{'other_name'}.":").
                    $q->td({-align=>"center",-width=>'40%'}, $message->{example}.'&nbsp;&nbsp;<b>:</b>&nbsp;&nbsp;00:A0:CC:73:30:66')
                   );
    $html .= $q->Tr($q->td({-align=>"center",-width=>'40%'}, $q->textfield(-name=>'other_name', -size=>20)));
    $html .= $q->Tr($q->td({-align=>"center",-width=>'40%'}, $q->checkbox(-name=>'teacher_ws', -label=>$message->{'teacher_ws'})));
    $html .= $q->Tr( $q->td({-align=>"center",-colspan=>2},
                       $q->submit(-class=>"button",-name=>"insert", -value=>"$message->{insert}").
                       $q->hidden(-name=>'do_register_realy', -value=>'do_register_realy').
                       $q->hidden(-name=>'sessionID', -value=>"$sessionID").
                       $q->hidden(-name=>'domain', -value=>"$domain").
                       $q->hidden(-name=>'if_in_domain', -value=>"$if_in_domain").
                       $q->hidden(-name=>'roomdn', -value=>"$roomdn").
                       $q->hidden(-name=>'tab', -value=>"$tab").
                       $q->hidden(-name=>'stab', -value=>"$stab").
                       $q->end_form()
                       )
                    );
    $html .= $q->end_table();
    $html .= $q->end_td();
    $html .= $q->end_table();

    $this->SUPER::display($html);
}

sub do_register_realy  {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"REGISTER","MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};
    my $user_dn       = "uid=".$LOGINuid.",".$ldapbase;

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $domain  = $q->param('domain');
    my $roomdn  = $q->param('roomdn');
    my $other_name = $q->param('other_name') || '';
    my $teacher_ws = $q->param('teacher_ws') || '';
    my $dhcpHWAddresses = $q->param('dhcpHWAddress');
    my $if_in_domain    = $q->param('if_in_domain');
    my $SCHOOL_NETMASK  = read_rcconfig("SCHOOL_NETMASK");
    my $USE_REMBO       = read_rcconfig("SCHOOL_USE_REMBO") || 'no';
    my $hostname = '';
    my $new_ip   = '';
    my $room     = '';
    my $tmp      = '';
    my $newpc    = '';
    my $choosen_host = $q->param('host');
    #Recovering avaiable hostnames
    my $fhosts   = $confParam->{'fhosts'};
    $fhosts      = pack('H*',$fhosts);
    my $hosts    = thaw($fhosts);
    my $host     = pop (@{$hosts});
    my $WARNINGS = '';
     
    #seeking $hosts to the choosen host
    if($choosen_host ne '') {
	while( $choosen_host ne $host && $host ne '' ) { 
		$host     = shift (@{$hosts});
	}
    }
    # Connecting the LDAP-Server
    my $url = "$confParam->{cgi_path}/register_pc.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";
    my $ldap= Net::LDAP->new($confParam->{LDAPserver},port=>$confParam->{LDAPport},version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server", $url);
	exit;
    }
    my $result = $ldap->bind(dn=>$user_dn, password=>$confParam->{"passwd"});
    if($result->code != 0) {
	my $errs = "bind $message->{failed}\n\n";
	$errs .= &LDAPerror($result);
	$this->display_error($errs, $url);
	exit;
    }
    if( scalar(split /\n/, $dhcpHWAddresses) > 1 && ($other_name ne '' || $teacher_ws )) {
	$this->display_error("If registering teacher pc or computer with alternete name, ".
			     "you may only use one hardware address.", $url);
	exit;
    }
    foreach my $dhcpHWAddress (split /\n/, $dhcpHWAddresses) {
	    # No more free host exist	
	    if($host eq '') {
		$WARNINGS .= "$message->{no_more_host}$dhcpHWAddress<br>";
		next;
	    }
	   chomp $dhcpHWAddress; $dhcpHWAddress =~ s/\r//;
	    # Proofing the MAC address	
	    if( $dhcpHWAddress !~ /^\w\w:\w\w:\w\w:\w\w:\w\w:\w\w$/) {
		$WARNINGS .= $message->{invalid_hwa}.': "'.$dhcpHWAddress.'"<br>';
		next;
	    }

	    # searching if dhcpHWAddress is uniqe
	    $result = $ldap->search( base   => 'o=DHCP,'.$ldapbase,
			   filter => "dhcpHWAddress=ethernet $dhcpHWAddress",
			   attrs  => ['cn']
			 );
	    my $host_exist = 0;
            if($result->count() > 0) {
		$WARNINGS .= $message->{existing_hwa}.': "'.$dhcpHWAddress.'" host:'.$result->entry(0)->get_value('cn').'<br>';
	        next;
            }
    	   ($hostname,$new_ip) = split /:/, $host;
	   ($room,$tmp)        = split /-/, $hostname;  
            if($other_name ne '') {
                $tmp        = $other_name;
		$other_name = $hostname;
		$hostname   = $tmp;
            }
	    $host = shift (@{$hosts});
	    my $newdn  = "cn=$hostname,$roomdn";
	    my $hostdn = "relativeDomainName=$hostname,ou=$domain,o=DNS,$ldapbase";

	    # create dhcpHost entry
	    $result =$ldap->add( dn => $newdn,
			attr => [
				  objectClass => [ 'top', 'dhcpHost','dhcpOptions' ,'SuSEdhcpHost' ],
				  dhcpHWAddress   => "ethernet ".$dhcpHWAddress,
				  cn              => "$hostname",
				  dhcpStatements  => "fixed-address ".$new_ip,
				  dnsZoneDN       => "$hostdn" 
				]
				);
	    if($result->code != 0) {
		my $errs = "add $newdn : $message->{failed}\n\n";
		$errs .= &LDAPerror($result);
		$this->display_error($errs, $url);
		$ldap->unbind;
		exit;
	    }
	    # insert dhcpHost entry into the room entry 
	    $ldap->modify( dn => $roomdn, add=> { dhcpHostDN=>$newdn } );
	    # Zuerst kommt Forwardauflsung
	    $result = $ldap->add( 
		  dn   => $hostdn,
		  attr => [
		     'objectclass' => [ 'top' , 'dNSZone' , 'SuSEdhcpHost'],
		     'relativeDomainName' => $hostname,
		     'dNSTTL'     => '172800',
		     'zoneName'   => "$domain",
		     'aRecord'    => "$new_ip",
		     'dhcpHostDN' => "$newdn"
		    ]   
		);  
	    if($result->code != 0) {
		my $errs = "add $hostdn : $message->{failed}\n\n";
		$errs .= &LDAPerror($result);
		$this->display_error($errs, $url);
		$ldap->unbind;
		exit;
	    }
	    if($other_name ne '') {
	       $ldap->add( 
		  dn   => "relativeDomainName=$other_name,ou=$domain,o=DNS,$ldapbase",
		  attr => [ 
		     'objectclass' => [ 'top' , 'dNSZone' , 'SuSEdhcpHost'],
		     'relativeDomainName' => $other_name,
		     'dNSTTL'     => '172800',
		     'zoneName'   => "$domain",
		     'aRecord'    => "$new_ip",
		     'dhcpHostDN' => "$newdn"
		    ]   
		);  
	        $ldap->modify( dn => $newdn, add=> { dnsZoneDN=>"relativeDomainName=$other_name,ou=$domain,o=DNS,$ldapbase" } );
	    }
	    if($teacher_ws) {
	       $ldap->add( 
		  dn   => "relativeDomainName=$room-tpc,ou=$domain,o=DNS,$ldapbase",
		  attr => [ 
		     'objectclass' => [ 'top' , 'dNSZone' , 'SuSEdhcpHost'],
		     'relativeDomainName' => "$room-tpc",
		     'dNSTTL'     => '172800',
		     'zoneName'   => "$domain",
		     'aRecord'    => "$new_ip",
		     'dhcpHostDN' => "$newdn"
		    ]   
		);  
	        $ldap->modify( dn => $newdn, add=> { dnsZoneDN=>"relativeDomainName=$room-tpc,ou=$domain,o=DNS,$ldapbase" } );
	    }
	    # Und jetzt kommt Reversauflsung
	    my $result = $ldap->search(
		  base=>"o=DNS,".$ldapbase,
		  filter=>"pTRRecord=mailserver.$domain.",
		  attrs=>[ 'zoneName','relativeDomainName' ]
		);
	    if($result->code != 0) {
		my $errs = "search $message->{failed}\n\n";
		$errs .= &LDAPerror($result);
		$this->display_error($errs, $url);
		$ldap->unbind;
		exit;
	    }
	    my $zone  = '';
	    my $subip = '';
	    
	    if( $SCHOOL_NETMASK =~ /255\.255/ ) {
	       $new_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
	       $subip = "$4.$3"; 
	       $zone  = "$2.$1.IN-ADDR.ARPA";
	    } else {
	       $new_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
	       $subip = "$4.$3.$2";
	       $zone  = "$1.IN-ADDR.ARPA";
	    } 
	    $result = $ldap->add( 
		  dn   => "relativeDomainName=$subip,ou=$zone,o=DNS,$ldapbase",
		  attr => [
		     'objectclass' => [ 'top' , 'dNSZone' ],
		     'relativeDomainName' => $subip,
		     'dNSTTL' => '172800',
		     'zoneName' => "$zone",
		     'hInfoRecord' => "$dhcpHWAddress",
		     'pTRRecord'   => "$hostname.$domain."
		    ]   
		);  
	    if($result->code != 0) {
		my $errs = "add relativeDomainName=$subip,ou=$zone,o=DNS,$ldapbase : $message->{failed}\n\n";
		$errs .= &LDAPerror($result);
		$this->display_error($errs, $url);
		$ldap->unbind;
		exit;
	    }
	    if($other_name ne '') {
               $ldap->modify( dn => "relativeDomainName=$subip,ou=$zone,o=DNS,$ldapbase", add=> { pTRRecord=>"$other_name.$domain." } );
            }
	    if($teacher_ws) {
               $ldap->modify( dn => "relativeDomainName=$subip,ou=$zone,o=DNS,$ldapbase", add=> { pTRRecord=>"$room-tpc.$domain." } );
            }

	    my $homedir="/home/workstations/$hostname";
	#   jetz machen wir einen Benutzer fr dieses Account    
	    # nun brauch er auch ein passwort
	    my $salt =  pack("C2",(int(rand 26)+65),(int(rand 26)+65));
	    my $crypt_passwd = crypt $hostname,$salt;
	    $crypt_passwd = "{crypt}".$crypt_passwd;

	    # was bin ich 
	    $result = $ldap->search( base   => "$ldapbase" ,
			   filter => "uid=cyrus" ,
			   attrs  => ['o','c','mailDomain']
			 );
	    my $org     = $result->entry(0)->get_value('o');
	    my $country = $result->entry(0)->get_value('c');
	    my $domain  = $result->entry(0)->get_value('mailDomain');
	   
	    my $command = $confParam->{bin_path}."/add_user_ldap -";   
	    my $cmd_args = "H=$confParam->{LDAPserver}\nP=$confParam->{LDAPport}\nal=$LOGINuid\n";
	    $cmd_args.= "bDN=$ldapbase\nap=$confParam->{passwd}\n";
	    $cmd_args.= "uid=$hostname\n";
	    $cmd_args.= "sn=$hostname Workstation-User\n";
	    $cmd_args.= "up=$crypt_passwd\n";
	    $cmd_args.= "mail=$hostname\@$domain\n";
	    $cmd_args.= "o=$org\n";
	    $cmd_args.= "c=$country\n";
	    $cmd_args.= "gidnumber=103\n";
	    $cmd_args.= "homedirectory=$homedir\n";
	    $cmd_args.= "writeglobaladdress=denied\n";
	    $cmd_args.= "mailenabled=no\n";
	    my $errs = cmd_pipe("$command", "$cmd_args");
	#    print STDERR "$errs\n$command\n$cmd_args";
	#	    if($errs ne "") {
	#		return (-1, $errs);
	#	    }

	    my $mesg = $session->suad_mkdir($homedir, '770');
	    if( $mesg->code() ne "OK" ){
	       $errs .= "create_homedir ".$message->{'failed'}.": ".$mesg->code()."\n";
	    }else{
	       $errs .= "create_homedir ".$message->{'success'}."\n";
	    }
	    $mesg = $session->suad_copydir('/etc/skel/', $homedir);
	    if( $mesg->code() ne "OK" ){
	       $errs .= "prepare_homedir ".$message->{'failed'}.": ".$mesg->code()."\n";
	    }else{
	       $errs .= "prepare_homedir ".$message->{'success'}."\n";
	    }
	    $mesg = $session->suad_chown($homedir,"$hostname",'102');
	    if( $mesg->code() ne "OK" ){
	       $errs .= "prepare_homedir2 ".$message->{'failed'}.": ".$mesg->code()."\n";
	    }else{
	       $errs .= "prepare_homedir2 ".$message->{'success'}."\n";
	    }
	    my $ret = $session->suad_smbadduser($hostname, $hostname);
	    if(  $ret->code() ne "OK" ) {
		$this->display_error($ret->code(),
				     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
		exit;
	    }

	    # Aufnahme in die Windowsdomaine
	    if( $if_in_domain ) {
		    $mesg = $session->suad_mkdir('/home/profile/'.$hostname, '700');
		    $mesg = $session->suad_chown('/home/profile/'.$hostname,"$hostname",'root');
		    # Jetzt wird ein Benutzeraccount fr den Rechner erzeugt
		    $command = $confParam->{bin_path}."/add_host_ldap -";   
		    $cmd_args = "H=$confParam->{LDAPserver}\nP=$confParam->{LDAPport}\nal=$LOGINuid\n";
		    $cmd_args.= "bDN=$ldapbase\nap=$confParam->{passwd}\ngidnumber=103\n";
		    $cmd_args.= "homedirectory=/dev/null\n";
		    $cmd_args.= "uid=$hostname\$\ndescription=Workstation $hostname ip:$new_ip\n";
		    my $errs = cmd_pipe("$command", "$cmd_args");
	#	    if($errs ne "") {
	#		return (-1, $errs);
	#	    }
        	    my $ret = $session->suad_smbaddmachine($hostname);
		    if(  $ret->code() ne "OK" ) {
		       $newpc .= $ret->code()."<br>";
		    }
                    if($USE_REMBO eq 'yes') { 
		        $ret = $session->suad_stdout_exec("/usr/bin/smbpasswd -c /etc/samba/smb.wst.conf -a \'$hostname\$\' 12345678");
			my $output = $ret->code();
			$output =~ /OK:\s+(\d+)/mg;
			if( $1 != 0 ) {
			    $newpc .= $ret->code()."<br>";
			}
		    }
	    }

	    $newpc .=   "Hostname: $hostname.$domain<br>";
	    $newpc .=   "IP-Address: $new_ip<br>";
	    $newpc .=   "MAC-Address: $dhcpHWAddress<br>";
	    if( $if_in_domain ) {
	      $newpc .=  "Netbiosname: $hostname<br>";
	    }
    }
    $ldap->unbind;
    $newpc = "<font color='red'>$WARNINGS</font><br>$newpc";
    $q->param('newpc',$newpc);
    # $this->display_msg($message->{register_success}, $url, 1);

}
sub startup {
    my $this = shift;

    my $q = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"REGISTER"))->data();

    my $LOGINuid      = $confParam->{uid};
    my $ldapbase      = $confParam->{baseDN};
    my $user_dn       = "uid=".$LOGINuid.",".$ldapbase;

    my $tab     = $q->param('tab');
    my $stab    = $q->param('stab');
    my $domain  = $q->param('domain');

    my @rooms = ();
    my @doms  = ();
    my %room_label = ();

    my $url = "$confParam->{cgi_path}/register_pc.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";
    my $ldap= Net::LDAP->new($confParam->{LDAPserver},port=>$confParam->{LDAPport},version => 3);
    if(!defined $ldap) {
        $this->display_error("can't contact LDAP Server", $url);
        exit;
    }
    my $result = $ldap->bind(dn=>$user_dn, password=>$confParam->{"passwd"});
    if($result->code != 0) {
        my $errs = "bind $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        exit;
    }
#-- search for virtual domains
    $result = $ldap->search( base => 'o=DNS,'.$confParam->{baseDN},
                           scope => 'sub',
                           attrs => [ 'zoneName' ],
                           filter => 'relativeDomainName=@' );
    if ($result->count == 0) {
        my $text = "search failed\n".&LDAPerror($result);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }
    foreach my $i ($result->entries) {
        foreach my $zn ( $i->get_value('zoneName') ) {
            if( ! ( $zn =~ /IN-ADDR\.ARPA/i ) ) {
                push @doms, $zn;
            }
        }
    }    


    # find all registered rooms
    $result = $ldap->search(base=>"o=DHCP,".$ldapbase,
                            filter=>"(&(Objectclass=SuSESchoolRoom)(description=*))",
                            attrs=>[ 'description','dhcpRange','dhcpNetMask' ]);
    if($result->code != 0) {
        my $errs = "search $message->{failed}\n\n";
        $errs .= &LDAPerror($result);
        $this->display_error($errs, $url);
        $ldap->unbind;
        exit;
    }

    foreach my $entry ($result->all_entries) {
       my $key = $entry->dn;
       my $val = $entry->get_value('description').' : '.$entry->get_value('dhcpRange').'/'.$entry->get_value('dhcpNetMask');
       push @rooms,$key;
       $room_label{"$key"} = $val;
    } 

    my $html = "";
    $html .= $q->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .=	$q->start_Tr();
    $html .=	    $q->start_td();
    $html .=		$q->start_table({-border=>0, -cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .=		    $q->Tr( $this->help_th(2,  $message->{title}, 'REGISTER') );
    $html .=            $q->start_form(-action=>$confParam->{cgi_path}."/register_pc.pl");
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=                $q->td({  -width=>"40%"},"$message->{rooms_list}");
    $html .=                $q->td({},$q->popup_menu(-name=>'rooms', -values=>\@rooms, -labels=>\%room_label, -width=>160));
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=                $q->td({  -width=>"40%"},"$message->{new_room}");
    $html .=                $q->td({},$q->textfield(-name=>'newroom', -size=>20));
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=                $q->td({  -width=>"40%"},"$message->{register_only_room}");
    $html .=                $q->td({},$q->checkbox(-name=>'register_only_room', -label=>"" ));
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=                $q->td({  -width=>"40%"},"$message->{virt_domain}");
    $html .=                $q->td({},$q->popup_menu(-name=>'domain', -values=>\@doms, -default=>$domain, -width=>100));
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=                $q->td({  -width=>"40%"},"$message->{if_in_domain}");
    $html .=                $q->td({}, $q->checkbox(-name=>'if_in_domain', -checked=>'yes', -label=>"" ));
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->start_Tr();
    $html .=               $q->td({ -align=>"center", -colspan=>2},
				  $q->submit(-name=>"do_register", -value=>"$message->{do_register}").
				  $q->hidden(-name=>"sessionID", -value=>$sessionID).
				  $q->hidden(-name=>"tab", -value=>$tab).
				  $q->hidden(-name=>"stab", -value=>$stab).
				  $q->end_form());
    $html .=            $q->end_Tr();
    $html .= "\n";
    $html .=            $q->end_table();
    $html .=        $q->end_td();
    $html .=    $q->end_Tr();
    $html .= $q->end_table();
    
    $this->back_url("$confParam->{cgi_path}/menu.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab");
    $this->SUPER::display($html);
}

sub create_dns_setup {
    my $this = shift;
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM"))->data();


    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $url = "$confParam->{cgi_path}/register_pc.pl?sessionID=$sessionID\&tab=$tab\&stab=$stab";

    my $ldap= Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    if(!defined $ldap) {
	$this->display_error("can't contact LDAP Server",
			     "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.", ".
			  $confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }

    my $base = "o=DNS,".$confParam->{baseDN};
    $mesg = $ldap->search(	base	=> $base,
				scope	=> 'sub',
				filter	=> "objectclass=dNSZone" );
    if ($mesg->count == 0) {
        my $text = "search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
	$ldap->unbind;
        exit;
    }

    my $nsdn = $mesg->as_struct;

    my $namedtempl = "/etc/named.conf";
    my @namedconf = ();
    my $lastgen = localtime;
    my @pfmydest;

    push @namedconf,
    "//
// THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
// BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
//
// to change some options manually, edit $namedtempl
//
// Last generated at \"$lastgen\"
//
";

    # deassemble LDAP and reassemble DNS zonefiles

    $mesg = $ldap->search(	base => $base,
				scope => 'sub',
				filter => '(&(relativeDomainName=@)(objectclass=dNSZone))'
				);
    if( $mesg->code != 0 ){
        my $text = "LDAP: search failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }elsif( $mesg->count == 0) {
        my $text = "search failed. No DNS zones found\n";
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }	
    $ldap->unbind;
    foreach my $zone_entry ( $mesg->entries ) {
	my $zone = $zone_entry->get_value('zoneName');
	my $soaent = $nsdn->{$zone_entry->dn};

	# disassemble NS Forward maps
	if( ! ( $zone =~ /IN-ADDR.ARPA/i ) ) {

	    if( $zone ne $confParam->{mailDomain} ) {
		push @pfmydest,$zone;
	    }

	    my $zonefile = "/var/named/$zone.zone";
	    my @soa = split(/ /,$soaent->{soarecord}->[0]);
	    my @outf = ();
	    push @outf,
	    ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
	    push @outf, "\$TTL $soaent->{dnsttl}->[0]";
	    push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
	    push @outf, "\t\t\t".$soa[2];
	    push @outf, "\t\t\t".$soa[3];
	    push @outf, "\t\t\t".$soa[4];
	    push @outf, "\t\t\t".$soa[5];
	    push @outf, "\t\t\t".$soa[6].")";
	    foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
		push @outf, "\tIN\tNS\t".$nsrr;
	    }
	    foreach my $arr ( @{$soaent->{arecord}} ) {
		push @outf, "\tIN\tA\t".$arr;
	    }
	    foreach my $mxrr ( @{$soaent->{mxrecord}} ) {
		push @outf, "\tIN\tMX\t".$mxrr;
	    }

	    # now disassemble host entries
	    foreach my $dn (sort ( keys %$nsdn )) {
		if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
		    my $hent = $nsdn->{$dn};
		    foreach my $hzone ( @{$hent->{zonename}} ) {
			if( $hzone eq $zone ) {
			    push @outf, $hent->{relativedomainname}->[0]."\tIN\tA\t".$hent->{arecord}->[0];
			}
		    }
		}
	    }
	    my $ret = $session->suad_save(\@outf, $zonefile);
	    if(  $ret->code() ne "OK" ) {
		$this->display_error($ret->code(),
				     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
		exit;
	    }

	    # add zone to named.conf
	    push @namedconf, "zone \"$zone\" {";
	    push @namedconf, "\ttype master;";
	    push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            if( $zone eq $confParam->{mailDomain} ) {
#                push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
#            }
# E SCHULSERVER 
	    push @namedconf, "};";
	    
	} else { # disassemble NS Reverse maps
	    my $zonefile = "/var/named/$zone.zone";
	    my @soa = split(/ /,$soaent->{soarecord}->[0]);
	    my @outf = ();
	    push @outf,
	    ";
; THIS FILE IS AUTO-GENERATED, DO NOT EDIT MANUALLY OR YOUR CHANGES WILL
; BE LOST AT THE NEXT GENERATION OF THIS FILE!!!
;
; Last generated at \"$lastgen\"
;
";
	    push @outf, "\$TTL $soaent->{dnsttl}->[0]";
	    push @outf, "\@\tIN\tSOA\t".$soa[0]." ".$soa[1]." (";
	    push @outf, "\t\t\t".$soa[2];
	    push @outf, "\t\t\t".$soa[3];
	    push @outf, "\t\t\t".$soa[4];
	    push @outf, "\t\t\t".$soa[5];
	    push @outf, "\t\t\t".$soa[6].")";
	    foreach my $nsrr ( @{$soaent->{nsrecord}} ) {
		push @outf, "\tIN\tNS\t".$nsrr;
	    }

	    # now disassemble host entries
	    foreach my $dn ( sort ( keys %$nsdn )) {
		if( ! ( $dn =~ /^relativeDomainName=\@/i ) ) {
		    my $hent = $nsdn->{$dn};
		    foreach my $hzone ( @{$hent->{zonename}} ) {
			if( $hzone eq $zone ) {
			    foreach my $arr ( @{$hent->{ptrrecord}} ) {
				push @outf, $hent->{relativedomainname}->[0]."\tIN\tPTR\t".$arr;
			    }
			}
		    }
		}
	    }
	    my $ret = $session->suad_save(\@outf, $zonefile);
	    if(  $ret->code() ne "OK" ) {
		$this->display_error($ret->code(),
				     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
		exit;
	    }

	    # add zone to named.conf
	    push @namedconf, "zone \"$zone\" {";
	    push @namedconf, "\ttype master;";
	    push @namedconf, "\tfile \"$zone.zone\";";
# C SCHULSERVER 
# THERE IS NO DHCP_UPDATER POSSIBLE
#            push @namedconf, "\tallow-update { key DHCP_UPDATER; };";
# E SCHULSERVER 
	    push @namedconf, "};";
	}
    }

    my $errors = "";

    my $ret = $session->suad_save(\@namedconf, "/etc/named.d/ldap.dump");
    if(  $ret->code() ne "OK" ) {
	$errors .= "named: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }
    
    $ret = $session->suad_exec("nsreload");
    if(  $ret->code() ne "OK" ) {
	$errors .= "named: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }

    
    $ret = $session->suad_exec("fetchdreload");
    if(  $ret->code() ne "OK" ) {
	$errors .= "fetchd: ".$ret->code()."\n";
	#$this->display_error($ret->code(),
	#		     $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
	#exit;
    }
    my $newpc = $cgi->param('newpc');
    $this->display_msg("$message->{register_success}<br>$newpc", $url, 30);
    exit();
}

sub create_dhcp_setup {
    my $this = shift;
    my $cgi = $this->{"cgi"};
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();
    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');
    my $act     = $cgi->param("act");

    $this->{"ldap"} = Net::LDAP->new($confParam->{LDAPserver}, port => $confParam->{LDAPport}, version => 3);
    my $ldap      = $this->{"ldap"};
    if(!defined $ldap) {
		$this->display_error("can't contact LDAP Server",
		      "$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
		exit;
    }
    my $mesg=$ldap->bind ("uid=".$confParam->{uid}.",".$confParam->{baseDN}, password=>$confParam->{passwd});
    if($mesg->code != 0) {
        my $text = "bind failed\n".&LDAPerror($mesg);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        exit;
    }
	my @dhcp_conf = $this->create_dhcp_setup_rec("cn=DHCP,o=DHCP,".$confParam->{baseDN},-1);

	my $ret = $session->suad_save(\@dhcp_conf, '/etc/dhcpd.conf');
	my $errors = "";
	if( $ret->code ne "OK" ){
		$errors .= "saving dhcpd.conf: failed<br>";
		$errors .= $ret->data()."<br>";
	}else{
		$ret = $session->suad_service_exec('dhcpd', 'restart');
		$ret->code() =~ /OK:\s(\d+?)\s(.*)/ms;
		if( $1 != 0 ){
			$errors .= "restarting dhcpd: failed<br>$2";
		}
	}
	if($errors ne ""){
	    $this->display_error($errors,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
	}else{
		$errors = $this->create_dns_setup();
		if($errors ne ""){
			$this->display_error($errors,$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab");
		}
	    $this->display_msg($message->{export_success},$confParam->{"cgi_path"}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
	}
	exit();
}

sub create_dhcp_setup_rec {
    my $this   = shift;
    my $akt_basedn = shift;
    my $deepth     = shift;
    my $akt_depth  = ($akt_basedn=~s/,/,/g);
    
    my $cgi = $this->{"cgi"};

    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $ldapbase  = $confParam->{baseDN};
    my $base_depth = ($ldapbase=~s/,/,/g);

    my $message = ($session->getLanguage($confParam->{LANG},"MANAGE_VDOM","DHCP","DNS_HOST"))->data();

    my $tab     = $cgi->param('tab');
    my $stab    = $cgi->param('stab');

    my $treelink=$confParam->{cgi_path}."/dhcp.pl?sessionID=$sessionID&tab=$tab&stab=$stab&act=edit";

    my $ldap  = $this->{"ldap"};

	my @dhcp_conf = ();

    my $result = $ldap->search(base=>$akt_basedn,scope=>"one",filter=>"objectclass=*");
    my @entries=$result->entries;
	my $left = "";
	for( my $i=0; $i < $deepth; $i++) {$left .= '  ';}
	$deepth++;

    foreach my $i (@entries) {
        my $dn = $i->dn;
        my $block = "";
		my @actions = ();
        foreach my $objectClass ($i->get_value('objectClass')) {
            if( $objectClass eq 'dhcpSubnet' ) {
                $block = '  ';
				my $network = new Net::Netmask($i->get_value('cn').'/'.$i->get_value('dhcpNetMask'));
				push @dhcp_conf, $left.'subnet '.$i->get_value('cn').' netmask '.$network->mask().' {';
            } elsif ( $objectClass eq "dhcpGroup" ) {
                $block = '  ';
				push @dhcp_conf, $left.'group { #Group: '.$i->get_value('description').' : '.$i->get_value('cn');
            } elsif ( $objectClass eq 'dhcpPool' ) {
                $block = '  '; 
				push @dhcp_conf, $left.'pool { #Pool: '.$i->get_value('cn');
				push @dhcp_conf, $left.'  range '.$i->get_value('dhcpRange').';';
            } elsif ( $objectClass eq 'dhcpHost' ) {
                $block = '  ';
				push @dhcp_conf, $left.'host '.$i->get_value('cn').' {';
				push @dhcp_conf, $left.$block.'hardware '.$i->get_value('dhcpHWAddress').';';
			}
        }
		foreach my $statement ($i->get_value('dhcpStatements')){
			push @dhcp_conf, $left.$block.$statement.';';
		}
		foreach my $option    ($i->get_value('dhcpOption')){
			push @dhcp_conf, $left.$block.'option '.$option.';';
		}
        push @dhcp_conf, $this->create_dhcp_setup_rec($dn,$deepth);
        if( $block ne "" ) {
			push @dhcp_conf, $left.'}';
        }
    }


    return @dhcp_conf;
}

1;
