#!/usr/bin/perl -w
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: vacation.pm,v 1.3 2003/10/29 10:35:40 varkoly Exp $
#
# user frontend for 'vacation'
#


package vacation;
use strict;
use CGI qw(-no_xhtml);
use CGI::Carp qw(fatalsToBrowser);
use Net::IMAP;
use Net::LDAP qw( :all );
use Storable qw(thaw freeze);
use Date::Calc qw( Mktime );
use DBI;
use LDAPUtils;
use Utils;
use Display;
use ManageSieve;
use sieve;
use vars qw(@ISA $confParam);
use subs qw(exit);
# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

my $javascript=<<EOF;
  function toggleButtons(name) {
    document.form.insert_groupware.checked = document.form.use_period.checked;
  }
EOF

sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

##############################################
#display
#
sub display{
    my $this = shift;
    $this->{'filtername'}="filter.sieve";

    my $q = $this->{"cgi"};
    my $session=$this->{"session"};
    my $confParam= $session->{"confParam"};
    my $sessionID = $confParam->{'sessionID'};
    my $uid = $confParam->{'uid'};
    my $passwd = $confParam->{'passwd'};
    my $imapserver = $confParam->{"IMAPserver"};
    my $sieveport = $confParam->{"SIEVEport"};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $proxyuid = $q->param('USER');
    if( (! defined $proxyuid) || ( $proxyuid eq "") ){
	if( (! defined $confParam->{'proxyuid'}) || ( $confParam->{'proxyuid'} eq "") ){
	    $proxyuid = $uid;
	}else{
	    $proxyuid = $confParam->{'proxyuid'};
	}
    }
#    print STDERR "PROXY $proxyuid\n";

    my $filtername=$this->{"filtername"};
    $this->{'HTMLOUT'} = "" ;
    
    my $mode = $q->param('mode');
    if (! defined $mode ){
        $mode = 'display';
    }
    my $cancel = $q->param('cancel');
    if( (defined $cancel) && ($cancel ne "") ){
	$mode = "display";
    }
#    print STDERR "mode $mode \n";
    
    if ( $mode eq 'display' ){
        my $sieve="";
	my $sieve_obj = ManageSieve->new($imapserver, $sieveport);
	my ($res, $msg) = $sieve_obj->connect();
	if ($res ne "OK"){
	    $this->display_error("connection failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
	    exit;
	}
	($res, $msg) = $sieve_obj->authplain($proxyuid,$uid,$passwd);
	if ($res ne "OK"){
	    $this->display_error("authentication failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID.
			"&tab=$tab", 3);
	    exit;
	}
	($res, $msg) = $sieve_obj->getScript($filtername);
	if ($res ne "OK"){
	    $sieve="";
	}else{
	    $sieve=$msg;
#	    print STDERR $sieve."\n";
	}
	$sieve_obj->logout();
	
        my $script_ref=parse_script($sieve);
        if(defined $script_ref){
#            print STDERR "Script parsed!\n";
            my $frozen = freeze $script_ref;      #freeze data
            $frozen = unpack('H*',$frozen);   #convert to ASCII
            my %tmp = ( "sieveScript" => $frozen,
			"proxyuid" => $proxyuid);
            my $s_reply=$session->modify(\%tmp);
            if( $s_reply->code() ne "OK" ) {
                 print STDERR "VACATION: unable to store data in session\n";
            }else {
#                print STDERR "Data saved\n" ;
            }
            $s_reply=$session->load($sessionID);
            if( ! defined($s_reply->data() ) ) {
		$this->display_error("Unable to reload session data: ".$s_reply->code(),
			$confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID);
            }
            $this->display_vacation($script_ref);
        }else{
	    $this->display_msg($message->{'vac_unable_to_parse'}, $confParam->{'cgi_path'}.'/edit_sieve.pl?'.
			   "sessionID=$sessionID&tab=sieve&stab=sieve_edit",3);
	    exit;
        }
    } elsif ( $mode eq 'modify' ){
        my @vac = $this->get_vacation_from_session();
        $this->modify_vacation(\@vac);
    } elsif ( $mode eq 'comment' ) {
        my @vac = $this->get_vacation_from_session();
        $this->vacation_switch_comment(\@vac);
    } elsif ( $mode eq 'comment_redir' ) {
        my @vac = $this->get_vacation_from_session();
        $this->redir_switch_comment(\@vac);
    } elsif($mode eq 'submit'){
        $this->get_vacation_from_form();
        $this->vacation_to_script();
    } elsif( $mode eq 'submit_sure'){    
        $this->vacation_to_script();
    } elsif($mode eq 'delete'){
        $this->delete_vacation();
    }

    $this->SUPER::display($this->{'HTMLOUT'});
}
#
##############################################

##############################################
#
sub display_vacation($){
    my $this = shift;
    my $script_ref = shift;
    my @script = @{$script_ref};
    my $vac_ref = $script[0];
    my %vacation = %{$vac_ref};
    my %redirect = %{$script[1]};
    my $q = $this->{"cgi"};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $uid = $confParam->{'uid'};
    my $proxyuid = $confParam->{'proxyuid'};
    my $authproxy = 0;
    if( $uid ne $proxyuid ){
	# AUTH proxy mode
	$authproxy = 1;
    }

    my $title = $message->{'vac_disp_rule_title'};
    if( $authproxy == 1){
	$title = $message->{'vac_disp_rule_title'}." (". $message->{'user'}.": $proxyuid)";
    }

    my $HTMLOUT = ""; 
    $HTMLOUT .= $q->start_table({-class=>'AdminBorder', -cellspacing => 2, -cellpadding=>0, -width=>"100%"}).
            $q->start_Tr().$q->start_td().
                $q->start_table({-cellspacing=>0, -cellpadding=>3, -width =>"100%"});
    if( (keys %vacation) || (keys %redirect) ){
        $HTMLOUT .= $q->Tr([ $this->help_th(3, $title, 'VACATION', $sessionID) ]);
    }
    if( (keys %vacation) != 0){
        my $vac_title;
        my $text_color;
        my $bg_color;
        if ($vacation{'comment'} ){
            $text_color = '#666666';
            $bg_color = '#e9e9e9';
            $vac_title = $message->{'vac_disp_comment_title'};
        }else{
            $text_color = '#000000';
            $bg_color = '#e9e9e9';
            $vac_title = $message->{'vac_disp_vac_title'};
        }

        #convert start and end intervall from UTC to localtime
	my $ldap = Net::LDAP->new($confParam->{'LDAPserver'},
				  port => $confParam->{'LDAPport'},
				  version => 3);
	if( ! defined($ldap) ){
	    $this->display_error("Can't contact LDAP server",
				 $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
	my $ldap_res = $ldap->bind( "uid=".$confParam->{'uid'}.",".$confParam->{'baseDN'},
				    password => $confParam->{'passwd'} );
	if( $ldap_res->code != LDAP_SUCCESS ){
	    my $err = LDAPerror($ldap_res);
	    $this->display_error("bind failed: ".$err,
				 $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
	
	$ldap_res = $ldap->search( base =>  "uid=".$confParam->{'proxyuid'}.",".$confParam->{'baseDN'},
				   scope => "base",
				   filter => "objectClass=*",
				   attrs => ["susetimezone"]
				 );
	if( $ldap_res->code != LDAP_SUCCESS ){
	    my $err = LDAPerror($ldap_res);
	    $this->display_error("LDAP search failed: ".$err,
				 $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
	if( $ldap_res->count == 0 ){
	    $this->display_error("couldn't read user entry",
				 $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	    exit;
	}
	my $entry = $ldap_res->entry(0);
	my $user_timezone = $entry->get_value("susetimezone") || "";
	my $orig_timezone = $ENV{'TZ'} || "";
	if( $user_timezone ne "" ){
	    $ENV{'TZ'} = $user_timezone;
	}
	my $start_local = "";
	if($vacation{'start'} ne ""){
	    my ($y, $m, $d, $h, $min) = split /[-: ]/, $vacation{'start'};
	    my ($sly, $slm, $sld, $slh, $slmin) = gmt2local($y, $m, $d, $h, $min);
	    $start_local = $sly."-".$slm."-".$sld;
	}
        
	my $end_local = "";
	if($vacation{'end'} ne ""){
	    my ($y, $m, $d, $h, $min) = split /[-: ]/, $vacation{'end'};
	    my ($ely, $elm, $eld, $elh, $elmin) = gmt2local($y, $m, $d, $h, $min);
	    $end_local = $ely."-".$elm."-".$eld;
	}
	$ENV{'TZ'} = $orig_timezone;

        my $reason = $q->escapeHTML($vacation{'reason'});
	my $addresses = $vacation{'addresses'};
        $reason =~ s/\n/<BR>/g;
        $HTMLOUT .= 
                $q->start_Tr().
                    $q->td({-colspan=>3, -class=>'AdminSubHeader' },
                        [$vac_title]).
                $q->end_Tr();
        $HTMLOUT .= $q->start_Tr().
                    $q->td({-width=>"15%"},
			    [$q->b($message->{'vac_subject'})]);
                    if($vacation{'subject'}){
                        $HTMLOUT .= $q->td(["<font color=$text_color>". $q->escapeHTML($vacation{'subject'}) ."</font>"]);
                    }else{
                        $HTMLOUT .= $q->td(["<font color=$text_color><i>$message->{vac_subject_from_org}</i></font>"]);
                    }
                    $HTMLOUT .= $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl').
                            $q->start_td({-valign=>"middle", -align => "center", -rowspan=>"4",
                                    -width=>"15%"});
                            if( $vacation{'comment'} ){
                                $HTMLOUT .= $q->image_button(-name=>$message->{'vac_activate_rule'},
					-class=>"image",
                                        -src=>'/images/uncomment.gif',
                                        -align=>"middle",
                                        -border=>"no").
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                    $q->hidden(-name=>'mode',
                                        -value=>'comment',
                                        -override=>1);
                            }else{ 
                                $HTMLOUT .= $q->image_button(-name=>$message->{'vac_deactivate_rule'},
					-class=>"image",
                                        -src=>'/images/comment.gif',
                                        -align=>"middle",
                                        -border=>"no").
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                    $q->hidden(-name=>'mode',
                                        -value=>'comment',
                                        -override=>1);
                            }
                            $HTMLOUT .= $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override=>1).
                                $q->hidden(-name=>"sure",
                                    -value=>1,
                                    -override=>1).
                        $q->end_td().
                    $q->end_form().

                $q->end_Tr().
                $q->start_Tr().
                    $q->td({-valign=>"top"},[$q->b($message->{'vac_text'})]).
                    $q->td(["<font color=$text_color>". $reason ."</font>"]).
                $q->end_Tr().
                $q->start_Tr().
                    $q->td({-valign=>"top"},
			   [$q->b($message->{'vac_addresses'}.":")]).
                    $q->td(["<font color=$text_color> ". join("<br>", @{$addresses}) ." </font>"]).
                $q->end_Tr();
		if( $vacation{'start'} ){
		    $HTMLOUT .= $q->start_Tr();
		    $HTMLOUT .=	    $q->start_td({-width=>"15%"});
		    $HTMLOUT .=		$q->b($message->{'vac_period'});		
		    $HTMLOUT .=	    $q->end_td();
		    $HTMLOUT .=	    $q->start_td({-width=>"15%"});
		    $HTMLOUT .=		"<font color=$text_color>";
		    $HTMLOUT .=		$message->{'vac_start'}.": ".$start_local."<br>";		
		    $HTMLOUT .=		"</font>";
		    if( $vacation{'end'} ){
			$HTMLOUT .=		"<font color=$text_color>";
			$HTMLOUT .=		$message->{'vac_end'}.": ".$end_local;	    
			$HTMLOUT .=		"</font>";
		    }
		    $HTMLOUT .=	    $q->end_td();
		    $HTMLOUT .= $q->end_Tr();
		}
                $HTMLOUT .= $q->start_Tr().
                    $q->td([$q->b($message->{'vac_repeat_intervall'})]).
                    $q->td(["<font color=$text_color> $vacation{'days'} $message->{'vac_days'} <font>"]).
                $q->end_Tr();    
    }            
    if( (keys %redirect) ){
        my $title; 
        my $text_color;
        my $bg_color;
        if($redirect{'comment'}){
            $text_color = '#666666';
            $bg_color = '#e9e9e9';
            $title = $message->{'vac_disp_redirect_comment'};
        }else{
            $text_color = '#000000';
            $bg_color = '#e9e9e9';
            $title = $message->{'vac_disp_redirect_title'};
        }
        $HTMLOUT .= $q->start_Tr({-class=>'AdminSubHeader'}).
                    $q->td({-colspan=>3,-class => "AdminSubHeader"},[$title]).
                $q->end_Tr().
                $q->start_Tr({-align=>"center"}).
                    $q->start_td({-colspan=>2}).
			"<font color=$text_color>".$q->escapeHTML($redirect{'email'});
	if( $redirect{'stop'} eq "1" ){
	    $HTMLOUT .= "(".$message->{'vac_redirect_not_local'}.")";
	}
	$HTMLOUT .=     "</font>".
                    $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl').
                            $q->start_td({-valign=>"middle", -align => "center", 
                                    -width=>"15%"});
                            if( $redirect{'comment'} ){
                                $HTMLOUT .= $q->image_button(-name=>$message->{'vac_activate_rule'},
					-class=>"image",
                                        -src=>'/images/uncomment.gif',
                                        -align=>"middle",
                                        -border=>"no").
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                    $q->hidden(-name=>'mode',
                                        -value=>'comment_redir',
                                        -override=>1);
                            }else{ 
                                $HTMLOUT .= $q->image_button(-name=>$message->{'vac_deactivate_rule'},
					-class=>"image",
                                        -src=>'/images/comment.gif',
                                        -align=>"middle",
                                        -border=>"no").
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                    $q->hidden(-name=>'mode',
                                        -value=>'comment_redir',
                                        -override=>1);
                            }
                            $HTMLOUT .= $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override=>1).
                                $q->hidden(-name=>"sure",
                                    -value=>1,
                                    -override=>1).
                        $q->end_td().
                    $q->end_form().
                $q->end_Tr();    
    }if( ((keys %redirect)==0) && ( (keys %vacation) == 0) ) {
        $HTMLOUT .= $q->Tr([ $this->help_th(1, $title, 'VACATION', $sessionID) ]).
                $q->start_Tr().
                    $q->start_td({-class=>'AdminSubHeader'}).
                        "$message->{vac_no_rule_defined}".
                    $q->end_td().
                $q->end_Tr().
                $q->start_Tr().
                    $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl').
                    $q->start_td({-align=>"right", -valign=>"middle"}).
                        $q->submit(-class=>"button",-value=>$message->{"vac_create"}).
                        $q->hidden(-name=>'mode',
                                -value=>'modify',
                                -override=>1).
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                        $q->hidden(-name=>'sessionID',
                                -value=>$sessionID,
                                -override=>1).
                    $q->end_td().
                    $q->end_form().
                $q->end_Tr();
    }else{
       $HTMLOUT .= $q->start_Tr.
                $q->td({-colspan=>3 , -align=>'center'},[ 
                    $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
                ]).
            $q->end_Tr.    
            $q->start_Tr({-valign=>"middle"}).
                $q->start_td({-colspan=>3, -align => 'right'}).
                $q->start_table().
                $q->start_Tr().
                    $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl').
                            $q->start_td({-valign=>"middle"}).
                            $q->submit(-class=>"button",-value=>$message->{"vac_modify"}).
                            $q->hidden(-name=>'mode',
                                    -value=>'modify',
                                    -override=>1).
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                            $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override=>1).
                        $q->end_td().
                    $q->end_form().
                    $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl').
                        $q->start_td({-valign=>"middle"}).
                            $q->submit(-class=>"button",-value=>$message->{"vac_delete"}).
                            $q->hidden(-name=>"mode",
                                        -value=>"delete",
                                        -override=>1).
				    $q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
				    $q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                            $q->hidden(-name=>'sessionID',
                                    -value=>$sessionID,
                                    -override=>1).
                        $q->end_td().
                    $q->end_form().
           $q->end_Tr.
           $q->end_table.
           $q->end_td.
                $q->end_Tr();    
    }
    $HTMLOUT .= $q->end_table().$q->end_td().$q->end_Tr().$q->end_table;
    $this->{"HTMLOUT"} = $HTMLOUT;
}
#
#####################################

#####################################
#
sub modify_vacation($){
    my $this = shift;
    my $tmp = shift;

    my $month = [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12' ],
    my $days = [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12',
	    '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24',
    	    '25', '26', '27', '28', '29', '30', '31'];
    my @vac_rules = @{$tmp};
    my %vacation = %{$vac_rules[0]}; 
    my $session = $this->{"session"};
    my $confParam = $session->{"confParam"};
    my $sessionID = $confParam->{"sessionID"};
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $HTMLOUT ="";
    my $uid = $confParam->{'uid'};
    my $proxyuid = $confParam->{'proxyuid'};
    my $authproxy = 0;
    if( $uid ne $proxyuid ){
	# AUTH proxy mode
	$authproxy = 1;
    }
    ## Search for mail addresses on which the use can receive mail 
    ## (mail, alias, vaddress Attribute from the users LDAP entry)
    my $ldap = Net::LDAP->new($confParam->{'LDAPserver'},
			      port => $confParam->{'LDAPport'},
			      version => 3);
    if( ! defined($ldap) ){
	$this->display_error("Can't contact LDAP server",
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $ldap_res = $ldap->bind( "uid=$uid,".$confParam->{'baseDN'},
				password => $confParam->{'passwd'} );
    if( $ldap_res->code != LDAP_SUCCESS ){
	my $err = LDAPerror($ldap_res);
	$this->display_error("bind failed: ".$err,
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    ## get all Domains
    $ldap_res = $ldap->search( base => "o=DNS,".$confParam->{baseDN},
                           scope => 'sub',
                           attrs => [ 'zoneName' ],
                           filter => "(&(relativeDomainName=@)(MTALocaldomain=true))" );
    if ($ldap_res->code != 0) {
        my $text = "search failed\n".&LDAPerror($ldap_res);
        $this->display_error($text,"$confParam->{cgi_path}/menu.pl?sessionID=$sessionID");
        $ldap->unbind;
        exit;
    }

    my $maildomain = $confParam->{"mailDomain"};
    my @doms;
    push @doms, $maildomain;
    foreach my $i ($ldap_res->entries) {
        my @zones = $i->get_value('zoneName');
        foreach my $zn ( @zones ) {
            if( $zn !~ /IN-ADDR\.ARPA/i && $zn ne $confParam->{mailDomain} ) {
                push @doms, $zn;
            }
        }
    }
    @doms = sort(@doms);
    
    ## Get users email addresses
    $ldap_res = $ldap->search( base =>  "uid=$proxyuid,".$confParam->{'baseDN'},
			       scope => "base",
			       filter => "objectClass=*",
			       attrs => ["vaddress", "alias", "mail", "susetimezone"]
			     );
    if( $ldap_res->code != LDAP_SUCCESS ){
	my $err = LDAPerror($ldap_res);
	$this->display_error("LDAP search failed: ".$err,
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    if( $ldap_res->count == 0 ){
	$this->display_error("couldn't read user entry",
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $entry = $ldap_res->entry(0);
    my @vaddresses = $entry->get_value("vaddress");
    my @alias = $entry->get_value("alias");
    my $mail = $entry->get_value("mail");
    
    ## for later use
    my $user_timezone = $entry->get_value("susetimezone");

    my @addresses=();
    ## append mailDomain to each value in alias
    foreach my $alias ( @alias ){
	foreach my $domain (@doms){
	    my $address = $alias."@".$domain;
	    if(! grep /^$address$/, @addresses ){
		push @addresses, $address;
	    }
	}
    }
    foreach my $domain (@doms){
	my $address = $proxyuid."@".$domain;
	if(! grep /^$address$/, @addresses ){
	    push @addresses, $address;
	}
    }
    foreach my $vaddress ( @vaddresses ){
	if(! grep /^$vaddress$/, @addresses ){
	    push @addresses, $vaddress;
	}
    }
    # store all addresses as keys of a hash to make sure each address appears only once
    my $vacaddresses = $vacation{'addresses'};
    if( (! defined $vacaddresses) || (scalar(@{$vacaddresses}) == 0) ){
	$vacaddresses = \@addresses;
    }
    my $more_addresses ="";
    foreach my $address (@{$vacaddresses}){
	if( ! grep /^$address$/, @addresses ){
	    $more_addresses .= $address." ";
	}
    }

    ## Read the the allowed Forward domains from LDAP
    $ldap_res = $ldap->search( base => "uid=cyrus,".$confParam->{'baseDN'},
			       scope => "base",
			       filter => "objectClass=*",
			       attrs => ["allowedFwdDomain" ]
			     );
    if( $ldap_res->code != LDAP_SUCCESS ){
	my $err = LDAPerror($ldap_res);
	$this->display_error("LDAP search failed: ".$err,
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    if( $ldap_res->count == 0 ){
	$this->display_error("LDAP search failed.",
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }

    my @fwddoms = $ldap_res->entry(0)->get_value("allowedFwdDomain");
    my $redir;
    if( ! defined $fwddoms[0] ) {
	$redir = $q->textfield(-name=>"redirect",
			       -default=> (defined $vac_rules[1]->{'email'} ? $vac_rules[1]->{'email'} : ""),
			       -size=>30);
    } else {
	my ( $localpart, $dompart ) = split(/@/,$vac_rules[1]->{'email'});
	$redir = $q->textfield(-name =>'param_localpart', 
			       -default => ( defined $vac_rules[1]->{'email'} ? $localpart : ""))."\@".
				   $q->popup_menu(-name =>'param_domainpart',
						  -values => \@fwddoms,
						  -default => $dompart );
    }
    $redir .= $q->checkbox(-name => "redirect_stop",
			  -checked => ( defined $vac_rules[1]->{'stop'} ? 1 : 0),
			  -value => "1",
			  -label => $message->{'vac_dont_keep'});
    my $title = $message->{'vac_disp_rule_title'};
    if( $authproxy ){
	$title = $message->{'vac_disp_rule_title'}." (". $message->{'user'}.": $proxyuid)";
    }
    
    #convert start and end intervall from UTC to localtime
    my $use_period = 0;
    my $orig_timezone = $ENV{'TZ'} || "";
    if( $user_timezone ne "" ){
	$ENV{'TZ'} = $user_timezone;
    }
    my ($sly, $slm, $sld) = Date::Calc::Today();
    # the year is needed later on
    my $act_year = $sly;
    my $slh = 0;
    my $slmin = 0;
    if( $vacation{'start'} ne "" ){
	my ($y, $m, $d, $h, $min) = split /[-: ]/, $vacation{'start'};
	($sly, $slm, $sld, $slh, $slmin) = gmt2local($y, $m, $d, $h, $min);
	$use_period = 1;
    }
    my $start_local = $sly."-".$slm."-".$sld;
    
    my ($ely, $elm, $eld) = Date::Calc::Today();
    my $elh = 23;
    my $elmin = 59;
    if( $vacation{'end'} ne "" ){
	my($y, $m, $d, $h, $min) = split /[-: ]/, $vacation{'end'};
	($ely, $elm, $eld, $elh, $elmin) = gmt2local($y, $m, $d, $h, $min);
    }
    my $end_local = $ely."-".$elm."-".$eld;
    $ENV{'TZ'} = $orig_timezone;
    my $reason = $vacation{'reason'};
   
    $HTMLOUT .= $q->start_table({-class=>'AdminBorder', -width=>"100%", -cellspacing=>2, -cellpadding => 0}).
                $q->start_Tr().
                    $q->start_td().
		    $q->start_form(-action=>$confParam->{'cgi_path'}.'/vacation.pl', -name=>"form").
                        $q->start_table({-width=>"100%", -cellpadding=>3, -cellspacing=>0}).
                            $q->Tr([ $this->help_th(3,$title,'EDIT_VACATION',$sessionID)]).
                            $q->start_Tr().
                                $q->td({-colspan=>3, -class=>'AdminSubHeader'} , [
                                    $q->center($message->{'vac_notification'})
                                ]).
                            $q->end_Tr();
    $HTMLOUT .=             $q->start_Tr().
                                $q->td({-width=>"25%"},[
                                    $message->{'vac_subject'}
                                ]).
                                $q->td([
                                    $q->textfield(-name=>"subject",
                                                -default=> $vacation{'subject'} || "",
                                                -size=>30)
                                ]).
                                $q->td([ $q->i($message->{'vac_subject_if_empty'}) ]).
                            $q->end_Tr().
                            $q->start_Tr().
                                $q->td({-valign=>"top"},[
                                    $message->{'vac_text'}
                                ]).
                                $q->td({-colspan => 2},[
                                    $q->textarea(-name=>"reason",
                                                -default=> $reason || "",
                                                -rows=>10,
                                                -cols=>60)
                                ]). 
                            $q->end_Tr();
    $HTMLOUT .=		    $q->start_Tr();
    $HTMLOUT .=			$q->start_td({});
    #$HTMLOUT .=			    $message->{'vac_use_period'};
    $HTMLOUT .=			    " ";
    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=			$q->start_td({-colspan=>1});
    $HTMLOUT .=			    $q->checkbox(-name => "use_period",
						 -value => "1",
						 -checked => $use_period,
						 -label => $message->{'vac_use_period'},
						 -onClick => "$javascript; toggleButtons('groupware')");
    $HTMLOUT .=			$q->end_td();
# C SCHULSERVER
#    $HTMLOUT .=			$q->start_td({-colspan=>1});
#    $HTMLOUT .=			    $q->checkbox(-name => "insert_groupware",
#						 -value => "1",
#						 -checked => $use_period,
#						 -label => $message->{'vac_insert_groupware'});
#    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=	 	    $q->hidden(-name=>"insert_groupware", -value=>"0");
# E SCHULSERVER
    $HTMLOUT .=		    $q->end_Tr();
    $HTMLOUT .=		    $q->start_Tr();
    $HTMLOUT .=			$q->start_td({-valign=>"middle"});
    $HTMLOUT .=			    $message->{'vac_start'};
    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=			$q->start_td({-colspan=>2});
    $HTMLOUT .=			    $message->{'vac_year'}.": ";
    $HTMLOUT .=			    $q->popup_menu(-name => "start_year",
					      -values => [ $act_year, $act_year+1, $act_year+2 ],
					      -default => $sly);
    $HTMLOUT .=			    " ".$message->{'vac_month'}.": ";
    $HTMLOUT .=			    $q->popup_menu(-name => "start_month",
					      -values => $month,
					      -default => $slm);
    $HTMLOUT .=			    " ".$message->{'vac_day'}.": ";
    $HTMLOUT .=			    $q->popup_menu(-name => "start_day",
					      -values => $days,
					      -default => $sld);
    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=		    $q->end_Tr();
    $HTMLOUT .=		    $q->start_Tr();
    $HTMLOUT .=			$q->start_td({-valign => "middle"});
    $HTMLOUT .=			    $message->{'vac_end'};
    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=			$q->start_td({-colspan=>2});
    $HTMLOUT .=			    $message->{'vac_year'}.": ";
    $HTMLOUT .=			    $q->popup_menu(-name => "end_year",
					      -values => [ '2003', '2004' ],
					      -default => $ely);
    $HTMLOUT .=			    " ".$message->{'vac_month'}.": ";;
    $HTMLOUT .=			    $q->popup_menu(-name => "end_month",
					      -values => $month,
					      -default => $elm);
    $HTMLOUT .=			    " ".$message->{'vac_day'}.": ";
    $HTMLOUT .=			    $q->popup_menu(-name => "end_day",
					      -values => $days,
					      -default => $eld);
    $HTMLOUT .=			$q->end_td();
    $HTMLOUT .=		    $q->end_Tr();
    $HTMLOUT .=             $q->start_Tr().
                                $q->td({-valign=>"middle"},[
                                    $message->{'vac_repeat_intervall'}
                                ]).
                                $q->td({-colspan => 2},[
                                    $q->textfield(-name=>"days",
                                                -default=> (defined $vacation{'days'} ? $vacation{'days'} :"7"),
                                                -size=>2).
                                    " $message->{'vac_days'}"            
                                ]). 
                            $q->end_Tr().    
			    $q->start_Tr().
				$q->td($message->{'vac_addresses'}.":").
				$q->td(
					$q->scrolling_list(-name => "addresses",
							   -values => \@addresses,
							   -defaults => $vacaddresses,
							   -multiple => 1,
							   -size => 3
							   )
				      ).
				$q->td(
					$message->{'vac_more_addresses'}.":",$q->br().
					$q->textfield( -name => "more_addresses",
						       -default => $more_addresses,
						       -size => 30
						     )
				      ).
			    $q->end_Tr().
                            $q->start_Tr({-align=>"center"}).
                                $q->td({-colspan=>3, -class=>"AdminSubHeader"},[$message->{'vac_forwarding'}]).
                            $q->end_Tr().
                            $q->start_Tr({-align=>"center"}).
                                $q->td({-colspan=>3},['<font color="#ff0000">'.$message->{'vac_forward_warn'}.'</font>']).
                            $q->end_Tr().
                            $q->start_Tr().
                                $q->td({-width=>"25%", -valign=>"middle"},[
                                    $message->{'vac_forward_to'}
                                ]).
			    $q->td({-colspan => 2},[
					   $redir
                                ]).
                            $q->end_Tr().
                            $q->start_Tr.
                                 $q->td({-colspan=>3 , -align=>'center'},[ 
                                     $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
                                 ]).
                            $q->end_Tr.    
                            $q->start_Tr().
					$q->td({-valign=>"middle", -align=>"right", -colspan => 3},[
					    $q->submit(-class=>"button",-name => "cancel",
						       -value => $message->{'vac_cancel'})." ".
					    $q->submit(-class=>"button",-value=>$message->{'vac_save'}).
					    $q->hidden(-name=>"mode",
						    -value=>"submit",
						    -override=>1).
					    $q->hidden(-name=>"groupware_id",
						    -value=> $vacation{'groupware_id'} || 0,
						    -override=>1).
					    $q->hidden( -name => "tab",
							-value => $tab,
							-override => 1).
					    $q->hidden( -name => "stab",
							-value => $stab,
							-override => 1).
					    $q->hidden(-name=>"sessionID",
						    -value=>$sessionID,
						    -override=>1)
					]).
                            $q->end_Tr().
                        $q->end_table().    
		    $q->end_form().
                    $q->end_td().    
                $q->end_Tr().
            $q->end_table(); 
    $this->{'HTMLOUT'} = $HTMLOUT;        
}
#
#######################################

#######################################
#
sub vacation_switch_comment($){
    my $this = shift;
    my $script_ref = shift;
    my @script = @{$script_ref};
    my $vac_ref = $script[0];
    my %vacation = %{$vac_ref};
    my %redir_hash = %{$script[1]};
    if( $vacation{'comment'} ){
        $vacation{'comment'} = 0;
    }else{
        $vacation{'comment'} = 1;
    }
    $this->put_vacation_to_session(\%vacation, \%redir_hash);
    $this->vacation_to_script();
}    
#
#######################################

#######################################
#
sub redir_switch_comment($){
    my $this = shift;
    my $script_ref = shift;
    my @script = @{$script_ref};
    my $vac_ref = $script[0];
    my %vacation = %{$vac_ref};
    my %redir_hash = %{$script[1]};
    if( $redir_hash{'comment'} ){
        $redir_hash{'comment'} = 0;
    }else{
        $redir_hash{'comment'} = 1;
    }
    $this->put_vacation_to_session(\%vacation, \%redir_hash);
    $this->vacation_to_script();
}    
#
#######################################
#######################################
#
sub get_vacation_from_form{
    my $this = shift;
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $confParam = $this->{'session'}->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($this->{'session'}->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $end = $q->param('end');
    my $days = $q->param('days');
    my $subject = $q->param('subject');
    my $reason = $q->param('reason');
    my @addresses = $q->param('addresses');
    my $more_addresses = $q->param('more_addresses');
    my @more_addresses = split / /, $more_addresses;
    
    my $ldap = Net::LDAP->new($confParam->{'LDAPserver'},
			      port => $confParam->{'LDAPport'},
			      version => 3);
    if( ! defined($ldap) ){
	$this->display_error("Can't contact LDAP server",
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $ldap_res = $ldap->bind( "uid=".$confParam->{'uid'}.",".$confParam->{'baseDN'},
				password => $confParam->{'passwd'} );
    if( $ldap_res->code != LDAP_SUCCESS ){
	my $err = LDAPerror($ldap_res);
	$this->display_error("bind failed: ".$err,
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    
    $ldap_res = $ldap->search( base =>  "uid=".$confParam->{'proxyuid'}.",".$confParam->{'baseDN'},
			       scope => "base",
			       filter => "objectClass=*",
			       attrs => ["mail", "susetimezone"]
			     );
    if( $ldap_res->code != LDAP_SUCCESS ){
	my $err = LDAPerror($ldap_res);
	$this->display_error("LDAP search failed: ".$err,
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    if( $ldap_res->count == 0 ){
	$this->display_error("couldn't read user entry",
			     $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
	exit;
    }
    my $entry = $ldap_res->entry(0);
    my $mail = $entry->get_value("mail");
    my $user_timezone = $entry->get_value("susetimezone") || "";
    ### evaluate time period
    my $start_gmt="";
    my $end_gmt="";
    ##should it be used?
    my $use_period = $q->param('use_period') || "";
    my $insert_groupware = 0;
    my $groupware_id = 0;
    if( $use_period eq "1" ){
	### transform start and end from localtime to UTC 
	my $sy = $q->param('start_year');
	my $sm = $q->param('start_month');
	my $sd = $q->param('start_day');
	my $sh = 0;
	my $smin = 0;
	my $orig_timezone = $ENV{'TZ'} || "";
	if( $user_timezone ne "" ){
	    $ENV{'TZ'} = $user_timezone;
	}
	my ($ly, $lm, $ld, $lh, $lmin) = local2gmt($sy, $sm, $sd, $sh, $smin);
	$start_gmt = $ly."-".$lm."-".$ld." ".$lh.":"."$lmin";
	
	my $ey = $q->param('end_year');
	my $em = $q->param('end_month');
	my $ed = $q->param('end_day');
	my $eh = 23;
	my $emin = 59;
	($ly, $lm, $ld, $lh, $lmin) = local2gmt($ey, $em, $ed, $eh, $emin);
	$end_gmt = $ly."-".$lm."-".$ld." ".$lh.":"."$lmin";
	$ENV{'TZ'} = $orig_timezone;
	
	$insert_groupware = $q->param('insert_groupware');
	$groupware_id = $q->param('groupware_id');
    }

    my $localpart = $q->param('param_localpart');
    my $dompart   = $q->param('param_domainpart');
    my $redirect;
    if( defined $subject && ($subject =~ /[\x80-\xFF]+/)){
	$this->display_error($message->{'vac_subject_noascii'}, 
			     $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.
			     $sessionID."&mode=modify&tab=$tab&stab=$stab", 4);
	exit;
    }
#    if( ! defined $reason || $reason eq "" ) {
#	$this->display_error($message->{'vac_no_vacation_text'}, 
#			     $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.
#			     $sessionID."&mode=modify&tab=$tab&stab=$stab", 4);
#	exit;
#    }

    if( defined $dompart && $dompart ne "" ) {
	if( (! defined $localpart) || ($localpart =~ /\@/) ){
            $this->display_error($message->{'vac_redirect_no_email'}, 
				 $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.
				 $sessionID."&mode=modify&tab=$tab&stab=$stab", 4);
	    exit;
	}
	if($localpart ne ""){
	    $redirect = $localpart."\@".$dompart;
	}else{
	    $redirect ="";
	}
    } else {
	$redirect = $q->param('redirect');
    }
    my $redirect_stop = $q->param('redirect_stop');
    
    
    # make sure that the mail-Attribute from LDAP appears first in the :addresses parameter
    my $addresses_sort = [];
    if( grep /^$mail$/, @addresses ){
	push @{$addresses_sort}, ($mail);
	foreach my $i (@addresses){
	    if( $i ne $mail ){
		push @{$addresses_sort}, ($i);
	    }
	}
    }else{
	$addresses_sort = \@addresses;
    }

    foreach my $address (@more_addresses){
	if( $address !~ /^.+@.+$/ ){
            $this->display_error($message->{'vac_more_address_invalid'}, 
                    $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.$sessionID."&mode=modify&tab=$tab&stab=$stab", 4); 
            exit;
	}
    }
    push @{$addresses_sort}, (@more_addresses);
    if( (! @{$addresses_sort}) || (scalar(@{$addresses_sort}) < 1 ) ){
            $this->display_error($message->{'vac_no_address'}, 
                    $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.$sessionID."&mode=modify&tab=$tab&stab=$stab", 4); 
            exit;
    }
    my %redir_hash;
    my %vacation;
#    print STDERR "$days, $reason, $subject \n";
    if( ($days ne "") && ($reason ne "") ){
        %vacation = ('days' => $days,
                    'subject' => $subject,
		    'addresses' => $addresses_sort,
                    'reason' => $reason,
		    'start' => $start_gmt,
		    'end' => $end_gmt,
		    'insert_groupware' => $insert_groupware,
		    'groupware_id' => $groupware_id);
    }
    if ($redirect eq ""){
#        print STDERR "1 $redirect \n";
        undef $redirect;
    }else{
        %redir_hash = ( 'email' => $redirect);
#        print STDERR "2 $redir_hash{'email'} \n";
	if(defined $redirect_stop && $redirect_stop eq "1"){
	    $redir_hash{'stop'} = 1;
	}else{
	    $redir_hash{'stop'} = 0;
	}
        if( ! ($redirect =~ /.+\@.+/)){
#            print STDERR "3 $redir_hash{'email'} \n";
            $this->put_vacation_to_session(\%vacation, \%redir_hash);        
            $this->display_error($message->{'vac_redirect_no_email'}, 
                    $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.$sessionID."&mode=modify&tab=$tab&stab=$stab", 4); 
            exit;
        }    
    }
    $this->put_vacation_to_session(\%vacation, \%redir_hash);
}
#
#######################################

#######################################
#
sub delete_vacation{
    my $this = shift;
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $sure = $q->param('sure');
    my $HTMLOUT = "";
    if( defined $sure ){
        my %vacation;
        my %redirect;
        $this->put_vacation_to_session(\%vacation, \%redirect);
        $this->vacation_to_script();
    }else{
        $HTMLOUT .= $q->start_table({-class=>'AdminBorder',-width=>"100%",-cellpadding => 0, -cellspacing=>1 }).
                    $q->start_Tr().
                        $q->start_td().
                            $q->start_table({-width=>"100%", -cellspacing=>0, -cellpadding=>5}).
                                $q->start_Tr().
				    $q->td({-class=>'AdminHeader', -colspan=>2 },
					     "<b>$message->{'vac_del_vacation'}</b>"
				    ).
                                $q->end_Tr().
                                $q->start_Tr().
                                    $q->td({-class=>'AdminSubHeaderRed', -colspan=>2},
                                            [$q->b($message->{'vac_del_vacation_confirm'})]).
                                $q->end_Tr().
                                $q->start_Tr.
                                    $q->td({-colspan=>3 , -align=>'center'},[ 
                                        $q->hr({-noshade => '1', -size => '1', -width=>'98%'}) 
                                    ]).
                                $q->end_Tr.    
                                $q->start_Tr({-valign=>"middle"}).
                                    $q->start_td({-align=>"center",-width=>"50%"}).
					$q->start_form((-action=>$confParam->{'cgi_path'}.'/vacation.pl')).
                                        $q->submit(-class=>"button",-value=>$message->{'vac_cancel'}).
                                        $q->hidden(-name=>"mode",
                                                   -value=>"display",
                                                   -override=>1).
					$q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
					$q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                        $q->hidden(-name=>"sessionID",
                                                  -value=>$sessionID,
                                                  -override=>1).
					$q->end_form().        
                                    $q->end_td().    
                                    $q->start_td({-align => "center", -width=>"50%"}).
					$q->start_form((-action=>$confParam->{'cgi_path'}.'/vacation.pl')).
                                        $q->submit(-class=>"button",-value=>$message->{'vac_ok'}).
                                        $q->hidden(-name=>"mode",
                                                -value=>"delete",
                                                -override=>1).
					$q->hidden( -name => "tab",
						-value => $tab,
						-override => 1).
					$q->hidden( -name => "stab",
						-value => $stab,
						-override => 1).
                                        $q->hidden(-name=>"sure",
                                                -value=>1,
                                                -override=>1).
                                        $q->hidden(-name=>"sessionID",
                                                -value=>$sessionID,
                                                -override=>1).
					$q->end_form().        
                                    $q->end_td().    
                                $q->end_Tr().
                            $q->end_table().
                        $q->end_td().
                    $q->end_Tr().
                $q->end_table();    
    }
    $this->{'HTMLOUT'}=$HTMLOUT;
}
#
#######################################

#######################################
#
sub get_vacation_from_session(){
    my $this = shift;
    my $confParam = $this->{'session'}->{'confParam'};
    my $frozen = $confParam->{"sieveScript"};
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);     # melt down... :-)
   
    my @script = @{$script_ref};
    my %vac_rule = %{$script[0]};
    my $redirect = $script[1];
    my @vac_rules = (\%vac_rule, $redirect);
    
    return @vac_rules;            
}
#
########################################

########################################
#
sub put_vacation_to_session(){
    my $this = shift;
    my $vac_rule = shift;
    my $redirect = shift;

    my $confParam = $this->{'session'}->{'confParam'};
    my $frozen = $confParam->{"sieveScript"};
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);     # melt down... :-)

    $$script_ref[0] = $vac_rule;
    $$script_ref[1] = $redirect;

    $frozen = freeze $script_ref;      #freeze data
    $frozen = unpack('H*',$frozen);   #convert to ASCII
    my %tmp = ( "sieveScript" => $frozen );
    my $s_reply=$this->{'session'}->modify(\%tmp);
    if( $s_reply->code() ne "OK" ) {
         print STDERR "unable to store data in session\n";
    }else {
#        print STDERR "Data saved\n" ;
    }
}
#
########################################

########################################
#
sub vacation_to_script($){
    my $this = shift;
    my $session = $this->{'session'};
    $session->load($session->{'confParam'}->{'sessionID'});
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $imapserver = $confParam->{'IMAPserver'};
    my $sieveport = $confParam->{'SIEVEport'};
    my $uid = $confParam->{'uid'};
    my $passwd = $confParam->{'passwd'};
    my $q = $this->{'cgi'};
    my $tab = $q->param('tab');
    my $stab = $q->param('stab');
    my $frozen = $confParam->{"sieveScript"};
    my $proxyuid = $confParam->{"proxyuid"};
#    print STDERR "PROXY $proxyuid\n";
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    $frozen = pack('H*',$frozen); # convert back to binary
    my $script_ref = thaw($frozen);
    my @script = @{$script_ref};
    my $filtername = $this->{'filtername'};
    my $HTMLOUT = "";

    # insert Date into Groupware (if requested)
    my $dbh = undef;
    my $db = "comfire";
    my $DBuser = "comfire";
    my $DBpasswd = "";
    my $DBhost = $confParam->{'GroupwareDBserver'} || "localhost";
    my $DBport = $confParam->{'GroupwareDBport'} || "5432";
    my $vacation = $script[0];
    my $seq = 0;
    if($vacation->{'insert_groupware'}){
        $dbh = DBI->connect("dbi:Pg:dbname=$db;host=$DBhost;port=$DBport", 
    	    "$DBuser", "$DBpasswd");  
        if( ! $dbh ){  
    	    my $errs .= $dbh->errstr;  
#    	    print STDERR "Database Connection failed: $errs"; 
    	    return ( -1, "Database Connection failed: $errs");  
        }
        $dbh->{"AutoCommit"} = 0;
	my %start = ();
	my %end = ();
	($start{"year"}, $start{"month"}, 
	    $start{"day"}, $start{"hour"}, 
	    $start{"min"}) = split /[-: ]/, $vacation->{'start'};
	($end{"year"}, $end{"month"}, 
	    $end{"day"}, $end{"hour"}, 
	    $end{"min"}) = split /[-: ]/, $vacation->{'end'};
	my $db_id = $vacation->{'groupware_id'};
	my $msg="";
	($seq,$msg) = $this->addVacationToGroupware($dbh, \%start,\%end, $vacation->{'reason'}, $db_id);
	if($seq <= 0){
	    $this->display_error($message->{'vac_groupware_error'}.": $msg", 
		    $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID.
		    "&mode=display&tab=$tab&stab=$stab", 3);
	    $dbh->disconnect();
	    exit;
	}else{
	    $vacation->{'groupware_id'}=$seq;
	}
    }
    my $sieve = struct_to_script(\@script);

    # put the script to the server
    my $sieve_obj = ManageSieve->new($imapserver, $sieveport);
    my ($res, $msg) = $sieve_obj->connect();
    if ($res ne "OK"){
	$this->display_error("connection failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID, 3);
        if( $dbh ){
	   $dbh->rollback();
	   $dbh->disconnect();
        }
	exit;
    }
    ($res, $msg) = $sieve_obj->authplain($proxyuid,$uid,$passwd);
    if ($res ne "OK"){
	$this->display_error("authentication failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID.
		    "&mode=display&tab=$tab&stab=$stab", 3);
        if( $dbh ){
	   $dbh->rollback();
	   $dbh->disconnect();
        }
	exit;
    }
    ($res, $msg) = $sieve_obj->putScript($filtername, $sieve);
    if ($res ne "OK"){
	$this->display_error("Sieve put failed: $msg", $confParam->{'cgi_path'}.'/vacation.pl?sessionID='.$sessionID.
		    "&mode=display&tab=$tab&stab=$stab", 3);
        if( $dbh ){
	   $dbh->rollback();
	   $dbh->disconnect();
        }
	exit;
    }
    ($res, $msg) = $sieve_obj->setActive($filtername);
    if ($res ne "OK"){
	$this->display_error("Sieve activate failed: $msg", $confParam->{'cgi_path'}.'/menu.pl?sessionID='.$sessionID.
		    "&mode=display&tab=$tab&stab=$stab", 3);
        if( $dbh ){
	   $dbh->rollback();
	   $dbh->disconnect();
        }
	exit;
    }
    $sieve_obj->logout();
    if( $uid eq $proxyuid ){
        $this->display_msg($message->{'vac_saved_changes'},
		$confParam->{'cgi_path'}."/vacation.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
    }else{
        $this->display_msg($message->{'vac_saved_changes'},
		$confParam->{'cgi_path'}."/browse_user.pl?sessionID=$sessionID&tab=$tab&stab=$stab",1);
    }
    if( $dbh ){
	$dbh->commit();
	$dbh->disconnect();
    }
    my %tmp = ( "sieveScript" => "" ,
		"proxyuid" => "");

    my $s_rep = $session->modify(\%tmp);
    if( $s_rep->code() ne "OK" ) {
	print STDERR "unable to store data in session\n";
    }
    exit;
}
#
########################################

######################
# gmt2local
sub gmt2local{
    my ($y, $m, $d, $h, $min) = @_;
    my $local = Date::Calc::Mktime($y, $m, $d, $h, $min, 0);
    my ($dummy, $dummy, $dummy, $off_h, $off_m) = Date::Calc::Timezone($local);
    return Date::Calc::Add_Delta_DHMS($y, $m, $d, $h, $min, 0, 0, $off_h, $off_m, 0 );
}
    

######################
# local2gmt
sub local2gmt{
    my ($y, $m, $d, $h, $min) = @_;
    my $localtime = Date::Calc::Mktime($y, $m, $d, $h, $min, 0);
    return Date::Calc::Gmtime($localtime);
}

#####################
# addVacationToGroupware
#   all supplied dates are in GMT/UTC
sub addVacationToGroupware{
    my $this = shift;
    my $dbh = shift;
    my $start = shift;
    my $end = shift;
    my $description = shift;
    my $seq = shift;
    my $session=$this->{"session"};
    my $confParam= $session->{"confParam"};
    my $message = ($session->getLanguage($confParam->{LANG},"VACATION"))->data();
    my $uid = $confParam->{'uid'};
    if( (defined $confParam->{'proxyuid'}) && ( $confParam->{'proxyuid'} ne "") ){
	$uid = $confParam->{'proxyuid'};
    }
    my @dflt_timezone = parse_file("/etc/sysconfig/clock", "TIMEZONE=");
    my $default_timezone = $dflt_timezone[0];
    $default_timezone =~ s/\"//g;
    my $orig_timezone = $ENV{'TZ'} || "";
    $ENV{'TZ'} = $default_timezone;
    
    my ($y, $m, $d, $h, $min) = gmt2local($start->{'year'},$start->{'month'},
		$start->{'day'}, $start->{'hour'},$start->{'min'});
    my $start_date = $y."-".$m."-".$d." ".$h.":".$min;
    ($y, $m, $d, $h, $min) = gmt2local($end->{'year'},$end->{'month'},$end->{'day'},
		$end->{'hour'},$end->{'min'});
    my $end_date = $y."-".$m."-".$d." ".$h.":".$min;
    my $comment = $message->{'vac_comment'};
    my $run_insert = 1;
    $description = $dbh->quote($description);
    $comment = $dbh->quote($comment);
    $ENV{'TZ'} = $orig_timezone;
    # Update existing Calendar Entry if $seq is set
    if( $seq > 0){
	my $sql = "UPDATE \"prg_dates\" set timestampfield02='$end_date', 
					    timestampfield01='$start_date',
					    field01=$description
					where intfield01='$seq' AND 
					      user_right='$uid' AND
					      field04=$comment;";
        my $sth = $dbh->prepare($sql);
        if( ! $sth ){  
    	my $errs .= $dbh->errstr;
            return ( -1, "Command preparation failed: $errs");  
        }
        my $res = $sth->execute();
	if( $res != 0E0 ){ ## 0 Rows affected
	    $run_insert = 0;
	}
        if(! $res){
        	my $errs .= $dbh->errstr;
        	$dbh->rollback;
        	return ( -1, "excute failed: $errs");  
        }
    }
    if($run_insert){
        my $name = $confParam->{'fullname'};
        $seq = $dbh->selectrow_array("select nextval('serial_id')") || undef;
        my $sql = "INSERT INTO \"prg_dates\" (
    			        field10,  
    			        intfield02,
    			        intfield01,
    			        changing_date,
    			        timestampfield02,
    			        user_right,
    			        timestampfield01,
    			        changed_from,
    			        creating_date,
    			        field09,
    			        field08,
    			        created_from,
    			        field07,
    			        field06,
    			        field05,
    			        field04,
    			        field03,
    			        field02,
    			        field01,
    			        group_right
    			    ) VALUES (
    			        null,
    			        null,
    			        $seq,
    			        null,
    			        '$end_date',
    			        '$uid',
    			        '$start_date',
    			        null,
    			        now(),
    			        null,
    			        null,
    			        '$name',
    			        null,
    			        'ds|no|',
    			        'gtz',
    			        $comment,
    			        'date4',
    			        null,
    			        $description,
    			        's'
    			    );";
        my $sth = $dbh->prepare($sql);
        if( ! $sth ){  
    	my $errs .= $dbh->errstr;
            return ( -1, "Command preparation failed: $errs");  
        }
        my $res = $sth->execute();
        if(! $res){
	   my $errs .= $dbh->errstr;
	   $dbh->rollback;
	   return ( -1, "excute failed: $errs");  
        }
    }
    return ($seq,"");
}

1;
