#!/usr/bin/perl -w
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: wall.pm,v 1.3 2003/11/08 10:53:54 varkoly Exp $
#
# write mail to all

package wall;
use strict;
use POSIX 'setsid';
use URI::Escape;
use Net::LDAP;
use Net::SMTP;
use vars qw(@ISA $confParam);
use subs qw(exit);
use MIME::Words qw(:all);


# Select the correct exit function
*exit = $ENV{MOD_PERL} ? \&Apache::exit : sub { CORE::exit };
@ISA = qw(Display);

sub new {
    my $this = shift;
    my $self = new Display(@_);
    
    return bless $self, $this;
}

sub display{
    my $this = shift;
    my $cgi = $this->{'cgi'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $mode = $cgi->param('mode');
    my $message = ($session->getLanguage($confParam->{LANG},"WALL"))->data();
    if( ! defined $mode ){
        $mode = "edit";
    }
    if($confParam->{uid} ne "cyrus") {
        display_error($cgi, $message->{access_denied} ,
                      $confParam->{'cgi_path'}."/menu.pl?sessionID=$sessionID");
        exit;
    }
    if( $mode eq "edit"){
        $this->edit();
    }elsif ( $mode eq "send" ){
        $this->sendmail();
    }
}

sub edit{
    my $this = shift;
    my $cgi = $this->{'cgi'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"WALL"))->data();

    my $tab = $cgi->param("tab");
    my $stab = $cgi->param("stab");
    my $msg_body = $cgi->param("msg_body");
    if(defined $msg_body){
        $msg_body = uri_unescape($msg_body);
    }else{
        $msg_body = "";
    }
    my $subject = $cgi->param("subject");
    if(defined $subject){
        $subject = uri_unescape($subject);
    }else{
        $subject = "";
    }
    my $html = "";
    $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
    $html .= $cgi->start_form(-method=>"post", -action=>$confParam->{'cgi_path'}."/wall.pl");
    $html .= $cgi->start_Tr();
    $html .= $cgi->start_td({});
    $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
    $html .= $cgi->start_Tr();
    $html .= $this->help_th(2, $message->{'wall_title'}, 'WALL');
    $html .= $cgi->end_Tr();   
    $html .= $cgi->start_Tr();
    $html .=	$cgi->td({-colspan => 2, -class=>'AdminSubHeaderRed'},
			$message->{'wall_subtitle'});
    $html .= $cgi->end_Tr();   
    $html .= $cgi->Tr(
		      $cgi->td(
			       [$message->{'wall_subject'} ] ),
		      $cgi->td({-valign=>"top"},
			       [$cgi->textfield(-name=>"subject", -size=>45)] ),
		      );
    $html .= $cgi->Tr(
		      $cgi->td({-valign=>"top"},
			       [$message->{'wall_message'}] ),
		      $cgi->td({-valign=>"top"},
			       [$cgi->textarea(-name=>'msg_body', -rows=>20, -columns=>50, -default=>$msg_body)])
		      );
    $html .= $cgi->Tr(
		      $cgi->td({-align=>"center", -valign=>"top", -colspan => 2},
			       [$cgi->submit(-class=>"button",-name=>"submit",
					     -value=>$message->{'wall_sendmail'}).
				$cgi->hidden(-name=>"mode", -value=>"send", override=>1).
				$cgi->hidden(-name=>"tab", -value=>"$tab", override=>1).
				$cgi->hidden(-name=>"stab", -value=>"$stab", override=>1).
				$cgi->hidden(-name=>"sessionID", -value=>"$sessionID", override=>1).
				$cgi->end_form()
				] )
		      );
    $html .= $cgi->end_table();
    $html .= $cgi->end_td();
    $html .= $cgi->end_Tr();
    $html .= $cgi->end_table();
	
    $this->SUPER::display($html);
}

sub sendmail{
    my $this = shift;
    my $cgi = $this->{'cgi'};
    my $session = $this->{'session'};
    my $confParam = $session->{'confParam'};
    my $sessionID = $confParam->{'sessionID'};
    my $message = ($session->getLanguage($confParam->{LANG},"WALL"))->data();
    my $tab = $cgi->param('tab');
    my $stab = $cgi->param('stab');
    
    my $subject  = $cgi->param("subject");
    $subject = encode_mimewords($subject);
    my $msg_body = $cgi->param("msg_body");
    if( (!(defined $subject)) || ($subject eq "") ){
        $msg_body = uri_escape($msg_body, "^A-Za-z");
        $this->display_error($message->{'wall_nosubject'},
              $confParam->{cgi_path}."/wall.pl?sessionID=$sessionID&tab=$tab&stab=$stab&msg_body=$msg_body");
        exit;
    }
    if( (!(defined $msg_body)) || ($msg_body eq "") ){
        $subject = uri_escape($subject, "^A-Za-z");
        $this->display_error($message->{'wall_notext'},
              $confParam->{cgi_path}."/wall.pl?sessionID=$sessionID&tab=$tab&stab=$stab&subject=$subject");
        exit;
    }

    my $ldap= Net::LDAP->new($confParam->{'LDAPserver'}, port => $confParam->{'LDAPport'}, version => 3);
    if(!defined $ldap) {
        $this->display_error($message->{'wall_connect_errror'},
              $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
        exit;
    }

    $ldap->bind ("uid=".$confParam->{'uid'}.",".$confParam->{'baseDN'},
		 password=>$confParam->{'passwd'});

    my $exist=$ldap->search(base=>"$confParam->{baseDN}",
			    scope  => "one",
			    attrs => ['uid'],
			    filter=>"(& (objectclass=SuSESchoolAccount) (!(uid=mailadmin)) (!(uid=cyrus)) )");
    
    my @MailAddr;


    my $Message = "Subject: $subject\nTo: (all)\n\n$msg_body\n";
   
    my @entries = $exist->entries();
    foreach my $entry (@entries){
        push @MailAddr, $entry->get_value("uid");
    }
    my $i = $exist->count();


    # fork and detach mail sending process
    # see: "http://apachetoday.com/news_story.php3?ltsn=2001-02-27-001-06-NW-LF-PL" why all this
    # is done
    $SIG{'CHLD'} = 'IGNORE';
    my $pid;
    my $apache_req = Apache->request();
    if (! defined ($pid=fork())){
        $this->display_error($cgi, "couldn't fork",
              $confParam->{cgi_path}."/menu.pl?sessionID=$sessionID");
        exit;
    #parent    
    }elsif($pid){
        my $html = "";
        $html .= $cgi->start_table({-class=>'AdminBorder', -cellspacing=>2, -cellpadding=>0, -width=>"100%"});
        $html .= $cgi->start_Tr();
        $html .= $cgi->start_td();
        $html .= $cgi->start_table({-cellspacing=>0, -cellpadding=>3, -width=>"100%"});
        $html .= $cgi->Tr(
			  $this->help_th(1, $message->{'wall_title'}, "WALL")
			  );
        $html .= $cgi->Tr(
			  $cgi->td({
				    -valign=>"top"},
				   $message->{'wall_sentto'}." ".$i." ".
				   $message->{'wall_num_users'}.$cgi->br.
				   $message->{'wall_time_warn'}
				   )
			  );
    	$html .= $cgi->end_table();
    	$html .= $cgi->end_td();
    	$html .= $cgi->end_Tr();
    	$html .= $cgi->end_table();
	
        $this->SUPER::display($html);
	
        #print STDERR "Mail process started PID: $pid\n";
        exit;
    }
    #child
    chdir '/'                or die "Can't chdir to /: $!";
    close STDIN;
    close STDOUT;
    setsid or die "Can't start a new session: $!";
    my $count = 0;
    my $error = 0;
    for(my $j=0; $j < $i; $j+=50){
        #print STDERR "SENDING MAIL $j TO: ";
        my $smtp = Net::SMTP->new("localhost");
        if(! defined $smtp){
            print STDERR "Connection to mailserver failed, mail was not sent to all users\n";
            my $error="Connection to mailserver failed\n"; 
            last;
        }else{
            $smtp->mail("mailadmin");
            for( my $k=$j; ( ($k < $j+50) && ($k < $i)) ; $k++){
                my $addr = $MailAddr[$k]; 
                $smtp->to($addr);
                $count++;
            }
            $smtp->data();
            $smtp->datasend($Message);

            $smtp->dataend();
            $smtp->quit;
        }
        #print STDERR "\n";
    }
    my $txt="";
    if($error){
        $txt = "Subject: Failed to send Mail to all users\n".
                  "To: SuSE Linux School Server Administrator <mailadmin>\n\n".
                  "There was a problem with sending the email with the subject:\n".
                  "    ".$subject."\n".
                  "to all users.\n\n".
                  "the following error occured:\n".
                  "    ".$error."\n\n";
                  
    }else{
        $txt = "Subject: Mail sent to all users\n".
                  "To: SuSE Linux School Server Administrator <mailadmin>\n\n".
                  "The email with the subject:\n".
                  "    \"".$subject."\"\n".
                  "was successfully sent to ".$count." users.\n";
    }
    my $smtp = Net::SMTP->new("localhost");
    $smtp->mail("mailadmin");
    $smtp->to("mailadmin");
    $smtp->data();
    $smtp->datasend($txt);
    $smtp->dataend;
    $smtp->quit;
##    print STDERR "Mail process finished\n";
    CORE::exit;
}

