#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: openssl.pm,v 1.4 2003/10/24 08:07:54 varkoly Exp $
#
require Exporter;

package openssl;

use strict;
use Expect;

my $dir           = "/etc/ssl";
my $cert_dir      = $dir."/certs";
my $CA_collection = $dir."/CA";
my $CA_derForm    = "/srv/www/htdocs/cacert.der";
my $newcerts      = $dir."/newcerts";
my $crl_dir       = $dir."/crl";
my $openssl = "/usr/bin/openssl";
my $timeout = 10;
$Expect::Log_Stdout=0;
#$Expect::Debug=1;
#$Expect::Exp_Internal = 1;
my $warn_proc_in = "an error occurred during the processing of the input into openssl\n";

#-----------------------------------------------------------------------
sub revoke_usercert($$$) {
    my $user = shift;
    my $CApasswd = shift;
    my $ldap = shift;
    my $ret = "";

    my $confParam = $::confParam;
    my $message   = $::message;


    my $user_dn = "uid=$user,$confParam->{baseDN}";

    my @rev_cert_param = ("ca",
			  "-config $dir/openssl_imap.cnf",
			  "-revoke $cert_dir/revoke_cert.pem",
			  "-keyfile $dir/private/cakey.pem",
			  "-cert $CA_collection/usedCA.pem",
			  "2>&1");

    my @gen_crl_param = ("ca ",
			 "-config $dir/openssl_imap.cnf",
			 "-keyfile $dir/private/cakey.pem",
			 "-cert $CA_collection/usedCA.pem",
			 "-out $crl_dir/crl.pem -gencrl",
			 "2>&1");

    my $exist = $ldap->search(base=>$user_dn,
                              scope  => "base",
                              attrs => ['userCertificate'],
                              filter=> "userCertificate=*");
    if ($exist->code != 0) {
        return $message->{warn_ldap_add}." (userCertificate)".$exist->error;
    }
    if ($exist->count == 0) {
        return $message->{warn_find_serial};
    }

    my $entry = $exist->entry(0);
    my @rev_cert = $entry->get_value('userCertificate;binary');
    
    # convert from DER to PEM to revoke the certificate
    open (OUT,"|/usr/bin/openssl x509 -out $cert_dir/revoke_cert.pem -inform DER -outform PEM") or return $message->{warn_file}." revoke_cert.pem";
    print OUT @rev_cert;
    close(OUT);

    $ret =rev_cert(\@rev_cert_param, $CApasswd);

    if($ret ne "0") {
	print STDERR "return to creadel_usercert.pl\n$ret\n";
	return $ret;
    }

    $ret = gen_crl(\@gen_crl_param, $CApasswd);

    if($ret ne "0") {
	print STDERR "return to creadel_usercert.pl\n$ret\n";
	return $ret;
    }

    unlink("$cert_dir/revoke_cert.pem") or return "can't remove revoke_cert.pem";

  #LDAP
    my $msg = $ldap->modify("uid=$user, ".$confParam->{baseDN},
                            changes => [
                                        delete=> [ "userCertificate;binary" => [] ],
                                        delete=> [ userPKCS12 => [] ],
                                        delete=> [ relayClientcert => [] ]
                                        ]);
    
    if($msg->code != 0) {
        return $message->{warn_ldap_remove}." ".$msg->error;
    }

    system("/usr/sbin/rcapache reload > /dev/null");

    return "";
}

sub revoke_ServCert($) {
    my $CApasswd = shift;
    my $ret = "";

    my @rev_cert_param = ("ca",
			  "-config $dir/openssl_imap.cnf",
			  "-revoke $cert_dir/cert.pem",
			  "-keyfile $dir/private/cakey.pem",
			  "-cert $CA_collection/ownCA.pem",
			  "2>&1");

    my @gen_crl_param = ("ca",
			 "-config $dir/openssl_imap.cnf",
			 "-keyfile $dir/private/cakey.pem",
			 "-cert $CA_collection/ownCA.pem",
			 "-out $crl_dir/crl.pem -gencrl",
			 "2>&1");


    $ret =rev_cert(\@rev_cert_param, $CApasswd);

    if($ret ne "0") {
	return $ret;
    }

    $ret = gen_crl(\@gen_crl_param, $CApasswd);

    if($ret ne "0") {
	return $ret;
    } else {
	$ret = "";
    }	

    unlink("$cert_dir/cert.pem") or $ret .= "can't remove cert.pem: $!\n";
    unlink("$cert_dir/skey.pem") or $ret .= "can't remove skey.pem: $!\n";
    unlink("$CA_derForm")        or $ret .= "can't remove cacert.der: $!\n";
 
    return "$ret";
}

sub gen_crl($$) {
    my $gen_crl_param = shift;
    my $CApasswd      = shift;
    my $ERROR = "0";

    my $robot = Expect->spawn($openssl." ".join(" ", @$gen_crl_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Enter PEM pass phrase:',
			       sub {
				   my $fh = shift;
				   print $fh "$CApasswd\n";
				   exp_continue;
			       }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]',
			       sub {
				   $ERROR = "$warn_proc_in (CRL)";
			       }
			      ],
			      [
			       timeout =>
			       sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (CRL)";
			       }
			      ]
			     );
    return "$ERROR";
}

sub rev_cert($$) {
    my $rev_cert_param = shift;
    my $CApasswd       = shift;
    my $ERROR = "0";

    my $robot = Expect->spawn($openssl." ".join(" ", @$rev_cert_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    $robot->expect(2, "Enter PEM pass phrase:") || return "Don't got it...".$robot->exp_error();

    print $robot "$CApasswd\r";

    my $pattern = $robot->expect(10,"-re","^phrase is too short.*",
				 "-re","[Ee][Rr][Rr][Oo][Rr]",
				 "-re","^Revoking.*");

    #print STDERR "----------------------------------------------------------------\n";
    #print STDERR "PATTERN= <$pattern>\n";
    #print STDERR "----------------------------------------------------------------\n";
    $robot->hard_close();

    return "ERROR: ".$robot->exp_error() if ! defined $pattern;
    return "ERROR: Password/-phrase to short: ".$robot->exp_error() if $pattern == 1;
    return "ERROR: Wrong Password or already revoked: ".$robot->exp_error() if $pattern == 2;
    return "0" if $pattern == 3;

#      my $pattern = $robot->expect($timeout,
#  			      [
#  			       'Enter PEM pass phrase:',
#  			       sub {
#  				   my $fh = shift;
#  				   print $fh "$CApasswd\n";
#  				   exp_continue;
#  			       }	
#  			      ],
#                                [
#  			       '-re', '[Ee][Rr][Rr][Oo][Rr]',
#  			       sub {
#  				   $ERROR = "$warn_proc_in (REVOKE)";
#  			       }
#  			      ],
#  			      [
#  			       timeout =>
#  			       sub {
#  				   $ERROR = "TIMEOUT:$warn_proc_in (REVOKE)";	
#  			       }
#  			      ]
#  			     );
#    return "$ERROR";
}

sub create_req($$$$$$$$$$) {
    my $cn      = shift || '';
    my $passwd  = shift || '';
    my $country = shift || '';
    my $state   = shift || '';
    my $local   = shift || '';
    my $org     = shift || '';
    my $orgu    = shift || '';
    my $email   = shift || '';
    my $keyout  = shift || "$cert_dir/newreq.pem";
    my $out     = shift || "$cert_dir/newreq.pem";

    my $ERROR = "0";

    my @ureq_param = ("req",
		      "-config $dir/openssl_imap.cnf",
		      "-new",
		      "-nodes",
		      "-keyout $keyout",
		      "-out $out",
		      "-days 1000",
		      "2>&1");

    my $robot = Expect->spawn($openssl." ".join(" ", @ureq_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Country Name', sub {
				   my $fh = shift; print $fh "$country\n"; exp_continue; }	
			      ],
			      [
			       'State or Province Name', sub {
				   my $fh = shift; print $fh "$state\n"; exp_continue; }	
			      ],
			      [
			       'Locality Name', sub {
				   my $fh = shift; print $fh "$local\n"; exp_continue; }	
			      ],
			      [
			       'Organization Name', sub {
				   my $fh = shift; print $fh "$org\n"; exp_continue; }	
			      ],
			      [
			       'Organizational Unit Name', sub {
				   my $fh = shift; print $fh "$orgu\n"; exp_continue; }	
			      ],
			      [
			       'Common Name', sub {
				   my $fh = shift; print $fh "$cn\n"; exp_continue; }	
			      ],
			      [
			       'Email Address', sub {
				   my $fh = shift; print $fh "$email\n"; exp_continue; }	
			      ],
			      [
			       'A challenge password', sub {
				   my $fh = shift; print $fh "$passwd\n"; exp_continue; }	
			      ],
			      [
			       'An optional company name', sub {
				   my $fh = shift; print $fh "$org\n"; exp_continue; }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]', sub {
				   $ERROR = "$warn_proc_in (REQ)"; }
			      ],
			      [
			       timeout => sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (REQ)"; }
			      ]
			     );
    return "$ERROR";

}

sub create_ureq($$$$$$$$) {
    my $cn      = shift || '';
    my $passwd  = shift || '';
    my $country = shift || '';
    my $state   = shift || '';
    my $local   = shift || '';
    my $org     = shift || '';
    my $orgu    = shift || '';
    my $email   = shift || '';
    my $keyout  = "$cert_dir/newreq.pem";
    my $out     = "$cert_dir/newreq.pem";

    return create_req($cn, $passwd, $country, $state, $local, $org,
		      $orgu, $email, $keyout, $out);
}

sub create_sreq($$$$$$$$) {
    my $cn      = shift || '';
    my $passwd  = shift || '';
    my $country = shift || '';
    my $state   = shift || '';
    my $local   = shift || '';
    my $org     = shift || '';
    my $orgu    = shift || '';
    my $email   = shift || '';
    my $keyout  = "$cert_dir/skey.pem";
    my $out     = "$cert_dir/skey.pem";

    return create_req($cn, $passwd, $country, $state, $local, $org,
		      $orgu, $email, $keyout, $out);
}

# I SCHULSERVER
sub create_schoolserver_sreq($$$$$$$$) {
    my $cn      = shift || '';
    my $passwd  = shift || '';
    my $country = shift || '';
    my $state   = shift || '';
    my $local   = shift || '';
    my $org     = shift || '';
    my $orgu    = shift || '';
    my $email   = shift || '';
    my $keyout  = "$cert_dir/schoolserver_skey.pem";
    my $out     = "$cert_dir/schoolserver_skey.pem";

    return create_req($cn, $passwd, $country, $state, $local, $org,
		      $orgu, $email, $keyout, $out);
}
# E SCHULSERVER

sub sign_ureq($$) {
    my $extension = shift || 'client_cert';
    my $pemPasswd = shift || '';
    my $out = "$cert_dir/newcert.pem";
    my $in  = "$cert_dir/newreq.pem";
    
    return sign_req($extension, $pemPasswd, $out, $in);
}

sub sign_sreq($$) {
    my $extension = shift || 'server_cert';
    my $pemPasswd = shift || '';
    my $out = "$cert_dir/cert.pem";
    my $in  = "$cert_dir/skey.pem";

    return sign_req($extension, $pemPasswd, $out, $in);
}

# I SCHULSERVER
sub sign_schoolserver_sreq($$) {
    my $extension = shift || 'server_cert';
    my $pemPasswd = shift || '';
    my $out = "$cert_dir/schoolserver_cert.pem";
    my $in  = "$cert_dir/schoolserver_skey.pem";

    return sign_req($extension, $pemPasswd, $out, $in);
}
# E SCHULSERVER

sub sign_req($$$$) {
    my $extension = shift || 'client_cert';
    my $pemPasswd = shift || '';
    my $out       = shift || '$cert_dir/newcert.pem';
    my $in        = shift || '$cert_dir/newreq.pem';
    my $ERROR = "0";

    my @sign_param = ("ca",
		      "-config $dir/openssl_imap.cnf",
		      "-keyfile $dir/private/cakey.pem",
		      "-cert $CA_collection/usedCA.pem",
		      "-policy policy_anything",
		      "-extensions $extension",
		      "-out $out",
		      "-infiles $in",
		      "2>&1");


    my $robot = Expect->spawn($openssl." ".join(" ", @sign_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Enter PEM pass phrase:', sub {
				   my $fh = shift; print $fh "$pemPasswd\n"; exp_continue; }	
			      ],
			      [
			       'Sign the certificate?', sub {
				   my $fh = shift; print $fh "y\n"; exp_continue; }	
			      ],
			      [
			       'certificate requests certified, commit?', sub {
				   my $fh = shift; print $fh "y\n"; exp_continue; }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]', sub {
				   $ERROR = "$warn_proc_in (SIGN)"; }
			      ],
			      [
			       timeout => sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (SIGN)"; }
			      ]
			     );
    return "$ERROR";
}

sub cert2pkcs12($) {
    my $passwd = shift || '';
    my $ERROR = "0";

    my @pkcs12_param = ("pkcs12",
			"-export",
			"-inkey $cert_dir/newreq.pem",
			"-in $cert_dir/newcert.pem",
			"-out $cert_dir/newcert.p12",
			"-name 'SuSE Linux School Server Client Certificate'",
			"2>&1");

    my $robot = Expect->spawn($openssl." ".join(" ", @pkcs12_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Enter Export Password:', sub {
				   my $fh = shift; print $fh "$passwd\n"; exp_continue; }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]', sub {
				   $ERROR = "$warn_proc_in (PKCS12)"; }
			      ],
			      [
			       timeout => sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (PKCS12)"; }
			      ]
			     );
    return "$ERROR";
}

sub create_fingerprint {
    my @fingerprint_param = ("x509",
			     "-fingerprint",
			     "-in $cert_dir/newcert.pem",
			     "|head -1 |");

    open (OUT,"$openssl ".join(" ", @fingerprint_param))
      or return ("couldn't execute openssl", "");
    my $i='';
    while (<OUT>) {
        $i.=$_ or return ("$warn_proc_in (fingerprint)", "");
    }

    if ($i =~ /\d+:error:\d+:\w/) {
        return ("$warn_proc_in $i", "");
    }

    if (! defined ($i)) {
        return ("$warn_proc_in (fingerprint)", "");
    }
    $i =~ s/^.*=(.*)\s*/$1/;
    # this is no longer needed and even not possible with new pfixtls extensions
    # $i =~ s/:/_/g;
    close OUT;

    return ("0", $i);
}

sub create_selfsign_CA {
    my $cn      = shift || '';
    my $passwd  = shift || '';
    my $country = shift || '';
    my $state   = shift || '';
    my $local   = shift || '';
    my $org     = shift || '';
    my $orgu    = shift || '';
    my $email   = shift || '';

    my $ERROR = "0";

    my @CA_param= ("req",
		   "-config $dir/openssl_imap.cnf",
		   "-new",
		   "-x509",
		   "-keyout $dir/private/cakey.pem",
		   "-out $CA_collection/ownCA.pem",
		   "-days 1825", # -days 1825 => 5 years
		   "2>&1");
    my $robot = Expect->spawn($openssl." ".join(" ", @CA_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Country Name', sub {
				   my $fh = shift; print $fh "$country\n"; exp_continue; }	
			      ],
			      [
			       'State or Province Name', sub {
				   my $fh = shift; print $fh "$state\n"; exp_continue; }	
			      ],
			      [
			       'Locality Name', sub {
				   my $fh = shift; print $fh "$local\n"; exp_continue; }	
			      ],
			      [
			       'Organization Name', sub {
				   my $fh = shift; print $fh "$org\n"; exp_continue; }	
			      ],
			      [
			       'Organizational Unit Name', sub {
				   my $fh = shift; print $fh "$orgu\n"; exp_continue; }	
			      ],
			      [
			       'Common Name', sub {
				   my $fh = shift; print $fh "$cn\n"; exp_continue; }	
			      ],
			      [
			       'Email Address', sub {
				   my $fh = shift; print $fh "$email\n"; exp_continue; }	
			      ],
			      [
			       'Enter PEM pass phrase', sub {
				   my $fh = shift; print $fh "$passwd\n"; exp_continue; }	
			      ],
			      [
			       'An optional company name', sub {
				   my $fh = shift; print $fh "$org\n"; exp_continue; }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]', sub {
				   $ERROR = "$warn_proc_in (CREATE CA)"; }
			      ],
			      [
			       timeout => sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (CREATE CA)"; }
			      ]
			     );
    return "$ERROR";
}

sub del_all_Certs($$$) {
    my $ldap      = shift;
    my $confParam = shift;
    my $message   = shift;
    #my $confParam = $::confParam;
    #my $message   = $::message;

    my $text = "";


    if ( -f "$CA_collection"."/ownCA.pem") {
	unlink("$CA_collection"."/ownCA.pem");
    }
    if ( -f "$CA_collection"."/usedCA.pem") {
	unlink("$CA_collection"."/usedCA.pem");
    }
    if ( -f "$CA_derForm") {
	unlink ("$CA_derForm");
    }
    if ( -f "$dir"."/private/cakey.pem") {
	unlink("$dir"."/private/cakey.pem");
    }
    if ( -f "$crl_dir"."/crl.pem") {
	unlink("$crl_dir"."/crl.pem");
    }
    open(FOO,">$crl_dir/crl.pem");
    close(FOO);
    if ( -f "$cert_dir"."/cert.pem") {
	unlink("$cert_dir"."/cert.pem");
    }
    if ( -f "$cert_dir"."/skey.pem") {
	unlink("$cert_dir"."/skey.pem");
    }

    opendir(DIR, "$newcerts/");
    while(my $d = readdir(DIR)) {
	if($d =~ /^([A-Fa-f0-9]+\.pem)$/) { # untaint data
#	print STDERR "$1\n";
	    unlink ("$newcerts/$1");
	}
    }

    open (TMPOUT, ">$dir"."/serial");
    print TMPOUT "01\n";
    close TMPOUT;
    if ( -f "$dir"."/index.txt") {
	unlink("$dir"."/index.txt");
        open TMPOUT, ">$dir"."/index.txt";
	close TMPOUT;
    }


    # scope => one is important! dont delete userPKCS12 in Addressbooks!
    my $search=$ldap->search(base=>$confParam->{baseDN},
			     scope  => "one",
			     attrs => ['uid', 'userCertificate;binary'],
			     filter=>"(&(uid=*)(userPKCS12=*))"
			    );

    if( $search->code != 0 ) {
	print STDERR "LDAP SEARCH ERROR=".&LDAPerror($search)."\n";
    }
    for (my $i=0; $i<$search->count; $i++) {
	my $entry=$search->entry($i);
	my $user=join("",$entry->get_value("uid"));
	my $changes = [ delete=> [ userPKCS12 => [] ],
			delete=> [ relayClientcert => [] ]
		      ];
	my $cert_bin = join ("",$entry->get_value("userCertificate;binary"));	
	if( (defined $cert_bin) && ($cert_bin ne "") ) {
	    $changes = [ delete=> [ userPKCS12 => [] ],
	   		 delete=> [ relayClientcert => [] ],
			 delete => ['userCertificate;binary' => [] ]
		      ];
	}
	my $msg = $ldap->modify( "uid=$user, ".$confParam->{baseDN},
				changes => $changes);

	if($msg->code != 0) {
	    $text .= $user.": ".$message->{warn_ldap_remove}." ".$msg->error;
	}
    }

    return $text;
}

sub decrypt_Key($) {
    my $passwd = shift;
    my $ERROR = "0";

    my @dec_key_param = ("rsa",
			 "-in $cert_dir/skey.pem",
			 "-out $cert_dir/skey.pem",
			 "2>&1");

    my $robot = Expect->spawn($openssl." ".join(" ", @dec_key_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       'Enter PEM pass phrase:', sub {
				   my $fh = shift; print $fh "$passwd\n"; exp_continue; }	
			      ],
			      [
			       '-re', '[Ee][Rr][Rr][Oo][Rr]', sub {
				   $ERROR = "$warn_proc_in (DECRYPT KEY)"; }
			      ],
			      [
			       timeout => sub {
				   $ERROR = "TIMEOUT:$warn_proc_in (DECRYPT KEY)"; }
			      ]
			     );
    return "$ERROR";
}

sub verify_certificate($$) {
    my $CAfile = shift;
    my $certFile = shift;
    my $ERROR = "FAILED";

    if ($CAfile =~ /^([\w\d\-_\.,]+)$/) {
	$CAfile = $1;
    } else {
	return "FAILED";
    }
    if ($certFile =~ /^([\w\d\-_\.,]+)$/) {
	$certFile = $1;
    } else {
	return "FAILED";
    }

    my @verify_param = ("verify",
			"-CAfile $CA_collection/$CAfile",
			"-purpose sslserver",
			"-verbose",
			"$cert_dir/$certFile",
			"2>&1");

    my $robot = Expect->spawn($openssl." ".join(" ", @verify_param));
    if(!defined $robot) {
	return "couldn't execute openssl";
    }

    my $pattern = $robot->expect($timeout,
			      [
			       "$cert_dir/$certFile: OK", sub {
				   $ERROR = "OK";
				   exp_continue; }	
			      ],
			      [
			       timeout => sub {
				   $ERROR = "FAILED"; }
			      ]
			     );
    return "$ERROR";
}

sub change_cert_format($$$$) {
    my $in       = shift || '';
    my $inform   = shift || 'PEM';
    my $out      = shift || '';
    my $outform  = shift || 'DER';

    if ($in =~ /^([\w\d\-_\.,\/]+)$/) {
	$in = $1;
    } else {
	return 1;
    }
    if ($out =~ /^([\w\d\-_\.,\/]+)$/) {
	$out = $1;
    } else {
	return 1;
    }
    if ($inform =~ /^([\w]+)$/) {
	$inform = $1;
    } else {
	return 1;
    }
    if ($outform =~ /^([\w]+)$/) {
	$outform = $1;
    } else {
	return 1;
    }

    my @format_param = ("x509",
			"-in $in",
			"-inform $inform",
			"-out $out",
			"-outform $outform");

    return system($openssl." ".join(" ", @format_param));
}


@openssl::ISA = qw(Exporter);
# C SCHULSERVER
@openssl::EXPORT = qw(
		      &revoke_usercert
		      &revoke_ServCert
		      &create_req
		      &create_ureq
		      &create_sreq
		      &sign_req
		      &sign_ureq
		      &sign_sreq
		      &cert2pkcs12
		      &create_fingerprint
		      &create_selfsign_CA
		      &del_all_Certs
		      &decrypt_Key
		      &verify_certificate
		      &change_cert_format
		      &create_schoolserver_sreq
		      &sign_schoolserver_sreq
		     );
# E SCHULSERVER
$openssl::VERSION = "0.3";
	
