# 
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: sieve.pm,v 1.2 2003/05/06 11:30:44 varkoly Exp $
#
require Exporter;
package sieve;

use strict;

#######################
# parses the SIEVE script provided as a parameter and returns
# the parsed script as an array
# The array has the following structure:
# @script( 
#       \%vacation('days' => days,
#                  'subject' => subject,
#		   'addresses' => \@adresses,
#                  'reason' => reason,
#                  'comment' => 0|1,
#		   'start' => start-date (YYYY-MM-DD HH:MM)
#		   'end' => end-date (YYYY-MM-DD HH:MM)
#                 ),
#       \%redirect('email' => emailaddress,
#                  'stop' => 0|1
#                   'comment' => 0|1
#                 ),
#       \@filterules(
#               \@filterrule(
#                       \%sizerule('size' => size,
#                           'op' => operator,
#                           'unit' => unit
#                           ),
#                       \@headerules( (\%header_rule( 'type' => header|envelope
#						      'headers' => \@headers,
#						      'match' => match,
#                                                     'content' => content
#                                                     'not' => 1|0
#                                                   )
#                                  ), 
#                                  (\%header_rule ...)
#                                ),
#                       \%action('action' => action,
#                                'param' => action_param
#                               ),
#                       $conj_op(allof, anyof),
#                       $stop 1|0
#                       $comment 1|0
#                       ),(\@filterule ...)
#		    ),
#	\%spam_rule('action' => (fileinto|reject),
#                   'param' => (folder)
#                   )
# )
#       
sub parse_script($){
    my $script = shift;
    my $script_bak = $script;
    my $len = Encode::from_to($script, "utf8", "iso-8859-2");
    if(! $len){
	print STDERR "lib/sieve.pm: SIEVE-Script is not UTF8 encoded assuming iso-8859-2\n";
	$script = $script_bak;
    }
    my @parsed ; 
    my @filterrules = ();
    my %vacation = ();
    my %redirect = ();
    my %spam_rule = ();
    $/ = "\r\n";
    if(!defined $script || $script eq ""){
        @parsed = (\%vacation, \%redirect, \@filterrules, \%spam_rule);
        return \@parsed;
    }
    my @script_array=split /\r?\n/,$script;
    my $status="unknown";
    my $filter_rules_count=-1;
    my $header_rules_count=-1;
    my $groupware_id;
    foreach(@script_array){
#	print STDERR "$_\n-----------------> $status\n";
	if($status eq "redirectstop"){
            if( /^(#\s*)?stop\s*;\s*$/ ){
		$redirect{'stop'} = 1;
		$status = "unknown";
		next;
	    }else{	
		$status = "unknown";
	    }
	}
        if($status eq "unknown"){
        #read until either a vacation-, redirect- or if-statement appears
            if( /^(#\s*)?(if|vacation|redirect|stop|## GROUPWARE_ID:).*$/){
                if( $2 eq "vacation"){
                    $status = "vacation";
                }elsif( $2 eq "redirect" ){
                    $status = "redirect";
                }elsif( $2 eq "if"){
                    $status = "filterrule";
                }elsif( $2 eq "## GROUPWARE_ID:"){
		    if( /^(#\s*)?(## GROUPWARE_ID: (\d+)).*$/){
#			print STDERR "GROUPWARE ID: $3\n";
			$groupware_id = $3;
		    }
		    next;
		}
            }else{
                next;
            }    
        }
        if($status eq "vacation"){
            if( /^(#\s*)?vacation\s+:days\s+(\d+)\s+(:start\s+"([^"]*)"\s+:end\s+"([^"]*)"\s+)?:addresses\s+\[([^]]+)\]\s+(:subject\s+"((\\\\|\\"|[^"\\])*)"\s+)?text:\s*$/ ){
                $vacation{'comment'} = (defined $1 ? 1 : 0);
                $vacation{'days'} = $2;
		if( $3 ne ""){
#		    print STDERR "Got interval: $3\n";
#		    print STDERR "    Start <$4>, End <$5>\n";
		    $vacation{'start'} = $4;
		    $vacation{'end'} = $5;
		}
                my @tmp_addresses = split /,/ ,$6;
                my @addresses;
                for(my $k=0; $k < @tmp_addresses; $k++){
                    if($tmp_addresses[$k] =~ /\s*"([^"]+)"/){
                        push @addresses, $1;
                    }
                }
		if( $groupware_id){
		    $vacation{'groupware_id'} = $groupware_id;
		}
		$vacation{'addresses'} = \@addresses;
                if(defined $4){
                    $vacation{'subject'} = remove_escaping($8);
                }else{
                    $vacation{'subject'} = "";
                }    
                $status = "vac_text";
#                printf STDERR "Read vacation rule with param:\n".
#                        "   Comment= ".$vacation{'comment'}."\n".
#                        "   Days=    ".$vacation{'days'}."\n".
#                        "   Subject= ".$vacation{'subject'}."\n".
#                        "   Start= ".$vacation{'start'}."\n".
#                        "   End= ".$vacation{'end'}."\n";
                next;        
            } else {
#                printf STDERR "Couldn't parse vacation rule, ignoring it!\n";
                $status = "unknown";
                return undef;
#		next;
            }    
        }
        if($status eq "vac_text"){
        #read until a single "." is found 
        #remove the dot stuff
            if( $vacation{'comment'} == 1){
                s/^\#//;  #remove the comment sign 
            }    
            if( $_ eq "." ){
                $status = "unknown";
                $vacation{'reason'} = remove_escaping($vacation{'reason'});
                chomp($vacation{'reason'}); 
#                printf STDERR "Reason=  ".$vacation{'reason'}."ENDE REASON\n";
                next;
            }elsif ( /^\.(\.+)$/ ){
                s/^\.(\.+)$/$1/;
            }
            $vacation{'reason'} .= $_."\r\n";
        }
        if($status eq "redirect"){
            if( /^(#\s*)?redirect\s+"([^"]+)"\s*;\s*$/ ){
                $redirect{'comment'} = (defined $1 ? 1 : 0);
                $redirect{'email'} = $2;
#                printf STDERR "Read redirect rule with param:\n".
#                        "   Comment= ".$redirect{'comment'}."\n".
#                        "   Mail=    ".$redirect{'email'}."\n";
            }else{
#                printf STDERR "Couldn't parse redirect rule, ignoring it!\n";
                return undef;
            }
            $status = "redirectstop";
            next;
        }
        if($status eq "filterrule"){
#            printf STDERR "Parsing filterrule\n";
            if( /^(#\s*)?if\s*(anyof|allof)\s*\(\s*$/ ){
                $filter_rules_count++;
                $filterrules[$filter_rules_count]->[0] = {};
                $filterrules[$filter_rules_count]->[1] = [];
                $filterrules[$filter_rules_count]->[2] = {};
                $filterrules[$filter_rules_count]->[3] = $2;
                $filterrules[$filter_rules_count]->[4] = 0;
                $filterrules[$filter_rules_count]->[5] = (defined $1 ? 1 : 0);
                $status = "condition";
#                printf STDERR "Parsing Condition\n";
                next;
            }elsif( /^if\s+header\s+:is\s+\[\"X-Spam-Flag\"\]\s+\[\"YES\"\]\s*$/ ){
#                printf STDERR "SPAM Condition\n";
		$status = "spam_action";
                next;
            }else{
#                printf STDERR "Couldn't parse filterrule, ignoring it!\n";
                $status = "unknown";
                return undef;
#                next;
            }    
        }
        if($status eq "condition"){
            if( /^(#)?\s*size\s+:(over|under)\s+(\d+)(K|M)?\s*\,?\s*$/){
                $filterrules[$filter_rules_count]->[0]->{'op'} = $2; 
                $filterrules[$filter_rules_count]->[0]->{'size'} = $3; 
                if(defined $4){
                    $filterrules[$filter_rules_count]->[0]->{'unit'} = $4; 
                }else{
                    $filterrules[$filter_rules_count]->[0]->{'unit'} = "B"; 
                }
#                printf STDERR "Read sizerule:\n".
#                        "   OP  =".$filterrules[$filter_rules_count]->[0]->{'op'}."\n".
#                        "   SIZE=".$filterrules[$filter_rules_count]->[0]->{'size'}."\n". 
#                        "   UNIT=".$filterrules[$filter_rules_count]->[0]->{'unit'}."\n"; 
                next;        
            }elsif(/^(#)?\s*(not)?\s+(header|envelope)\s+:(contains|regex|is)\s+\[([^]]+)\] \["((\\\\|\\"|[^"\\])+)"\]\s*\,?\s*$/){
                $header_rules_count++;
                $filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'not'} = (defined $2 ? 1:0); 
		$filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'type'} = $3;
                $filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'match'} = $4; 
                $filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'content'} = remove_escaping($6); 
                my @tmp_headers = split /,/ ,$5;
                my @headers;
                for(my $k=0; $k < @tmp_headers; $k++){
                    if($tmp_headers[$k] =~ /\s*"([^"]+)"/){
                        push @headers, $1;
                    }
                }
                $filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'headers'} = \@headers; 
#                printf STDERR "Read Headerrule:\n".
#                    "   NOT    =".$filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'not'}."\n".
#                    "   MATCH  =".$filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'match'}."\n".
#                    "   CONTENT=".$filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'content'}."\n"; 
#                 printf STDERR  "   TYPE=".$filterrules[$filter_rules_count]->[1]->[$header_rules_count]->{'type'}."\n"; 
                next;    
            }elsif(/^#?\s*\)\s*$/){
                next;
            }elsif(/^#?\s*\{\s*$/){
                $status = "action";
                $header_rules_count=-1;
                next;
            }else{
#                printf STDERR "Couldn't parse condition, ignoring it!\n";
                undef $filterrules[$filter_rules_count];
                $filter_rules_count--; 
                $header_rules_count=-1;
                $status = "unknown";
                return undef;
#                next;
            }    
        }
        if($status eq "action"){
            if( /^(#)?\s*(fileinto|redirect)\s+"((\\\\|\\"|[^"\\])+)"\s*\;\s*$/ ){
                $filterrules[$filter_rules_count]->[2]->{'action'} = $2;
                $filterrules[$filter_rules_count]->[2]->{'param'} = $3;
#                printf STDERR "Read action command:\n".
#                    "   ACTION =".$filterrules[$filter_rules_count]->[2]->{'action'}."\n".
#                    "   PARAM  =".$filterrules[$filter_rules_count]->[2]->{'param'}."\n";
		$status = "stop";
                next;    
            }elsif( /^(#)?\s*(reject)\s+text:\s*$/ ){
                $filterrules[$filter_rules_count]->[2]->{'action'} = $2;
#                printf STDERR "Read action command:\n".
#                    "   ACTION =".$filterrules[$filter_rules_count]->[2]->{'action'}."\n";
                $status = "rej_param";    
                next;    
            }elsif( /^(#)?\s*(discard)\s*\;\s*$/ ){
                $filterrules[$filter_rules_count]->[2]->{'action'} = $2;
#                printf STDERR "Read action command:\n".
#                    "   ACTION =".$filterrules[$filter_rules_count]->[2]->{'action'}."\n".
		$status = "stop";
                next;    
            }elsif( /^(#)?\s*\;\s*$/ ){
                next;
            }else{
#                printf STDERR "Couldn't parse action command, ignoring it!\n";
                $filter_rules_count--;
                $status = "unknown";
                return undef;
#                next;
            }    
        }
        if($status eq "rej_param"){
        #read until a single "." is found 
        #remove the dot stuff
            if( $filterrules[$filter_rules_count]->[5] == 1){
                s/^\#//;  #remove the comment sign 
            }    
            if( $_ eq "." ){
                $status = "stop";
                $filterrules[$filter_rules_count]->[2]->{'param'} = 
                            remove_escaping($filterrules[$filter_rules_count]->[2]->{'param'});
                chomp($filterrules[$filter_rules_count]->[2]->{'param'}); 
#                printf STDERR "Reason=  ".$filterrules[$filter_rules_count]->[2]->{'param'}."ENDE REASON\n";
                next;
            }elsif ( /^\.(\.)+$/ ){
                s/^\.(\.+)$/$1/;
            }
            $filterrules[$filter_rules_count]->[2]->{'param'} .= $_."\r\n";
        }
	if( $status eq "stop"){
#	    print STDERR "looking for stop\n";
            if( /^(#)?\s*stop\s*\;\s*$/ ){
#		print STDERR "got it\n";
                $filterrules[$filter_rules_count]->[4] = 1;
		next;
            }elsif( /^(#)?\s*\}\s*$/ ){
                $status = "unknown";
                next;
	    }elsif( /^(#)?;?\s*$/ ){
		next;
	    }else{
		return undef;
	    }
	}
        if($status eq "spam_action"){
            if( /^\s*(fileinto)\s+"((\\\\|\\"|[^"\\])+)"\s*\;\s*$/ ){
                $spam_rule{'action'} = $1;
                $spam_rule{'param'} = $2;
		$status = "unknown";
                next;
            }elsif( /^\s*(discard)\s*\;\s*$/ ){
                $spam_rule{'action'} = $1;
		$status = "unknown";
                next;
	    }
	}
    }

    @parsed= (\%vacation, \%redirect, \@filterrules, \%spam_rule); 
    return \@parsed;
}

#######################
# converts a script array (as return by "parse_script") into the
# corresponding SIEVE-script
#
sub struct_to_script($){
#    print STDERR "sieve::struct_to_script()\n";
    my $tmp = shift;
    my @parsed = @{$tmp};
    my $sieve = "require \"fileinto\";\r\n".
            "require \"reject\";\r\n".
            "require \"vacation\";\r\n".
            "require \"vacation-period\";\r\n".
            "require \"regex\";\r\n";
    
    my %spamfilter = %{@parsed[3]};
    if( keys %spamfilter && $spamfilter{'action'} ){
	$sieve .= "if header :is [\"X-Spam-Flag\"] [\"YES\"] \r\n";
	$sieve .= "{\r\n";
	if( $spamfilter{'action'} eq "fileinto" ){
	    $sieve .= "    fileinto \"".$spamfilter{'param'}."\";\r\n";
	}else{
	    $sieve .= "    discard;\r\n";
	}
	$sieve .= "    stop;\r\n}\r\n";
    }
    
    my %vacation = %{$parsed[0]};
    if (keys %vacation && defined $vacation{'reason'} && ($vacation{'reason'} ne "") ){
        my $comment = "";
        my $reason = "";
	my $addresses = $vacation{'addresses'};
	my $address_param = ":addresses [";
	foreach my $address (@{$addresses}){
	    $address = "\"$address\"";
	}
	$address_param .= join( ', ' , @{$addresses});
	$address_param .= "]";
	my @reason_lines = split /\r?\n/, $vacation{'reason'};
	foreach my $line ( @reason_lines ){
	    $line =~ s/^(\.+)$/.$1/g ;
	    $reason .= $line."\r\n";
	}
        if ( (defined $vacation{'comment'}) && ($vacation{'comment'} == 1) ){
            $comment = "#";
            $reason =~ s/^/\#/mg; 
        }
        $reason = do_escaping($reason);
#       if ( (substr $reason,length($reason)-1,1) eq "\\"){
#            $reason .= " ";
#       }
        my $subject = do_escaping($vacation{'subject'});
#	print STDERR $vacation{'start'}." ".$vacation{'end'}."\n";
	my $groupware_id = $vacation{'groupware_id'} || 0;
#	print STDERR "groupware_id  =".$vacation{'groupware_id'}."\n";
	my $start = $vacation{'start'} || "";
	my $end = $vacation{'end'} || "";
	my $period = "";
	if( $start ne "" ){
	    $period = ":start \"$start\"";
	    if( $end ne "" ){
		$period .= " :end \"$end\"";
	    }
	}
	if($groupware_id){
	    $sieve .= "## GROUPWARE_ID: $groupware_id\r\n";
	}
        $sieve .= $comment."vacation :days $vacation{'days'} $period $address_param ".
                ($subject ? ":subject \"$subject\" " : "")."text:\r\n".$reason."\r\n$comment.\r\n$comment;\r\n"; 
    }
    my %redirect = %{$parsed[1]};
    if (keys %redirect){
        my $comment = "";
        if ( (defined $redirect{'comment'}) && ($redirect{'comment'} == 1) ){
            $comment = "#";
        }    
        $sieve .= $comment."redirect \"$redirect{'email'}\";\r\n";
	if($redirect{'stop'}){
	    $sieve .= $comment."stop;\r\n";
	}
    }

    my @filterrules = @{$parsed[2]};
    my $c = @filterrules;
#    print STDERR "COUNT $c\n";
    for(my $i=0; $i < @filterrules; $i++){
        my @act_rule=@{$filterrules[$i]};
        my $conj_op = $act_rule[3];
        my %size_rule = %{$act_rule[0]};
        my @header_rules = @{$act_rule[1]};
        if ( ( (keys %size_rule) ) || (@header_rules > 0) ){
            my $comment = (defined $act_rule[5] && $act_rule[5]) ? "#" : "";
            $sieve .= $comment."if $conj_op (\r\n";
            my %size_rule = %{$act_rule[0]};
            my @header_rules = @{$act_rule[1]};
            my $c = @header_rules;
#            print STDERR " RULES              $c\n";
            if( (keys %size_rule) ){
                $sieve .= $comment."    size :$size_rule{'op'} $size_rule{'size'}". 
                        ( ($size_rule{'unit'} eq "B")? "" : $size_rule{'unit'});
                if(@header_rules > 0){
                    $sieve .= ",";
                }
                $sieve .= "\r\n";
            }
            if( @header_rules > 0){
#                print STDERR "HEADER_RULES: @header_rules\n";
                for(my $j=0; $j < @header_rules; $j++){
                    my %cur_rule= %{$header_rules[$j]};
                    if( (defined $cur_rule{'not'}) && ($cur_rule{'not'} == 1)){
                        $sieve .= $comment."    not ".$cur_rule{'type'}." ";
                    }else{
                        $sieve .= $comment."    ".$cur_rule{'type'}." ";
                    }    
                    $sieve .= ":$cur_rule{'match'} [";
                    my @headers = @{$cur_rule{'headers'}};
                    for(my $k=0; $k < @headers; $k++){
                        $sieve .= "\"$headers[$k]\"";
                        if($k != @headers-1){
                            $sieve .= ", ";
                        }
                    }
                    my $content= do_escaping($cur_rule{'content'});
                    $sieve .= "] [\"$content\"] ";
                    if($j != @header_rules-1){
                        $sieve .= ",";
                    }
                    $sieve .= "\r\n"
                }
            }
            $sieve .= $comment.")\r\n".$comment."{\r\n"; 
            my %action = %{$act_rule[2]};
            $sieve .= $comment."    $action{'action'}";
            if($action{'action'} eq "reject"){
                my $param = "";
		my @param_lines = split /\r?\n/, $action{'param'};
		foreach my $line ( @param_lines ){
		    $line =~ s/^(\.+)$/.$1/g ;
		    $param .= $line."\r\n";
		}
                if ( (defined $act_rule[5]) && ($act_rule[5] == 1) ){
                    $param =~ s/^/\#/mg; 
                }
                $param = do_escaping($param);
                $sieve .= " text:\r\n".$param."\r\n".$comment.".\r\n".$comment.";\r\n";
            }elsif( (defined $action{'param'}) && ($action{'param'} ne "") ){
                my $param = do_escaping($action{'param'});
                if ( (substr $param,length($param)-1,1) eq "\\"){
                    $param .= " ";
                }
                $sieve .= " \"$param\" ;\r\n";
            }else{
                $sieve .= " ;\r\n";
            }    
            if( (defined $act_rule[4]) && ($act_rule[4] == 1) ){
                $sieve .= $comment."    stop;\r\n";
            }
            $sieve .= $comment."}\r\n";
        }    
    }
    $sieve .= "keep;\r\nstop;\r\n";
    Encode::from_to($sieve, "iso-8859-2", "utf8");
#    print STDERR  "end struct_to_script  $sieve\n";
    return $sieve;
}

####
# this function puts a \ in front of all " and \
### ATTENTION: The cyrus sieve implementation cannot parse the sequence \\" (escaped backslash at end of string)
### so if the string ends with \\ a addtional space will be added \\ "
sub do_escaping($){
    my $source = shift;
    $source =~ s/\\/\\\\/g;
    $source =~ s/"/\\"/g;
    if ( (substr $source,length($source)-1,1) eq "\\"){
        $source .= " ";
    }
    return $source;
}

sub remove_escaping($){
    my $source = shift;
    $source =~ s/\\"/"/g;
    $source =~ s/\\\\/\\/g;
    return $source;
}

@sieve::ISA = qw(Exporter);
@sieve::EXPORT = qw(
                    parse_script
                    struct_to_script
                  );

1;

