#! /bin/bash
#
# $Id: mksessiondcert.sh,v 1.4 2003/09/10 12:27:33 varkoly Exp $
#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#

test -z "$1" && {
    echo "syntax: $0 certificatename"
    exit 1
}
name=$1

openssl=/usr/bin/openssl

comment="sessiond client certificate"
C=XY
ST=unknown
L=unknown
OU="session daemon"
O="SuSE Linux School Server"
CN=$name
email=none

ckey=/etc/sessiond/certs/${name}key.pem
creq=/etc/sessiond/certs/${name}req.pem
ccert=/etc/sessiond/certs/${name}cert.pem
ccertder=/etc/sessiond/certs/${name}cert.der

test -f /etc/sessiond/cacert.pem || {
    echo "no CA found"
    exit 1
}

TMPFILE=`mktemp /tmp/mksessiondcert.sh.XXXXXX` || exit 1

myexit() {
    rm -f $TMPFILE $ckey $creq $ccert
    exit 1
}

umask 077

cat<<EOF > $TMPFILE
[ ca ]
default_ca      = CA_default

[ CA_default ]

dir             = /etc/sessiond
certs           = \$dir/certs
crl_dir         = \$dir/crl
database        = \$dir/index.txt
new_certs_dir   = \$dir/newcerts

certificate     = \$dir/cacert.pem
serial          = \$dir/serial
crl             = \$dir/crl.pem
private_key     = \$dir/private/cakey.pem
RANDFILE        = \$dir/private/.rand

x509_extensions = usr_cert

default_days    = 2000
default_md      = md5
policy          = policy_anything

[ policy_anything ]

countryName             = optional
stateOrProvinceName     = optional
localityName            = optional
organizationName        = optional
organizationalUnitName  = optional
commonName              = supplied
emailAddress            = optional

[ req ]
default_bits           = 1024
default_keyfile        = privkey.pem
distinguished_name     = req_distinguished_name
attributes             = req_attributes
x509_extensions        = v3_ca
prompt                 = no

[ req_distinguished_name ]
countryName            = $C
stateOrProvinceName    = $ST
L                      = $L
O                      = $O
OU                     = $OU
commonName             = $CN
emailAddress           = $email

[ req_attributes ]
challengePassword              = $RANDOM$RANDOM challenge password

[ server_cert ]

basicConstraints=CA:FALSE
nsCertType = server
nsComment = $comment
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid,issuer:always
subjectAltName=email:copy
issuerAltName=issuer:copy


[ client_cert ]

basicConstraints=CA:FALSE
nsCertType = client, email
nsComment = $comment
keyUsage = nonRepudiation, digitalSignature, keyEncipherment
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid,issuer:always
subjectAltName=email:copy
issuerAltName=issuer:copy

[ v3_ca ]

subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always,issuer:always
basicConstraints = CA:true
keyUsage = cRLSign, keyCertSign
nsCertType = sslCA, emailCA
subjectAltName=email:copy
issuerAltName=issuer:copy

[ usr_cert ]

basicConstraints=CA:FALSE
nsComment                 = $comment
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid,issuer:always

EOF

echo "creating certificate request..."
$openssl req -config $TMPFILE -new -nodes -keyout $ckey -out $creq || {
	echo "error creating certificate request"
	myexit
}
	    
echo "signing server certificate..."
$openssl ca -config $TMPFILE -notext -batch -out $ccert -infiles $creq || {
	echo "error signing server certificate"
	myexit
}
rm -f $TMPFILE

echo "converting to DER format..."
$openssl x509 -in $ccert -inform PEM -out $ccertder -outform DER || {
	echo "error converting to DER"
}
