#
# $Id: HotSaNIC.pm,v 1.1.1.1 2003/03/25 10:31:47 ray Exp $
#

package HotSaNIC;

use Exporter;

# our ($VERSION, @ISA, @EXPORT);

($VERSION = '$Revision: 1.1.1.1 $') =~ s/.*(\d+\.\d+).*/$1/;

@ISA = qw(Exporter);
@EXPORT = qw (
  snmp_sysbulk
  snmp_sysquery
  get_module_name
  get_config
  read_settings
  check_config
  strip_unwanted
  identify_os_type
  parse_line
  get_diagram_properties
  opt2arg
  arg2opt
  check_args
  dupe_control
  convert_units
  );

######################################################################
#
# control function to make sure that a module is running only once
#
sub dupe_control {
  my ($what,$mod,$message)=@_;
  if (!defined $mod) { $mod=get_module_name(); }
  if (!defined $message) { $message="(unknown reason)"; }
  if ($what eq "start") {
    if (-e "running.dat") {
      open FILE,"running.dat";
      $pid=<FILE>;
      close FILE;
      if ((defined $pid) && ($pid ne "") && (-e "/proc/$pid")) {
        print time," ",$mod,": process already running.\n";

	#
	# TODO: check how long it is running
	# if too long (maybe 1 hour ?) -> check process-name, kill if it's
	# a read-data process
	#

        exit 1;
        }
      }
    open FILE,">running.dat";
    print FILE $$;
    close FILE;
    }
  elsif ($what eq "stop") {
    if (-e "running.dat") { unlink "running.dat"; }
    exit 0;
    }
  elsif ($what eq "die") {
    if (-e "running.dat") { unlink "running.dat"; }
    die time." ".$mod.": ".$message."\n";
    }
  elsif ($what eq "warn") {
    print time," ",$mod," warn: ",$message,"\n";
    }
  else { print time," ",$mod," method \"",$what,"\" not supported.\n"; }
  }

######################################################################
#
# checks if all necessary arguments are given in the corosponding
# hash. If not, try to reconstruct them.
#
sub check_args {
  ($check,%ARGS)=@_;
  foreach $value (split /,/,$check) {
    if ( (!defined $ARGS{$value}) || ($ARGS{$value} eq "") || ($ARGS{$value} eq "not configured") ) {
      if ($value eq "MODNAME") { $ARGS{$value}=get_module_name(); }
      if ($value eq "DEBUGLEVEL") { $ARGS{$value}=-1; }
      if ($value eq "OSTYPE") { $ARGS{$value}=identify_os_type(); }
      if ($value eq "SNMPWALK") { $ARGS{$value}="echo >/dev/null"; dupe_control("warn",$ARGS{"MODNAME"},"SNMPWALK not configured in main settings!"); }
      if ($value eq "SNMPGET") { $ARGS{$value}="echo >/dev/null"; dupe_control("warn",$ARGS{"MODNAME"},"SNMPGET not configured in main settings!"); }
      }
    }
  return %ARGS;
  }

######################################################################
#
# converts an option-hash to cmdline argument string which can be
# passed to a module script
#
sub opt2arg {
  my $line="";
  my %opts=@_;
  foreach $var (keys(%opts)) {
    if (index("DEBUGLEVEL MODNAME OSTYPE SNMPWALK SNMPGET",$var) >= 0) {
      $line=$line." ".$var."=\"".$opts{$var}."\"";
      }
    }
  if ($opts{"DEBUGLEVEL"}>=0) { print "CLIENT-LINE: $line\n" };
  return $line;
  }

######################################################################
#
# converts cmdline arguments array to an options-hash
#
sub arg2opt {
  my %HASH;
  foreach $pair (@_) {
    ($var,$value)=split /=/, $pair;
    $HASH{$var}=$value;
    }
  if (! defined $HASH{"MODNAME"}) { $HASH{"MODNAME"}=get_module_name(); }
  if (! defined $HASH{"OSTYPE"}) { $HASH{"OSTYPE"}=`uname`; chomp $HASH{"OSTYPE"}; }
  if (! defined $HASH{"DEBUGLEVEL"}) { $HASH{"DEBUGLEVEL"}=-1; }
  if ($HASH{"DEBUGLEVEL"} >=6 ) {
    print "\n";
    foreach (keys %HASH) { printf "     %-15s %s\n",$_,$HASH{$_}; }
    }
  return %HASH;
  }

######################################################################
#
# evaluates some diagram properties from the given range
#
sub get_diagram_properties {
  my ($range)=@_;
  my ($descr,$file,$build);
  if    ($range eq "1h")     { $descr="hour";    $file=$descr; $build=0; }
  elsif ($range eq "6h")     { $descr="6 hours"; $file="6h";   $build=30; }
  elsif ($range eq "1day")   { $descr="day";     $file=$descr; $build=120; }
  elsif ($range eq "1week")  { $descr="week";    $file=$descr; $build=1000; }
  elsif ($range eq "1month") { $descr="month";   $file=$descr; $build=4300; }
  elsif ($range eq "1year")  { $descr="year";    $file=$descr; $build=52500; }
  return ($descr,$file,$build);
  }

#sub snmp_new {
#  use Net::SNMP;
#  my ($community) = @_;
#  my $response = "";
#  my $port=161;
#  my $sysDescr = '1.3.6.1.2.1.1.1.0';
#    
#  my ($session, $error) = Net::SNMP->session(
#    Hostname        => $hostname,
#    Community       => $community,
#    Port            => $port
#    );
#    
#  if(!defined($session)) {
#    printf("ERROR: %s\n", $error);
#    return 0;
#    }
#    
#  $session->timeout(1);           # Numbers of secs to wait for response
#  $session->retries(1);           # Number of retries to send
#    
#  if(!defined($response = $session->get_request($sysDescr))) {
#    $session->close;
#    return 0;
#    }
#    
#  print "sysDescr: $response->{$sysDescr}\n";
#    
#  $session->close;
#    
#  return 1;
#  }
#


######################################################################
#
# Query remote host via SNMP and store data in tempfile
# GetBulk method
#
sub snmp_sysbulk {
  my ($binpath,$host,$community,$OIDs,$nums)=@_;
  my $query="";
  my @OIDs=split / /,$OIDs;
  my @nums=split / /,$nums;
  my $OIDcount=(@OIDs);
  foreach $num (@nums) {
    foreach $OID (@OIDs) {
      $query=$query.$OID.".".$num." ";
      }
    }
  my @result=`$binpath $host $community $query`;

  $num=0;
  $count=0;
  foreach (@result) {
    chomp;
    s/.* = //; 
    $num++;
    if ($num==1) { $line=$nums[$count]; $count++ }
    $line=$line." ".$_;
    if ($num>=$OIDcount) {$num=0; push @results,$line;}
    }

  return @results;
  }


######################################################################
#
# Query remote host via SNMP and store data in tempfile
# snmpwalk method
#
sub snmp_sysquery {
  my ($binpath,$host,$community,$item,@grep)=@_;
  @result=`$binpath $host $community $item`;
  foreach $contains (@grep) {
    foreach (grep /$contains/, @result) {
      chomp;
      if (/\.([0-9]*) = /, $num=$1) {
        ($var,$value)=split / = /;
        if (defined $lines{$num}) { $lines{$num}=$lines{$num}." $value"; } else { $lines{$num}=$value; }
        }
      }
    }

  foreach $line (keys %lines) {
    push @results,$line." ".$lines{$line};
    }

  return @results;
  }

######################################################################
#
#  evaluate name of module by its subdir
#
sub get_module_name {
  my $name=uc `pwd|awk -F "-" '{print \$2}'`;
  chomp $name;
  return $name;
}


######################################################################
#
#  strip unwanted chars like " and multiple spaces at end / beginning
#  of each element and return the processed array.
#
sub strip_unwanted {
  foreach (@_) {
    $_ =~ s/ *= */=/g;    # spaces near a =
    $_ =~ s/\"//g;        # " chars
    $_ =~ s/  +/ /g;      # multiple spaces to single
    $_ =~ s/^ *//g;       # spaces at beginning
    $_ =~ s/ *$//g;       # spaces at end
    }
  return @_;
  }


######################################################################
#
# Parse a line, strip all unwanted chars, split it by "=" into
# a VAR and a VALUE part and make sure those parts are defined.
#
sub parse_line {
  my ($line)=@_;
  chomp $line;

# weed out comments (i.e. everything following a "#"
  my ($important,$comment)=split /#/,$line,2;
  if ((! defined $comment) || ( $comment eq "" )) {
    if (index($line,"#")>=0) { $comment=" "; }
    else { $comment=""; }
    }
  if ((! defined $important) || ($important eq "")) { return ("","",$comment); }

# trim the edges ;)
  ($important)=strip_unwanted($important);

# separate the variable name from its assigned value
  my ($var,$value)=split /=/,$important,2;

# avoid undefined variables
  $var="" if (! defined $var);
  $value="" if (! defined $value);

# force varnames to be upcase
  $var=uc $var;

  return ($var,$value,$comment)
}


######################################################################
#
#  evaluate settings file and check config...
#
sub get_config {
  my ($location)=@_;
  my ($var,$value,$BINPATH,$DAEMONDIR,$LOGDIR,$PIDFILE,$DTIME,$CTIME,$STIME,$WEBDIR,$ORDER,$LOGSIZE,$LOGBACKUPS);

  my @lines=read_settings($location);
  foreach (@lines) {
    ($var,$value)=parse_line($_);
    $config{$var}=$value;
    }

  # sanity check for all config variables
  #
  $config{"BINPATH"}=check_config_item("BINPATH","path","","","path to \"rrdtool\"",%config);
  $config{"DAEMONDIR"}=check_config_item("DAEMONDIR","path","","","path to \"HotSaNIC\"",%config);
  $config{"WEBDIR"}=check_config_item("WEBDIR","path","","","path to HotSaNIC's output directory",%config);
  $config{"LOGDIR"}=check_config_item("LOGDIR","path","\$DAEMONDIR/log","","path to HotSaNIC's logfiles",%config);
  $config{"PIDFILE"}=check_config_item("PIDFILE","var","\$DAEMONDIR/log/rrdtimer.pid","","path to HotSaNIC's PID file",%config);
  $config{"DTIME"}=60*check_config_item("DTIME","var","15","minutes","diagram rebuild time",%config);
  $config{"CTIME"}=3600*check_config_item("CTIME","var","12","hours","diagram conversion time",%config);
  $config{"STIME"}=check_config_item("STIME","var","120","seconds","module scan time",%config);
  $config{"LOGSIZE"}=check_config_item("LOGSIZE","var","500000","bytes","max. logfile size",%config);
  $config{"LOGBACKUPS"}=check_config_item("LOGBACKUPS","var","4","","logfiles to keep as backups",%config);
  $config{"IMAGEFORMAT"}=check_config_item("IMAGEFORMAT","var","gif","","format of generated pictures",%config);
  $config{"ORDER"}=check_config_item("ORDER","var","","","order of modules",%config);
  $config{"DEBUGLEVEL"}=check_config_item("DEBUGLEVEL","var","-1","","debug level",%config);

  return %config;
  }


######################################################################
#
# read config-file and strip comments and empty entries
#
sub read_settings {
  my ($location)=@_;
  if ( (! defined $location) || ($location eq "")) { $location="."; }
  open CONF,"$location/settings" or die "Could not open configuration-file.\nPlease make sure there is a file named \"settings\" in HotSaNIC's main directory. Probably you have to run setup.pl first.\n";
  while (<CONF>) {
    chomp;
    my ($important,$comment)=split /#/;
    if ((defined $important) && ( index($important,"=") >= 0)) {
      push @config,$important
      }
    }
  close CONF;

  return @config;
  }


######################################################################
#
# sanity check config-item
#
sub check_config_item {
  my ($name,$type,$default,$unit,$description,%confhash)=@_;

  my $DAEMONDIR=$confhash{"DAEMONDIR"};
  my $var=$confhash{$name};

  if (defined $var) { $var=~ s/\$DAEMONDIR/$DAEMONDIR/g; }

  if ($type eq "path") {
    if ((! defined $var) || ($var eq "")) {
      print $name," (",$description,") not configured.\n";
      if ($default eq "") { print "no default value given.\n"; exit 1; }
      else { print "using default value: ",$default," ",$unit,"\n"; ($var=$default)=~ s/\$DAEMONDIR/$DAEMONDIR/g; }
      }
    if ( ! -e $var) { print $name," (",$description,") does not exist.\n"; exit 1; }
    }

  if ($type eq "var") {
    if ((! defined $var) || ($var eq "")) {
      print $name," (",$description,") not configured.\n";
      if ($default eq "") { print "no default value given.\n"; exit 1; }
      else { print "using default value: ",$default," ",$unit,"\n"; $var=$default; }
      }
    }
  return $var;
  }


######################################################################
#
# identify OS type
#
sub identify_os_type {
  my $os=`uname`;
  chomp $os;
  return $os;
  }

######################################################################
#
# convert - converts GB/MB/KB into Bytes
#
sub convert_units {
  my @in = @_;
  my @out = ();

  foreach $i (@in) {
    $i =~ s/(^\d+)K/$1 * 1024/;
    $i =~ s/(^\d+)M/$1 * 1024 * 1024/;
    $i =~ s/(^\d+)G/$1 * 1024 * 1024 * 1024/;
    #print "$i\n";
    $bytes = (eval "$i");
    #print "$bytes\n";
    push @out, $bytes;
  }
  return @out;
}

1;

