#!/usr/bin/perl -w

use lib "/usr/lib/sysMonitor";
use HotSaNIC;
use RRDs;
use strict;

# read global settings
#
my $MODNAME=get_module_name();
my %CONFIG=get_config("/usr/lib/sysMonitor");

my $OUTDIR=$CONFIG{"WEBDIR"}."/".lc $MODNAME;
my $IMGFMT=lc $CONFIG{"IMAGEFORMAT"};

my (@DEVS,$STYLE,$MULT,$device,$maxin,$maxout,$name,$IOMIN,$range,$descr,$file,$build);
my $SWAPIO=1;

my $COLOR_IN_LINE="00a000";
my $COLOR_IN_AREA="c0ffc0";
my $COLOR_OUT_LINE="c00000";
my $COLOR_OUT_AREA="ffc0c0";
my $COLOR_BORDER="a0a0a0";
my $COLOR_MAXSPEED="ff00ff";
my $COLOR_WRONGDATA="ffffa0";

# read module-specific settings
#
my @lines=read_settings(".");

# evaluate settings
#
foreach (@lines) {
  my ($var,$value)=parse_line($_);
  if ($var eq "STYLE") {
    $STYLE=lc $value;
    if ($STYLE eq "bits") { $MULT=8; } else { $STYLE="bytes"; $MULT=1; }
    }
  if ($var eq "SWAPIO") {
    if (index("1 yes true",lc $value) >=0 ) { $SWAPIO=-1; }
    }
  if ($var eq "DEV") {
    push @DEVS,$value;
    }
  }

my $SWAPOI=-$SWAPIO;

# generate diagrams
#
foreach (@DEVS) {
  ($device,$maxin,$maxout)=split /,/;
  if (index ($device,"SNMP")>=0) {
    my (undef,$host,$oid)=split /:/,$device;
    ($name=$host."_".$oid) =~ s/\./_/g;
    $device=$host.":".$oid;
    }
  else { $name=$device; }

# if database exists, create images...
  if ( -e "/var/lib/sysMonitor/data-traffic/rrd/$name.rrd") {
    if ( ! -e $OUTDIR ) { mkdir $OUTDIR,0755; }
    print "creating images for $device...\n";
    if ((! defined $maxin) || ($maxin eq "") || ($maxin == 0)) {
      print "  Maximum input speed not set!\n  Please check settings-file!\n  setting speed to 100 MBit\n";
      $maxin=12500000;
      }
    if ((! defined $maxout) || ($maxout eq "") || ($maxout == 0)) {
      print "  Maximum output speed not set!\n  Please check settings-file!\n  setting speed to 100 MBit\n";
      $maxout=12500000;
      }
    $maxin*=$MULT;
    $maxout*=$MULT;
    $IOMIN=1000*$MULT;

    foreach $range ("1h","6h","1day","1week","1month","1year") {
      ($descr,$file,$build)=get_diagram_properties($range);
      if ($range eq "1h") { draw_hourly($name,$device,$STYLE,$MULT,$IOMIN,$maxin,$maxout,$IMGFMT); }
      else { draw_other($range,$name,$device,$descr,$file,$build,$STYLE,$MULT,$IOMIN,$maxin,$maxout,$IMGFMT); }
      }
    }
  else { print "  No database found\n  Please check if the daemon is running.\n";next; }
  print "\n";
  }
exit 0;

sub draw_hourly {

  my($name,$device,$STYLE,$MULT,$IOMIN,$maxin,$maxout,$IMGFMT)=@_;

  my $FILENAME="$OUTDIR/$name-$file.$IMGFMT";
  my $TEMPNAME="$OUTDIR/temp.$IMGFMT";

  my $DATE=time;
  my $DATESTRING=`date "+%a %Y-%m-%d %H:%M"`;
  chomp $DATESTRING;

# the scaring ;) definition of "wrong1":
# if TIME < now { if in+out unknown { INF else 0 } else 0 }
#   CDEF:wrong1=TIME,$DATE,LT,maxin,maxout,+,UN,INF,0,IF,0,IF \

  my ($prints,$xs,$ys)=RRDs::graph $TEMPNAME, "-i", "-b","1024", "-v","$STYLE/s",
    "-s","-3600", "-w",$CONFIG{"WIDTH"}, "-h",$CONFIG{"HEIGHT"},
    "-u",$IOMIN, "-l",(-$IOMIN), "--title", "traffic on $device - last hour ($DATESTRING)",
    "DEF:dbavgin=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:in:AVERAGE",
    "DEF:dbmaxin=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:in:MAX",
    "DEF:dbavgout=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:out:AVERAGE",
    "DEF:dbmaxout=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:out:MAX",
    "CDEF:avgin=dbavgin,$MULT,*",
    "CDEF:maxin=dbmaxin,$MULT,*",
    "CDEF:avgout=dbavgout,$MULT,*",
    "CDEF:maxout=dbmaxout,$MULT,*",
    "CDEF:wrong1=TIME,$DATE,LT,maxin,maxout,+,UN,INF,0,IF,0,IF",
    "CDEF:wrong2=0,wrong1,-",
    "CDEF:avgin_draw=avgin,$SWAPIO,*",
    "CDEF:avgout_draw=avgout,$SWAPOI,*",
    "AREA:avgin_draw#d0d0d0:",
    "LINE1:avgin_draw#$COLOR_IN_LINE:input\\g",
    "GPRINT:avgin:AVERAGE:  (avg\\:%7.2lf %s /\\g",
    "GPRINT:maxin:MAX: max\\:%7.2lf %s $STYLE/s)\\n",
    "AREA:avgout_draw#d0d0d0:",
    "LINE1:avgout_draw#$COLOR_OUT_LINE:output\\g",
    "GPRINT:avgout:AVERAGE: (avg\\:%7.2lf %s /\\g",
    "GPRINT:maxout:MAX: max\\:%7.2lf %s $STYLE/s)\\n",
    "AREA:wrong1#$COLOR_WRONGDATA:data unknown\\n",
    "AREA:wrong2#$COLOR_WRONGDATA:",
    "HRULE:$maxin#$COLOR_MAXSPEED:Maximum transmission speed without compression (in\\: $maxin / out\\: $maxout $STYLE/s)\\n",
    "HRULE:-$maxout#$COLOR_MAXSPEED",
    "HRULE:0#000000";

  system "mv $TEMPNAME $FILENAME";
  print "  hour    ",$xs,"x",$ys," ",$FILENAME,"\n";
# foreach (@$prints) { print $_,"\n"; }
  }

sub draw_other {

  my($range,$name,$device,$descr,$file,$build,$STYLE,$MULT,$IOMIN,$maxin,$maxout,$IMGFMT)=@_;

  my $FILENAME="$OUTDIR/$name-$file.$IMGFMT";
  my $TEMPNAME="$OUTDIR/temp.$IMGFMT";
  
  my $DATE=time;
  my $DATESTRING=`date "+%a %Y-%m-%d %H:%M"`;
  chomp $DATESTRING;

  my $make=0;
  if (! -e "$FILENAME") { print "creating missing $FILENAME ..."; $make++; }
  else  {
    my (undef,undef,undef,undef,undef,undef,undef,undef,undef,$mtime,undef,undef,undef) = stat( "$FILENAME" );
    if ($DATE > ($mtime+$build)) { $make++; }
    }

  if ($make>0) {
    my ($prints,$xs,$ys)=RRDs::graph $TEMPNAME, "-i", "-b","1024", "-v","$STYLE/s",
      "-s","-$range", "-w",$CONFIG{"WIDTH"}, "-h",$CONFIG{"HEIGHT"},
      "-u",$IOMIN, "-l",(-$IOMIN), "--title", "traffic on $device - last $descr ($DATESTRING)",
      "DEF:dbminin=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:in:MIN",
      "DEF:dbmaxin=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:in:MAX",
      "DEF:dbavgin=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:in:AVERAGE",
      "DEF:dbminout=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:out:MIN",
      "DEF:dbmaxout=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:out:MAX",
      "DEF:dbavgout=/var/lib/sysMonitor/data-traffic/rrd/$name.rrd:out:AVERAGE",
      "CDEF:minin=dbminin,$MULT,*",
      "CDEF:maxin=dbmaxin,$MULT,*",
      "CDEF:avgin=dbavgin,$MULT,*",
      "CDEF:minout=dbminout,$MULT,*",
      "CDEF:maxout=dbmaxout,$MULT,*",
      "CDEF:avgout=dbavgout,$MULT,*",
      "CDEF:wrong1=maxin,maxout,+,UN,INF,UNKN,IF",
      "CDEF:wrong2=0,wrong1,-",
      "CDEF:minin_draw=minin,$SWAPIO,*",
      "CDEF:maxin_draw=maxin,$SWAPIO,*",
      "CDEF:avgin_draw=avgin,$SWAPIO,*",
      "CDEF:minout_draw=minout,$SWAPOI,*",
      "CDEF:maxout_draw=maxout,$SWAPOI,*",
      "CDEF:avgout_draw=avgout,$SWAPOI,*",
      "AREA:maxin_draw#$COLOR_IN_AREA:min/max in\\g",
      "LINE1:maxin_draw#$COLOR_BORDER:\\g",
      "AREA:minin_draw#ffffff:\\g",
      "LINE1:minin_draw#$COLOR_BORDER:\\g",
      "GPRINT:minin:MIN:  (%7.2lf %s /\\g",
      "GPRINT:maxin:MAX: %7.2lf %s $STYLE/s)",
      "LINE2:avgin_draw#$COLOR_IN_LINE:avg in\\g",
      "GPRINT:avgin:MIN:  (min\\:%7.2lf %s /\\g",
      "GPRINT:avgin:AVERAGE: avg\\:%7.2lf %s /\\g",
      "GPRINT:avgin:MAX: max\\:%7.2lf %s $STYLE/s)\\n",
      "AREA:maxout_draw#$COLOR_OUT_AREA:min/max out\\g",
      "LINE1:maxout_draw#$COLOR_BORDER:\\g",
      "AREA:minout_draw#ffffff:\\g",
      "LINE1:minout_draw#$COLOR_BORDER:\\g",
      "GPRINT:minout:MIN: (%7.2lf %s /\\g",
      "GPRINT:maxout:MAX: %7.2lf %s $STYLE/s)",
      "LINE2:avgout_draw#$COLOR_OUT_LINE:avg out\\g",
      "GPRINT:avgout:MIN: (min\\:%7.2lf %s /\\g",
      "GPRINT:avgout:AVERAGE: avg\\:%7.2lf %s /\\g",
      "GPRINT:avgout:MAX: max\\:%7.2lf %s $STYLE/s)\\n",
      "AREA:wrong1#$COLOR_WRONGDATA:data unknown\\n",
      "AREA:wrong2#$COLOR_WRONGDATA:",
      "HRULE:$maxin#$COLOR_MAXSPEED:Maximum transmission speed without compression (in\\: $maxin / out\\: $maxout $STYLE/s)",
      "HRULE:-$maxout#$COLOR_MAXSPEED",
      "HRULE:0#000000";
  
    system "mv $TEMPNAME $FILENAME";
    printf "  %-7s %d"."x%d %s\n",$descr,$xs,$ys,$FILENAME;
    }
  }

