#!/usr/bin/perl -w

use lib "/usr/lib/sysMonitor";
use HotSaNIC;
use RRDs;

# get cmdline arguments (i.e. options passed by the daemon)
# and other global parameters
#
%ARGS=arg2opt(@ARGV);
$MODNAME=$ARGS{"MODNAME"};
$OSTYPE=$ARGS{"OSTYPE"};
$DEBUGLEVEL=$ARGS{"DEBUGLEVEL"};

dupe_control("start",$MODNAME,"");

# read module-specific settings
#

undef %devin;
undef %devout;
undef %maxin;
undef %maxout;
undef %sampletime;

# read config-file to check which interfaces should be sampled
# and which are the min/max settings for interfaces
open (CF,"settings");
while (<CF>) {
  ($var,$values)=parse_line($_);
  if ($var eq "DEV") {
    $values=~ s/\"//g;
    ($dev,$maxin,$maxout)=split(/,/,$values);
    if ( index($dev,"SNMP:") >= 0) {
      (undef,$host,$interface,$community,$method)=split /:/,$dev;
      if (defined $method) { if ($method eq "walk") { $SNMPWALK{$host}=1 }; }
      ($dev=$host."_".$interface) =~ s/\./_/g;
      if (defined $snmpinterfaces{$host.":".$community}) {
        $snmpinterfaces{$host.":".$community}=$snmpinterfaces{$host.":".$community}." ".$interface;
        }
      else { $snmpinterfaces{$host.":".$community}=$interface; }
      }
    $maxin{$dev}=$maxin;
    $maxout{$dev}=$maxout;
    $devin{$dev}=0;
    $devout{$dev}=0;
    $sampletime{$dev}=time;
    }
  }
close CF;

# query local interfaces

if ($OSTYPE =~ /Linux/) {
  open(IN,"/proc/net/dev");
  while(<IN>) {
    chomp;
    #if ((index($_,"Int")<0) && (index($_,"byte")<0)) {
    if ( /:/ ) {
      ($combined, $a, $d, $d, $d, $d, $d, $d, $output, $e)=split;
      ($name, $input)=split(":",$combined);
      if(!$input) { ($input, $output) = ($a, $e); }
      $devin{$name}=$input;
      $devout{$name}=$output;
      $sampletime{$name}=time;
      }
    }
  close(IN);

} else {
  dupe_control("warn",$ARGS{"MODNAME"},": Sorry, ".$OSTYPE." is not supported until you've hacked read-data.pl...");
  }

# then query SNMP hosts and check interfaces
#

if (%snmpinterfaces) {
  if ($DEBUGLEVEL >= 6) { print "SNMP defined, checking args...\n"; }
  %ARGS=check_args("SNMPWALK,SNMPGET",%ARGS);
  $SNMPGET_CMD=$ARGS{"SNMPGET"};
  $SNMPWALK_CMD=$ARGS{"SNMPWALK"};
  }
foreach $combined (sort keys %snmpinterfaces) {
  ($host,$community)=split /:/,$combined;
  $interfaces=$snmpinterfaces{$combined};
  if ( ! defined $SNMPWALK{$host} ) {
    if ( ($SNMPGET_CMD ne "") && ($SNMPGET_CMD ne "not configured") ) {
      @results=snmp_sysbulk($SNMPGET_CMD,$host,$community,".1.3.6.1.2.1.2.2.1.10 .1.3.6.1.2.1.2.2.1.16",$interfaces); }
    else { dupe_control("warn",$MODNAME,"Can't query SNMP target $host - SNMPGET not configured in global settings!"); }
    
    }
  else {
    if ( ($SNMPWALK_CMD ne "") && ($SNMPWALK_CMD ne "not configured") ) {
      @unfiltered=snmp_sysquery($SNMPWALK_CMD,$host,$community,"interfaces","ifInOctets","ifOutOctets");
      foreach $line (@unfiltered) {
# check if interface number is configured
        ($interface)=split / /,$line;
        if (" $interfaces " =~ / $interface /) { push @results,$line; }
        }
      }
    else { dupe_control("warn",$MODNAME,"Can't query SNMP target $host - SNMPGET not configured in global settings!"); }
    }
  foreach $line (@results) {
    $line =~ s/[A-Za-z]*[0-9]*: //g;
    ($interface,$input,$output)=split / /,$line;
    ($nn=$host."_$interface") =~ s/\./_/g;
    $devin{$nn}=$input;
    $devout{$nn}=$output;
    $sampletime{$name}=time;
    }
  }
    

# feed databases with samples

foreach $name (keys %maxin) {

  # build new database if needed
  #
  if (! -e "/var/lib/sysMonitor/data-traffic/rrd/$name.rrd" ) {
    
    # get max. transmission value
    #
    my $max=$maxin{$name};
    if ($max < $maxout{$name}) { $max=$maxout{$name} };
    
    # if max-transmission is not set, assume 100 MBit.
    #
    if ($max == 0) { $max=12500000; }
    system("./makerrd","$name","$maxin") }

# update database
  RRDs::update "/var/lib/sysMonitor/data-traffic/rrd/$name.rrd",$sampletime{$name}.":".$devin{$name}.":".$devout{$name}."\n";
  if ($ERROR = RRDs::error) { dupe_control("die",$ARGS{"MODNAME"},": unable to update `$name.rrd': $ERROR"); }
  }

dupe_control("stop",$ARGS{"MODNAME"},"");
