#!/usr/bin/perl
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: fetchd.pl,v 1.3 2003/09/10 12:27:33 varkoly Exp $
#

use strict;
use lib qw(/srv/www/lib/fetchd);

use FetchEntry;
use POPFetch;
use IMAPFetch;
use SMTPSender;

use Socket;
use IO::Socket::UNIX;
use IO::Handle;
use Storable qw(thaw freeze);
use Crypt::Blowfish;
use Crypt::CBC;
use Net::LDAP;
use Net::LDAP::Util qw( ldap_error_name
                        ldap_error_text) ;
use Unix::Syslog qw(:macros);  # Syslog macros
use Unix::Syslog qw(:subs);    # Syslog functions
use Config::IniFiles;
use POSIX;
use FileHandle;
use File::Temp qw/ tempfile tempdir /;
use IPC::SysV qw(&ftok IPC_STAT IPC_PRIVATE IPC_CREAT IPC_NOWAIT IPC_EXCL IPC_RMID);
use IPC::Semaphore;

use vars qw($AUTOLOAD $maxsize $debug $keep $shm_length $shm_id
	    $thread_max $sem $madmin $maildomain $fetchheader $dropundeliverable);
use subs qw(&logit &dolock &unlock);

my $cf = "/etc/imap/globals.conf";
my $socket = "/var/run/fetchd.sock";
my $pidfile = "/var/run/fetchd.pid";
my $lockdir = "/var/lock";

$thread_max = 5;
$debug = 1;
$maxsize = 10240000; # postfix default
$keep = 1;
$shm_length = 5;
$fetchheader = 1;
$dropundeliverable = 0;

my $ldaphost   = "localhost";
my $mailhost   = "localhost";
my $basedn;

if( -z $cf || ! -f $cf ) {
    logit("unable to open $cf\n");
    exit(1);
}

my $jobs = {};   # global joblist
my $loser = {};

my $alarm = 60; # in seconds
my $npri  = 10; # job priority granularity


my $ldap_reconnect_interval;
my $ldap_max_reconnect;
my $ldap;
init_config();


# read master key and create cipher
if( open(KEY, '/etc/imap/bf_key') == 0 ) {
    $ldap->unbind;
    die "can't read bf_key";
}
my $key;
sysread KEY,$key,16;
close KEY;
#my $cipher = new Crypt::Blowfish $key;

# delete old socket
unlink($socket);

# set some sighandlers
$SIG{CHLD} = 'IGNORE';
#$SIG{CHLD} = \&handlesigterm;
$SIG{ALRM} = \&handlesigalarm;
$SIG{TERM} = \&handlesigterm;
$SIG{HUP} = \&init_config;
$SIG{PIPE} = 'IGNORE';

logit("fetchd.pl: preparing to start...\n");

umask(0177);
my $fserver = new IO::Socket::UNIX(Proto  => IO::Socket::SOCK_STREAM,
				   Local => $socket,
				   Listen => SOMAXCONN,
				   Reuse => 1);
logit("can't setup server\n") unless $fserver;

my $uid = (getpwnam("wwwrun"))[2];
my $gid = getgrnam("uucp");
chown($uid, -1, $socket);

if( -e $pidfile ) {
    unlink($pidfile);
}
if( open(OUT,">$pidfile") == 0 ) {
    $ldap->unbind;
    die "unable to open $pidfile";
}

# set the effective gid and uid, see man perlvar
$)=$gid;
$>=$uid;

# initialize Shared Memory segment
my $skey = ftok("/usr/sbin/fetchd.pl",0);
if( ! defined $skey ) {
    $ldap->unbind;
    die "ftok: $!\n";
}

$shm_id = shmget($skey, $shm_length, IPC_CREAT | 0600);
if( ! defined $shm_id ) {
    $ldap->unbind;
    die "shmget: $!\n";
}
$sem = new IPC::Semaphore(IPC_PRIVATE, 1, S_IRWXU | IPC_CREAT);

shmwrite($shm_id, "0|$thread_max", 0, $shm_length);

# fetch list of jobs from LDAP Server
init_joblist();

my $pid;
if( ! $debug ) {
    if( ! defined ($pid = fork()) ) {
	print "unable to fork\n";
	do_exit(1);
    } elsif( $pid ) {
	$ldap->unbind;
	exit(0);
    }

    chdir("/");
    &setsid();
    close(STDIN);
    close(STDOUT);
    close(STDERR);
}

print OUT getpid()."\n";
close(OUT);


alarm($alarm);
logit("fetchd.pl successfully started...\n");

# ------------------ LOOOOP ------------------------------------
while( 1 ) {
    my $clnt = $fserver->accept();
    next if ! defined $clnt;
    my $cmd = <$clnt>;

    my @arg;
    while( <$clnt> ) {
	chomp($_);
	last if /\0/;
	push @arg, $_;
    }

    #evaluate command and args and call corresponding function
    eval $cmd."(\$clnt, \@arg)";
}
# ------------------ LOOOOP ------------------------------------

logit("WE SHOULD NEVER GET HERE\n");
logit("ERRNO=$!\n");

# MAIN end
1;

#---------------------------------------------------------------
# Functions
#---------------------------------------------------------------
use constant RED      => 1;
use constant YELLOW   => 2;
use constant BLUE     => 3;
use constant GREEN    => 4;
use constant MAGENTA  => 5;
use constant CYAN     => 6;

sub rebind {
    my @args = @_;

    my $ret = $ldap->bind(@args);

    # try to handle a bogous LDAP connection
    if( $ret->code != 0 ) {
	logit("rebind: error binding to LDAP Server: ".ldap_error_text($ret->code)."\n");
	logit("rebind: establishing a new connection\n");
	my $count = 0;
	my $firstattempt = 1;
	do {
	    if( $ret->code != 0 ) {
		$ldap = undef;
		do {
		    $ldap = new Net::LDAP($ldaphost, version => 3);
		    if( ! defined $ldap ) {
			logit("rebind: Unable to contact LDAP Server at $ldaphost: $!\n");
			if( $count == $ldap_max_reconnect ) {
			    do_exit(1);
			}
		    }
		    sleep($ldap_reconnect_interval) if ! $firstattempt;
		    $firstattempt = 0;
		    $count++;
		} while( ! defined $ldap );
	    }
	    $ret = $ldap->bind(@args);
	} while( $ret->code != 0 );
    }

    return $ret;
}

sub logit {
    my $msg = shift;
    my $col = shift || undef;

    my $colseq = undef;
    $colseq = "\e[31m" if defined $col && $col == RED;
    $colseq = "\e[32m" if defined $col && $col == GREEN;
    $colseq = "\e[33m" if defined $col && $col == YELLOW;
    $colseq = "\e[34m" if defined $col && $col == BLUE;
    $colseq = "\e[35m" if defined $col && $col == MAGENTA;
    $colseq = "\e[36m" if defined $col && $col == CYAN;
    
    if( $debug ) {
	my $txt;
	if( defined $colseq ) {
	    $txt = "$colseq$msg\e[0m";
	} else {
	    $txt = $msg;
	}
	print $txt;
    } else {
	syslog(LOG_INFO, $msg);
    }
}

sub error {
    my ($handle, $msg) = @_;

    $msg =~ s/\0//g;
    chomp($msg);
    $handle->write("ERROR: $msg\0\n");
}

sub okay {
    my ($handle, $msg) = @_;
    
    $msg =~ s/\0//g;
    chomp($msg);
    $handle->write("OK: $msg\0\n");
}

sub init_config {

    # read fetchd.conf
    # ----------------------------------------------------------------------------------
    my $cfg = new Config::IniFiles( -file => $cf, -reloadwarn => 1 );
    
    $debug             = $cfg->val("FETCHD", "debug");
    $keep              = ( $cfg->val("FETCHD", "keeponserver") eq "true" ) ? 1 : 0;
    $socket            = $cfg->val("FETCHD", "unixsocket");
    $ldaphost          = $cfg->val("FETCHD", "ldaphost");
    $madmin            = $cfg->val("FETCHD", "mailadmin");
    $fetchheader       = ( $cfg->val("FETCHD", "append_fetch_header") eq "true" ) ? 1 : 0;
    $dropundeliverable = ( $cfg->val("FETCHD", "drop_undeliverable_mail") eq "true" ) ? 1 : 0;

    $thread_max   = $cfg->val("FETCHD", "thread_max");
    $thread_max = 5  if ! defined $thread_max;
    $thread_max = 99 if $thread_max > 99;

    my $tmp;
    shmread($shm_id, $tmp, 0, $shm_length);
    $tmp =~ s/\0//g;
    $tmp =~ /(\d+)\|(\d+)/;
    my $cur = $1;
    shmwrite($shm_id, "$cur|$thread_max", 0, $shm_length);

    $npri         = $cfg->val("FETCHD", "priority_granularity");
    $npri = 10 if ! defined $npri;
   
    $ldap_reconnect_interval = $cfg->val("FETCHD", "ldap_reconnect_interval");
    $ldap_max_reconnect      = $cfg->val("FETCHD", "ldap_max_reconnect");
    
    $ldap_reconnect_interval = 5 if ! defined $ldap_reconnect_interval;
    $ldap_max_reconnect      = 20 if ! defined $ldap_max_reconnect;
    $madmin = "mailadmin" if ! defined $madmin;

    if( ! $debug ) {
	openlog("fetchd.pl", LOG_PID | LOG_NOTICE, LOG_DAEMON);
    }
    logit("init_config...\n");

    if( $debug ) {
	logit("fetchd.conf:
-------------------------------------------------------------
debug                   = $debug
keeponserver            = $keep
unixsocket              = $socket
ldaphost                = $ldaphost
mailadmin               = $madmin
ldap_reconnect_interval = $ldap_reconnect_interval
ldap_max_reconnect      = $ldap_max_reconnect
append_fetch_header     = $fetchheader
drop_undeliverable_mail = $dropundeliverable
thread_max              = $thread_max
priority_granularity    = $npri
-------------------------------------------------------------
");
    }
    
    # ----------------------------------------------------------------------------------

    # find out postfix message_size_limit
    # ----------------------------------------------------------------------------------
    open(IN, "/usr/sbin/postconf message_size_limit|");
    my $mstmp = <IN>;
    close(IN);
    
    if( ! defined $mstmp ) {
	logit("WARNING: unable to call postconf to detect max. message size!!!\n");
	logit("WARNING: using default of $maxsize!!!\n");
    } else {
	chomp($mstmp);
	$mstmp =~ s/message_size_limit\s+=\s+//;
	$maxsize = $mstmp;
    }
    if( $debug ) {
	logit("postfix:
-------------------------------------------------------------
message_size_limit      = $maxsize
-------------------------------------------------------------
");
    }
    # ----------------------------------------------------------------------------------

    # ----------------------------------------------------------------------------------
    # Get basedn from LDAP Server (this requires LDAPv3, this will and is not
    # designed to work with OpenLDAP < 2.x
    $maildomain = undef;

    # contact LDAP server
    $ldap = new Net::LDAP($ldaphost, version => 3);
    if( ! defined $ldap ) {
	print "Unable to contact LDAP Server at $ldaphost: $!\n";
	exit(1);
    }

    my $msg = rebind();

    if( $msg->code != 0 ) {
	die "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n";
    }

    $msg = $ldap->search( scope => "base",
			  filter=> "(objectclass=*)",
			  attrs=> [ "namingContexts" ] );
    if( $msg->count <= 0 ) {
	$ldap->unbind;
	die "Unable to query LDAProotDSE, giving up: $!\n";
    }
    
    my @attr = $msg->entry(0)->attributes;
    my @ncs = $msg->entry(0)->get_value($attr[0]);
    
    foreach my $suffix ( @ncs ) {
	if( ! ( $suffix =~ /^o=AddressBook/ ) ) {
	    $basedn = $suffix;
	}
    }
    
    if( ! defined $basedn ) {
	$ldap->unbind;
	die "Unable to determine baseDN!!!";
    }
    
    my $cbase = "uid=cyrus,".$basedn;
    $msg = $ldap->search( base => $cbase,
			  scope => "base",
			  filter=> "(uid=cyrus)",
			  attrs=> [ "smtpServer", "mailDomain" ] );
    if( $msg->count <= 0 ) {
	$ldap->unbind;
	die "Unable to search for smtpServer and mailDomain attribute: $!\n";
    }
    
    my $res = $msg->as_struct;
    $mailhost   = join("", @{$res->{$cbase}->{smtpserver}});
    push @$maildomain, join("", @{$res->{$cbase}->{maildomain}});
    
    if( ! defined $maildomain->[0] ) {
	$ldap->unbind;
	die "Unable to get the name of the Maildomain!\n";
    }
    
    if( (! defined $mailhost) || $mailhost eq "" ) {
	$ldap->unbind;
	die "Unable to get the name of the SMTP Server!\n";
    }
    
    my $dnsbase = "o=DNS,".$basedn;
    $msg = $ldap->search( base => $dnsbase,
			  scope => 'sub',
			  attrs => [ 'zoneName' ],
			  filter => "relativeDomainName=@" );
    if ($msg->count == 0) {
	die "Unable to search for DNS Domain Names: $!\n";
	$ldap->unbind;
    }
    
    foreach my $e ($msg->entries) {
        my @zones = $e->get_value('zoneName');
        foreach my $zn ( @zones ) {
	    if( $maildomain->[0] ne $zn && ! ( $zn =~ /IN-ADDR\.ARPA/i ) ) {
		push @$maildomain, $zn;
	    }
        }
    }

    if( $debug ) {
	logit("LDAP:
-------------------------------------------------------------
mailhost                = $mailhost
maildomains             =\n");
	foreach ( @$maildomain ) {
	    logit("                          $_\n");
	}
	logit("-------------------------------------------------------------\n");
    }
    # ----------------------------------------------------------------------------------
    logit("init_config: reading configuration data successfull.\n");
}

sub init_joblist {
    logit("init_joblist...\n");

    my $msg = rebind();
    
    if( $msg->code != 0 ) {
	logit("unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	do_exit();
    }


    $msg = $ldap->search( base=> $basedn,
			  filter=> "(&(uid=cyrus)(popaccount=*))",
			  attrs=> [ 'popaccount' ] );
    
    my $nrjobs = 0;
    foreach my $entry ($msg->all_entries) {
	foreach my $attr ($entry->attributes) {
	    foreach my $val ( $entry->get_value($attr) ) {
		# get popaccounts out, decrypt, unfreeze and sync with joblist
		my $cbc = new Crypt::CBC ( { 'key' => $key,
					     'cipher' => 'Blowfish' });
		my $endata = $cbc->decrypt($val);
		$endata = pack('H*', $endata);
		$endata = thaw($endata);
		my $tmp = $endata->get();
		$jobs->{$tmp->{id}} = $endata;
		if( $debug ) {
		    logit("INIT job: ".$tmp->{id}."\n");
		}
		$nrjobs++;
	    }
	}
    }
    logit("init_joblist: successfully loaded $nrjobs jobs\n");
}

sub statentry {
    my ($handle, $suser, $pass, $data) = @_;

    logit("statentry...\n");
    if( $debug ) {
	logit("called statentry suser=$suser, pass=$pass\n");
    }

    my $dn = "uid=$suser,$basedn";
    
    my $msg = rebind( dn=> $dn,
		      password => $pass );
    if( $msg->code != 0 ) {
	logit("statentry: unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	error($handle, "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	return;
    }

    $data = pack('H*', $data);
    my $fe = thaw($data);
    my $entry = $fe->get();

    my $fetcher;
    if( $entry->{proto} eq "pop" ) {
	$fetcher = new POPFetch($entry, $maxsize);
    } else {
	$fetcher = new IMAPFetch($entry, $maxsize);
    }

    # ATTENTION!!!! this will fork a fetcher child:
    # if $ret->code starts with OK, we are the child and we have to exit somewhere
    # else we are the parent and have to continue our job.
    if( $debug ) {
	logit("about to stat mail fetching for user <$entry->{user}> from host <$entry->{host}>\n");
    }
    my $ret = $fetcher->fetch(1);

    if( $ret->code() eq "OK" ) {
# --------- if code is OK, than we are a forked child and have to exit later ----------
	# The Error Object is only carrying the information about
	# the success of spawning a new child. The real data is in another
	# Error Object:
	my $fret = $ret->data();
	if( $fret->code() =~ /^OK/ ) {
	    my $statlog = $fret->data();
	    if( $debug ) {
		logit("$statlog\n");
	    }
	    okay($handle, $statlog);
	} else {
	    logit($fret->code()."\n");
	    error($handle, $fret->code()."\n");
	}
	$ldap->unbind;
	exit;
# ---------------------------------- End Of Child -------------------------------------
    } else {
	#logit($ret->code()."\n");
    }
    
}

sub removeentry {
    my ($handle, $suser, $pass, $data) = @_;
    
    my $cbc;
    my $encdata;
    my $backup;

    logit("removeentry...\n");
    if( $debug ) {
	logit("called removeentry suser=$suser, pass=$pass\n");
    }

    $data = pack('H*', $data);
    my $fe = thaw($data);
    my $hash = $fe->get();
    
    my $dn = "uid=$suser,$basedn";
    
    my $msg = rebind( dn=> $dn,
		      password => $pass );
    if( $msg->code != 0 ) {
	logit("removeentry: unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	error($handle, "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	return;
    }
    
    # delete entry in the joblist
    if( defined $jobs->{$hash->{id}} ) {
	$backup = $jobs->{$hash->{id}};
	delete $jobs->{$hash->{id}};
	logit("removeentry: removed job ".$hash->{id}."...\n");
    } else {
	logit("removeentry: no such job...\n");
	error($handle, "no such job");
	return;
    }

    if( keys %$jobs ) {
	# create array of encrypted jobs from active joblist
	my @all;
	foreach my $j ( keys %$jobs ) {
	    my $cbc = new Crypt::CBC ( { 'key' => $key,
					 'cipher' => 'Blowfish' });
	    my $data = freeze($jobs->{$j});
	    $data = unpack('H*', $data);
	    $encdata = $cbc->encrypt($data);
	    push @all, $encdata;
	}
	
	$msg = $ldap->modify( $dn,
			      replace=> { popaccount => \@all }
			      );

	if( $msg->code ) {
	    logit("removeentry: unable to replace: ".ldap_error_text($msg->code)."\n");
	    error($handle, "unable to replace: ".ldap_error_text($msg->code)."\n");
	    $jobs->{$hash->{id}} = $backup;
	    return;
	}
	
    } else {
	$msg = $ldap->modify( $dn,
			      delete =>  [ "popaccount" ]
			      );
	if( $msg->code ) {
	    logit("removeentry: unable to delete: ".ldap_error_text($msg->code)."\n");
	    error($handle, "unable to delete: ".ldap_error_text($msg->code)."\n");
	    $jobs->{$hash->{id}} = $backup;
	    return;
	}
    }

    logit("removeentry: successfully removed entry\n");
    okay($handle, "removed entry");
}

sub newentry {
    my ($handle, $suser, $pass, $data) = @_;

    my $cbc;
    my $encdata;
    my $backup;

    logit("newentry...\n");
    if( $debug ) {
	logit("called newentry suser=$suser, pass=$pass\n");
    }

    $data = pack('H*', $data);
    my $fe = thaw($data);
    my $hash = $fe->get();

    my $dn = "uid=$suser,$basedn";

    my $msg = rebind( dn=> $dn,
		      password => $pass );
    if( $msg->code != 0 ) {
	logit("newentry: unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	error($handle, "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	return;
    }

    # insert/override new entry in the joblist
    $backup = $jobs->{$hash->{id}} if defined $jobs->{$hash->{id}};
    $jobs->{$hash->{id}} = $fe;
    logit("newentry: added job ".$hash->{id}."...\n");
    
    # create array of encrypted jobs from active joblist
    my @all;
    foreach my $j ( keys %$jobs ) {
	$cbc = new Crypt::CBC ( { 'key' => $key,
				  'cipher' => 'Blowfish' });
	my $data = freeze($jobs->{$j});
	$data = unpack('H*', $data);
	$encdata = $cbc->encrypt($data);
	push @all, $encdata;
    }

    $msg = $ldap->search( base=> $basedn,
			  filter=> "(&(uid=$suser)(popaccount=*))",
			  attrs=> [ 'popaccount' ] );
    
    # delete popaccounts, if there are some avalable
    if( $msg->count ) {
	$msg = $ldap->modify( $dn,
			      delete=> { popaccount => [] }
			      );
	if( $msg->code ) {
	    logit("newentry: unable to delete: ".ldap_error_text($msg->code)."\n");
	    error($handle, "unable to delete: ".ldap_error_text($msg->code)."\n");
	    $jobs->{$hash->{id}} = $backup if defined $backup;
	    return;
	}
    }

    $msg = $ldap->modify( $dn,
			  add=> { popaccount => \@all }
			  );
    if( $msg->code ) {
	# shit! disaster! end of life!
	# let's dump out all entries so that not all the data is gone
	# UNDO:
	if( defined $backup ) {
	    $jobs->{$hash->{id}} = $backup;
	} else {
	    delete $jobs->{$hash->{id}};
	}
	# DUMP
	my $tmpdir = tempdir( CLEANUP => 0 );
	foreach my $j ( @all ) {
	    my ($fh, $filename) = File::Temp::tempfile( "fetchd_job-XXXXXXXXX", DIR => $tmpdir);
	    binmode $fh;
	    print $fh "$j";
	    close($fh);
	}
	logit("newentry: unable to add: ".ldap_error_text($msg->code)."\nDumped all jobs to $tmpdir\n");
	error($handle, "unable to modify jobs in LDAP, Dumped all jobs to directory $tmpdir.<br>Use <pre>/usr/sbin/recover-jobs.pl $tmpdir password</pre> to restore them manually\n");
	return;
    }

    logit("newentry: successfully added new entry\n");
    okay($handle, "added new entry");
}

sub getentries {
    my ($handle, $suser, $pass) = @_;
    
    logit("getentries...\n");
    if( $debug ) {
	logit("called getentry suser=$suser, pass=$pass\n");
    }

    my $dn = "uid=$suser,$basedn";
    
    my $msg = rebind( dn=> $dn,
		      password => $pass );
    if( $msg->code != 0 ) {
	logit("getentries: unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	error($handle, "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	return;
    }
    
    my $nrjobs = 0;
    foreach my $job ( keys %$jobs ) {
	my $data = freeze($jobs->{$job});
	$data = unpack('H*', $data);
	$handle->write("$data\n");
	$nrjobs++;
    }
    $handle->write("\0\n");
    logit("getentries: successfully sent $nrjobs jobs to user interface\n");
}

sub fetchallnow {
    my ($handle, $suser, $pass) = @_;
    
    logit("fetchallnow...\n");
    if( $debug ) {
	logit("called fetchallnow suser=$suser, pass=$pass\n");
    }

    my $dn = "uid=$suser,$basedn";
    
    my $msg = rebind( dn=> $dn,
		      password => $pass );
    if( $msg->code != 0 ) {
	logit("getentries: unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	error($handle, "unable to bind to LDAP Server: ".ldap_error_text($msg->code)."\n");
	return;
    }
    
    # order jobs: The losers will be the winners this time
    my $jobstack = undef;
    for(my $i=$npri; $i>0; $i--) {
	foreach my $job ( keys %$jobs ) {
	    if( defined $loser->{$job} && $loser->{$job} == $i ) {
		if( $debug >= 3 ) {
		    my $j = $jobs->{$job}->get();
		    logit("Pushing job $j->{user}\@$j->{host} to the top, importance = $i.\n", CYAN);
		}
		push @$jobstack, $job;
	    }
	}
    }
    # push the rest to the bottom
    foreach my $job ( keys %$jobs ) {
	if( ! defined $loser->{$job} ) {
	    push @$jobstack, $job;
	}
    }

    my $status = "";
    foreach my $job ( @$jobstack ) {
	my $j = $jobs->{$job}->get();
	
	my $user      = $j->{user};
	my $pass      = $j->{passwd};
	my $host      = $j->{host};
	my $localaddr = $j->{localaddr};
	my $proto     = $j->{proto};
	my $multidrop = $j->{multidrop};

	my $fe;
	if( $proto eq "pop" ) {
	    $fe = new POPFetch($j, $maxsize);
	} else {
	    $fe = new IMAPFetch($j, $maxsize);
	}
	
	# ATTENTION!!!! this will fork a fetcher child:
	# if $ret->code starts with OK, we are the child and we have to exit
	# somewhere else we are the parent and have to continue our job.
	if( $debug ) {
	    logit("about to fetch mail for user <$user> from host <$host>\n", BLUE);
	}
	my $ret = $fe->fetch();
	my $bak = $loser->{$job};
	if( defined $bak ) {
	    delete $loser->{$job};
	}
	if( $ret->code() eq "OK" ) {
# ----------- if code is OK, than we are a forked child and have to exit later ----------
	    # The Error Object is only carrying the information about
	    # the success of spawning a new child. The real data is in another
	    # Error Object:
	    &delThread("FETCHALLNOW, FETCHER FINISHED for $user\@$host"); # fetcher finished
	    my $fret = $ret->data();
	    if( $fret->code() =~ /^OK/ ) {
		my $msga = $fret->data();
		deliver($j, $msga);
	    } else {
		&delThread("FETCHALLNOW, FETCHER FAILED for $user\@$host"); # sender not started, remove his threadstatus
		logit($fret->code()."\n", YELLOW);
	    }
	    exit;
# ---------------------------------------------- End Of Child -------------------------------------
	} else {
	    if( $ret->code() =~ /max Threads reached/ ) {
		$status .= "Job $user\@$host not fetched: reached max Thread limit, try again!\n";
		if( $debug >= 3 ) {
		    logit("Adding job $user\@$host to the loser queue...", CYAN);
		}
		if( defined $bak && $bak < $npri ) {
		    if( $debug >= 3 ) {
			logit(" increasing the importance level... ", CYAN);
		    }
		    $loser->{$job} = ++$bak;
		} else {
		    $loser->{$job} = defined $bak ? $bak : 1;
		}
		if( $debug >= 3 ) {
		    logit(" level=$loser->{$job}\n", CYAN);
		}
	    }
	}
	if( ! defined $loser->{$job} ) {
	    $status .= "PROCESSED $user\@$host, check log for errors\n";
	}
	if( $debug >= 2 ) {
	    if( ! defined $loser->{$job} ) {
		logit("*** SUCCESSFULLY PROCESSED $user\@$host ***\n", MAGENTA);
	    }
	}
    }
    
    logit("fetchallnow: ".$status);
    okay($handle, $status);
}

sub AUTOLOAD {
    my $prog = $AUTOLOAD;

    print $prog."\n";

    $prog =~ s/^.*:://;
    logit("AUTOLOAD: UNKNOWN command: $prog\n");
    #error($handle, "UNKNOWN command: $prog\n");
}

sub addThread {
    my $caller = shift || "UNKNOWN";

    # increases thread num with 2 ( fetching + sending)
    my $tmp = "";
    $sem->op(0, 0, 0,  # wait until semaphore is unlocked and
	     0, 1, 0); # lock
    shmread($shm_id, $tmp, 0, $shm_length);
    $tmp =~ s/\0//g;
    $tmp =~ /(\d+)\|(\d+)/;
    my $cur = $1;
    my $max = $2;
    if( $cur >= $max*2 ) {
	logit("ERROR: max threads reached!!!\n", YELLOW);
	$sem->op(0, -1, 0); # free semaphore
	return 0;
    }
    $cur+=2;
    logit("ADDTHREAD($caller): THREADNUM=$cur\n",RED) if $debug >=4;
    shmwrite($shm_id, "$cur|$max", 0, $shm_length);
    $sem->op(0, -1, 0); # free semaphore
    return 1;
}

sub delThread {
    my $caller = shift || "UNKNOWN";

    my $tmp;
    $sem->op(0, 0, 0,  # wait until semaphore is unlocked and
	     0, 1, 0); # lock
    shmread($shm_id, $tmp, 0, $shm_length);
    if( ! defined $tmp ) {
	# no shared mem left, maybe we are the last living
	# child and our parent is already gone to lunch...
	# So hurry up, leave and finish your job...
	return;
    }
    $tmp =~ s/\0//g;
    $tmp =~ /(\d+)\|(\d+)/;
    my $cur = $1;
    my $max = $2;
    if( $cur == 0 ) {
	logit("ERROR: numThreads already 0, this should not happen!!!\n", RED);
	$sem->op(0, -1, 0); # free semaphore
	return;
    }
    $cur--;
    logit("DELTHREAD($caller): THREADNUM=$cur\n",RED) if $debug >=4;
    shmwrite($shm_id, "$cur|$max", 0, $shm_length);
    $sem->op(0, -1, 0); # free semaphore
}

sub handlesigterm {
    do_exit();
}

sub do_exit {
    logit("cleaning up...\n");
    my $code = shift || 0;
    my $shmid_ds = {};
    my $ret = shmctl($shm_id, IPC_RMID, $shmid_ds);
    $sem->remove if defined $sem;
    $ldap->unbind;
    logit("cul8ter...\n");
    exit($code);
}

sub handlesigalarm {
    if( defined $jobs ) {
	my $timenow   = strftime("%s",localtime);
	my $dayofweek = strftime("%u",localtime);
	my $hourofday = strftime("%H",localtime);
	
	# order jobs: The losers will be the winners this time
	my $jobstack = undef;
	for(my $i=$npri; $i>0; $i--) {
	    foreach my $job ( keys %$jobs ) {
		if( defined $loser->{$job} && $loser->{$job} == $i ) {
		    if( $debug >= 3 ) {
			my $j = $jobs->{$job}->get();
			logit("Pushing job $j->{user}\@$j->{host} to the top, importance = $i.\n", CYAN);
		    }
		    push @$jobstack, $job;
		}
	    }
	}
	# push the rest to the bottom
	foreach my $job ( keys %$jobs ) {
	    if( ! defined $loser->{$job} ) {
		push @$jobstack, $job;
	    }
	}
	
	my $cnt = 0;
	foreach my $job ( @$jobstack ) {
	    my $j = $jobs->{$job}->get();
	    
	    # an interval of 0 indicates "ignore job"
	    next if $j->{timetable}->[2] == 0;
	    
	    my $user      = $j->{user};
	    my $pass      = $j->{passwd};
	    my $host      = $j->{host};
	    my $localaddr = $j->{localaddr};
	    my $ctime     = $j->{ctime};
	    my $proto     = $j->{proto};
	    my $multidrop = $j->{multidrop};
	    my $hourstart = $j->{timetable}->[0];
	    my $hourend   = $j->{timetable}->[1];
	    my $interv    = $j->{timetable}->[2];
	    
	    my $isactiveday = 0;
	    for(my $i=0; $i<=$#{$j->{weekdays}}; $i++) {
		if( $j->{weekdays}->[$i] == $dayofweek ) {
		    $isactiveday = 1;
		    last;
		}
	    }
	    
	    my $tdiff = int(($timenow - $ctime)/60);
	    if( $tdiff > 0 ) {
		if( ( ($tdiff % $interv) == 0  &&
		      $hourofday >= $hourstart && 
		      $hourofday <= $hourend   &&
		      $isactiveday )           ||
		    defined $loser->{$job} ) # last time we losed, so we ignore whether this is
		                             # our turn or not this time
		{
		    my $fe;
		    if( $proto eq "pop" ) {
			$fe = new POPFetch($j, $maxsize);
		    } else {
			$fe = new IMAPFetch($j, $maxsize);
		    }
		    
		    # ATTENTION!!!! this will fork a fetcher child:
		    # if $ret->code starts with OK, we are the child and we have to exit
		    # somewhere else we are the parent and have to continue our job.
		    if( $debug ) {
			logit("about to fetch mail for user <$user> from host <$host>\n", BLUE);
		    }
		    my $ret = $fe->fetch();
		    my $bak = $loser->{$job};
		    if( defined $bak ) {
			delete $loser->{$job};
		    }
		    if( $ret->code() eq "OK" ) {
# -------------------- if code is OK, than we are a forked child and have to exit later ----------
			# The Error Object is only carrying the information about
			# the success of spawning a new child. The real data is in another
			# Error Object:
			&delThread("ALARM, FETCHER FINISHED for $user\@$host"); # fetcher finished
			my $fret = $ret->data();
			if( $fret->code() =~ /^OK/ ) {
			    my $msga = $fret->data();
			    deliver($j, $msga);
			} else {
			    &delThread("ALARM, FETCHER FAILED for $user\@$host"); # sender not started, remove his threadstatus
			    logit($fret->code()."\n", YELLOW);
			}
			exit;
# ---------------------------------------------- End Of Child -------------------------------------
		    } else {
			if( $ret->code() =~ /max Threads reached/ ) {
			    if( $debug >= 3 ) {
				logit("Adding job $user\@$host to the loser queue...", CYAN);
			    }
			    if( defined $bak && $bak < $npri ) {
				if( $debug >= 3 ) {
				    logit(" increasing the importance level... ", CYAN);
				}
				$loser->{$job} = ++$bak;
			    } else {
				$loser->{$job} = defined $bak ? $bak : 1;
			    }
			    if( $debug >= 3 ) {
				logit(" level=$loser->{$job}\n", CYAN);
			    }
			}
		    }
		    if( $debug >= 2 ) {
			if( ! defined $loser->{$job} ) {
			    logit("*** SUCCESSFULLY PROCESSED $user\@$host ***\n", MAGENTA);
			}
		    }
		}
	    }
	    $cnt++;
	}
	if( $debug ) {
	    logit("*********************** $cnt jobs ***********************\n", GREEN);
	}
    }
    
    alarm($alarm);
}

sub deliver {
    my $job  = shift;
    my $mail = shift;

    my $user      = $job->{user};
    my $localaddr = $job->{localaddr};
    
    my $ms = new SMTPSender($job, $mailhost);

    # ATTENTION!!!! this will fork a sender child:
    # if $ret->code starts with OK, we are the child and we have to exit somewhere
    # else we are the parent and have to continue our job.
    if( $debug ) {
	logit("about to send mail to address <$localaddr> from fetch account <$user>\n", BLUE);
    }
    my $ret = $ms->send($mail);
    
    if( $ret->code() eq "OK" ) {
# -------------------- if code is OK, than we are a forked child and have to exit later ----------
	# The Error Object is only carrying the information about
	# the success of spawning a new child. The real data is in another
	# Error Object:
	&delThread("DELIVER, SENDER FINISHED for $user\@$job->{host}"); # sender is thru...
	my $fret = $ret->data();
	if( $fret->code() ne "OK" ) {
	    logit($fret->code()."\n");
	}
	exit;
# ---------------------------------------------- End Of Child -------------------------------------
    } else {
	if( $ret->code() =~ /no data to send/ ) {
	    &delThread("DELIVER, SENDER FINISHED(no data) for $user\@$job->{host}"); # sender is thru...
	}
	#logit($ret->code()."\n", RED);
    }
}

sub dolock($$) {
    my $user = shift;
    my $host = shift;
    
    my $lfile = "__fetchd_".$user."\@".$host;
    my $ulock = $lockdir."/".$lfile;
    
    if( -l $ulock ) {
	return new Error(undef, "ERROR: $ulock is a symbolic link, that must not happen");
    }

    if ( -e $ulock ) {
	if( ! open(PID, "<$ulock") ) {
	    return new Error(undef, "ERROR: unable to read pid from lock file $ulock");
	}

	my $pid = <PID>;
	chomp($pid);
	close(PID);
	if( ! defined $pid || $pid eq "" ) {
	    return new Error(undef, "ERROR: unable to read pid from lock file $ulock");
	}

	if( ! opendir(PROC, "/proc") ) {
	    return new Error(undef, "ERROR: unable to opendir /proc filesystem");
	}
	my @pids = grep(/[0-9]/, readdir(PROC));
	if( ! defined $pids[0] ) {
	    return new Error(undef, "ERROR: unable to readdir /proc filesystem");
	}
	closedir(PROC);

	# check if lockfile is stale, so we do not need to wait for any
	# fetcher child
	my $isstale = 1;
	foreach my $p ( @pids ) {
	    if( $p eq $pid ) {
		$isstale = 0;
		last;
	    }
	}

	if( ! $isstale ) {
	    return new Error(undef, "ERROR: The mail of user $user is still being fetched");
	} else {
	    logit("removing stale lockfile $ulock\n");
	    unlink($ulock);
	}
    }

    if( ! open(OUT, ">$ulock") ) {
	return new Error(undef, "ERROR: unable to create lock file");
    }
    
    print OUT $$."\n";
    close OUT;

    return new Error(undef, "OK");
}

sub unlock($$) {
    my $user = shift;
    my $host = shift;
    
    my $lfile = "__fetchd_".$user."\@".$host;
    my $ulock = $lockdir."/".$lfile;

    if( -l $ulock ) {
	return new Error(undef, "ERROR(POPFetch): $ulock is a symbolic link, that must not happen!!");
    }

    unlink($ulock);
    return new Error(undef, "OK");
}

