#!/usr/bin/perl
#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: report_quota.pl,v 1.1.1.1 2003/03/25 10:31:40 ray Exp $
#

use strict;
use POSIX qw(strftime);
use lib qw(/srv/www/lib);

use Utils;

my $filter = "";
my $mbody  = "";
my $lang   = "EN";
my $imaplib = "/var/lib/imap";
my %ulist  = ();

my $qwarn = (parse_file("/etc/imapd.conf", "quotawarn:"))[0];

if( ! defined $qwarn || $qwarn eq "" ) {
    print STDERR "unable to read quotawarn from /etc/imapd.conf\n";
    exit;
}

my @allfiles;
readFilesRecursive("$imaplib/quota",\@allfiles);

foreach my $f (@allfiles) {
    if( $f =~ /.*\.(.*)/ ) {
	my $uid = $1;
	if(defined $uid && $uid ne "") {
	    open(Q, "$f");
	    my @qv = <Q>;
	    close(Q);
	    next if ! defined $qv[0];
	    chomp(@qv);
	    my $used  = ($qv[0]/1024);
	    my $total = $qv[1];

	    $ulist{$uid}{limit} = $total;
	    $ulist{$uid}{used}  = int($used);
	    $ulist{$uid}{pused} = int($used/$total*100);
	    $ulist{$uid}{warn}  = $qwarn;
	}
    }
}

my $date = strftime("%a, %d %b %Y %H:%M:%S %z\n",localtime);
    
if( $lang eq "DE" ) {
    $mbody ="To: \$uid
From: mailadmin
Subject: Automatischer Quota Report
Reply-To: mailadmin
Date: $date

Die groesse Ihrer Mailbox hat den im System definierten Warning Level
ueberschritten. Bitte loeschen Sie ein paar Mails.

Quota Report fuer Ihre Mailbox:

Max. Quota verfuegbar = \$ulist{\$uid}{limit} (KBytes)
Quota in Gebrauch     = \$ulist{\$uid}{used} (KBytes)
Quota Warning Level   = \$ulist{\$uid}{warn}%
Quota in Gebrauch     = \$ulist{\$uid}{pused}%
";
} else {
    $mbody ="To: \$uid
From: mailadmin
Subject: Automatic Quota Report
Reply-To: mailadmin
Date: $date

Your mailbox size has reached the system defined quota warning level.
Please delete some mails.

Quota usage report for your mailbox:

Max. Quota available = \$ulist{\$uid}{limit} (KBytes)
Quota used           = \$ulist{\$uid}{used} (KBytes)
Quota warning level  = \$ulist{\$uid}{warn}%
Quota in use         = \$ulist{\$uid}{pused}%
";
}

foreach my $uid (keys %ulist) {
    
    if( $ulist{$uid}{pused} > $ulist{$uid}{warn} ) {
	my $eval = eval "sprintf \"%s\", \"$mbody\"";
	if( ! open(OUT, "| /usr/lib/cyrus/bin/deliver -q -r mailadmin -a cyrus $uid") ) {
	    print STDERR "ERROR sending quota warning to $uid: $!\n";
	}
	print OUT $eval;
	close(OUT);
    }
}

1;
