#! /usr/bin/perl -w
#
# Copyright (c) 2003 SuSE Linux AG Nuernberg, Germany.  
#                         All rights reserved.
#
#
# $Id: addprivaddrbase.pl,v 1.1 2003/05/06 11:37:52 varkoly Exp $
#
BEGIN {
    push @INC,"/srv/www/lib";
}

use strict;
use POSIX;
use Net::LDAP qw(:all);
use Net::LDAP::LDIF;
use File::Temp qw(tempfile);
use Getopt::Long;
use Utils;

$Getopt::Long::error = 0;
my $baseDN = "";
my $CYRUSPW = "";
my $ldaphost = "localhost";
my $ldapport = 389;
my $ldap;

if( $#ARGV >= 0 ) {
    GetOptions("base=s" => \$baseDN,
	       "passwd=s" => \$CYRUSPW,
	       );
} else {
    do_help();
    exit;
}

if( $baseDN eq ""){
    print STDERR "The option --base is required\n";
    do_help();
    exit;
}
if( $CYRUSPW eq ""){
    print STDERR "The option --passwd is required\n";
    do_help();
    exit;
}

my $admin_dn = "uid=cyrus,".$baseDN;
my @dflt_timezone = parse_file("/etc/sysconfig/clock", "TIMEZONE=");
my $default_timezone = $dflt_timezone[0];
$default_timezone =~ s/\"//g;

$ldap = Net::LDAP->new($ldaphost, port => $ldapport, version => 3 );
my $res = $ldap->bind( $admin_dn, password => $CYRUSPW );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP authentication failed!");
}
update_users();
exit 0;

sub do_help{
    print STDERR "\naddprivaddrbase.pl --base <ldapbase> --passwd <cyrus-password>\n\n";
}



sub update_users{
    # read all users
    my $res = $ldap->search( base => $baseDN,
		   scope => "one",
		   filter => "(&(objectClass=posixAccount)(objectClass=SuSEemailObject))",
		   attrs => [ "uid" , "suseTimezone", "openldapaci"]
		);
    if( $res->code != 0 ){
	print STDERR "Error reading User objects\n";
	LDAPerror($res);
	return;
    }
    my $attrs;
    $attrs = [ "objectclass" => ["top","organizationalUnit" ],
	       "ou" => "addr" ];

    my @users = $res->entries();

    foreach my $act_user ( @users ){
	my $update = 0;
	my $dn = $act_user->dn();
	my $addrdn = "ou=addr,$dn";
	$res = $ldap->search( base => $addrdn,
			      scope => "base",
			      filter => "(objectclass=*)",
			      attrs => "ou" );
	if($res->code() == LDAP_NO_SUCH_OBJECT ){
	    print STDERR "    Adding ou=addr for $addrdn\n";
	    $res = $ldap->add($addrdn, attrs => $attrs);
	}else{
	    print STDERR "    User $dn does not need an update\n";
	}
    }
}

sub LDAPerror($){
    my $res = shift;
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "    Error message:".$res->error()."\n";
	if($res->server_error()){
	    print STDERR "    Additional info:".$res->server_error()."\n";
	}
    }
}
