#!/bin/bash
#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: backup.sh,v 1.4 2003/10/24 08:07:54 varkoly Exp $
#

LDAPDATA="/tmp/imapold.ldif"
POSTGRES_DATA="/tmp/skydb.sql"
MAILBOXES_DB="/tmp/mailboxes.db.txt"
POPACCOUNTS="/tmp/fetchjobs"
CACERT="/usr/ssl/"
BACKUP="/etc/imap $LDAPDATA $MAILBOXES_DB $POSTGRES_DATA $POPACCOUNTS $CACERT"
BACKUP="$BACKUP /var/imap /var/spool/imap/ /usr/sieve/ /var/named /etc/named.conf"

RELFILE="/etc/IMAP-release"
BASEDN=$(awk 'BEGIN { IGNORECASE=1; } /^base/  { print $2; next; } { next; }' \
    < /etc/openldap/ldap.conf)

TAR_OPT=""
BACKUP_FILE=""
USAGE="usage: $0 [-t <special tar options>] <backup file>"

if [ "$UID" -ne "0" ]; then
    echo "$0 must be started as user root!!!"
    echo "Exiting..."
    exit 1
fi

if [ -f /var/www/htdocs/cacert.der ]; then
    BACKUP="$BACKUP /var/www/htdocs/cacert.der"
fi

if [ ! -f $RELFILE ]; then
    echo "Unable to find $RELFILE. This is no SuSE Linux School Server"
    echo
    echo "  This tool in indented only for migrating an SuSE Linux School Server"
    echo "  to SuSE Linux Openexchange Server 4. It is not capable of making normal"
    echo "  backups of Your SuSE Linux Openexchange Server."
    echo
    exit 1
fi

VERSION=`grep "^VERSION" $RELFILE`
VERSION=`echo $VERSION | sed -e 's/^VERSION.*=.*\([0-9]\+.[0-9]\+\)/\1/'`
if [ $VERSION != "3.1" ]; then
    echo "This is no SuSE Linux School Server 3.1"
    echo
    echo "  This tool in indented only for migrating an SuSE Linux School Server"
    echo "  to SuSE Linux Openexchange Server 4. It is not capable of making normal"
    echo "  backups of Your SuSE Linux Openexchange Server."
    echo
    exit 1
fi

if [ $# -lt 1 ]; then { echo "$USAGE"; exit 0; }; fi

TEMP=`getopt -o t: -- "$@"`
eval set -- "$TEMP"
while true; do
    case "$1" in
	-t)
	    TAR_OPT=$2;
	    shift 2
	    ;;
	--)
	    shift
	    break
	    ;;
    esac
done

BACKUP_FILE=$1
if [ -z $BACKUP_FILE ]; then { echo "$USAGE"; exit 1; }; fi

PASSWORD=""
OIFS=$IFS
IFS=""
while [ -z "$PASSWORD" ]; do
    read -a PASSWORD -p "Enter Cyrus Password: " -r -s
    echo
done
IFS=$OIFS

rccyrus	stop
rcnamed stop
rcldap start
echo "Saving fetchd jobs ..."
mkdir -p $POPACCOUNTS
cd $POPACCOUNTS
/usr/bin/perl -e '
    use strict;
    use Storable qw(thaw freeze);
    use Crypt::Blowfish;
    use Crypt::CBC;
    use Net::LDAP;

    # read master key and create cipher
    if( open(KEY, "/etc/imap/bf_key") == 0 ) {
	die "can not read bf_key";
    }
    my $key;
    sysread KEY,$key,16;
    close KEY;
    my $cipher = new Crypt::Blowfish $key;

    my $ldap = Net::LDAP->new("localhost", port => 389, version => 3);
    my $res = $ldap->search( base => "", 
			     scope => "base", 
			     filter => "objectclass=*", 
			     attrs => ["namingContexts"]
			   );
    if( $res->code != 0){
	die("reading BaseDN failed");
    }
    if( $res->count != 1){
	die("reading BaseDN failed");
    }
    my $rootdse = $res->entry(0);
    my $basedn = $rootdse->get_value("namingContexts");
    my $msg = $ldap->search( base=> $basedn,
			    filter=> "(&(uid=cyrus)(popaccount=*))",
			    attrs=> [ "popaccount" ] );
    my $nrjobs = 0;
    my $fh;
    foreach my $entry ($msg->all_entries) {
	foreach my $attr ($entry->attributes) {
	    foreach my $val ( $entry->get_value($attr) ) {
		# get popaccounts out, decrypt, unfreeze and sync with joblist
		my $cbc = new Crypt::CBC ( $cipher );
		my $endata = $cbc->decrypt($val);
		$endata = pack("H*", $endata);
		open($fh,">fetchjob_$nrjobs") || die( "Unable to open: $!" );
		print $fh $endata;
		close($fh);
		$nrjobs++;
	    }
	}
    }
'
cd -

echo "Saving LDAP database ..."
ldapsearch -x -LLL -D "uid=cyrus,$BASEDN" -w "$PASSWORD" -b "$BASEDN" -s sub "+" "*" > $LDAPDATA

echo "Saving PostgreSQL database ..."
su - postgres -c "pg_dump -d SkyDB > $POSTGRES_DATA"

echo "Saving Cyrus IMAP mailboxes.db ..."
su - cyrus -c "ctl_mboxlist -d > $MAILBOXES_DB"

echo "Creating archive ..."
tar cpf$TAR_OPT $BACKUP_FILE $BACKUP

echo "Cleanup up ..."
rm -rf $LDAPDATA $POSTGRES_DATA $POP_ACCOUNTS $MAILBOXES_DB


echo "Backup completed"
