#! /bin/bash
#
# Copyright (c) 2003 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: bdb2ldbm.sh,v 1.1 2003/05/06 11:37:52 varkoly Exp $
#
# This tool is for switching the default database backend of openldap2 from 
# bdb to ldbm and converting the required database to the new format.
#

SLAPD_CONF=/etc/openldap/slapd.conf
TEMPDIR=`mktemp -d /tmp/ldapbak.XXXXXX`
LOGFILE="$TEMPDIR/slapadd.log"
BDB_BACKENDS=""

BDB_BACKENDS=`grep "^database[[:space:]]\+bdb" $SLAPD_CONF`
if [ -z "$BDB_BACKENDS" ]; then
    echo "Didn't find any bdb backends in $SLAPD_CONF. Aborting...";
    exit
fi

if [ -x "/usr/bin/db_recover" ]; then
    rcldap stop
    DIRECTORIES=`grep ^directory $SLAPD_CONF | awk '{print $2}'`
    echo "Running db_recover for all bdb backends"
    for i in $DIRECTORIES; do
        if [ -f $i/id2entry.bdb ]; then
            db_recover -v -h $i
        fi
    done
    echo
else
    echo "  Could not find the db_recover utility, please install the db-utils package."
    exit -1
fi

BACKENDS=`grep ^database $SLAPD_CONF | awk '{print $2}'`

num=0
echo "Saving LDAP data"
for i in $BACKENDS; do
	num=$((num+1));
	if [ "x$i" = "xbdb" ]; then
		slapcat -f $SLAPD_CONF -n $num -l $TEMPDIR/ldapbak.ldif.$num
		echo "    $TEMPDIR/ldapbak.ldif.$num"
	fi
done
echo
echo "Cleaing up backend directories"

for i in $DIRECTORIES; do
	mkdir -p $TEMPDIR/$i
	mv $i/*.bdb $TEMPDIR/$i
	mv $i/log.* $TEMPDIR/$i
	mv $i/__db.* $TEMPDIR/$i
done

echo "Updating configuration file $SLAPD_CONF"
cp -v $SLAPD_CONF $TEMPDIR/slapd.conf.bdb

sed -e 's;^database\([[:space:]]\+\)bdb;database\1ldbm;' \
    -e 's;^checkpoint.*;;' $TEMPDIR/slapd.conf.bdb > $SLAPD_CONF

echo "Restoring LDAP data"
for i in $TEMPDIR/ldapbak.ldif.*; do
	n=`echo $i | cut -d . -f 4`
	slapadd -n $n -f $SLAPD_CONF -v -l $i 2>> $LOGFILE
done
echo "  The conversation from back-bdb to back-ldbm is finished. You can find a"
echo "  backup of your original data at $TEMPDIR "

rcldap start

