#! /usr/bin/perl -w
#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: cmp_lang_ini.pl,v 1.2 2003/05/06 11:36:55 varkoly Exp $
#

use strict;

my $debug = 0;

my $L1;
my $L2;

open($L1, "$ARGV[0]") || die "unable to open $ARGV[0]: $!";
open($L2, "$ARGV[1]") || die "unable to open $ARGV[1]: $!";

exit(1) if syntax_ok($L1);
exit(1) if syntax_ok($L2);

seek($L1, 0, 0);
seek($L2, 0, 0);

my @l1 = <$L1>;
my @l2 = <$L2>;

close($L1);
close($L2);

chomp(@l1);
chomp(@l2);

print "scanning $ARGV[0]...\n";
my $l1hash = make_hash(\@l1);

print "scanning $ARGV[1]...\n";
my $l2hash = make_hash(\@l2);

print "$ARGV[0] has ".keys(%$l1hash)." sections\n";
print "$ARGV[1] has ".keys(%$l2hash)." sections\n";

if( keys(%$l1hash) != keys(%$l2hash) ) {
    print "Number of sections differ!\n";
    exit(1);
}

foreach my $sect ( keys %$l1hash ) {
    if( ! defined($l2hash->{$sect}) ) {
	print "$sect missing in $ARGV[1]!\n";
	last;
    }
    foreach my $key ( keys %{$l1hash->{$sect}} ) {
	my $key2 = $l2hash->{$sect}->{$key};
	if( ! defined $key2 ) {
	    print "MISSING \e[31m$key\e[0m in section \e[31m$sect\e[0m in $ARGV[1]\n";
	    next;
	} else {
	    my $val1 = $l1hash->{$sect}->{$key};
	    my $val2 = $l2hash->{$sect}->{$key};
	    if( $val1 ne $val2 ) {
		print "values of \e[31m$key\e[0m different in section \e[31m$sect\e[0m:\n";
		print "-\e[37m$val1\e[0m\n";
		print "+\e[33m$val2\e[0m\n";
	    }
	}
    }
}

1;

sub syntax_ok {
    my $fd = shift;

    my $ret = 0;
    my $line;
    my $linecounter = 0;
    while( ($line = <$fd>) ) {
	$linecounter++;
	next if $line =~ /^#/;
	next if $line =~ /^\s+/;
	next if $line =~ /^\[[A-Z]*/;
	#print $line;
	if( $line !~ /^[a-zA-Z0-9_-]+\s*=/ ) {
	    print "SYNTAX ERROR IN LINE $linecounter:\n";
	    print $line;
	    $ret++;
	}
    }
    return $ret;
}

sub make_hash {
    my $arr = shift;

    my $ahash;

    for(my $i=0; $i<scalar(@$arr); $i++) {
	my $l = $arr->[$i];
	if( $l =~ /^\[/ ) {
	    $l =~ /\[(.*)\]/;
	    my $sect = $1;
	    print "found section [$sect]...\n" if $debug;
	    while(1) {
		$l = $arr->[++$i];
		last if  $i >= scalar(@$arr);
		if( $l =~ /(^[a-zA-Z0-9_-]+)\s*=\s*(.*)$/ ) {
		    print "adding key <$1> to sect <$sect>\n" if $debug;
		    $ahash->{$sect}->{$1} = $2;
		    #push @{$ahash->{$sect}}, { "$1" => "$2"};
		}
		if(  $l =~ /^\[/ ) {
		    $i--;
		    last;
		}
	    }
	}
	last if  $i >= scalar(@$arr);
	next if $l =~ /^#/;
	next if $l =~ /^\s+/;
    }

    return $ahash;
}
