#!/usr/bin/perl -w
use strict;
use Storable qw(thaw freeze);
use Crypt::Blowfish;
use Crypt::CBC;
use Net::LDAP;

if( ! defined $ARGV[0] ) {
    die "No input file given!\n";
}

if( ! defined $ARGV[1] ) {
    die "No LDAP base given!\n";
}

if( ! defined $ARGV[2] ){
    die "No administrator password given!\n";
}

my $filename = $ARGV[0];
my $basedn = $ARGV[1];
my $passwd = $ARGV[2];

# read master key and create cipher
if( open(KEY, '/etc/imap/bf_key') == 0 ) {
    die "can't read bf_key";
}
my $key;
sysread KEY,$key,16;
close KEY;

my $ldap = Net::LDAP->new("localhost", port => 389, version => 3);
my $res = $ldap->bind("uid=cyrus,".$basedn, password => $passwd);
if( $res->code != 0){
    die("LDAP bind failed");
}

$res = $ldap->search( base => "uid=cyrus,".$basedn,
		      scope => "base",
		      filter => "objectclass=*",
		      attrs => ["popaccount"]
		    );
if( $res->code != 0){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP search failed");
}

my @popaccount = $res->entry(0)->get_value("popaccount");

my $fh;
open $fh, "<$filename" || die("unable to open: $!\n");
my @tmp = <$fh>;
my $content = join( "", @tmp);
my $cbc = new Crypt::CBC ( { 'key' => $key,
			  'cipher' => 'Blowfish' });
$content = unpack('H*', $content);
my $encdata = $cbc->encrypt($content);
push @popaccount, $encdata;
$res = $ldap->modify( "uid=cyrus,".$basedn,
		      replace => { "popaccount" => \@popaccount }
		    );

if( $res->code != 0){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP modify failed");
}
