#! /usr/bin/perl -w
#
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: initgroupid.pl,v 1.2 2003/05/06 11:36:55 varkoly Exp $
#

use strict;
use POSIX;
use Net::LDAP;

if( ! defined $ARGV[0] ) {
    die "No LDAP base given!\n";
}

if( ! defined $ARGV[1] ){
    die "No administrator password given!\n";
}

my $baseDN = $ARGV[0];
my $CYRUSPW = $ARGV[1];
my $ldaphost = "localhost";
my $ldapport = 389;
my $res;
my $ldap;

my $admin_dn = "uid=cyrus,".$baseDN;

$ldap = Net::LDAP->new($ldaphost, port => $ldapport, version => 3 );
$res = $ldap->bind( $admin_dn, password => $CYRUSPW );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP authentication failed!");
}

# read maildomain
$res = $ldap->search( base => "uid=cyrus,".$baseDN,
	       scope => "base",
	       filter => "objectClass=*",
	       attrs => [ "maildomain" ]
	    );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("reading users from LDAP server failed!");
}
my $maildomain = $res->entry(0)->get_value("maildomain");

# read all users
$res = $ldap->search( base => $baseDN,
	       scope => "one",
	       filter => "objectClass=posixAccount",
	       attrs => [ "uid", "gidNumber", "comfireGroupId" ]
	    );
if( $res->code != 0 ){
    die("reading users from LDAP server failed!");
}

my @users = $res->entries();

foreach my $act_user ( @users ){
    my $prim_group = $act_user->get_value("gidNumber");
    my $uid = $act_user->get_value("uid");
    print "Updating user: ".$uid."\n";

    # get groupsmemberships of each user
    my @groupid = ($prim_group);

    my $group_res = $ldap->search( base => $baseDN,
				   scope => "one",
				   filter => "(&(objectclass=posixgroup)(memberUid=$uid))",
				   attrs => [ "gidnumber" ]
				);
    if( $group_res->code != 0 ){
	die("reading groups from LDAP server failed!");
    }
    my @groups = $group_res->entries();
    foreach my $act_group (@groups){
	my $gidnumber = $act_group->get_value("gidnumber");
	if( ! (grep /^$gidnumber$/, @groupid) ){
	    push @groupid, $gidnumber;
	}
    }
    if( $act_user->exists("comfireGroupId") ){
	$act_user->replace( "comfireGroupId" => \@groupid );
    }else{
	$act_user->add( "comfireGroupId" => \@groupid );
    }
    
    my $up_res = $act_user->update( $ldap );
    if( $up_res->code != 0){
	print $up_res->error()." ".$up_res->server_error()."\n";
    }
}
