#! /usr/bin/perl -w
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
#
# $Id: ldapconv.pl,v 1.2 2003/05/06 11:36:55 varkoly Exp $
#
BEGIN {
    push @INC,"/srv/www/lib";
}

use strict;
use POSIX;
use Net::LDAP;
use DBI;
use Utils qw(%CONF_GLOBAL);

if( ! defined $ARGV[0] ) {
    die "No LDAP base given!\n";
}

if( ! defined $ARGV[1] ){
    die "No administrator password given!\n";
}

&Utils::read_Config || die "Unable to read global configuration file!\n";

my $groupwarehost = $::CONF_GLOBAL{'GENERAL'}{'GroupwareHostname'};
if( ! defined $groupwarehost || $groupwarehost eq "" ) {
    $groupwarehost = "localhost";
}

my $baseDN = $ARGV[0];
my $CYRUSPW = $ARGV[1];
my $ldaphost = "localhost";
my $ldapport = 389;
my $db = "comfire";
my $DBuser = "comfire";
my $DBpasswd = "";
my $DBport = "5432";
my $dbh = undef;

my $res;
my $ldap;

my $admin_dn = "uid=cyrus,".$baseDN;

$ldap = Net::LDAP->new($ldaphost, port => $ldapport, version => 3 );
$res = $ldap->bind( $admin_dn, password => $CYRUSPW );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP authentication failed!");
}

print "Converting LDAP information for the Groupware ...\n";
# read maildomain
$res = $ldap->search( base => "uid=cyrus,".$baseDN,
	       scope => "base",
	       filter => "objectClass=*",
	       attrs => [ "maildomain" ]
	    );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("reading users from LDAP server failed!");
}
my $maildomain = $res->entry(0)->get_value("maildomain");

# read all users
$res = $ldap->search( base => $baseDN,
	       scope => "one",
	       filter => "objectClass=posixAccount",
	       attrs => [ "uid", "gidNumber", "vaddress", "alias", "mailenabled", "skyrixGreenConfig" ]
	    );
if( $res->code != 0 ){
    die("reading users from LDAP server failed!");
}

$dbh = DBI->connect("dbi:Pg:dbname=$db;host=$groupwarehost;port=$DBport", 
    	    "$DBuser", "$DBpasswd");  

if( ! $dbh ){  
    my $errs .= $dbh->errstr;  
    die ("Database Connection failed: $errs");  
}
$dbh->{"AutoCommit"} = 0;


my @users = $res->entries();

foreach my $act_user ( @users ){
    my $prim_group = $act_user->get_value("gidNumber");
    my $uid = $act_user->get_value("uid");
    print "Updating user: ".$uid."\n";

    # get groupsmemberships of each user
    my @groupid = ($prim_group);

    my $group_res = $ldap->search( base => $baseDN,
				   scope => "one",
				   filter => "(&(objectclass=posixgroup)(memberUid=$uid))",
				   attrs => [ "gidnumber" ]
				);
    if( $group_res->code != 0 ){
	die("reading groups from LDAP server failed!");
    }
    my @groups = $group_res->entries();
    foreach my $act_group (@groups){
	my $gidnumber = $act_group->get_value("gidnumber");
	if( ! (grep /^$gidnumber$/, @groupid) ){
	    push @groupid, $gidnumber;
	}
    }
    $act_user->add( "comfireGroupId" => \@groupid );
    
    if( lc($act_user->get_value("mailenabled")) ne "ok"){
	my @alias = $act_user->get_value("alias");
	my @reject = ();
	foreach( @alias ){
	    push @reject, $_."@".$maildomain;
	}
	my @vaddress = $act_user->get_value("vaddress");
	foreach( @vaddress ){
	    push @reject, $_;
	}
	push @reject, $uid."@".$maildomain;
	if( scalar(@reject) > 0 ){
	    $act_user->add( "reject" => \@reject );
	}
    }
    my $drafts="drafts";
    my $sent="sent-mail";
    my $spam="spam";
    my $trash="trash";

    my $signature="";
    my $skyConf = $act_user->get_value("skyrixGreenConfig");
    if( (defined $skyConf) && ($skyConf ne "" ) ){
	$act_user->delete("skyrixGreenConfig");
	my @skyConfLines = split /\n/, $skyConf;
	foreach my $line (@skyConfLines){
	    if( $line =~ /\s*signature\s+=\s+\"(.*)\";$/ ){
		$signature = $1;
		$signature =~ s/\\n//g;
	    }
	    if( $line =~ /\s*draftsFolderPath\s+=\s+\"(.*)\";$/ ){
		$drafts = $1;
		$drafts =~ s/^\/inbox\/(.*)$/$1/i;
	    }
	    if( $line =~ /\s*sentFolderPath\s+=\s+\"(.*)\";$/ ){
		$sent = $1;
		$sent =~ s/^\/inbox\/(.*)$/$1/i;
	    }
	    if( $line =~ /\s*spamFolderPath\s+=\s+\"(.*)\";$/ ){
		$spam = $1;
		$spam =~ s/^\/inbox\/(.*)$/$1/i;
	    }
	    if( $line =~ /\s*trashFolderPath\s+=\s+\"(.*)\";$/ ){
		$trash = $1;
		$trash =~ s/^\/inbox\/(.*)$/$1/i;
	    }
	}
    }

    my $up_res = $act_user->update( $ldap );
    if( $up_res->code != 0){
	print $up_res->error()." ".$up_res->server_error()."\n";
    }
    if( $signature ne "" ){
	my $quote_login = $dbh->quote($uid);
        my $serial = $dbh->selectrow_array("select nextval('serial_id')") || undef;
    	my $login = $dbh->selectrow_array("SELECT username FROM signature WHERE username=$quote_login;");
	my $commit = 1;
    	if(! $login){
    	    $signature = $dbh->quote($signature);
    	    my $sql = "INSERT INTO \"signature\" ( username, description, stext, object_id ) VALUES
    	    	    ( $quote_login, 'default', $signature, $serial );";
    	    my $sth = $dbh->prepare($sql);
    	    if( ! $sth ){  
    	        my $errs = $dbh->errstr;
		$commit = 0;
    	    }
    	    my $res = $sth->execute();
    	    if(! $res){
		my $errs = $dbh->errstr;
		$dbh->rollback;
		$commit = 0;
    	    }
	    if( $commit ) {
		$dbh->commit();
	    }
	    my $oid = $dbh->selectrow_array("SELECT object_id FROM signature WHERE username=$quote_login;");
	    $oid = $dbh->quote($oid);
	    my $test = $dbh->selectrow_array("SELECT username FROM usrdata WHERE username=$quote_login");
	    if(! $test){
		$sql = "INSERT INTO \"usrdata\" (username, signature, vcard, quotation, 
		                                 savecopy, htmlview, delete, maxrows, disposition) 
			       VALUES ($quote_login, $oid, 'NO', 'YES', 'YES', 'NO', 'NO', '25', '1');";
		$sth = $dbh->prepare($sql);
		if( ! $sth ){  
		    my $errs = $dbh->errstr;
		    $commit = 0;
		}
		my $res = $sth->execute();
		if(! $res){
		    my $errs = $dbh->errstr;
		    $dbh->rollback;
		    $commit = 0;
		}
		if( $commit ) {
		    $dbh->commit();
		}
	    }
    	}
    }
    system("add_user_comfire", "-al", "comfire", "-uid", $uid, "-H", "$groupwarehost",
		    "-imapadminpwd", "$CYRUSPW", "-sent", "$sent",
		    "-drafts", "$drafts", "-spam", "$spam", "-trash", "$trash");
}

$dbh->disconnect();

