#!/usr/bin/perl -w
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: rename_domain.pl,v 1.4 2003/10/24 08:07:54 varkoly Exp $
#
# Files/Locations to modify:
#
# /etc/openldap/slapd.conf
# /etc/openldap/ldap.conf
# /etc/postfix/main.cf
# LDAP Database 
#

use strict;
use Net::LDAP;
use Getopt::Long;
use lib qw(/srv/www/lib);

use Utils;

my $odom = "";
my $ohost = "";
my $basedn = "";
my $ldapserver = "localhost";
my $ldapport = "389";
my $newdom = "";
my $newhost = "";
my $po = undef;
my $outtxt = "";

my @stdfiles = ( "/etc/openldap/slapd.conf",
		 "/etc/openldap/ldap.conf",
		 "/etc/postfix/main.cf",
		 "/etc/resolv.conf",
		 "/etc/httpd/httpd.conf" );

$Getopt::Long::error = 0;

if( $#ARGV >= 0 ) {
    GetOptions("oldhost=s" => \$ohost,
	       "olddomain=s" => \$odom,
	       "newhost=s" => \$newhost,
	       "newdomain=s" => \$newdom,
	       "printold" => \$po
	       );
} else {
    do_help();
    exit(1);
}

if( defined $po ) {
    open(IN,"hostname -f|") || die "unable to exec hostname -f";
    my $hn = <IN>;
    close(IN);
    chomp($hn);
    my @ha = split(/\./,$hn);
    print "Determined with 'hostname -f':\n";
    print "Host   = ".$ha[0]."\n";
    print "Domain = ";
    for(my $i=1; $i<=$#ha; $i++) {
	print $ha[$i].($i == $#ha ? "" : ".");
    }
    print "\n";
    print "Determined with 'grep -i ^base /etc/openldap/ldap.conf':\n";
    open(IN,"grep -i ^base /etc/openldap/ldap.conf|") || die "unable to exec grep";
    my $base = <IN>;
    close(IN);
    chomp($base);
    $base =~ s/^base\s+//i;
    print "BaseDN = $base\n";
    exit(0);
}

if( ( ! defined $ohost || $ohost eq "" ) ||
    ( ! defined $odom || $odom eq "" ) ||
    ( ! defined $newhost || $newhost eq "" ) ||
    ( ! defined $newdom || $newdom eq "" ) ) {
    do_help();
    exit(1);
}

my $CYRUSPW="";
system('/bin/stty -echo');
print "Enter cyrus Password: ";
chomp($CYRUSPW = <STDIN>);
system('/bin/stty echo');


my $newdn = $newdom;
$newdn =~ s/\./,dc=/g;
$newdn = "dc=".$newdn;

# read basedn
($basedn, $ldapserver, $ldapport) =
    parse_file("/etc/openldap/ldap.conf", "BASE", "HOST", "PORT");

foreach my $cf ( @stdfiles ) {
    rename_domain($cf);
}
do_ldif("ldapdata_new.ldif");

$outtxt = "#!/bin/bash

CYRUS_PASSWORD=\"$CYRUSPW\"

# shut down services
rcapache stop
rcpostfix stop
rccyrus stop
rcldap stop

# backup old data
";

foreach my $cf ( @stdfiles ) {
    $cf =~ /^.*\/(.*)/;
    $outtxt .= "cp -f $cf $1.backup\n";
}

$outtxt .= "cp -arf /var/lib/ldap ldap.backup\n";

foreach my $cf ( @stdfiles ) {
    $outtxt .= "cp -f $cf.new $cf\n";
}

$outtxt .= "rm -f /var/lib/ldap/*
rcldap start
sleep 3
ldapadd -x -v -Duid=cyrus,$newdn -w \$CYRUS_PASSWORD -f ldapdata_new.ldif
if [ \$? -ne 0 ]; then
    echo \"failed to populate LDAP Server with new data\"
    exit
fi
ldapsearch -x uid=cyrus
if [ \$? -ne 0 ]; then
    echo \"unable to find user cyrus in LDAP directory\"
    exit
fi

/sbin/SuSEconfig

# starting services
rcapache start
rcpostfix start
rccyrus start
rcldap start
";

open(OUT,">rename.sh") || die "unable to create rename.sh";
print OUT $outtxt;
close(OUT);
chmod(0755, "rename.sh");

print<<EOF;

Now execute rename.sh using "sh rename.sh" within this directory.

************************************************************************
** Don\'t forget to Export the modified data for the nameserver, if you
** are using the builtin nameserver function of SuSE Linux School Server!
************************************************************************

EOF

1;

sub do_help {
    print<<EOF;
Syntax: $0 -oldhost oldhost -olddomain olddomain -newhost newhost -newdomain newdomain
        where oldhost and newhost is only the host part of the name.
        Example:
               Host+Domainname: mail.domain.com
               Hostpart = mail
               Domain   = domain.com
        $0 -printold
        Tries to determine the old settings and prints it to STDOUT
EOF
}

sub do_ldif {
    my $dest = shift;
    open(IN, "/usr/bin/ldapsearch -L -x -Duid=cyrus,$basedn -w $CYRUSPW|")
	|| die "unable to execute /usr/bin/ldapsearch";

    my @ldif = <IN>;
    close(IN);

    my $dc = (split(/dc=/,$newdn))[1];
    $dc =~ s/,//;

    open(OUT,">$dest") || die "unable to write to $dest";
    foreach my $l (@ldif) {
	chomp $l;
	next if $l =~ /creatorsname/i;
	next if $l =~ /modifiersname/i;
	next if $l =~ /createtimestamp/i;
	next if $l =~ /modifytimestamp/i;

	$l =~ s/$odom/$newdom/gi if $l =~ /^zonename:.*/i;
	$l =~ s/$ohost\.$odom/$newhost\.$newdom/gi if $l =~ /^nSRecord:.*/i;
	$l =~ s/$ohost\.$odom/$newhost\.$newdom/gi if $l =~ /^mXRecord:.*/i;
	$l =~ s/$ohost\.$odom/$newhost\.$newdom/gi if $l =~ /^sOARecord:.*/i;
	$l =~ s/$ohost\.$odom/$newhost\.$newdom/gi if $l =~ /^pTRRecord:.*/i;
	$l =~ s/$ohost/$newhost/gi if $l =~ /relativeDomainName(?:\:|=)\s*$ohost/i;

	$l =~ s/$basedn/$newdn/gi if $l =~ /^dn:.*/i;
	$l =~ s/$basedn/$newdn/gi if $l =~ /^member:.*/i;
	$l =~ s/$basedn/$newdn/gi if $l =~ /^mailDeliveryProgram:.*/i;
	$l =~ s/^dc:.*/dc: $dc/ if $l =~ /^dc:.*/;
	$l =~ s/maildomain:.*/maildomain: $newdom/i if $l =~ /^maildomain:.*/i;
	$l =~ s/imapserver:.*/imapserver: $newhost\.$newdom/i
	    if $l =~ /^imapserver:.*/i;
	$l =~ s/smtpserver:.*/smtpserver: $newhost\.$newdom/i
	    if $l =~ /^smtpserver:.*/i;
	$l =~ s/mail:\s+(.+\@).*/mail: $1$newdom/i if $l =~ /^mail:.*/i;
	print OUT "$l\n";
    }
    close(OUT);
}

sub rename_domain {
    my $cfile = shift;

    open(IN, "$cfile") || die "unable to open $cfile";
    my @rc = <IN>;
    close(IN);
    
    open(OUT,">$cfile.new") || die "unable to write to $cfile.new";
    foreach my $l (@rc) {
	chomp $l;
	$l =~ s/$basedn/$newdn/g if $l =~ /.*$basedn.*/;
	$l =~ s/$ohost\.$odom/$newhost\.$newdom/ if $l =~ /.*$ohost\.$odom.*/;
	$l =~ s/$odom/$newdom/g if $l =~ /.*$odom.*/;
	print OUT "$l\n";
    }
    close(OUT);
}

