#!/bin/bash
#
# Copyright (c) 2000-2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002 SuSE Linux AG Nuernberg, Germany.  All rights reserved.
#
# $Id: restore.sh,v 1.4 2003/10/24 08:07:54 varkoly Exp $
#

#
# Path, where to extract the backup of old email server installation
#
TMPDIR="/tmp/imapbak"

# DO NOT CHANGE ANYTHING BELOW THIS LINE!!!

OLD_LDIF="$TMPDIR/tmp/imapold.ldif"
NEW_LDIF="$TMPDIR/tmp/imapnew.ldif"
DER_DIR="$TMPDIR/tmp"
RELFILE="/etc/SLOX-release"
BASEDN=$(awk 'BEGIN { IGNORECASE=1; } /^base/  { print $2; next; } { next; }' \
    < /etc/openldap/ldap.conf)

LDIFCONV="/usr/share/doc/packages/imapweb32/tools/ldifconv.pl"
LDAPCONV="/usr/share/doc/packages/imapweb32/tools/ldapconv.pl"
LDAPUPDATE40="/usr/share/doc/packages/imapweb32/tools/ldapupdate40.pl"
UPDATESYSTEMUSER=/opt/comfire/application/scripts/insUpdSystemUser
EN_FETCH="/usr/share/doc/packages/imapweb32/tools/encrypt_fetchentry.pl"

######################## FOLDER RESTORE #################################
i_RESTORE()
{   
    rccyrus stop

    echo "Restoring mails and mail folders";
    cp -a $TMPDIR/var/imap/quota/* /var/lib/imap/quota/
    cp -a $TMPDIR/var/imap/user/* /var/lib/imap/user/
    cp -a $TMPDIR/var/spool/imap/* /var/spool/imap/

    echo "Recreating mailboxes.db"
    rm -f /var/lib/imap/mailboxes.db
    rm -f /var/lib/imap/db/*
    su - cyrus -c "ctl_mboxlist -u < $TMPDIR/tmp/mailboxes.db.txt"
    rccyrus start

}

####################### SIEVE RESTORE ##############################
s_RESTORE()
{
    echo "Restoring mailfilters and vacation data ..."

    cp -a $TMPDIR/usr/sieve/* /var/lib/sieve/

}
######################## FETCHD RESTORE ###############################
f_RESTORE(){
    if [ "$(echo $TMPDIR/tmp/fetchjobs/*)" != "$TMPDIR/tmp/fetchjobs/*" ]; then
	rcfetchd stop
	if [ -z "$PASSWORD" ] ; then
	    read_password
	fi
	for i in $TMPDIR/tmp/fetchjobs/* ; do 
	    $EN_FETCH $i "$BASEDN" "$PASSWORD";
	done
	rcfetchd start
    fi
}


######################## LDAP RESTORE #################################
l_RESTORE()
{
    if [ -z "$PASSWORD" ] ; then
	read_password
    fi

    $LDIFCONV $OLD_LDIF $NEW_LDIF
    if [ -z $NEW_LDIF ]; then
	echo "generation of new LDAP data failed"
	return
    fi
    
    rcldap stop
    echo "Loading LDAP data into LDAP server ..."
    
    rm -f /var/lib/ldap/*
    slapadd -c -l $NEW_LDIF
    if [ $? != 0 ]; then
	echo "adding LDAP data failed, maybe you should try again"
    else
	echo "done"
	rcldap start
    fi
    echo "Waiting 5 seconds for LDAP server to be started."
    sleep 5
    $LDAPCONV "$BASEDN" "$PASSWORD"
    $LDAPUPDATE40 --base "$BASEDN" --passwd "$PASSWORD" --all
    echo "Creating Contacts for all system users"
    $UPDATESYSTEMUSER all_users
}

######################## Groupware Restore ##########################
g_RESTORE()
{
    echo "Restoring Groupware data (Dates, ToDo items and addressbooks)"
    if [ -z "$PASSWORD" ]; then
	read_password
    fi
    createuser -q --no-adduser --no-createdb -U postgres SkyUser
    createdb -q -E LATIN1 -U postgres SkyDB
    psql -q -U postgres SkyDB -f $TMPDIR/tmp/skydb.sql
    echo -e "cyrus\n$PASSWORD" | /opt/comfire/application/scripts/migration SkyDB SkyUser SkyUser
    sleep 5
    dropdb -q -U postgres SkyDB
    dropuser -q -U postgres SkyUser
}


######################## CA RESTORE #################################
c_RESTORE()
{
    cp -av $TMPDIR/usr/ssl/serial* /etc/ssl/
    cp -av $TMPDIR/usr/ssl/index.txt* /etc/ssl
    cp -av $TMPDIR/usr/ssl/CA/usedCA.pem /etc/ssl/CA/
    cp -av $TMPDIR/usr/ssl/CA/ownCA.pem /etc/ssl/CA
    chgrp mail $TMPDIR/usr/ssl/certs/skey.pem
    cp -av $TMPDIR/usr/ssl/certs/skey.pem /etc/ssl/certs/
    cp -av $TMPDIR/usr/ssl/certs/cert.pem /etc/ssl/certs/
    cp -av $TMPDIR/usr/ssl/private/cakey.pem /etc/ssl/private/
    cp -av $TMPDIR/usr/ssl/newcerts /etc/ssl/newcerts/
    mkdir -p /etc/ssl/crl
    cp -av $TMPDIR/usr/ssl/crl/crl.pem /etc/ssl/crl/
    
    # restore cacert for apache
    if [ -f $TMPDIR/var/www/htdocs/cacert.der ]; then
	cp -av $TMPDIR/var/www/htdocs/cacert.der /srv/www/htdocs/
    fi
}

######################## NAMED RESTORE ###############################
n_RESTORE(){
    echo "Restoring name server configuration"
    rcnamed stop
    cp -a $TMPDIR/var/named/* /var/named/
    cp -a $TMPDIR/etc/named.conf /etc/
    rcnamed start
}

# read password from stdin
read_password()
{
    PASSWORD=""
    OIFS=$IFS
    IFS=""
    while [ -z "$PASSWORD" ]; do
            read -a PASSWORD -p "Enter Cyrus Password: " -r -s
            echo
    done
    export PASSWORD
    IFS=$OIFS
}


do_help()
{
    cat<<EOF ;

$0 -x file [-t flags]
    -x extract
    -t with additional flags "flags" for tar

 or

$0 [-i] [-p] [-l] [-f] [-c] [-g] [-n] [-a]
    -i restore only imap folder
    -s restore only sieve mailfilter rules
    -l restore only ldap directory
    -f restore only fetch accounts
    -c restore only CA and certs
    -g restore only Groupware data
    -n restore only DNS data
    -a restore all
EOF
}

checkit()
{
    if [ -d $TMPDIR ]; then
	return 1
    else
	echo "Please extract archive first!"
	return 0
    fi
}

############################# MAIN ######################################

if [ "$UID" -ne "0" ]; then
    echo "$0 must be started as user root!!!"
    echo "Exiting..."
    exit 1
fi

if [ ! -f $RELFILE ]; then
    echo "unable to find $RELFILE. This is no SuSE Linux School Server"
    exit 1
fi

VERSION=`grep "^VERSION" $RELFILE`
VERSION=`echo $VERSION | sed -e 's/^VERSION.*=.*\([0-9]\+.[0-9]\+\)/\1/'`

TAR_OPT=""
BACKUP_FILE=""
for i in x t i s l f c g n; do
    eval OPT_${i#-}="false"
done


if [ $# -lt 1 ]; then do_help; exit 0; fi


TEMP=`getopt -o islfcagnx:t: -- "$@"`
eval set -- "$TEMP"

while true ; do
    case "$1" in
	-x)
	    OPT_x="true"
	    BACKUP_FILE=$2
	    shift 2
	    ;;
	-t)
	    OPT_t="true"
	    TAR_OPT=$2
	    shift 2
	    ;;
	-i|-s|-l|-f|-c|-g|-n)
	    eval OPT_${1#-}="true"
	    shift
	    ;;
	-a)
	    for i in i s l f c g n; do
		eval OPT_${i#-}="true"
	    done
	    shift
	    ;;
	--)
	    break
	    ;;
    esac
done

if [ $OPT_x == "true" ]; then
    if [ ! -e $BACKUP_FILE ]; then echo "$BACKUP_FILE is no file nor tar archive"; exit 1; fi
    mkdir -p $TMPDIR
    echo "Extracting archive to <$TMPDIR>, please wait..."
    tar xpf$TAR_OPT $BACKUP_FILE -C $TMPDIR
fi

for i in i s l f c g n; do
    if eval [ "\$OPT_$i" == "true" ]; then
	checkit || eval $i"_RESTORE"
    fi
done

exit 0
