#! /usr/bin/perl -w
# Copyright (c) 2002-2003 SuSE Linux AG Nuernberg, Germany.
#                         All rights reserved.
#
#
# $Id: wantspamcheck.pl,v 1.1 2003/05/06 11:37:52 varkoly Exp $

use strict;
use Net::LDAP;
use Net::LDAP::Util qw( ldap_error_name
                        ldap_error_text) ;

my $TEMPFAIL = 75;

my $SENDER = $ARGV[0];
if( ! defined $SENDER ) {
    print "sender missing in commandline\n";
    exit($TEMPFAIL);
}

if( ! defined $ARGV[1] ) {
    print "recipient missing in commandline\n";
    exit($TEMPFAIL);
}

my @RECIPIENTS;

for(my $i=1; defined $ARGV[$i]; $i++) {
    #print "$ARGV[$i]\n";
    push @RECIPIENTS, $ARGV[$i];
}

my $ldap = new Net::LDAP("localhost", version => 3);
if( ! defined $ldap ) {
    print "Unable to contact LDAP Server\n";
    exit($TEMPFAIL);
}

my $ret = $ldap->bind();

if( $ret->code != 0 ) {
    print "error binding to LDAP Server: ".ldap_error_text($ret->code)."\n";
    exit($TEMPFAIL);
}

my @MAIL = <STDIN>;

foreach my $rec ( @RECIPIENTS ) {
    my $lp = (split(/\@/,$rec))[0];
    #print "<$lp> -- <$rec>\n";
    my $msg = $ldap->search( base => "dc=suse,dc=de",
			     scope => "one",
			     filter => "(&(objectclass=posixaccount)(|(uid=$lp)(mail=$rec)))",
			     attrs => [ 'spamcheck' ]
			     
			     );
    if( $msg->code != 0 ) {
	print "error searching LDAP Server: ".ldap_error_text($msg->code)."\n";
    }
    my $spamcheck;
    if( $msg->count > 0 ) {
	$spamcheck = $msg->entry(0)->get_value('spamcheck');
    }
    if( defined $spamcheck && $spamcheck eq "on" ) {
	open(OUT, "|/usr/bin/spamc |/usr/sbin/sendmail -i -f \"$SENDER\" $rec");
	foreach my $l ( @MAIL ) {
	    print OUT $l;
	}
	close(OUT);
    } else {
	open(OUT, "|/usr/sbin/sendmail -i -f \"$SENDER\" $rec");
	foreach my $l ( @MAIL ) {
	    print OUT $l;
	}
	close(OUT);
    }
}


