#!/usr/bin/perl
use Net::LDAP;
use Net::IMAP;
use Config::IniFiles;

use strict;
my ($passwd,$ldaphost,$ldapport,$basedn,$maildomain,$report) = @ARGV;

my $ldap = Net::LDAP->new($ldaphost, port=>$ldapport, version => 3);

my $mess = $ldap->bind(
                        dn => "uid=cyrus,$basedn",
                        password => $passwd
                    );

my $imap = new Net::IMAP("localhost", Debug => 0);
my $resp = $imap->login("cyrus", $passwd);

open(OUT,">>$report");
# hier comes the code
$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(&(gidNumber=103)(objectclass=SuSESchoolAccount))',
			attrs	=> ['uid']
		     );

foreach my $entry ( $mess->entries ) {
	$ldap->modify(	dn => $entry->dn,
			delete=> { objectclass =>  "SuSESchoolAccount"}
		     );
}
$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(&(gidNumber=103)(!(objectclass=SuSESchoolWorkstation)))',
			attrs	=> ['uid']
		     );
foreach my $entry ( $mess->entries ) {
	$ldap->modify(	dn => $entry->dn,
			add=> { objectclass =>  "SuSESchoolWorkstation"}
		     );
	print OUT "modified dn : ".$entry->dn."\n";
}

$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(&(!(mail=*))(objectclass=SuSESchoolAccount))',
			attrs	=> ['uid']
		     );

foreach my $entry ( $mess->entries ) {
	my $uid = $entry->get_value('uid');
	$ldap->modify(	dn => $entry->dn,
			add=> { mail =>  "$uid\@$maildomain"}
		     );
	print OUT "modified dn : ".$entry->dn."\n";
}

# repair bad birthday
$mess = $ldap->search (
                        base   => $basedn,
                        scope  => 'one',
                        attrs  => ['birthday','uid'],
                        filter => '(&(uid=*)(birthday=*))'
                        );

foreach my $entry ($mess->all_entries) {
        my $uid      = $entry->get_value('uid');
        my $birthday = $entry->get_value('birthday');
        my $dn       = $entry->dn();

        my $newbirthday = $birthday;
        $newbirthday =~ tr/./-/;
        if( $newbirthday =~ /(\d{2})-(\d{2})-(\d{4})/ ) {
                $newbirthday = "$3-$2-$1";
        }
        $newbirthday =~ /(\d{4})-(\d+)-(\d+)/;
        $newbirthday = sprintf("$1-%02d-%02d",$2,$3);
        print OUT "analysing $uid: birthday befor: $birthday birthday after: $newbirthday\n";
        if($newbirthday ne $birthday) {
                print OUT "modify birthday for $uid\n";
                $ldap->modify(  dn      => $dn,
                                replace => { 'birthday' => $newbirthday }
                        );
        }

}

# do this only if it was beta
if( -e "/var/adm/updates/was_beta") {
	# repair all mailboxes with .
	my $mailbox="user/mailadmin";
	$resp = $imap->delete("user.mailadmin");
	$resp = $imap->create($mailbox);
	$resp = $imap->setacl($mailbox,"cyrus","lrswipcda");
	$resp = $imap->create($mailbox.'/Gesendet');
	$resp = $imap->create($mailbox.'/Papiekorb');
	$mess = $ldap->search(
				base    => $basedn,
				scope   => 'one',
				filter  => 'objectclass=SuSESchoolAccount',
				attrs   => ['uid']
			     );

	foreach my $entry ( $mess->entries ) {
		my $uid = $entry->get_value('uid');
		$resp = $imap->delete("user/$uid/INBOX.Papierkorb");
		$resp = $imap->delete("user/$uid/INBOX.Gesendet");
		$resp = $imap->create("user/$uid/Papierkorb");
		$resp = $imap->create("user/$uid/Gesendet");
		print OUT "modify mailboxes for $uid\n";
	}
	$mess = $ldap->search(
				base    => $basedn,
				scope   => 'one',
				filter  => 'objectclass=SuSESchoolClass',
				attrs   => ['cn']
			     );

	foreach my $entry ( $mess->entries ) {
		my $cn = $entry->get_value('cn');
		$imap->deleteacl($cn,'group:teachers');
		my $msg = $ldap->search(
				base    => $basedn,
				scope   => 'one',
				filter  => "cn=$cn",
				attrs   => ['memberuid']
			     );
		my @members = $msg->entry(0)->get_value('memberuid');
		foreach(@members){
			$msg = $ldap->search(
				base    => $basedn,
				scope   => 'one',
				filter  => "(&(uid=$_)(gidnumber=102))",
				attrs   => ['uid']
			     );
			if($msg->count > 0) {
				$imap->setacl($cn,$_,"lrswipcd");
				print OUT "set rights on $cn for $_\n";
			}
		}
	}

	#repair phpgw_prefs 
	my $langini     = new Config::IniFiles( -file => "/srv/www/lib/imap_lang_DE.ini" );
	my $phpgw_prefs = $langini->val('CREATE_USER','phpgw_prefs');
	my $mysqlc = "mysql -u cyrus -p\'$passwd\' -h mailserver phpgroupware";
	my $mysqla = " UPDATE phpgw_preferences SET preference_value=\'$phpgw_prefs\'";
	system("echo -E \"$mysqla\" | $mysqlc");
}

#repair bad acl settings
$resp = $imap->setacl("workgroups","anyone","");
$resp = $imap->setacl("workgroups","group:users","");
$resp = $imap->setacl("workgroups","group:administration","");
$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(&(objectclass=posixgroup)(!(objectclass=SuSESchoolGroup))(!(objectclass=SuSESchoolClass)))',
			attrs	=> ['cn']
		     );

foreach my $entry ( $mess->entries ) {
	my $cn = $entry->get_value('cn');
	my $mailbox = $cn;
        $resp = $imap->setacl($mailbox,$cn,"");	
        $resp = $imap->setacl($mailbox,"group:$cn","lrswi");	
	print OUT "modified mailbox : $cn\n";
}

$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(objectclass=SuSESchoolGroup)',
			attrs	=> ['cn']
		     );

foreach my $entry ( $mess->entries ) {
	my $cn = $entry->get_value('cn');
	my $mailbox = "workgroups/$cn";
        $resp = $imap->setacl($mailbox,$cn,"");	
        $resp = $imap->setacl($mailbox,"anyone","");	
        $resp = $imap->setacl($mailbox,"group:users","");	
        $resp = $imap->setacl($mailbox,"group:teachers","");	
        $resp = $imap->setacl($mailbox,"group:administration","");	
        $resp = $imap->setacl($mailbox,"group:$cn","lrswi");	
	print OUT "modified mailbox : $cn\n";
}

$mess = $ldap->search(
			base    => $basedn,
			scope	=> 'one',
			filter	=> '(objectclass=SuSESchoolClass)',
			attrs	=> ['cn']
		     );

foreach my $entry ( $mess->entries ) {
	my $cn = $entry->get_value('cn');
	my $mailbox = "$cn";
        $resp = $imap->setacl($mailbox,$cn,"");	
        $resp = $imap->setacl($mailbox,"anyone","");	
	print OUT "modified mailbox : $cn\n";
}


#now we close it all
close(OUT);
$resp = $imap->logout();
$ldap->unbind;
